/*
 * File: CellConstantInputPredictLookup.c
 *
 * MATLAB Coder version            : 2.7
 * C/C++ source code generated on  : 02-Apr-2015 17:30:11
 */

/* Include Files */
#include "rt_nonfinite.h"
#include "CellConstantInputPredict.h"
#include "CellConstantInputPredictLookup.h"
#include "UKFInitialize.h"
#include "UKFStep.h"
#include "computeSOCEstimate.h"
#include "getInitialBatteryState.h"
#include "BHM_emxutil.h"
#include "rdivide.h"
#include "repmat.h"
#include <stdio.h>

/* Function Declarations */
static double rt_roundd_snf(double u);

/* Function Definitions */

/*
 * Arguments    : double u
 * Return Type  : double
 */
static double rt_roundd_snf(double u)
{
  double y;
  if (fabs(u) < 4.503599627370496E+15) {
    if (u >= 0.5) {
      y = floor(u + 0.5);
    } else if (u > -0.5) {
      y = u * 0.0;
    } else {
      y = ceil(u - 0.5);
    }
  } else {
    y = u;
  }

  return y;
}

/*
 * This function does prediction for the following conditions:
 *  1. State is given in weighted samples (eg sigma points). X is nx x N, w
 *  is 1 x N.
 *
 *  2. u is a vector 1 x N, where each sample represents a constant future
 *  input. Samples are equally weighted. Sampling is nested so for each x
 *  sample we simulate all u samples.
 *
 *  3. input says what model to use (constant current, power, or resistance)
 *
 *  4. No future process noise is considered, therefore the only uncertainty
 *  is in the state estimate (X,w) and the inputs (u)
 *
 *  5. Since we are dealing only with constant inputs, we can simulate all
 *  possibilities a priori for all (SOC,u) pairs, and store these in a lookup
 *  table. This function uses the lookup table rather than simulation.
 * Arguments    : double t0
 *                emxArray_real_T *X
 *                const emxArray_real_T *w
 *                const emxArray_real_T *u
 *                char input
 *                emxArray_real_T *EOL
 *                emxArray_real_T *RUL
 *                emxArray_real_T *W
 * Return Type  : void
 */
void CellConstantInputPredictLookup(double t0, emxArray_real_T *X, const
  emxArray_real_T *w, const emxArray_real_T *u, char input, emxArray_real_T *EOL,
  emxArray_real_T *RUL, emxArray_real_T *W)
{
  emxArray_real_T *b_X;
  int i11;
  int loop_ub;
  emxArray_real_T *U;
  double uBlockSize;
  int i;
  emxArray_int32_T *r787;
  double x;
  double d2;
  int i12;
  emxArray_real_T *c_X;
  emxArray_real_T *SOCs;
  boolean_T b_bool;
  static const short iv0[80601] = { 0, 2, 4, 7, 10, 15, 20, 29, 41, 57, 78, 100,
    124, 148, 173, 198, 223, 249, 274, 300, 325, 352, 378, 404, 430, 457, 483,
    509, 536, 562, 588, 615, 641, 668, 694, 720, 747, 773, 800, 826, 853, 879,
    906, 932, 959, 986, 1012, 1039, 1065, 1092, 1118, 1146, 1173, 1200, 1226,
    1253, 1280, 1307, 1333, 1360, 1388, 1414, 1442, 1469, 1496, 1522, 1550, 1576,
    1604, 1631, 1658, 1685, 1712, 1739, 1767, 1794, 1821, 1848, 1875, 1901, 1929,
    1955, 1983, 2010, 2037, 2065, 2092, 2118, 2146, 2173, 2200, 2227, 2255, 2281,
    2309, 2336, 2363, 2390, 2417, 2444, 2473, 2500, 2527, 2555, 2582, 2609, 2637,
    2664, 2692, 2719, 2747, 2774, 2802, 2829, 2857, 2883, 2912, 2940, 2966, 2995,
    3022, 3050, 3078, 3106, 3134, 3162, 3190, 3217, 3245, 3272, 3301, 3328, 3357,
    3385, 3413, 3441, 3469, 3498, 3526, 3554, 3582, 3610, 3638, 3667, 3695, 3722,
    3750, 3779, 3808, 3836, 3865, 3893, 3921, 3950, 3978, 4006, 4035, 4064, 4092,
    4121, 4150, 4178, 4207, 4236, 4264, 4293, 4322, 4351, 4379, 4407, 4437, 4466,
    4495, 4524, 4553, 4582, 4611, 4640, 4669, 4699, 4728, 4757, 4786, 4816, 4845,
    4874, 4904, 4934, 4963, 4993, 5021, 5052, 5082, 5112, 5141, 5171, 5201, 5231,
    5261, 5290, 5321, 0, 2, 4, 7, 10, 14, 20, 28, 40, 56, 76, 98, 121, 145, 170,
    194, 219, 244, 270, 295, 320, 347, 373, 399, 424, 450, 476, 502, 529, 555,
    581, 607, 633, 659, 685, 711, 737, 764, 790, 816, 842, 868, 895, 921, 947,
    974, 1000, 1027, 1052, 1079, 1105, 1132, 1159, 1186, 1211, 1239, 1265, 1292,
    1318, 1344, 1372, 1398, 1425, 1452, 1479, 1505, 1532, 1558, 1586, 1613, 1639,
    1666, 1693, 1719, 1746, 1773, 1800, 1827, 1854, 1880, 1907, 1933, 1961, 1988,
    2015, 2041, 2068, 2094, 2122, 2149, 2176, 2202, 2229, 2255, 2283, 2310, 2337,
    2363, 2390, 2417, 2445, 2472, 2499, 2526, 2553, 2580, 2607, 2635, 2662, 2689,
    2716, 2743, 2771, 2798, 2825, 2852, 2880, 2907, 2934, 2962, 2989, 3017, 3044,
    3072, 3099, 3127, 3155, 3182, 3210, 3236, 3265, 3292, 3320, 3348, 3376, 3404,
    3431, 3459, 3487, 3515, 3543, 3571, 3599, 3627, 3654, 3682, 3710, 3738, 3766,
    3795, 3823, 3851, 3879, 3907, 3935, 3962, 3991, 4020, 4048, 4076, 4105, 4133,
    4161, 4190, 4218, 4246, 4275, 4303, 4332, 4359, 4389, 4418, 4446, 4475, 4504,
    4532, 4561, 4590, 4619, 4648, 4677, 4706, 4735, 4764, 4793, 4821, 4851, 4880,
    4910, 4939, 4967, 4998, 5027, 5056, 5086, 5116, 5145, 5175, 5204, 5233, 5264,
    0, 2, 4, 7, 10, 14, 20, 27, 39, 54, 74, 96, 119, 142, 166, 191, 215, 240,
    266, 291, 315, 342, 367, 393, 418, 444, 470, 496, 522, 547, 573, 599, 625,
    651, 676, 702, 728, 754, 780, 806, 832, 858, 884, 910, 936, 962, 988, 1014,
    1040, 1067, 1092, 1119, 1145, 1172, 1197, 1224, 1251, 1277, 1303, 1329, 1356,
    1382, 1409, 1435, 1462, 1488, 1515, 1540, 1568, 1594, 1621, 1647, 1674, 1699,
    1727, 1753, 1780, 1806, 1833, 1859, 1886, 1912, 1939, 1966, 1992, 2019, 2045,
    2071, 2098, 2125, 2152, 2177, 2205, 2230, 2258, 2285, 2311, 2337, 2364, 2391,
    2418, 2445, 2472, 2498, 2525, 2552, 2579, 2606, 2633, 2659, 2686, 2713, 2740,
    2767, 2794, 2820, 2848, 2875, 2902, 2930, 2956, 2984, 3011, 3038, 3066, 3093,
    3120, 3148, 3175, 3201, 3230, 3256, 3284, 3312, 3339, 3367, 3394, 3422, 3449,
    3477, 3504, 3532, 3560, 3587, 3615, 3642, 3669, 3697, 3726, 3754, 3781, 3809,
    3837, 3865, 3893, 3920, 3948, 3976, 4004, 4032, 4060, 4088, 4116, 4145, 4173,
    4201, 4229, 4257, 4285, 4313, 4342, 4370, 4399, 4427, 4455, 4484, 4512, 4541,
    4569, 4598, 4627, 4655, 4684, 4713, 4741, 4769, 4799, 4828, 4857, 4886, 4914,
    4944, 4973, 5002, 5032, 5061, 5090, 5119, 5149, 5177, 5208, 0, 2, 4, 7, 10,
    14, 19, 27, 38, 53, 72, 93, 116, 139, 163, 187, 212, 236, 261, 286, 311, 337,
    362, 387, 412, 438, 464, 489, 515, 540, 566, 591, 617, 642, 668, 694, 719,
    745, 770, 796, 822, 848, 873, 899, 925, 951, 977, 1002, 1027, 1054, 1079,
    1106, 1132, 1158, 1183, 1210, 1236, 1262, 1288, 1314, 1341, 1366, 1393, 1419,
    1445, 1471, 1498, 1523, 1550, 1576, 1603, 1629, 1655, 1680, 1708, 1734, 1760,
    1786, 1813, 1838, 1865, 1890, 1918, 1944, 1970, 1996, 2023, 2048, 2075, 2102,
    2128, 2153, 2181, 2206, 2233, 2260, 2286, 2312, 2338, 2364, 2392, 2418, 2445,
    2471, 2498, 2524, 2551, 2577, 2604, 2631, 2657, 2684, 2711, 2737, 2764, 2790,
    2818, 2844, 2870, 2898, 2924, 2952, 2979, 3006, 3033, 3060, 3087, 3114, 3141,
    3167, 3195, 3221, 3249, 3276, 3303, 3331, 3358, 3385, 3412, 3440, 3467, 3494,
    3522, 3549, 3576, 3603, 3630, 3658, 3686, 3713, 3741, 3768, 3796, 3823, 3851,
    3878, 3906, 3934, 3962, 3989, 4017, 4045, 4073, 4100, 4128, 4156, 4184, 4212,
    4240, 4267, 4296, 4324, 4352, 4380, 4408, 4436, 4464, 4493, 4521, 4549, 4577,
    4606, 4634, 4663, 4691, 4719, 4748, 4777, 4805, 4834, 4862, 4892, 4921, 4949,
    4978, 5007, 5036, 5065, 5094, 5122, 5153, 0, 2, 4, 7, 10, 14, 19, 26, 37, 51,
    70, 91, 113, 136, 160, 184, 208, 233, 257, 282, 306, 332, 357, 382, 406, 432,
    457, 483, 508, 533, 558, 584, 609, 634, 660, 685, 710, 736, 761, 787, 812,
    837, 863, 888, 914, 939, 965, 991, 1015, 1042, 1067, 1093, 1119, 1145, 1170,
    1196, 1222, 1248, 1273, 1299, 1326, 1351, 1377, 1403, 1429, 1454, 1481, 1506,
    1533, 1559, 1585, 1611, 1637, 1662, 1689, 1715, 1741, 1767, 1793, 1818, 1845,
    1870, 1897, 1923, 1949, 1975, 2001, 2026, 2053, 2079, 2105, 2130, 2157, 2182,
    2209, 2235, 2261, 2286, 2313, 2339, 2366, 2392, 2418, 2444, 2471, 2497, 2523,
    2550, 2576, 2602, 2629, 2655, 2681, 2708, 2734, 2760, 2787, 2814, 2840, 2867,
    2893, 2920, 2947, 2973, 3000, 3027, 3054, 3080, 3107, 3133, 3161, 3187, 3214,
    3241, 3268, 3295, 3322, 3349, 3376, 3403, 3430, 3457, 3484, 3511, 3538, 3564,
    3592, 3619, 3647, 3674, 3701, 3728, 3756, 3783, 3810, 3837, 3865, 3892, 3920,
    3947, 3975, 4002, 4029, 4057, 4085, 4112, 4140, 4167, 4195, 4222, 4250, 4278,
    4306, 4334, 4361, 4389, 4417, 4445, 4473, 4501, 4529, 4557, 4585, 4614, 4642,
    4669, 4698, 4727, 4755, 4783, 4811, 4840, 4869, 4897, 4926, 4955, 4983, 5012,
    5041, 5069, 5099, 0, 2, 4, 7, 10, 13, 19, 26, 36, 50, 68, 89, 111, 134, 157,
    181, 205, 229, 253, 278, 301, 327, 352, 376, 401, 426, 451, 476, 501, 526,
    551, 576, 601, 626, 651, 677, 702, 727, 752, 777, 802, 827, 853, 878, 903,
    929, 954, 979, 1004, 1030, 1055, 1081, 1106, 1132, 1156, 1183, 1208, 1234,
    1259, 1284, 1311, 1335, 1362, 1388, 1413, 1438, 1465, 1489, 1516, 1542, 1567,
    1593, 1619, 1644, 1670, 1696, 1722, 1747, 1773, 1798, 1824, 1849, 1876, 1902,
    1927, 1953, 1979, 2004, 2030, 2056, 2082, 2107, 2134, 2159, 2185, 2211, 2237,
    2262, 2288, 2314, 2340, 2366, 2392, 2418, 2444, 2470, 2496, 2522, 2548, 2574,
    2601, 2627, 2653, 2679, 2705, 2731, 2758, 2784, 2809, 2837, 2862, 2889, 2916,
    2942, 2968, 2995, 3021, 3048, 3074, 3100, 3127, 3153, 3180, 3207, 3234, 3260,
    3287, 3314, 3340, 3367, 3394, 3421, 3447, 3474, 3501, 3527, 3554, 3581, 3608,
    3635, 3662, 3689, 3716, 3743, 3770, 3796, 3824, 3851, 3879, 3906, 3933, 3960,
    3987, 4014, 4042, 4069, 4096, 4124, 4151, 4178, 4206, 4233, 4261, 4288, 4316,
    4343, 4371, 4399, 4426, 4454, 4482, 4510, 4538, 4565, 4593, 4620, 4649, 4677,
    4705, 4734, 4761, 4790, 4818, 4846, 4875, 4903, 4931, 4960, 4988, 5016, 5045,
    0, 2, 4, 6, 9, 13, 18, 25, 35, 49, 66, 87, 108, 131, 154, 177, 201, 225, 249,
    273, 297, 322, 347, 371, 395, 421, 445, 470, 495, 519, 544, 569, 594, 619,
    643, 668, 693, 718, 743, 768, 793, 818, 843, 868, 893, 918, 943, 968, 992,
    1018, 1042, 1068, 1094, 1119, 1143, 1169, 1195, 1220, 1245, 1270, 1296, 1321,
    1347, 1372, 1398, 1422, 1448, 1473, 1499, 1525, 1550, 1576, 1601, 1626, 1652,
    1677, 1703, 1728, 1754, 1778, 1805, 1829, 1856, 1881, 1907, 1932, 1958, 1982,
    2009, 2034, 2060, 2084, 2111, 2135, 2162, 2187, 2213, 2238, 2263, 2289, 2316,
    2341, 2367, 2393, 2418, 2444, 2470, 2496, 2521, 2547, 2573, 2599, 2625, 2651,
    2677, 2702, 2729, 2755, 2780, 2807, 2832, 2859, 2885, 2911, 2937, 2963, 2990,
    3016, 3042, 3067, 3094, 3120, 3147, 3173, 3200, 3226, 3253, 3279, 3305, 3332,
    3358, 3385, 3411, 3438, 3464, 3490, 3517, 3543, 3571, 3597, 3624, 3651, 3678,
    3704, 3731, 3757, 3785, 3811, 3838, 3865, 3892, 3919, 3946, 3973, 4000, 4027,
    4054, 4081, 4108, 4134, 4162, 4189, 4217, 4244, 4271, 4298, 4326, 4353, 4381,
    4408, 4436, 4463, 4491, 4518, 4546, 4573, 4601, 4629, 4657, 4685, 4712, 4740,
    4768, 4796, 4824, 4852, 4881, 4909, 4937, 4964, 4993, 0, 2, 4, 6, 9, 13, 18,
    25, 34, 47, 64, 84, 106, 128, 151, 174, 198, 221, 245, 269, 292, 317, 342,
    366, 390, 415, 439, 464, 488, 513, 537, 562, 586, 611, 636, 660, 685, 709,
    734, 759, 783, 808, 833, 858, 882, 907, 932, 957, 981, 1007, 1031, 1056,
    1081, 1106, 1131, 1156, 1181, 1206, 1231, 1256, 1282, 1306, 1332, 1357, 1382,
    1407, 1433, 1457, 1483, 1508, 1533, 1558, 1584, 1608, 1634, 1659, 1684, 1710,
    1735, 1759, 1785, 1810, 1836, 1861, 1886, 1911, 1937, 1961, 1987, 2012, 2038,
    2062, 2088, 2113, 2139, 2164, 2190, 2214, 2239, 2265, 2291, 2317, 2342, 2367,
    2393, 2418, 2444, 2469, 2495, 2520, 2546, 2572, 2597, 2623, 2649, 2673, 2700,
    2726, 2751, 2777, 2802, 2829, 2855, 2881, 2907, 2932, 2958, 2984, 3010, 3035,
    3062, 3087, 3114, 3140, 3167, 3193, 3219, 3245, 3271, 3297, 3324, 3350, 3376,
    3402, 3429, 3454, 3480, 3507, 3534, 3560, 3587, 3613, 3640, 3666, 3692, 3718,
    3746, 3772, 3799, 3825, 3852, 3878, 3905, 3932, 3959, 3985, 4012, 4039, 4066,
    4092, 4119, 4146, 4173, 4200, 4227, 4254, 4281, 4308, 4336, 4363, 4390, 4417,
    4445, 4472, 4499, 4526, 4554, 4582, 4609, 4637, 4663, 4692, 4720, 4747, 4775,
    4803, 4831, 4858, 4886, 4913, 4942, 0, 2, 4, 6, 9, 13, 18, 24, 33, 46, 63,
    82, 104, 126, 148, 171, 194, 218, 241, 265, 288, 313, 337, 361, 384, 409,
    434, 458, 482, 506, 531, 555, 579, 604, 628, 652, 677, 701, 725, 750, 774,
    799, 823, 848, 872, 897, 921, 946, 970, 995, 1019, 1045, 1069, 1094, 1118,
    1144, 1168, 1193, 1217, 1242, 1268, 1292, 1317, 1342, 1367, 1391, 1417, 1441,
    1467, 1492, 1517, 1542, 1567, 1591, 1617, 1641, 1666, 1691, 1716, 1740, 1766,
    1790, 1816, 1841, 1866, 1891, 1916, 1940, 1966, 1991, 2016, 2040, 2066, 2091,
    2116, 2142, 2167, 2191, 2216, 2241, 2267, 2292, 2317, 2343, 2368, 2393, 2418,
    2444, 2469, 2494, 2520, 2545, 2570, 2596, 2621, 2646, 2672, 2697, 2722, 2749,
    2773, 2800, 2825, 2851, 2876, 2902, 2928, 2953, 2979, 3004, 3031, 3056, 3082,
    3108, 3134, 3160, 3186, 3212, 3237, 3263, 3289, 3315, 3341, 3367, 3393, 3419,
    3445, 3471, 3498, 3524, 3550, 3576, 3602, 3628, 3655, 3680, 3707, 3733, 3760,
    3786, 3812, 3839, 3865, 3892, 3918, 3945, 3971, 3998, 4024, 4050, 4077, 4104,
    4131, 4157, 4184, 4211, 4238, 4265, 4291, 4318, 4345, 4372, 4399, 4426, 4453,
    4480, 4508, 4535, 4562, 4590, 4616, 4644, 4672, 4699, 4727, 4754, 4782, 4809,
    4837, 4864, 4892, 0, 2, 4, 6, 9, 13, 17, 24, 32, 45, 61, 80, 101, 123, 145,
    168, 191, 214, 237, 261, 284, 308, 332, 356, 379, 404, 428, 452, 476, 500,
    524, 548, 572, 596, 620, 644, 669, 693, 717, 741, 765, 789, 814, 838, 862,
    887, 911, 935, 959, 984, 1008, 1033, 1057, 1082, 1106, 1131, 1156, 1180,
    1204, 1228, 1254, 1278, 1303, 1328, 1352, 1376, 1402, 1426, 1451, 1476, 1500,
    1525, 1550, 1574, 1599, 1624, 1649, 1673, 1698, 1722, 1748, 1772, 1797, 1822,
    1847, 1871, 1896, 1920, 1946, 1970, 1995, 2019, 2045, 2069, 2094, 2119, 2144,
    2168, 2193, 2218, 2244, 2268, 2293, 2318, 2343, 2368, 2393, 2418, 2443, 2468,
    2494, 2519, 2544, 2569, 2594, 2618, 2644, 2670, 2694, 2720, 2745, 2771, 2796,
    2822, 2847, 2872, 2898, 2923, 2949, 2973, 3000, 3024, 3051, 3076, 3102, 3128,
    3153, 3179, 3204, 3230, 3256, 3282, 3307, 3333, 3359, 3384, 3410, 3435, 3462,
    3488, 3514, 3540, 3566, 3592, 3618, 3643, 3670, 3696, 3722, 3748, 3774, 3800,
    3826, 3852, 3878, 3905, 3931, 3957, 3983, 4009, 4036, 4063, 4089, 4115, 4142,
    4168, 4195, 4222, 4248, 4275, 4301, 4328, 4355, 4382, 4409, 4435, 4462, 4489,
    4516, 4543, 4569, 4597, 4625, 4652, 4679, 4706, 4734, 4761, 4788, 4815, 4843,
    0, 2, 4, 6, 9, 12, 17, 23, 32, 44, 60, 78, 99, 120, 143, 165, 188, 211, 234,
    257, 280, 304, 328, 351, 374, 399, 422, 446, 470, 494, 517, 541, 565, 589,
    613, 637, 661, 685, 709, 732, 756, 780, 804, 828, 853, 877, 901, 925, 948,
    973, 997, 1022, 1046, 1070, 1093, 1119, 1143, 1167, 1191, 1215, 1240, 1264,
    1289, 1313, 1338, 1362, 1387, 1410, 1436, 1460, 1485, 1509, 1533, 1557, 1582,
    1607, 1631, 1656, 1680, 1704, 1729, 1753, 1778, 1803, 1827, 1852, 1876, 1900,
    1925, 1950, 1974, 1998, 2024, 2047, 2073, 2097, 2122, 2146, 2170, 2195, 2220,
    2245, 2270, 2295, 2319, 2344, 2369, 2394, 2418, 2443, 2468, 2493, 2518, 2543,
    2568, 2592, 2617, 2642, 2667, 2693, 2717, 2743, 2768, 2793, 2818, 2843, 2868,
    2894, 2919, 2943, 2969, 2994, 3020, 3045, 3070, 3096, 3121, 3147, 3172, 3197,
    3223, 3248, 3274, 3299, 3325, 3350, 3375, 3401, 3427, 3453, 3478, 3504, 3530,
    3555, 3581, 3606, 3633, 3658, 3684, 3710, 3736, 3762, 3788, 3814, 3839, 3865,
    3891, 3917, 3944, 3969, 3996, 4022, 4048, 4074, 4100, 4127, 4153, 4179, 4206,
    4232, 4258, 4285, 4311, 4338, 4364, 4390, 4418, 4444, 4471, 4498, 4524, 4552,
    4578, 4605, 4632, 4659, 4686, 4713, 4740, 4767, 4795, 0, 2, 4, 6, 9, 12, 17,
    23, 31, 43, 58, 76, 97, 118, 140, 162, 184, 207, 230, 253, 276, 300, 323,
    346, 369, 393, 417, 440, 464, 488, 511, 535, 558, 582, 606, 629, 653, 677,
    700, 724, 748, 772, 795, 819, 843, 867, 891, 915, 938, 962, 986, 1010, 1034,
    1058, 1082, 1106, 1131, 1155, 1178, 1202, 1227, 1250, 1275, 1299, 1324, 1347,
    1372, 1395, 1420, 1445, 1469, 1493, 1517, 1541, 1566, 1590, 1614, 1639, 1663,
    1686, 1711, 1735, 1760, 1784, 1808, 1833, 1857, 1880, 1905, 1930, 1954, 1978,
    2003, 2026, 2051, 2076, 2100, 2124, 2148, 2172, 2198, 2222, 2247, 2271, 2296,
    2320, 2345, 2369, 2394, 2418, 2443, 2468, 2492, 2517, 2542, 2565, 2591, 2616,
    2640, 2665, 2689, 2715, 2740, 2765, 2790, 2814, 2839, 2864, 2889, 2913, 2939,
    2964, 2989, 3014, 3040, 3065, 3090, 3115, 3140, 3165, 3191, 3216, 3241, 3266,
    3292, 3316, 3341, 3367, 3393, 3418, 3444, 3469, 3494, 3520, 3545, 3570, 3596,
    3622, 3647, 3673, 3699, 3724, 3750, 3776, 3801, 3827, 3853, 3878, 3904, 3929,
    3956, 3982, 4008, 4034, 4060, 4086, 4112, 4138, 4164, 4190, 4216, 4242, 4269,
    4295, 4321, 4347, 4374, 4400, 4427, 4453, 4479, 4506, 4533, 4560, 4586, 4613,
    4640, 4667, 4694, 4720, 4747, 0, 2, 4, 6, 9, 12, 16, 22, 30, 41, 56, 75, 95,
    116, 137, 159, 181, 204, 226, 249, 272, 295, 318, 342, 364, 388, 411, 435,
    458, 481, 505, 528, 552, 575, 598, 622, 645, 669, 692, 716, 739, 763, 786,
    810, 834, 857, 881, 904, 927, 952, 975, 999, 1023, 1047, 1070, 1095, 1118,
    1142, 1165, 1189, 1214, 1237, 1262, 1286, 1310, 1333, 1358, 1381, 1406, 1430,
    1454, 1478, 1502, 1525, 1550, 1574, 1598, 1622, 1646, 1669, 1694, 1717, 1742,
    1766, 1790, 1814, 1838, 1861, 1886, 1910, 1934, 1957, 1982, 2006, 2031, 2055,
    2079, 2102, 2126, 2150, 2175, 2200, 2224, 2248, 2272, 2297, 2321, 2345, 2370,
    2394, 2418, 2443, 2467, 2492, 2516, 2540, 2565, 2589, 2613, 2639, 2662, 2688,
    2712, 2737, 2762, 2786, 2811, 2836, 2860, 2884, 2910, 2934, 2960, 2984, 3009,
    3034, 3059, 3084, 3109, 3134, 3159, 3184, 3209, 3234, 3259, 3283, 3308, 3333,
    3359, 3384, 3409, 3435, 3460, 3485, 3510, 3535, 3561, 3586, 3611, 3637, 3662,
    3687, 3713, 3738, 3764, 3789, 3815, 3840, 3866, 3890, 3917, 3943, 3968, 3994,
    4020, 4045, 4071, 4097, 4123, 4149, 4175, 4201, 4227, 4253, 4279, 4304, 4331,
    4357, 4383, 4410, 4435, 4462, 4489, 4515, 4541, 4568, 4594, 4621, 4648, 4673,
    4701, 0, 2, 4, 6, 9, 12, 16, 22, 30, 40, 55, 73, 92, 113, 134, 156, 178, 201,
    223, 246, 268, 291, 314, 337, 359, 383, 406, 429, 452, 476, 499, 522, 545,
    568, 591, 615, 638, 661, 684, 708, 731, 754, 778, 801, 824, 848, 871, 895,
    917, 941, 964, 989, 1012, 1036, 1058, 1083, 1106, 1130, 1153, 1177, 1201,
    1224, 1249, 1272, 1296, 1319, 1343, 1366, 1391, 1415, 1438, 1462, 1486, 1509,
    1534, 1557, 1581, 1605, 1629, 1652, 1676, 1699, 1724, 1748, 1772, 1795, 1819,
    1842, 1867, 1891, 1914, 1938, 1962, 1985, 2010, 2034, 2058, 2081, 2105, 2129,
    2154, 2178, 2202, 2226, 2250, 2274, 2298, 2322, 2346, 2370, 2394, 2418, 2442,
    2467, 2491, 2514, 2539, 2564, 2587, 2612, 2636, 2661, 2685, 2710, 2734, 2759,
    2783, 2808, 2832, 2856, 2881, 2905, 2930, 2955, 2980, 3004, 3029, 3053, 3078,
    3103, 3128, 3152, 3177, 3202, 3227, 3251, 3276, 3300, 3326, 3351, 3376, 3401,
    3426, 3451, 3476, 3500, 3526, 3551, 3576, 3601, 3626, 3651, 3676, 3702, 3727,
    3752, 3777, 3803, 3828, 3852, 3879, 3904, 3929, 3955, 3980, 4006, 4031, 4057,
    4083, 4108, 4134, 4160, 4185, 4211, 4237, 4262, 4289, 4315, 4341, 4367, 4392,
    4419, 4445, 4471, 4497, 4523, 4550, 4576, 4602, 4628, 4655, 0, 2, 4, 6, 9,
    12, 16, 21, 29, 39, 54, 71, 90, 111, 132, 153, 175, 197, 220, 242, 264, 287,
    310, 332, 355, 378, 401, 424, 447, 470, 493, 516, 539, 561, 584, 607, 630,
    654, 677, 700, 723, 746, 769, 792, 815, 838, 862, 885, 907, 931, 954, 978,
    1001, 1025, 1047, 1071, 1095, 1118, 1141, 1164, 1188, 1211, 1235, 1259, 1282,
    1305, 1329, 1352, 1377, 1400, 1424, 1447, 1471, 1494, 1518, 1541, 1565, 1589,
    1612, 1635, 1659, 1682, 1706, 1730, 1754, 1777, 1801, 1824, 1848, 1872, 1895,
    1918, 1943, 1965, 1990, 2013, 2037, 2060, 2084, 2108, 2132, 2156, 2180, 2203,
    2227, 2251, 2275, 2299, 2323, 2346, 2370, 2394, 2418, 2442, 2466, 2489, 2514,
    2538, 2562, 2586, 2610, 2635, 2659, 2683, 2707, 2731, 2756, 2780, 2804, 2828,
    2853, 2876, 2901, 2926, 2950, 2975, 2999, 3024, 3048, 3072, 3097, 3121, 3146,
    3171, 3195, 3219, 3244, 3268, 3294, 3318, 3343, 3368, 3392, 3417, 3442, 3466,
    3491, 3516, 3541, 3566, 3591, 3616, 3641, 3666, 3691, 3716, 3741, 3766, 3791,
    3815, 3841, 3866, 3891, 3916, 3942, 3967, 3992, 4018, 4043, 4068, 4094, 4119,
    4145, 4170, 4196, 4221, 4247, 4273, 4299, 4324, 4349, 4376, 4402, 4428, 4454,
    4480, 4506, 4532, 4558, 4583, 4610, 0, 2, 4, 6, 8, 12, 16, 21, 28, 39, 52,
    69, 88, 109, 129, 151, 172, 194, 216, 238, 260, 283, 305, 328, 350, 373, 396,
    419, 441, 464, 487, 509, 532, 555, 578, 600, 623, 646, 669, 692, 715, 737,
    760, 783, 806, 829, 852, 875, 897, 921, 944, 967, 991, 1014, 1036, 1060,
    1083, 1106, 1129, 1152, 1176, 1199, 1223, 1246, 1269, 1292, 1316, 1338, 1362,
    1386, 1409, 1432, 1456, 1478, 1502, 1526, 1549, 1572, 1596, 1618, 1643, 1665,
    1689, 1713, 1736, 1759, 1783, 1805, 1829, 1853, 1876, 1899, 1923, 1946, 1970,
    1993, 2017, 2040, 2063, 2087, 2111, 2134, 2158, 2182, 2205, 2229, 2252, 2276,
    2300, 2323, 2347, 2371, 2394, 2418, 2442, 2465, 2490, 2513, 2536, 2561, 2584,
    2609, 2633, 2657, 2681, 2705, 2729, 2753, 2777, 2800, 2825, 2848, 2873, 2897,
    2922, 2946, 2970, 2994, 3018, 3043, 3067, 3091, 3115, 3140, 3164, 3188, 3212,
    3236, 3262, 3286, 3311, 3335, 3360, 3384, 3409, 3432, 3458, 3482, 3507, 3531,
    3556, 3581, 3605, 3630, 3655, 3680, 3704, 3729, 3754, 3778, 3804, 3829, 3854,
    3879, 3904, 3929, 3954, 3979, 4004, 4029, 4054, 4080, 4105, 4130, 4156, 4180,
    4206, 4232, 4257, 4283, 4308, 4334, 4360, 4385, 4411, 4437, 4463, 4488, 4514,
    4539, 4566, 0, 2, 4, 6, 8, 12, 15, 21, 28, 38, 51, 68, 86, 106, 127, 148,
    169, 191, 213, 235, 256, 279, 301, 324, 345, 368, 391, 413, 436, 458, 481,
    503, 526, 548, 571, 594, 616, 639, 661, 684, 707, 729, 752, 775, 797, 820,
    843, 866, 888, 911, 934, 957, 980, 1003, 1025, 1049, 1072, 1095, 1117, 1140,
    1164, 1186, 1210, 1233, 1256, 1278, 1302, 1325, 1349, 1372, 1395, 1418, 1441,
    1463, 1487, 1510, 1534, 1557, 1580, 1602, 1626, 1648, 1672, 1695, 1719, 1742,
    1765, 1787, 1811, 1835, 1858, 1880, 1904, 1927, 1951, 1974, 1997, 2020, 2043,
    2066, 2090, 2113, 2137, 2160, 2183, 2207, 2230, 2254, 2277, 2301, 2324, 2348,
    2371, 2395, 2418, 2441, 2465, 2489, 2512, 2536, 2559, 2584, 2607, 2631, 2655,
    2678, 2702, 2726, 2750, 2773, 2798, 2821, 2845, 2869, 2893, 2917, 2941, 2965,
    2989, 3013, 3037, 3061, 3085, 3110, 3134, 3157, 3181, 3205, 3230, 3255, 3279,
    3303, 3327, 3352, 3376, 3399, 3424, 3449, 3473, 3498, 3522, 3546, 3571, 3595,
    3620, 3644, 3669, 3694, 3718, 3742, 3768, 3792, 3817, 3842, 3866, 3891, 3916,
    3941, 3966, 3991, 4016, 4041, 4066, 4091, 4116, 4140, 4166, 4192, 4217, 4242,
    4267, 4293, 4318, 4344, 4369, 4395, 4420, 4446, 4471, 4496, 4523, 0, 2, 4, 6,
    8, 11, 15, 20, 27, 37, 50, 66, 84, 104, 124, 145, 166, 188, 210, 231, 252,
    275, 297, 319, 341, 364, 386, 408, 430, 453, 475, 497, 520, 542, 564, 587,
    609, 632, 654, 676, 699, 721, 744, 766, 789, 811, 834, 857, 878, 902, 924,
    947, 970, 993, 1014, 1038, 1061, 1084, 1106, 1128, 1152, 1174, 1198, 1221,
    1243, 1265, 1289, 1311, 1335, 1358, 1381, 1404, 1427, 1449, 1472, 1495, 1518,
    1541, 1564, 1586, 1610, 1632, 1656, 1679, 1702, 1725, 1748, 1770, 1793, 1816,
    1839, 1862, 1885, 1908, 1931, 1954, 1977, 2000, 2023, 2046, 2070, 2093, 2116,
    2139, 2162, 2185, 2209, 2232, 2255, 2278, 2301, 2325, 2348, 2371, 2395, 2417,
    2441, 2465, 2487, 2512, 2534, 2559, 2582, 2606, 2629, 2653, 2676, 2700, 2723,
    2746, 2771, 2794, 2818, 2842, 2866, 2889, 2913, 2937, 2961, 2984, 3008, 3032,
    3056, 3080, 3104, 3127, 3151, 3175, 3199, 3223, 3247, 3271, 3296, 3320, 3344,
    3367, 3392, 3416, 3440, 3464, 3489, 3513, 3537, 3561, 3586, 3610, 3634, 3659,
    3683, 3707, 3732, 3756, 3781, 3805, 3830, 3854, 3879, 3904, 3928, 3953, 3978,
    4003, 4027, 4052, 4077, 4101, 4127, 4152, 4177, 4202, 4226, 4252, 4277, 4303,
    4328, 4353, 4378, 4404, 4429, 4454, 4480, 0, 2, 4, 6, 8, 11, 15, 20, 27, 36,
    48, 64, 82, 102, 122, 143, 164, 185, 206, 228, 249, 271, 293, 315, 336, 359,
    381, 403, 425, 447, 469, 492, 514, 536, 558, 580, 602, 625, 647, 669, 691,
    713, 736, 758, 780, 803, 825, 847, 869, 892, 914, 937, 960, 982, 1004, 1027,
    1050, 1072, 1094, 1117, 1140, 1162, 1185, 1208, 1231, 1253, 1276, 1298, 1321,
    1344, 1367, 1390, 1412, 1434, 1458, 1480, 1503, 1526, 1549, 1571, 1594, 1616,
    1639, 1662, 1685, 1708, 1730, 1752, 1776, 1799, 1821, 1843, 1867, 1889, 1913,
    1935, 1958, 1980, 2003, 2026, 2050, 2073, 2095, 2118, 2141, 2164, 2187, 2210,
    2233, 2256, 2279, 2302, 2325, 2349, 2372, 2394, 2418, 2441, 2464, 2488, 2510,
    2534, 2557, 2581, 2604, 2627, 2651, 2674, 2697, 2720, 2744, 2767, 2791, 2815,
    2838, 2862, 2885, 2909, 2932, 2956, 2980, 3003, 3027, 3051, 3074, 3097, 3121,
    3145, 3169, 3193, 3217, 3241, 3264, 3288, 3312, 3335, 3360, 3384, 3408, 3432,
    3456, 3480, 3504, 3528, 3552, 3576, 3600, 3624, 3648, 3672, 3697, 3721, 3745,
    3769, 3794, 3818, 3843, 3867, 3891, 3916, 3940, 3965, 3990, 4014, 4039, 4063,
    4088, 4113, 4138, 4163, 4187, 4212, 4237, 4262, 4287, 4312, 4337, 4363, 4388,
    4412, 4438, 0, 2, 4, 6, 8, 11, 15, 20, 26, 35, 47, 63, 81, 100, 120, 140,
    161, 182, 203, 224, 245, 268, 289, 311, 332, 355, 376, 398, 420, 442, 464,
    486, 508, 530, 552, 574, 596, 618, 640, 662, 684, 706, 728, 750, 772, 794,
    816, 839, 860, 883, 904, 927, 950, 972, 994, 1017, 1039, 1061, 1083, 1105,
    1129, 1150, 1174, 1196, 1218, 1240, 1263, 1285, 1308, 1331, 1353, 1376, 1398,
    1420, 1443, 1466, 1488, 1511, 1533, 1555, 1578, 1600, 1623, 1646, 1668, 1691,
    1714, 1735, 1759, 1781, 1804, 1826, 1849, 1871, 1894, 1917, 1939, 1961, 1984,
    2007, 2030, 2053, 2075, 2098, 2121, 2144, 2166, 2189, 2212, 2235, 2258, 2280,
    2303, 2326, 2349, 2371, 2395, 2418, 2440, 2464, 2486, 2510, 2533, 2556, 2579,
    2602, 2626, 2649, 2672, 2694, 2718, 2741, 2765, 2788, 2811, 2835, 2858, 2881,
    2905, 2928, 2952, 2975, 2998, 3022, 3045, 3068, 3092, 3115, 3139, 3163, 3186,
    3210, 3234, 3257, 3281, 3304, 3328, 3352, 3376, 3399, 3423, 3447, 3471, 3495,
    3518, 3542, 3566, 3590, 3614, 3637, 3662, 3686, 3710, 3734, 3758, 3783, 3807,
    3831, 3855, 3879, 3904, 3928, 3952, 3977, 4001, 4025, 4050, 4075, 4099, 4124,
    4148, 4173, 4198, 4223, 4247, 4272, 4297, 4322, 4347, 4371, 4397, 0, 2, 4, 6,
    8, 11, 15, 19, 26, 34, 46, 61, 79, 98, 117, 138, 158, 179, 200, 221, 242,
    264, 285, 307, 328, 350, 372, 393, 415, 437, 458, 480, 502, 524, 545, 567,
    589, 611, 633, 654, 676, 698, 720, 742, 764, 786, 808, 830, 851, 874, 895,
    918, 940, 962, 983, 1006, 1029, 1051, 1072, 1094, 1117, 1139, 1162, 1184,
    1206, 1228, 1251, 1272, 1295, 1318, 1340, 1362, 1385, 1406, 1429, 1451, 1474,
    1496, 1518, 1540, 1563, 1585, 1608, 1630, 1652, 1675, 1697, 1719, 1742, 1764,
    1786, 1808, 1831, 1853, 1876, 1898, 1921, 1942, 1965, 1987, 2011, 2033, 2056,
    2078, 2101, 2123, 2146, 2168, 2191, 2213, 2236, 2259, 2281, 2304, 2327, 2349,
    2372, 2395, 2417, 2441, 2463, 2486, 2509, 2532, 2555, 2578, 2601, 2624, 2647,
    2669, 2693, 2715, 2739, 2762, 2785, 2808, 2831, 2854, 2878, 2901, 2924, 2947,
    2970, 2994, 3017, 3039, 3063, 3086, 3110, 3133, 3157, 3180, 3204, 3227, 3250,
    3273, 3297, 3321, 3344, 3368, 3391, 3415, 3439, 3462, 3486, 3509, 3533, 3557,
    3581, 3604, 3628, 3652, 3676, 3700, 3724, 3747, 3771, 3795, 3819, 3843, 3868,
    3892, 3916, 3940, 3964, 3988, 4013, 4037, 4061, 4086, 4109, 4135, 4159, 4184,
    4208, 4233, 4257, 4282, 4307, 4331, 4356, 0, 2, 4, 6, 8, 11, 14, 19, 25, 34,
    45, 60, 77, 96, 115, 135, 156, 176, 197, 218, 238, 260, 281, 303, 323, 346,
    367, 389, 410, 432, 453, 475, 496, 518, 539, 561, 582, 604, 626, 647, 669,
    691, 712, 734, 756, 778, 799, 821, 842, 865, 886, 908, 930, 952, 973, 996,
    1018, 1040, 1061, 1083, 1106, 1127, 1150, 1172, 1194, 1216, 1238, 1260, 1283,
    1305, 1327, 1349, 1371, 1392, 1415, 1437, 1459, 1482, 1504, 1525, 1548, 1569,
    1592, 1614, 1636, 1659, 1681, 1702, 1725, 1747, 1769, 1791, 1814, 1835, 1858,
    1880, 1902, 1924, 1946, 1968, 1991, 2014, 2036, 2058, 2081, 2103, 2125, 2148,
    2170, 2193, 2215, 2237, 2260, 2282, 2305, 2327, 2350, 2373, 2394, 2418, 2440,
    2463, 2486, 2508, 2531, 2554, 2577, 2599, 2622, 2644, 2668, 2690, 2713, 2736,
    2759, 2782, 2805, 2828, 2851, 2874, 2897, 2920, 2943, 2966, 2989, 3011, 3034,
    3057, 3081, 3104, 3128, 3151, 3174, 3197, 3220, 3243, 3267, 3290, 3313, 3337,
    3360, 3383, 3407, 3430, 3454, 3477, 3501, 3524, 3548, 3570, 3595, 3618, 3642,
    3666, 3689, 3713, 3737, 3761, 3784, 3808, 3832, 3856, 3880, 3904, 3928, 3951,
    3976, 4000, 4024, 4048, 4072, 4097, 4121, 4145, 4170, 4194, 4218, 4243, 4267,
    4291, 4317, 0, 2, 4, 6, 8, 11, 14, 19, 25, 33, 44, 58, 75, 94, 113, 133, 153,
    173, 194, 215, 235, 257, 278, 299, 319, 341, 363, 384, 405, 426, 448, 469,
    491, 512, 533, 555, 576, 597, 619, 640, 662, 683, 705, 726, 748, 769, 791,
    813, 834, 856, 877, 899, 921, 943, 964, 986, 1008, 1030, 1051, 1073, 1095,
    1116, 1139, 1161, 1183, 1204, 1226, 1247, 1270, 1292, 1314, 1336, 1358, 1379,
    1401, 1423, 1445, 1467, 1489, 1510, 1533, 1554, 1577, 1599, 1621, 1643, 1665,
    1686, 1709, 1731, 1752, 1774, 1796, 1818, 1840, 1862, 1884, 1906, 1928, 1950,
    1973, 1995, 2017, 2039, 2061, 2083, 2105, 2128, 2150, 2172, 2194, 2217, 2239,
    2261, 2283, 2305, 2328, 2350, 2372, 2395, 2417, 2440, 2463, 2485, 2508, 2530,
    2553, 2575, 2598, 2620, 2643, 2665, 2688, 2711, 2734, 2756, 2779, 2802, 2824,
    2847, 2870, 2893, 2916, 2938, 2961, 2983, 3006, 3029, 3053, 3076, 3099, 3122,
    3145, 3168, 3191, 3213, 3237, 3260, 3283, 3306, 3329, 3352, 3376, 3399, 3422,
    3445, 3469, 3492, 3515, 3538, 3562, 3585, 3609, 3632, 3656, 3679, 3703, 3726,
    3750, 3773, 3797, 3821, 3845, 3868, 3892, 3915, 3940, 3964, 3988, 4011, 4035,
    4060, 4084, 4108, 4132, 4156, 4180, 4204, 4229, 4252, 4277, 0, 2, 3, 5, 8,
    11, 14, 18, 24, 32, 43, 57, 74, 92, 111, 130, 150, 171, 191, 212, 232, 253,
    274, 295, 315, 337, 358, 379, 400, 421, 443, 464, 485, 506, 527, 549, 570,
    591, 612, 634, 655, 676, 697, 719, 740, 761, 783, 804, 825, 847, 868, 890,
    912, 933, 954, 976, 998, 1019, 1040, 1062, 1084, 1105, 1128, 1149, 1171,
    1192, 1214, 1235, 1258, 1279, 1301, 1323, 1345, 1366, 1388, 1410, 1431, 1453,
    1475, 1496, 1518, 1539, 1562, 1584, 1605, 1627, 1649, 1670, 1692, 1714, 1736,
    1757, 1780, 1801, 1823, 1845, 1867, 1888, 1910, 1932, 1954, 1976, 1998, 2020,
    2042, 2064, 2086, 2108, 2130, 2152, 2174, 2196, 2218, 2240, 2262, 2284, 2307,
    2329, 2350, 2373, 2395, 2418, 2440, 2462, 2484, 2507, 2529, 2551, 2574, 2596,
    2619, 2640, 2664, 2686, 2708, 2731, 2754, 2776, 2799, 2821, 2844, 2866, 2889,
    2912, 2934, 2956, 2979, 3002, 3025, 3048, 3071, 3093, 3116, 3139, 3162, 3184,
    3207, 3230, 3253, 3276, 3299, 3322, 3345, 3368, 3391, 3414, 3437, 3460, 3483,
    3506, 3530, 3553, 3576, 3599, 3623, 3646, 3669, 3692, 3716, 3739, 3763, 3786,
    3810, 3833, 3857, 3880, 3904, 3928, 3952, 3975, 3998, 4023, 4047, 4071, 4095,
    4118, 4142, 4166, 4191, 4214, 4239, 0, 2, 3, 5, 8, 10, 14, 18, 24, 32, 42,
    56, 72, 90, 109, 128, 148, 168, 188, 208, 228, 250, 270, 291, 311, 333, 354,
    375, 396, 417, 438, 459, 480, 501, 522, 543, 564, 585, 606, 627, 648, 669,
    690, 711, 732, 754, 775, 796, 817, 839, 859, 881, 902, 924, 944, 967, 988,
    1009, 1030, 1051, 1074, 1094, 1117, 1138, 1160, 1180, 1203, 1223, 1246, 1267,
    1289, 1310, 1332, 1352, 1375, 1396, 1418, 1439, 1461, 1482, 1504, 1525, 1547,
    1569, 1590, 1612, 1633, 1654, 1677, 1698, 1720, 1741, 1763, 1784, 1806, 1828,
    1849, 1870, 1892, 1914, 1936, 1958, 1980, 2001, 2023, 2045, 2067, 2088, 2110,
    2132, 2154, 2176, 2198, 2220, 2241, 2263, 2285, 2307, 2329, 2351, 2373, 2395,
    2418, 2440, 2462, 2484, 2506, 2528, 2550, 2572, 2595, 2616, 2639, 2662, 2684,
    2706, 2728, 2751, 2773, 2795, 2818, 2840, 2863, 2885, 2908, 2929, 2952, 2974,
    2998, 3020, 3043, 3065, 3088, 3110, 3133, 3155, 3178, 3201, 3224, 3246, 3269,
    3292, 3315, 3338, 3360, 3383, 3406, 3429, 3452, 3474, 3498, 3521, 3544, 3567,
    3590, 3613, 3636, 3659, 3682, 3706, 3729, 3752, 3776, 3799, 3822, 3845, 3869,
    3893, 3916, 3940, 3962, 3987, 4010, 4034, 4058, 4082, 4105, 4129, 4153, 4176,
    4201, 0, 2, 3, 5, 8, 10, 14, 18, 23, 31, 41, 54, 70, 88, 107, 126, 145, 165,
    185, 205, 225, 246, 267, 287, 307, 329, 349, 370, 391, 412, 433, 453, 474,
    495, 516, 537, 557, 578, 599, 620, 641, 662, 683, 704, 725, 746, 767, 788,
    808, 830, 850, 872, 893, 915, 935, 957, 978, 999, 1020, 1041, 1063, 1084,
    1106, 1127, 1148, 1169, 1191, 1212, 1234, 1255, 1276, 1298, 1319, 1340, 1362,
    1383, 1404, 1426, 1447, 1468, 1490, 1510, 1533, 1554, 1575, 1597, 1618, 1639,
    1661, 1682, 1704, 1724, 1747, 1767, 1789, 1811, 1832, 1853, 1875, 1896, 1918,
    1940, 1961, 1983, 2004, 2026, 2048, 2069, 2091, 2112, 2134, 2156, 2178, 2199,
    2221, 2242, 2264, 2286, 2307, 2330, 2351, 2374, 2396, 2417, 2439, 2461, 2483,
    2505, 2527, 2548, 2571, 2593, 2615, 2637, 2660, 2682, 2704, 2726, 2748, 2770,
    2792, 2815, 2837, 2859, 2881, 2903, 2925, 2948, 2971, 2993, 3015, 3038, 3060,
    3082, 3105, 3127, 3150, 3172, 3195, 3217, 3240, 3262, 3285, 3308, 3330, 3353,
    3376, 3398, 3421, 3443, 3467, 3489, 3512, 3535, 3558, 3581, 3604, 3627, 3650,
    3673, 3696, 3719, 3742, 3765, 3788, 3811, 3835, 3858, 3881, 3905, 3927, 3951,
    3975, 3998, 4022, 4045, 4069, 4092, 4116, 4139, 4164, 0, 2, 3, 5, 8, 10, 13,
    18, 23, 30, 40, 53, 69, 86, 104, 123, 143, 163, 182, 202, 222, 243, 263, 284,
    303, 325, 345, 366, 386, 407, 428, 448, 469, 489, 510, 531, 551, 572, 593,
    614, 634, 655, 676, 697, 717, 738, 759, 780, 800, 822, 842, 864, 885, 906,
    926, 948, 969, 990, 1010, 1031, 1053, 1073, 1095, 1116, 1137, 1158, 1180,
    1200, 1222, 1243, 1264, 1285, 1307, 1327, 1349, 1370, 1391, 1412, 1434, 1454,
    1476, 1496, 1518, 1539, 1561, 1582, 1603, 1624, 1646, 1667, 1688, 1708, 1730,
    1751, 1773, 1794, 1815, 1836, 1857, 1879, 1901, 1922, 1943, 1965, 1986, 2008,
    2029, 2050, 2072, 2093, 2115, 2136, 2158, 2179, 2201, 2222, 2244, 2266, 2286,
    2309, 2330, 2352, 2374, 2396, 2417, 2439, 2461, 2483, 2504, 2526, 2548, 2569,
    2592, 2614, 2636, 2658, 2679, 2701, 2723, 2745, 2767, 2789, 2811, 2834, 2856,
    2877, 2899, 2921, 2944, 2966, 2988, 3011, 3033, 3055, 3077, 3099, 3122, 3144,
    3166, 3189, 3211, 3233, 3256, 3278, 3301, 3323, 3346, 3368, 3391, 3412, 3436,
    3458, 3481, 3504, 3526, 3549, 3572, 3594, 3617, 3640, 3663, 3686, 3709, 3732,
    3755, 3777, 3801, 3824, 3847, 3870, 3892, 3916, 3940, 3963, 3986, 4010, 4033,
    4056, 4080, 4103, 4127, 0, 2, 3, 5, 7, 10, 13, 17, 23, 30, 39, 52, 67, 84,
    102, 121, 140, 160, 180, 200, 219, 240, 260, 280, 300, 321, 341, 361, 382,
    402, 423, 443, 464, 484, 505, 525, 546, 566, 587, 607, 628, 648, 669, 690,
    710, 731, 751, 772, 792, 814, 834, 855, 876, 897, 917, 938, 959, 980, 1000,
    1021, 1043, 1063, 1085, 1106, 1126, 1147, 1168, 1189, 1210, 1231, 1252, 1273,
    1294, 1315, 1336, 1357, 1378, 1399, 1420, 1441, 1462, 1483, 1504, 1525, 1546,
    1567, 1588, 1609, 1630, 1651, 1672, 1693, 1715, 1735, 1757, 1778, 1799, 1819,
    1840, 1862, 1883, 1905, 1926, 1947, 1968, 1989, 2011, 2032, 2053, 2074, 2096,
    2117, 2138, 2160, 2181, 2202, 2224, 2245, 2266, 2288, 2309, 2331, 2353, 2374,
    2396, 2417, 2439, 2460, 2482, 2503, 2525, 2546, 2569, 2590, 2612, 2634, 2656,
    2677, 2699, 2721, 2743, 2765, 2786, 2808, 2830, 2851, 2873, 2895, 2918, 2940,
    2962, 2984, 3006, 3028, 3050, 3071, 3094, 3116, 3138, 3160, 3183, 3205, 3227,
    3249, 3271, 3294, 3316, 3338, 3361, 3382, 3405, 3428, 3450, 3473, 3495, 3518,
    3540, 3563, 3585, 3608, 3631, 3653, 3676, 3699, 3722, 3744, 3767, 3790, 3813,
    3836, 3858, 3882, 3905, 3928, 3951, 3974, 3998, 4021, 4044, 4067, 4091, 0, 2,
    3, 5, 7, 10, 13, 17, 22, 29, 39, 51, 66, 82, 100, 119, 138, 157, 177, 197,
    216, 236, 256, 276, 296, 317, 337, 357, 377, 398, 418, 438, 458, 479, 499,
    519, 540, 560, 580, 601, 621, 642, 662, 682, 703, 723, 744, 764, 784, 806,
    825, 847, 867, 888, 908, 929, 950, 971, 991, 1011, 1033, 1053, 1074, 1095,
    1116, 1136, 1157, 1177, 1199, 1220, 1241, 1261, 1282, 1302, 1324, 1345, 1365,
    1386, 1407, 1427, 1449, 1469, 1490, 1511, 1532, 1553, 1574, 1594, 1615, 1636,
    1657, 1677, 1699, 1719, 1741, 1762, 1783, 1803, 1824, 1845, 1866, 1887, 1908,
    1929, 1950, 1971, 1992, 2014, 2035, 2056, 2077, 2098, 2119, 2140, 2161, 2182,
    2204, 2225, 2246, 2268, 2288, 2310, 2332, 2353, 2374, 2396, 2417, 2438, 2460,
    2481, 2503, 2524, 2546, 2567, 2589, 2610, 2632, 2654, 2675, 2697, 2719, 2740,
    2762, 2784, 2805, 2826, 2848, 2870, 2892, 2914, 2936, 2958, 2979, 3001, 3023,
    3044, 3067, 3089, 3111, 3133, 3155, 3177, 3199, 3221, 3243, 3265, 3287, 3309,
    3331, 3353, 3376, 3398, 3420, 3442, 3465, 3487, 3509, 3532, 3554, 3577, 3599,
    3622, 3644, 3667, 3689, 3711, 3734, 3757, 3780, 3803, 3825, 3848, 3871, 3894,
    3917, 3940, 3963, 3986, 4009, 4031, 4055, 0, 2, 3, 5, 7, 10, 13, 17, 22, 29,
    38, 50, 64, 81, 99, 117, 136, 155, 174, 194, 213, 233, 253, 273, 292, 313,
    333, 353, 373, 393, 413, 433, 453, 474, 494, 514, 534, 554, 574, 595, 615,
    635, 655, 676, 696, 716, 736, 757, 777, 798, 817, 838, 859, 879, 899, 920,
    941, 961, 981, 1002, 1023, 1043, 1064, 1085, 1105, 1125, 1146, 1166, 1188,
    1208, 1229, 1250, 1270, 1290, 1312, 1332, 1353, 1373, 1394, 1414, 1435, 1455,
    1477, 1497, 1518, 1539, 1559, 1579, 1601, 1621, 1642, 1662, 1684, 1704, 1725,
    1746, 1767, 1787, 1807, 1828, 1850, 1870, 1891, 1912, 1933, 1954, 1975, 1996,
    2016, 2037, 2058, 2079, 2100, 2121, 2142, 2163, 2184, 2205, 2226, 2248, 2268,
    2290, 2311, 2332, 2353, 2374, 2396, 2417, 2438, 2459, 2481, 2501, 2523, 2545,
    2566, 2588, 2609, 2630, 2652, 2673, 2695, 2716, 2738, 2759, 2781, 2801, 2823,
    2845, 2867, 2888, 2910, 2932, 2953, 2975, 2997, 3018, 3040, 3062, 3084, 3105,
    3127, 3149, 3171, 3193, 3215, 3236, 3258, 3280, 3302, 3324, 3346, 3368, 3390,
    3412, 3435, 3457, 3479, 3501, 3523, 3546, 3568, 3590, 3612, 3635, 3657, 3679,
    3702, 3725, 3747, 3770, 3792, 3815, 3838, 3860, 3883, 3906, 3929, 3951, 3974,
    3996, 4020, 0, 2, 3, 5, 7, 10, 13, 17, 22, 28, 37, 48, 63, 79, 97, 115, 133,
    152, 172, 191, 210, 230, 250, 269, 289, 309, 329, 349, 369, 389, 409, 429,
    448, 468, 488, 508, 528, 548, 568, 588, 609, 629, 649, 669, 689, 709, 729,
    749, 769, 790, 809, 830, 851, 871, 890, 911, 932, 952, 972, 992, 1013, 1033,
    1054, 1074, 1095, 1115, 1136, 1156, 1177, 1197, 1218, 1238, 1259, 1278, 1299,
    1320, 1340, 1361, 1381, 1401, 1422, 1442, 1463, 1484, 1504, 1525, 1545, 1565,
    1586, 1607, 1627, 1647, 1668, 1688, 1710, 1730, 1751, 1771, 1791, 1812, 1833,
    1854, 1874, 1895, 1916, 1936, 1957, 1978, 1999, 2019, 2040, 2061, 2082, 2102,
    2123, 2143, 2165, 2186, 2206, 2228, 2248, 2270, 2291, 2312, 2333, 2354, 2375,
    2396, 2417, 2437, 2459, 2479, 2501, 2523, 2544, 2565, 2586, 2607, 2629, 2650,
    2671, 2692, 2714, 2735, 2756, 2777, 2798, 2820, 2842, 2863, 2885, 2906, 2928,
    2949, 2971, 2992, 3014, 3035, 3057, 3078, 3100, 3122, 3143, 3165, 3187, 3208,
    3230, 3252, 3274, 3295, 3317, 3339, 3361, 3383, 3405, 3427, 3449, 3471, 3493,
    3515, 3537, 3559, 3581, 3604, 3626, 3647, 3670, 3693, 3715, 3737, 3759, 3782,
    3805, 3827, 3850, 3872, 3895, 3917, 3940, 3962, 3986, 0, 2, 3, 5, 7, 10, 13,
    16, 21, 28, 36, 47, 61, 77, 95, 113, 131, 150, 169, 188, 207, 227, 246, 266,
    285, 305, 325, 345, 364, 384, 404, 424, 444, 463, 483, 503, 523, 543, 563,
    582, 602, 622, 642, 662, 682, 702, 722, 742, 761, 782, 801, 822, 842, 862,
    882, 903, 923, 943, 963, 983, 1004, 1023, 1044, 1064, 1085, 1104, 1125, 1145,
    1166, 1186, 1206, 1227, 1247, 1267, 1288, 1308, 1328, 1349, 1369, 1388, 1409,
    1429, 1450, 1470, 1491, 1511, 1531, 1551, 1572, 1592, 1613, 1633, 1654, 1673,
    1694, 1715, 1735, 1755, 1775, 1796, 1817, 1837, 1858, 1878, 1899, 1919, 1940,
    1960, 1981, 2001, 2022, 2043, 2063, 2084, 2105, 2125, 2146, 2167, 2187, 2208,
    2228, 2250, 2271, 2291, 2312, 2333, 2354, 2375, 2396, 2416, 2438, 2458, 2480,
    2501, 2522, 2543, 2564, 2585, 2606, 2627, 2648, 2669, 2690, 2711, 2733, 2753,
    2774, 2795, 2817, 2839, 2860, 2881, 2902, 2924, 2945, 2966, 2988, 3009, 3031,
    3052, 3073, 3095, 3116, 3138, 3159, 3181, 3202, 3224, 3246, 3267, 3289, 3311,
    3332, 3354, 3376, 3398, 3419, 3441, 3463, 3485, 3507, 3529, 3551, 3573, 3595,
    3616, 3639, 3661, 3683, 3705, 3727, 3750, 3772, 3794, 3817, 3839, 3862, 3884,
    3907, 3928, 3952, 0, 2, 3, 5, 7, 10, 13, 16, 21, 27, 35, 46, 60, 76, 93, 111,
    129, 148, 166, 185, 204, 224, 243, 263, 281, 302, 321, 341, 360, 380, 400,
    419, 439, 458, 478, 498, 517, 537, 557, 576, 596, 616, 636, 655, 675, 695,
    715, 735, 754, 775, 794, 814, 834, 854, 874, 894, 914, 934, 954, 974, 994,
    1014, 1034, 1055, 1075, 1094, 1115, 1134, 1155, 1175, 1195, 1215, 1236, 1255,
    1276, 1296, 1316, 1336, 1356, 1376, 1397, 1416, 1437, 1457, 1477, 1498, 1518,
    1537, 1558, 1578, 1598, 1618, 1639, 1658, 1679, 1700, 1720, 1739, 1760, 1780,
    1801, 1821, 1841, 1862, 1882, 1902, 1923, 1943, 1964, 1984, 2004, 2025, 2045,
    2066, 2086, 2106, 2127, 2148, 2168, 2189, 2209, 2230, 2251, 2272, 2292, 2313,
    2334, 2354, 2375, 2395, 2417, 2437, 2458, 2479, 2500, 2521, 2542, 2562, 2583,
    2604, 2625, 2646, 2667, 2688, 2709, 2729, 2750, 2771, 2793, 2814, 2835, 2856,
    2878, 2899, 2920, 2940, 2962, 2983, 3005, 3026, 3047, 3068, 3090, 3111, 3132,
    3154, 3175, 3197, 3218, 3239, 3261, 3282, 3304, 3325, 3347, 3369, 3390, 3412,
    3434, 3455, 3477, 3499, 3521, 3542, 3564, 3585, 3608, 3630, 3652, 3674, 3695,
    3718, 3740, 3762, 3784, 3807, 3829, 3851, 3874, 3895, 3918, 0, 2, 3, 5, 7, 9,
    12, 16, 21, 27, 35, 45, 59, 74, 91, 109, 127, 145, 164, 183, 201, 221, 240,
    259, 278, 298, 317, 337, 356, 376, 395, 415, 434, 453, 473, 492, 512, 532,
    551, 571, 590, 610, 629, 649, 669, 688, 708, 728, 747, 767, 786, 807, 826,
    846, 865, 886, 906, 925, 945, 965, 985, 1004, 1025, 1045, 1065, 1084, 1105,
    1124, 1145, 1165, 1185, 1204, 1224, 1244, 1264, 1284, 1304, 1324, 1344, 1364,
    1384, 1404, 1424, 1444, 1464, 1484, 1504, 1524, 1544, 1564, 1584, 1604, 1624,
    1644, 1665, 1685, 1705, 1724, 1744, 1764, 1785, 1805, 1825, 1846, 1866, 1886,
    1906, 1926, 1946, 1967, 1987, 2007, 2028, 2048, 2068, 2088, 2109, 2129, 2149,
    2170, 2190, 2211, 2231, 2252, 2272, 2293, 2313, 2334, 2355, 2375, 2396, 2416,
    2437, 2458, 2478, 2499, 2520, 2540, 2561, 2582, 2603, 2624, 2644, 2665, 2686,
    2706, 2727, 2748, 2769, 2790, 2811, 2832, 2853, 2874, 2895, 2915, 2937, 2958,
    2979, 3000, 3021, 3042, 3064, 3085, 3106, 3127, 3148, 3169, 3191, 3211, 3233,
    3255, 3276, 3297, 3319, 3340, 3362, 3383, 3405, 3426, 3448, 3469, 3491, 3512,
    3534, 3555, 3578, 3599, 3621, 3643, 3664, 3687, 3709, 3731, 3753, 3775, 3797,
    3819, 3841, 3862, 3885, 0, 2, 3, 5, 7, 9, 12, 16, 20, 26, 34, 44, 57, 73, 89,
    107, 125, 143, 161, 180, 198, 218, 237, 256, 275, 294, 314, 333, 352, 371,
    391, 410, 429, 449, 468, 487, 507, 526, 545, 565, 584, 604, 623, 643, 662,
    682, 701, 721, 739, 760, 779, 799, 819, 838, 857, 877, 897, 917, 936, 956,
    976, 995, 1016, 1035, 1055, 1074, 1095, 1114, 1134, 1154, 1174, 1194, 1213,
    1233, 1253, 1273, 1293, 1313, 1332, 1352, 1372, 1391, 1412, 1431, 1451, 1471,
    1491, 1510, 1531, 1551, 1570, 1590, 1610, 1629, 1650, 1670, 1690, 1709, 1729,
    1749, 1770, 1790, 1809, 1829, 1849, 1870, 1890, 1910, 1930, 1950, 1970, 1990,
    2010, 2030, 2050, 2070, 2091, 2111, 2131, 2151, 2171, 2192, 2212, 2233, 2253,
    2273, 2294, 2314, 2334, 2354, 2375, 2395, 2416, 2437, 2457, 2478, 2498, 2519,
    2539, 2560, 2581, 2601, 2622, 2642, 2663, 2683, 2704, 2725, 2746, 2767, 2787,
    2808, 2829, 2850, 2871, 2891, 2912, 2933, 2954, 2975, 2996, 3017, 3038, 3059,
    3080, 3101, 3122, 3143, 3164, 3184, 3206, 3227, 3248, 3270, 3291, 3312, 3333,
    3355, 3376, 3397, 3419, 3440, 3462, 3483, 3505, 3525, 3548, 3569, 3591, 3613,
    3633, 3656, 3678, 3699, 3721, 3743, 3765, 3787, 3809, 3830, 3853, 0, 2, 3, 5,
    7, 9, 12, 16, 20, 26, 33, 43, 56, 71, 87, 105, 123, 141, 159, 178, 196, 215,
    234, 253, 271, 291, 310, 329, 348, 367, 386, 406, 425, 444, 463, 482, 501,
    521, 540, 559, 578, 598, 617, 636, 656, 675, 694, 714, 732, 752, 771, 791,
    811, 830, 849, 869, 889, 908, 927, 947, 967, 986, 1006, 1026, 1045, 1064,
    1085, 1104, 1124, 1144, 1163, 1183, 1203, 1222, 1242, 1262, 1281, 1301, 1321,
    1340, 1360, 1379, 1399, 1419, 1439, 1458, 1478, 1497, 1517, 1537, 1557, 1576,
    1596, 1615, 1636, 1655, 1675, 1694, 1714, 1734, 1754, 1774, 1794, 1814, 1834,
    1853, 1873, 1893, 1913, 1933, 1953, 1973, 1993, 2013, 2033, 2052, 2073, 2093,
    2112, 2133, 2153, 2173, 2193, 2214, 2234, 2254, 2274, 2294, 2315, 2334, 2355,
    2375, 2396, 2416, 2436, 2457, 2477, 2498, 2518, 2538, 2559, 2579, 2600, 2620,
    2641, 2661, 2681, 2702, 2723, 2743, 2764, 2785, 2805, 2826, 2846, 2866, 2888,
    2908, 2929, 2950, 2971, 2991, 3012, 3033, 3054, 3075, 3096, 3116, 3137, 3158,
    3179, 3200, 3221, 3242, 3263, 3284, 3306, 3327, 3348, 3369, 3390, 3411, 3433,
    3454, 3475, 3496, 3518, 3540, 3561, 3582, 3603, 3626, 3647, 3669, 3690, 3712,
    3734, 3756, 3777, 3798, 3821, 0, 2, 3, 5, 7, 9, 12, 15, 20, 25, 33, 43, 55,
    70, 86, 103, 120, 138, 157, 175, 193, 212, 231, 250, 268, 287, 306, 325, 344,
    363, 382, 401, 420, 439, 458, 477, 496, 515, 534, 554, 573, 592, 611, 630,
    649, 668, 688, 707, 725, 745, 764, 784, 803, 823, 841, 861, 881, 900, 919,
    938, 958, 977, 997, 1017, 1036, 1055, 1075, 1094, 1114, 1133, 1153, 1172,
    1192, 1211, 1231, 1250, 1270, 1289, 1309, 1328, 1348, 1367, 1387, 1406, 1426,
    1446, 1465, 1484, 1504, 1524, 1543, 1562, 1582, 1601, 1622, 1641, 1661, 1680,
    1699, 1719, 1739, 1759, 1778, 1798, 1818, 1838, 1857, 1877, 1897, 1917, 1936,
    1956, 1976, 1996, 2016, 2035, 2055, 2075, 2094, 2115, 2134, 2155, 2175, 2195,
    2215, 2235, 2255, 2275, 2295, 2315, 2335, 2355, 2376, 2396, 2416, 2436, 2456,
    2477, 2497, 2517, 2537, 2558, 2578, 2598, 2619, 2638, 2659, 2679, 2700, 2720,
    2741, 2761, 2782, 2802, 2823, 2843, 2864, 2884, 2905, 2925, 2946, 2967, 2987,
    3008, 3028, 3049, 3070, 3091, 3111, 3131, 3153, 3174, 3195, 3215, 3236, 3257,
    3278, 3299, 3320, 3341, 3362, 3383, 3404, 3425, 3447, 3467, 3489, 3510, 3532,
    3553, 3574, 3596, 3617, 3638, 3660, 3681, 3703, 3725, 3746, 3767, 3789, 0, 2,
    3, 5, 7, 9, 12, 15, 19, 25, 32, 42, 54, 68, 84, 101, 118, 136, 154, 173, 190,
    209, 228, 247, 265, 284, 303, 322, 340, 359, 378, 397, 416, 435, 453, 472,
    491, 510, 529, 548, 567, 586, 605, 624, 643, 662, 681, 700, 719, 738, 757,
    777, 796, 815, 833, 853, 872, 892, 910, 930, 949, 968, 988, 1007, 1027, 1045,
    1065, 1084, 1104, 1123, 1143, 1162, 1181, 1200, 1220, 1239, 1259, 1278, 1297,
    1316, 1336, 1355, 1375, 1394, 1414, 1433, 1452, 1471, 1491, 1511, 1530, 1549,
    1569, 1588, 1608, 1627, 1646, 1665, 1685, 1704, 1724, 1744, 1763, 1783, 1802,
    1822, 1841, 1861, 1881, 1900, 1920, 1940, 1959, 1979, 1999, 2018, 2038, 2058,
    2077, 2097, 2116, 2137, 2157, 2176, 2196, 2216, 2236, 2256, 2276, 2295, 2316,
    2335, 2356, 2376, 2396, 2416, 2436, 2456, 2476, 2496, 2516, 2536, 2556, 2577,
    2597, 2616, 2636, 2657, 2678, 2698, 2718, 2738, 2759, 2779, 2799, 2819, 2840,
    2860, 2881, 2901, 2922, 2942, 2962, 2983, 3003, 3024, 3045, 3065, 3086, 3106,
    3127, 3148, 3168, 3189, 3210, 3230, 3251, 3272, 3293, 3314, 3335, 3355, 3376,
    3397, 3418, 3439, 3460, 3481, 3503, 3524, 3544, 3566, 3587, 3609, 3630, 3651,
    3673, 3694, 3715, 3736, 3758, 0, 2, 3, 5, 7, 9, 12, 15, 19, 25, 32, 41, 53,
    67, 82, 99, 116, 134, 152, 170, 188, 207, 225, 243, 261, 281, 299, 318, 337,
    355, 374, 393, 411, 430, 449, 468, 486, 505, 524, 543, 561, 580, 599, 618,
    637, 656, 675, 693, 712, 731, 750, 769, 788, 807, 826, 845, 865, 884, 902,
    921, 941, 959, 979, 998, 1017, 1036, 1056, 1074, 1094, 1113, 1133, 1152,
    1171, 1189, 1209, 1229, 1248, 1267, 1286, 1305, 1325, 1343, 1363, 1382, 1401,
    1421, 1440, 1458, 1478, 1498, 1517, 1535, 1555, 1574, 1594, 1613, 1632, 1651,
    1670, 1690, 1710, 1729, 1748, 1768, 1787, 1807, 1826, 1845, 1865, 1884, 1904,
    1923, 1943, 1962, 1982, 2001, 2021, 2040, 2059, 2080, 2099, 2119, 2139, 2158,
    2178, 2198, 2217, 2237, 2257, 2276, 2297, 2316, 2336, 2356, 2376, 2396, 2416,
    2436, 2455, 2475, 2495, 2515, 2535, 2555, 2575, 2595, 2615, 2635, 2656, 2676,
    2696, 2716, 2736, 2756, 2776, 2796, 2817, 2837, 2857, 2877, 2898, 2918, 2938,
    2958, 2979, 2999, 3020, 3040, 3060, 3080, 3101, 3122, 3142, 3163, 3183, 3204,
    3225, 3245, 3266, 3287, 3307, 3328, 3349, 3370, 3390, 3411, 3432, 3453, 3474,
    3495, 3515, 3537, 3558, 3579, 3600, 3622, 3643, 3664, 3685, 3706, 3728, 0, 2,
    3, 5, 7, 9, 12, 15, 19, 24, 31, 40, 51, 65, 81, 97, 114, 132, 150, 168, 185,
    204, 222, 240, 258, 277, 296, 314, 333, 351, 370, 388, 407, 426, 444, 463,
    481, 500, 519, 537, 556, 575, 593, 612, 631, 649, 668, 687, 705, 725, 743,
    762, 781, 800, 818, 838, 857, 876, 894, 913, 932, 951, 970, 989, 1008, 1027,
    1046, 1065, 1085, 1104, 1123, 1142, 1161, 1179, 1199, 1218, 1237, 1256, 1275,
    1293, 1313, 1332, 1351, 1370, 1389, 1408, 1428, 1446, 1466, 1485, 1504, 1522,
    1542, 1561, 1580, 1599, 1619, 1637, 1656, 1675, 1695, 1714, 1734, 1753, 1772,
    1791, 1811, 1830, 1849, 1868, 1888, 1907, 1926, 1946, 1965, 1984, 2004, 2023,
    2042, 2062, 2081, 2101, 2121, 2140, 2160, 2179, 2199, 2219, 2238, 2257, 2278,
    2297, 2317, 2337, 2356, 2376, 2396, 2415, 2435, 2455, 2475, 2495, 2514, 2534,
    2554, 2573, 2593, 2613, 2634, 2654, 2674, 2694, 2714, 2734, 2754, 2773, 2794,
    2814, 2834, 2854, 2874, 2894, 2914, 2934, 2954, 2975, 2995, 3015, 3035, 3055,
    3076, 3096, 3117, 3137, 3157, 3178, 3198, 3219, 3239, 3260, 3280, 3301, 3322,
    3342, 3363, 3383, 3404, 3425, 3446, 3467, 3487, 3508, 3529, 3550, 3571, 3592,
    3613, 3634, 3655, 3676, 3698, 0, 2, 3, 5, 7, 9, 11, 15, 19, 24, 30, 39, 50,
    64, 79, 96, 113, 130, 147, 165, 182, 201, 219, 237, 255, 274, 292, 311, 329,
    347, 366, 384, 403, 421, 440, 458, 477, 495, 513, 532, 550, 569, 588, 606,
    625, 643, 662, 681, 699, 718, 736, 755, 774, 793, 811, 830, 849, 868, 886,
    905, 924, 942, 962, 981, 1000, 1018, 1037, 1056, 1075, 1094, 1113, 1132,
    1151, 1169, 1188, 1207, 1226, 1245, 1264, 1282, 1302, 1320, 1340, 1359, 1378,
    1396, 1415, 1434, 1453, 1472, 1491, 1510, 1529, 1547, 1567, 1586, 1605, 1623,
    1642, 1661, 1681, 1700, 1719, 1738, 1757, 1776, 1796, 1815, 1834, 1853, 1872,
    1891, 1910, 1930, 1949, 1968, 1987, 2007, 2025, 2045, 2064, 2084, 2103, 2123,
    2142, 2162, 2181, 2200, 2220, 2239, 2259, 2278, 2298, 2317, 2337, 2357, 2376,
    2396, 2415, 2435, 2455, 2474, 2494, 2514, 2533, 2552, 2572, 2592, 2612, 2632,
    2652, 2672, 2691, 2711, 2731, 2750, 2771, 2791, 2811, 2831, 2851, 2871, 2891,
    2911, 2931, 2951, 2971, 2991, 3011, 3030, 3051, 3071, 3091, 3112, 3132, 3152,
    3172, 3193, 3213, 3234, 3254, 3274, 3295, 3315, 3336, 3356, 3377, 3397, 3418,
    3439, 3459, 3480, 3501, 3522, 3542, 3563, 3584, 3605, 3626, 3646, 3668, 0, 2,
    3, 5, 7, 9, 11, 15, 18, 23, 30, 38, 49, 63, 78, 94, 111, 128, 145, 163, 180,
    198, 216, 235, 252, 271, 289, 307, 325, 344, 362, 380, 398, 417, 435, 453,
    472, 490, 508, 527, 545, 564, 582, 600, 619, 637, 656, 674, 692, 711, 729,
    748, 767, 785, 803, 823, 841, 860, 878, 897, 916, 934, 953, 972, 991, 1009,
    1028, 1046, 1066, 1084, 1103, 1122, 1141, 1159, 1178, 1197, 1216, 1234, 1253,
    1271, 1291, 1309, 1328, 1347, 1366, 1385, 1403, 1422, 1441, 1460, 1479, 1497,
    1516, 1534, 1554, 1573, 1592, 1610, 1629, 1648, 1667, 1686, 1705, 1724, 1743,
    1762, 1781, 1800, 1819, 1838, 1857, 1876, 1895, 1914, 1933, 1951, 1971, 1990,
    2009, 2028, 2047, 2067, 2086, 2105, 2125, 2144, 2163, 2182, 2202, 2220, 2240,
    2259, 2279, 2298, 2318, 2337, 2357, 2376, 2396, 2415, 2435, 2454, 2474, 2493,
    2513, 2532, 2551, 2571, 2591, 2611, 2630, 2650, 2670, 2689, 2709, 2728, 2748,
    2768, 2788, 2808, 2828, 2847, 2867, 2887, 2907, 2927, 2947, 2967, 2987, 3006,
    3027, 3047, 3067, 3087, 3107, 3127, 3147, 3167, 3187, 3208, 3228, 3248, 3268,
    3289, 3309, 3329, 3350, 3370, 3391, 3411, 3431, 3452, 3473, 3494, 3514, 3535,
    3556, 3576, 3597, 3617, 3639, 0, 2, 3, 5, 7, 9, 11, 14, 18, 23, 29, 38, 48,
    61, 76, 92, 109, 126, 143, 160, 178, 196, 214, 232, 249, 268, 286, 304, 322,
    340, 358, 376, 394, 412, 431, 449, 467, 485, 503, 522, 540, 558, 576, 595,
    613, 631, 650, 668, 686, 705, 722, 741, 760, 778, 796, 815, 834, 852, 870,
    889, 908, 926, 945, 963, 982, 1000, 1019, 1037, 1056, 1075, 1094, 1112, 1131,
    1149, 1168, 1187, 1205, 1224, 1243, 1261, 1280, 1298, 1317, 1336, 1354, 1373,
    1392, 1410, 1429, 1448, 1466, 1484, 1504, 1522, 1541, 1560, 1578, 1596, 1615,
    1634, 1653, 1672, 1691, 1710, 1728, 1747, 1766, 1785, 1804, 1822, 1841, 1860,
    1879, 1898, 1917, 1935, 1955, 1974, 1992, 2012, 2030, 2050, 2069, 2088, 2107,
    2126, 2145, 2165, 2184, 2202, 2222, 2241, 2261, 2280, 2299, 2318, 2338, 2357,
    2376, 2396, 2415, 2434, 2454, 2473, 2492, 2511, 2531, 2550, 2570, 2590, 2609,
    2629, 2648, 2668, 2687, 2706, 2726, 2746, 2766, 2785, 2805, 2825, 2844, 2864,
    2884, 2903, 2923, 2943, 2963, 2982, 3002, 3022, 3042, 3062, 3082, 3102, 3122,
    3142, 3162, 3182, 3202, 3222, 3242, 3262, 3283, 3302, 3323, 3343, 3364, 3384,
    3404, 3425, 3445, 3466, 3486, 3507, 3527, 3548, 3569, 3589, 3610, 0, 2, 3, 5,
    6, 9, 11, 14, 18, 23, 29, 37, 47, 60, 75, 90, 107, 124, 141, 158, 175, 193,
    211, 229, 246, 264, 282, 300, 318, 336, 354, 372, 390, 408, 426, 444, 462,
    480, 498, 517, 535, 553, 571, 589, 607, 625, 644, 662, 679, 698, 716, 735,
    753, 771, 789, 808, 826, 845, 862, 881, 900, 918, 937, 955, 974, 991, 1010,
    1028, 1047, 1066, 1084, 1103, 1121, 1139, 1158, 1177, 1195, 1214, 1232, 1250,
    1269, 1287, 1306, 1324, 1343, 1361, 1380, 1398, 1417, 1436, 1454, 1472, 1491,
    1509, 1528, 1547, 1565, 1583, 1602, 1620, 1640, 1658, 1677, 1695, 1714, 1733,
    1751, 1770, 1789, 1808, 1826, 1845, 1864, 1883, 1901, 1920, 1939, 1958, 1976,
    1996, 2014, 2033, 2052, 2071, 2090, 2109, 2128, 2147, 2166, 2185, 2204, 2223,
    2242, 2261, 2281, 2300, 2319, 2338, 2357, 2376, 2396, 2415, 2434, 2453, 2472,
    2491, 2510, 2530, 2550, 2569, 2588, 2608, 2627, 2646, 2666, 2685, 2705, 2724,
    2744, 2763, 2783, 2802, 2822, 2841, 2861, 2880, 2900, 2920, 2939, 2958, 2979,
    2998, 3018, 3038, 3058, 3077, 3097, 3117, 3137, 3157, 3177, 3197, 3217, 3237,
    3257, 3276, 3297, 3317, 3337, 3357, 3377, 3398, 3418, 3438, 3459, 3479, 3500,
    3520, 3540, 3560, 3581, 0, 2, 3, 5, 6, 9, 11, 14, 18, 22, 28, 36, 46, 59, 73,
    89, 105, 122, 139, 156, 173, 191, 208, 226, 243, 261, 279, 297, 315, 333,
    350, 368, 386, 404, 422, 440, 458, 476, 494, 512, 530, 548, 565, 583, 602,
    620, 638, 656, 673, 692, 709, 728, 746, 764, 782, 801, 819, 837, 855, 873,
    892, 910, 929, 947, 965, 983, 1002, 1019, 1038, 1057, 1075, 1093, 1112, 1129,
    1148, 1167, 1185, 1203, 1222, 1239, 1258, 1276, 1295, 1313, 1332, 1350, 1369,
    1386, 1405, 1424, 1442, 1460, 1479, 1497, 1516, 1534, 1552, 1570, 1589, 1607,
    1626, 1645, 1663, 1682, 1700, 1719, 1737, 1756, 1774, 1793, 1811, 1830, 1849,
    1867, 1886, 1904, 1923, 1942, 1960, 1979, 1998, 2017, 2036, 2055, 2073, 2092,
    2111, 2130, 2149, 2167, 2187, 2205, 2224, 2243, 2262, 2281, 2300, 2319, 2338,
    2357, 2376, 2395, 2415, 2434, 2453, 2471, 2490, 2510, 2529, 2549, 2568, 2587,
    2606, 2625, 2645, 2663, 2683, 2702, 2722, 2741, 2760, 2780, 2799, 2819, 2838,
    2857, 2877, 2896, 2916, 2935, 2955, 2975, 2994, 3014, 3033, 3053, 3073, 3092,
    3112, 3132, 3152, 3172, 3191, 3211, 3231, 3250, 3271, 3291, 3311, 3331, 3350,
    3371, 3391, 3411, 3432, 3452, 3472, 3492, 3513, 3532, 3553, 0, 2, 3, 5, 6, 8,
    11, 14, 17, 22, 28, 36, 45, 58, 72, 87, 103, 120, 137, 154, 170, 188, 206,
    223, 240, 258, 276, 294, 311, 329, 347, 364, 382, 400, 418, 435, 453, 471,
    489, 507, 524, 542, 560, 578, 596, 614, 632, 650, 667, 686, 703, 722, 740,
    758, 775, 794, 812, 830, 847, 865, 884, 902, 920, 939, 957, 974, 993, 1011,
    1029, 1048, 1066, 1084, 1102, 1120, 1139, 1157, 1175, 1193, 1211, 1229, 1248,
    1265, 1284, 1303, 1321, 1339, 1357, 1375, 1394, 1412, 1430, 1448, 1467, 1484,
    1503, 1521, 1540, 1557, 1576, 1594, 1613, 1631, 1650, 1668, 1686, 1705, 1723,
    1742, 1760, 1778, 1797, 1815, 1834, 1852, 1871, 1889, 1908, 1926, 1944, 1964,
    1982, 2001, 2019, 2038, 2057, 2075, 2094, 2113, 2132, 2150, 2169, 2187, 2207,
    2225, 2244, 2263, 2282, 2301, 2320, 2339, 2358, 2376, 2395, 2414, 2433, 2452,
    2471, 2490, 2509, 2528, 2547, 2566, 2586, 2605, 2624, 2642, 2662, 2681, 2700,
    2720, 2739, 2758, 2777, 2796, 2816, 2835, 2854, 2874, 2893, 2912, 2932, 2951,
    2971, 2990, 3010, 3029, 3049, 3068, 3088, 3107, 3127, 3147, 3166, 3186, 3206,
    3225, 3245, 3265, 3285, 3305, 3324, 3345, 3365, 3385, 3405, 3425, 3445, 3465,
    3485, 3505, 3526, 0, 2, 3, 5, 6, 8, 11, 14, 17, 22, 28, 35, 45, 56, 70, 85,
    101, 118, 135, 151, 168, 186, 203, 220, 237, 255, 273, 290, 308, 325, 343,
    361, 378, 396, 414, 431, 449, 466, 484, 502, 519, 537, 555, 573, 590, 608,
    626, 644, 661, 679, 697, 715, 733, 751, 768, 787, 805, 823, 840, 858, 877,
    894, 913, 931, 949, 966, 985, 1002, 1021, 1039, 1057, 1075, 1093, 1110, 1129,
    1147, 1165, 1183, 1201, 1219, 1238, 1255, 1274, 1292, 1310, 1328, 1346, 1364,
    1382, 1400, 1418, 1436, 1455, 1472, 1491, 1509, 1527, 1545, 1563, 1581, 1600,
    1618, 1636, 1654, 1673, 1691, 1709, 1727, 1746, 1764, 1782, 1801, 1819, 1837,
    1856, 1874, 1893, 1911, 1929, 1948, 1966, 1985, 2003, 2022, 2040, 2059, 2077,
    2096, 2115, 2133, 2152, 2170, 2189, 2208, 2227, 2245, 2264, 2283, 2301, 2320,
    2339, 2358, 2377, 2395, 2414, 2432, 2451, 2470, 2490, 2508, 2527, 2546, 2565,
    2584, 2603, 2622, 2641, 2660, 2679, 2698, 2717, 2736, 2755, 2775, 2794, 2813,
    2832, 2851, 2870, 2889, 2909, 2928, 2947, 2967, 2986, 3006, 3025, 3044, 3064,
    3083, 3103, 3122, 3142, 3161, 3181, 3200, 3220, 3240, 3260, 3279, 3298, 3319,
    3339, 3359, 3378, 3398, 3418, 3438, 3458, 3478, 3498, 0, 2, 3, 5, 6, 8, 11,
    14, 17, 21, 27, 34, 44, 55, 69, 84, 100, 116, 132, 149, 166, 183, 200, 218,
    234, 252, 270, 287, 305, 322, 339, 357, 374, 392, 409, 427, 444, 462, 479,
    497, 515, 532, 550, 567, 585, 603, 620, 638, 655, 673, 690, 709, 726, 744,
    761, 780, 798, 815, 833, 851, 869, 886, 905, 923, 940, 958, 976, 994, 1012,
    1030, 1048, 1066, 1084, 1101, 1120, 1138, 1156, 1173, 1191, 1209, 1227, 1245,
    1263, 1281, 1299, 1317, 1335, 1352, 1371, 1389, 1407, 1424, 1443, 1460, 1479,
    1497, 1515, 1532, 1550, 1568, 1587, 1605, 1623, 1641, 1659, 1677, 1696, 1714,
    1732, 1750, 1768, 1786, 1805, 1823, 1841, 1859, 1878, 1896, 1913, 1932, 1950,
    1969, 1987, 2006, 2024, 2043, 2061, 2079, 2098, 2116, 2135, 2153, 2172, 2190,
    2209, 2228, 2246, 2265, 2283, 2302, 2321, 2339, 2358, 2377, 2395, 2413, 2432,
    2451, 2470, 2489, 2508, 2526, 2545, 2564, 2583, 2601, 2621, 2639, 2658, 2677,
    2696, 2715, 2734, 2753, 2772, 2791, 2810, 2829, 2848, 2867, 2886, 2905, 2925,
    2944, 2963, 2982, 3001, 3021, 3040, 3059, 3079, 3098, 3117, 3137, 3156, 3175,
    3195, 3215, 3234, 3254, 3273, 3293, 3313, 3333, 3352, 3372, 3392, 3412, 3432,
    3451, 3471, 0, 2, 3, 4, 6, 8, 11, 13, 17, 21, 27, 34, 43, 54, 68, 82, 98,
    114, 130, 147, 163, 181, 198, 215, 232, 249, 267, 284, 301, 319, 336, 353,
    371, 388, 405, 423, 440, 457, 475, 492, 510, 527, 545, 562, 580, 597, 615,
    632, 649, 667, 684, 702, 720, 738, 755, 773, 791, 808, 826, 843, 862, 879,
    897, 915, 933, 950, 968, 985, 1004, 1021, 1039, 1057, 1075, 1092, 1110, 1128,
    1146, 1164, 1182, 1199, 1217, 1234, 1253, 1271, 1289, 1306, 1324, 1341, 1360,
    1378, 1395, 1413, 1431, 1448, 1467, 1485, 1503, 1520, 1538, 1556, 1574, 1592,
    1610, 1628, 1646, 1664, 1682, 1700, 1718, 1736, 1754, 1772, 1790, 1808, 1826,
    1844, 1863, 1881, 1898, 1917, 1935, 1954, 1972, 1990, 2008, 2027, 2045, 2063,
    2081, 2099, 2118, 2136, 2155, 2173, 2192, 2210, 2229, 2247, 2265, 2284, 2302,
    2321, 2339, 2358, 2377, 2395, 2413, 2432, 2451, 2470, 2488, 2507, 2525, 2544,
    2563, 2581, 2600, 2619, 2638, 2656, 2675, 2694, 2713, 2732, 2751, 2769, 2788,
    2807, 2826, 2844, 2864, 2883, 2902, 2921, 2940, 2959, 2978, 2997, 3017, 3036,
    3055, 3074, 3094, 3113, 3132, 3151, 3171, 3190, 3210, 3229, 3248, 3268, 3288,
    3307, 3327, 3346, 3366, 3386, 3405, 3424, 3445, 0, 2, 3, 4, 6, 8, 10, 13, 17,
    21, 26, 33, 42, 53, 66, 81, 96, 112, 129, 145, 161, 178, 195, 212, 229, 246,
    264, 281, 298, 315, 332, 350, 367, 384, 401, 419, 436, 453, 470, 488, 505,
    522, 540, 557, 574, 592, 609, 626, 643, 661, 678, 696, 714, 731, 748, 766,
    784, 801, 818, 836, 854, 871, 889, 907, 925, 942, 960, 977, 995, 1013, 1031,
    1048, 1066, 1083, 1101, 1119, 1137, 1154, 1172, 1189, 1207, 1224, 1243, 1260,
    1278, 1296, 1313, 1331, 1349, 1367, 1384, 1401, 1420, 1437, 1455, 1473, 1491,
    1508, 1526, 1543, 1562, 1580, 1597, 1615, 1633, 1651, 1669, 1687, 1704, 1722,
    1740, 1758, 1776, 1794, 1812, 1829, 1848, 1866, 1883, 1902, 1920, 1938, 1956,
    1974, 1993, 2011, 2029, 2047, 2065, 2083, 2102, 2119, 2138, 2156, 2175, 2193,
    2211, 2230, 2248, 2266, 2285, 2303, 2321, 2340, 2358, 2376, 2394, 2413, 2432,
    2450, 2469, 2487, 2506, 2524, 2543, 2561, 2580, 2599, 2617, 2636, 2655, 2673,
    2692, 2711, 2729, 2748, 2767, 2786, 2804, 2823, 2842, 2861, 2880, 2899, 2918,
    2937, 2956, 2975, 2994, 3013, 3032, 3051, 3070, 3089, 3108, 3127, 3147, 3166,
    3185, 3204, 3223, 3243, 3263, 3282, 3301, 3321, 3340, 3360, 3379, 3398, 3419,
    0, 2, 3, 4, 6, 8, 10, 13, 16, 21, 26, 33, 41, 52, 65, 79, 95, 110, 127, 143,
    159, 176, 193, 210, 226, 244, 261, 278, 295, 312, 329, 346, 363, 380, 397,
    414, 432, 449, 466, 483, 500, 517, 535, 552, 569, 586, 604, 621, 638, 655,
    672, 690, 708, 725, 742, 760, 777, 795, 811, 829, 847, 864, 882, 899, 917,
    934, 952, 969, 987, 1004, 1022, 1040, 1057, 1074, 1092, 1110, 1127, 1145,
    1162, 1179, 1197, 1214, 1233, 1250, 1268, 1285, 1303, 1320, 1338, 1356, 1373,
    1390, 1408, 1425, 1444, 1461, 1479, 1496, 1513, 1531, 1549, 1567, 1585, 1602,
    1620, 1638, 1656, 1673, 1691, 1709, 1727, 1744, 1762, 1780, 1798, 1815, 1834,
    1851, 1869, 1887, 1905, 1923, 1941, 1959, 1977, 1995, 2013, 2031, 2049, 2067,
    2085, 2103, 2122, 2140, 2158, 2176, 2194, 2212, 2231, 2249, 2267, 2285, 2303,
    2322, 2340, 2358, 2376, 2394, 2413, 2432, 2450, 2468, 2487, 2505, 2524, 2541,
    2560, 2579, 2597, 2616, 2634, 2653, 2671, 2690, 2709, 2727, 2746, 2764, 2783,
    2801, 2820, 2839, 2858, 2877, 2895, 2914, 2933, 2952, 2971, 2990, 3009, 3028,
    3047, 3066, 3085, 3103, 3123, 3142, 3161, 3180, 3199, 3219, 3238, 3257, 3276,
    3296, 3315, 3334, 3354, 3373, 3393, 0, 2, 3, 4, 6, 8, 10, 13, 16, 20, 25, 32,
    40, 51, 64, 78, 93, 109, 125, 141, 157, 174, 190, 207, 223, 241, 258, 275,
    292, 308, 325, 342, 359, 376, 393, 410, 427, 444, 461, 478, 496, 513, 530,
    547, 564, 581, 598, 615, 632, 650, 666, 684, 701, 719, 735, 753, 770, 788,
    805, 822, 840, 857, 874, 892, 909, 926, 944, 961, 979, 996, 1014, 1031, 1048,
    1065, 1083, 1101, 1118, 1136, 1153, 1170, 1188, 1205, 1223, 1240, 1258, 1275,
    1292, 1309, 1327, 1345, 1362, 1379, 1397, 1414, 1432, 1450, 1467, 1484, 1502,
    1519, 1537, 1555, 1572, 1590, 1607, 1625, 1643, 1660, 1678, 1695, 1713, 1731,
    1748, 1766, 1784, 1801, 1819, 1837, 1854, 1873, 1890, 1908, 1926, 1944, 1962,
    1980, 1998, 2015, 2033, 2051, 2069, 2087, 2105, 2123, 2141, 2159, 2177, 2195,
    2213, 2231, 2250, 2268, 2286, 2304, 2322, 2340, 2358, 2376, 2395, 2413, 2431,
    2449, 2468, 2486, 2504, 2522, 2541, 2559, 2578, 2596, 2614, 2633, 2651, 2670,
    2688, 2706, 2725, 2743, 2762, 2780, 2799, 2818, 2836, 2855, 2874, 2892, 2911,
    2930, 2948, 2967, 2986, 3005, 3024, 3042, 3061, 3080, 3099, 3118, 3137, 3156,
    3175, 3194, 3213, 3233, 3252, 3271, 3290, 3309, 3329, 3347, 3367, 0, 2, 3, 4,
    6, 8, 10, 13, 16, 20, 25, 31, 40, 50, 62, 76, 91, 107, 123, 139, 155, 172,
    188, 205, 221, 238, 255, 272, 288, 305, 322, 339, 356, 373, 390, 406, 423,
    440, 457, 474, 491, 508, 525, 542, 559, 576, 593, 610, 626, 644, 661, 678,
    695, 712, 729, 747, 764, 781, 798, 815, 833, 849, 867, 884, 902, 918, 936,
    953, 971, 988, 1005, 1023, 1040, 1057, 1074, 1092, 1109, 1126, 1144, 1160,
    1178, 1195, 1213, 1230, 1248, 1265, 1282, 1299, 1317, 1334, 1351, 1368, 1386,
    1403, 1421, 1438, 1456, 1472, 1490, 1507, 1525, 1543, 1560, 1577, 1595, 1612,
    1630, 1647, 1665, 1682, 1700, 1717, 1735, 1752, 1770, 1787, 1805, 1823, 1840,
    1858, 1875, 1894, 1911, 1929, 1947, 1964, 1982, 2000, 2018, 2035, 2053, 2071,
    2089, 2107, 2125, 2143, 2161, 2179, 2197, 2214, 2232, 2250, 2268, 2286, 2304,
    2322, 2340, 2358, 2377, 2395, 2413, 2431, 2449, 2467, 2485, 2503, 2522, 2540,
    2558, 2576, 2595, 2613, 2631, 2649, 2668, 2686, 2704, 2723, 2741, 2759, 2778,
    2796, 2815, 2833, 2852, 2870, 2889, 2908, 2926, 2945, 2963, 2982, 3001, 3020,
    3038, 3056, 3076, 3095, 3114, 3132, 3151, 3170, 3189, 3208, 3227, 3246, 3266,
    3285, 3304, 3322, 3342, 0, 2, 3, 4, 6, 8, 10, 13, 16, 20, 25, 31, 39, 49, 61,
    75, 90, 105, 121, 137, 152, 169, 186, 202, 218, 235, 252, 269, 285, 302, 319,
    335, 352, 369, 386, 402, 419, 436, 453, 470, 486, 503, 520, 537, 554, 571,
    588, 605, 621, 638, 655, 672, 689, 706, 723, 740, 757, 774, 791, 808, 826,
    842, 860, 877, 894, 911, 928, 945, 963, 980, 997, 1014, 1031, 1048, 1066,
    1083, 1100, 1117, 1134, 1151, 1169, 1185, 1203, 1220, 1238, 1255, 1272, 1289,
    1306, 1324, 1341, 1357, 1375, 1392, 1410, 1427, 1444, 1461, 1478, 1495, 1513,
    1531, 1548, 1565, 1582, 1600, 1617, 1634, 1652, 1669, 1687, 1704, 1721, 1739,
    1756, 1773, 1791, 1809, 1826, 1844, 1861, 1879, 1897, 1914, 1932, 1949, 1967,
    1985, 2002, 2019, 2038, 2055, 2073, 2091, 2109, 2126, 2144, 2162, 2180, 2198,
    2215, 2233, 2251, 2269, 2287, 2304, 2322, 2340, 2359, 2377, 2395, 2413, 2431,
    2449, 2467, 2484, 2503, 2521, 2539, 2557, 2575, 2593, 2611, 2629, 2648, 2666,
    2684, 2702, 2721, 2738, 2757, 2775, 2794, 2812, 2831, 2849, 2867, 2886, 2904,
    2923, 2941, 2960, 2978, 2997, 3016, 3034, 3053, 3072, 3090, 3109, 3127, 3147,
    3166, 3184, 3203, 3222, 3241, 3260, 3279, 3298, 3317, 0, 2, 3, 4, 6, 8, 10,
    13, 16, 20, 24, 30, 38, 48, 60, 74, 88, 103, 119, 135, 150, 167, 183, 200,
    216, 233, 249, 266, 282, 299, 315, 332, 349, 365, 382, 399, 415, 432, 449,
    465, 482, 499, 515, 532, 549, 566, 582, 599, 615, 633, 649, 667, 683, 700,
    717, 734, 751, 768, 784, 801, 819, 835, 853, 870, 887, 903, 921, 937, 955,
    972, 989, 1006, 1023, 1040, 1057, 1074, 1091, 1108, 1125, 1142, 1160, 1176,
    1194, 1211, 1228, 1245, 1262, 1278, 1296, 1313, 1330, 1347, 1364, 1381, 1399,
    1416, 1433, 1449, 1467, 1484, 1501, 1519, 1536, 1553, 1570, 1587, 1605, 1622,
    1639, 1656, 1674, 1691, 1708, 1726, 1743, 1760, 1778, 1795, 1812, 1830, 1847,
    1865, 1882, 1900, 1917, 1935, 1952, 1970, 1987, 2004, 2022, 2039, 2057, 2075,
    2093, 2110, 2128, 2146, 2163, 2181, 2199, 2216, 2234, 2252, 2270, 2287, 2305,
    2323, 2341, 2359, 2377, 2394, 2412, 2430, 2448, 2465, 2484, 2502, 2520, 2538,
    2556, 2574, 2592, 2610, 2628, 2646, 2664, 2682, 2700, 2718, 2737, 2755, 2773,
    2791, 2809, 2828, 2846, 2864, 2883, 2901, 2919, 2938, 2956, 2975, 2993, 3011,
    3030, 3049, 3067, 3086, 3104, 3123, 3142, 3161, 3180, 3198, 3217, 3236, 3255,
    3273, 3293, 0, 2, 3, 4, 6, 8, 10, 12, 16, 19, 24, 30, 38, 47, 59, 72, 87,
    102, 117, 133, 148, 165, 181, 197, 213, 230, 246, 263, 279, 296, 312, 329,
    345, 362, 378, 395, 411, 428, 444, 461, 478, 494, 511, 527, 544, 561, 577,
    594, 610, 627, 644, 661, 678, 694, 711, 728, 745, 762, 778, 795, 812, 828,
    846, 863, 880, 896, 913, 930, 947, 964, 981, 998, 1015, 1031, 1049, 1066,
    1083, 1100, 1116, 1133, 1150, 1167, 1184, 1201, 1218, 1235, 1252, 1268, 1286,
    1303, 1320, 1336, 1354, 1370, 1388, 1405, 1422, 1438, 1455, 1472, 1490, 1507,
    1524, 1541, 1558, 1575, 1592, 1609, 1627, 1644, 1661, 1678, 1695, 1712, 1730,
    1746, 1764, 1781, 1798, 1816, 1833, 1851, 1868, 1885, 1903, 1920, 1937, 1955,
    1972, 1989, 2007, 2024, 2042, 2059, 2077, 2094, 2112, 2130, 2147, 2165, 2182,
    2200, 2217, 2235, 2253, 2270, 2287, 2305, 2323, 2341, 2359, 2377, 2394, 2412,
    2430, 2447, 2465, 2483, 2501, 2519, 2537, 2555, 2573, 2590, 2608, 2626, 2644,
    2662, 2680, 2698, 2716, 2734, 2752, 2771, 2789, 2807, 2825, 2843, 2861, 2880,
    2898, 2916, 2934, 2953, 2971, 2989, 3008, 3026, 3045, 3063, 3081, 3100, 3119,
    3138, 3156, 3175, 3194, 3212, 3231, 3249, 3269, 0, 2, 3, 4, 6, 8, 10, 12, 15,
    19, 24, 30, 37, 46, 58, 71, 85, 100, 115, 131, 146, 163, 179, 195, 211, 227,
    244, 260, 276, 293, 309, 325, 342, 358, 375, 391, 407, 424, 440, 457, 473,
    490, 506, 523, 539, 556, 572, 589, 605, 622, 638, 655, 672, 688, 705, 722,
    738, 755, 771, 788, 805, 822, 839, 856, 872, 889, 906, 922, 940, 956, 973,
    990, 1007, 1023, 1040, 1057, 1074, 1091, 1108, 1124, 1141, 1158, 1175, 1192,
    1209, 1225, 1242, 1259, 1276, 1293, 1310, 1326, 1343, 1360, 1377, 1394, 1411,
    1427, 1444, 1461, 1478, 1495, 1512, 1529, 1546, 1563, 1580, 1597, 1614, 1631,
    1648, 1665, 1682, 1699, 1717, 1733, 1751, 1768, 1784, 1802, 1819, 1837, 1854,
    1871, 1888, 1905, 1923, 1940, 1957, 1974, 1992, 2009, 2027, 2044, 2061, 2079,
    2096, 2114, 2131, 2149, 2166, 2183, 2201, 2218, 2236, 2253, 2270, 2288, 2306,
    2324, 2341, 2359, 2377, 2394, 2412, 2429, 2447, 2465, 2483, 2500, 2518, 2536,
    2554, 2571, 2589, 2607, 2625, 2643, 2660, 2678, 2696, 2714, 2732, 2750, 2768,
    2786, 2804, 2822, 2840, 2858, 2877, 2895, 2913, 2931, 2949, 2967, 2986, 3004,
    3023, 3041, 3059, 3078, 3096, 3115, 3133, 3152, 3170, 3189, 3207, 3225, 3245,
    0, 2, 3, 4, 6, 8, 10, 12, 15, 19, 23, 29, 36, 46, 57, 70, 84, 98, 114, 129,
    144, 161, 177, 193, 208, 225, 241, 257, 273, 290, 306, 322, 338, 355, 371,
    387, 404, 420, 436, 452, 469, 485, 502, 518, 534, 551, 567, 584, 600, 617,
    633, 650, 666, 683, 699, 716, 732, 749, 765, 782, 799, 815, 832, 849, 865,
    881, 899, 915, 932, 949, 965, 982, 999, 1015, 1032, 1049, 1066, 1082, 1099,
    1115, 1132, 1149, 1166, 1182, 1199, 1216, 1233, 1249, 1266, 1283, 1300, 1316,
    1333, 1349, 1367, 1383, 1400, 1416, 1433, 1450, 1467, 1484, 1501, 1518, 1534,
    1551, 1568, 1585, 1602, 1619, 1636, 1653, 1670, 1687, 1704, 1720, 1738, 1755,
    1771, 1789, 1805, 1823, 1840, 1857, 1874, 1891, 1908, 1926, 1943, 1959, 1977,
    1994, 2012, 2029, 2046, 2063, 2081, 2098, 2115, 2133, 2150, 2167, 2185, 2202,
    2219, 2236, 2254, 2271, 2289, 2307, 2324, 2341, 2359, 2376, 2394, 2411, 2429,
    2447, 2464, 2482, 2499, 2517, 2535, 2552, 2570, 2588, 2605, 2623, 2641, 2658,
    2677, 2694, 2712, 2730, 2748, 2766, 2784, 2802, 2820, 2838, 2856, 2874, 2892,
    2910, 2928, 2945, 2964, 2982, 3000, 3019, 3036, 3055, 3074, 3092, 3110, 3129,
    3147, 3166, 3184, 3202, 3221, 0, 2, 3, 4, 6, 8, 10, 12, 15, 19, 23, 29, 36,
    45, 56, 68, 82, 97, 112, 127, 142, 159, 174, 190, 206, 222, 238, 254, 270,
    287, 303, 319, 335, 351, 367, 384, 400, 416, 432, 448, 465, 481, 497, 513,
    530, 546, 562, 579, 594, 611, 627, 644, 660, 677, 693, 710, 726, 743, 759,
    775, 792, 808, 825, 842, 858, 874, 891, 907, 925, 941, 958, 974, 991, 1007,
    1024, 1041, 1057, 1074, 1090, 1106, 1124, 1140, 1157, 1173, 1190, 1207, 1223,
    1239, 1256, 1273, 1290, 1306, 1323, 1339, 1356, 1373, 1389, 1405, 1422, 1439,
    1456, 1473, 1489, 1506, 1523, 1540, 1556, 1573, 1590, 1607, 1623, 1640, 1657,
    1674, 1691, 1707, 1725, 1741, 1758, 1775, 1792, 1809, 1826, 1843, 1860, 1877,
    1894, 1911, 1928, 1945, 1962, 1979, 1997, 2014, 2031, 2048, 2065, 2082, 2100,
    2117, 2134, 2151, 2169, 2186, 2203, 2220, 2237, 2254, 2272, 2290, 2307, 2324,
    2342, 2359, 2376, 2393, 2411, 2429, 2446, 2464, 2481, 2499, 2516, 2534, 2551,
    2569, 2586, 2604, 2622, 2639, 2657, 2675, 2692, 2710, 2728, 2746, 2763, 2781,
    2799, 2817, 2835, 2853, 2871, 2889, 2907, 2924, 2943, 2961, 2979, 2997, 3014,
    3033, 3051, 3070, 3088, 3106, 3124, 3143, 3161, 3179, 3198, 0, 2, 3, 4, 6, 8,
    10, 12, 15, 18, 23, 28, 35, 44, 55, 67, 81, 95, 110, 125, 140, 156, 172, 188,
    203, 220, 236, 252, 268, 284, 300, 316, 332, 348, 364, 380, 396, 412, 428,
    444, 460, 477, 493, 509, 525, 541, 557, 574, 589, 606, 622, 639, 655, 671,
    687, 704, 720, 737, 752, 769, 786, 802, 819, 835, 851, 867, 884, 900, 917,
    934, 950, 967, 983, 999, 1016, 1032, 1049, 1065, 1082, 1098, 1115, 1131,
    1148, 1164, 1181, 1197, 1214, 1230, 1247, 1263, 1280, 1296, 1313, 1329, 1346,
    1362, 1379, 1395, 1411, 1428, 1445, 1462, 1478, 1495, 1511, 1528, 1545, 1561,
    1578, 1595, 1611, 1628, 1645, 1661, 1678, 1694, 1712, 1728, 1745, 1762, 1778,
    1796, 1813, 1829, 1846, 1863, 1880, 1897, 1914, 1930, 1948, 1964, 1982, 1999,
    2016, 2033, 2050, 2067, 2084, 2101, 2118, 2135, 2153, 2170, 2187, 2203, 2221,
    2238, 2256, 2273, 2290, 2307, 2325, 2342, 2359, 2376, 2394, 2411, 2428, 2446,
    2463, 2480, 2498, 2515, 2533, 2550, 2568, 2585, 2603, 2620, 2638, 2655, 2673,
    2690, 2708, 2726, 2743, 2761, 2779, 2797, 2814, 2832, 2850, 2868, 2886, 2903,
    2921, 2939, 2957, 2975, 2993, 3011, 3029, 3047, 3066, 3084, 3102, 3120, 3138,
    3156, 3175, 0, 2, 3, 4, 6, 7, 9, 12, 15, 18, 22, 28, 35, 43, 54, 66, 79, 94,
    108, 124, 138, 154, 170, 186, 201, 217, 233, 249, 265, 281, 297, 312, 328,
    344, 360, 376, 392, 408, 424, 440, 456, 472, 488, 504, 520, 537, 553, 569,
    584, 601, 617, 633, 649, 666, 681, 698, 714, 731, 746, 763, 779, 795, 812,
    828, 845, 860, 877, 893, 910, 926, 943, 959, 975, 991, 1008, 1024, 1041,
    1057, 1073, 1089, 1106, 1122, 1139, 1155, 1172, 1188, 1204, 1220, 1237, 1254,
    1270, 1286, 1303, 1319, 1336, 1352, 1368, 1384, 1401, 1417, 1434, 1451, 1467,
    1484, 1500, 1517, 1533, 1550, 1566, 1583, 1599, 1616, 1632, 1649, 1666, 1682,
    1699, 1716, 1732, 1749, 1765, 1782, 1799, 1816, 1833, 1849, 1866, 1883, 1900,
    1916, 1934, 1950, 1967, 1984, 2001, 2018, 2035, 2052, 2069, 2086, 2103, 2120,
    2137, 2154, 2171, 2187, 2204, 2222, 2239, 2256, 2273, 2291, 2308, 2325, 2342,
    2359, 2376, 2394, 2411, 2428, 2445, 2463, 2480, 2497, 2514, 2532, 2549, 2566,
    2584, 2601, 2619, 2636, 2654, 2671, 2689, 2706, 2724, 2741, 2759, 2776, 2794,
    2812, 2829, 2847, 2865, 2882, 2900, 2918, 2936, 2954, 2971, 2990, 3008, 3026,
    3044, 3062, 3080, 3098, 3116, 3133, 3152, 0, 2, 3, 4, 6, 7, 9, 12, 15, 18,
    22, 27, 34, 42, 53, 65, 78, 92, 107, 122, 136, 152, 168, 183, 198, 215, 230,
    246, 262, 278, 294, 309, 325, 341, 357, 373, 389, 404, 420, 436, 452, 468,
    484, 500, 516, 532, 548, 564, 579, 596, 611, 628, 644, 660, 676, 692, 708,
    725, 740, 756, 773, 789, 805, 822, 838, 854, 870, 886, 903, 919, 935, 951,
    968, 983, 1000, 1016, 1033, 1049, 1065, 1081, 1098, 1113, 1130, 1146, 1163,
    1179, 1195, 1211, 1228, 1244, 1260, 1276, 1293, 1309, 1325, 1342, 1358, 1374,
    1390, 1407, 1423, 1440, 1456, 1473, 1489, 1505, 1522, 1538, 1555, 1571, 1587,
    1604, 1620, 1637, 1653, 1669, 1686, 1703, 1719, 1736, 1752, 1769, 1786, 1803,
    1819, 1836, 1853, 1869, 1886, 1902, 1919, 1936, 1953, 1970, 1987, 2003, 2020,
    2037, 2054, 2071, 2088, 2104, 2121, 2138, 2155, 2172, 2188, 2205, 2223, 2240,
    2257, 2274, 2291, 2308, 2325, 2342, 2359, 2376, 2393, 2411, 2428, 2445, 2462,
    2479, 2496, 2514, 2531, 2548, 2565, 2582, 2600, 2617, 2635, 2652, 2669, 2687,
    2704, 2722, 2739, 2757, 2774, 2792, 2809, 2827, 2844, 2861, 2880, 2897, 2915,
    2933, 2950, 2968, 2986, 3004, 3022, 3040, 3058, 3076, 3094, 3111, 3130, 0, 2,
    3, 4, 6, 7, 9, 12, 14, 18, 22, 27, 33, 42, 52, 63, 77, 91, 105, 120, 135,
    150, 166, 181, 196, 212, 228, 243, 259, 275, 291, 306, 322, 338, 353, 369,
    385, 401, 417, 432, 448, 464, 480, 496, 511, 527, 543, 559, 574, 591, 606,
    623, 639, 655, 670, 687, 703, 719, 734, 750, 767, 782, 799, 815, 831, 847,
    863, 879, 896, 912, 928, 944, 960, 976, 992, 1009, 1025, 1041, 1057, 1073,
    1089, 1105, 1122, 1138, 1154, 1170, 1186, 1202, 1218, 1235, 1251, 1266, 1283,
    1299, 1316, 1332, 1348, 1364, 1380, 1396, 1413, 1429, 1445, 1462, 1478, 1494,
    1510, 1527, 1543, 1559, 1576, 1592, 1608, 1625, 1641, 1657, 1674, 1691, 1706,
    1723, 1739, 1756, 1773, 1789, 1806, 1823, 1839, 1856, 1872, 1888, 1906, 1922,
    1939, 1955, 1972, 1989, 2006, 2022, 2039, 2056, 2072, 2089, 2106, 2123, 2140,
    2156, 2173, 2190, 2207, 2224, 2241, 2258, 2275, 2291, 2308, 2325, 2342, 2359,
    2376, 2393, 2410, 2427, 2444, 2461, 2478, 2496, 2513, 2530, 2547, 2564, 2581,
    2599, 2616, 2633, 2650, 2668, 2685, 2702, 2720, 2737, 2754, 2772, 2789, 2807,
    2824, 2841, 2859, 2877, 2894, 2912, 2929, 2947, 2965, 2983, 3000, 3018, 3036,
    3054, 3072, 3089, 3108, 0, 2, 3, 4, 6, 7, 9, 12, 14, 18, 22, 27, 33, 41, 51,
    62, 75, 89, 104, 118, 133, 148, 164, 179, 194, 210, 225, 241, 256, 272, 288,
    303, 319, 334, 350, 366, 381, 397, 413, 428, 444, 460, 476, 491, 507, 523,
    539, 554, 570, 586, 601, 618, 633, 649, 665, 681, 697, 713, 728, 744, 761,
    776, 793, 809, 825, 840, 857, 872, 889, 905, 921, 937, 953, 968, 985, 1001,
    1017, 1033, 1049, 1064, 1081, 1096, 1113, 1129, 1145, 1161, 1177, 1193, 1209,
    1225, 1241, 1257, 1274, 1289, 1306, 1322, 1338, 1353, 1370, 1386, 1402, 1418,
    1435, 1451, 1467, 1483, 1499, 1515, 1532, 1548, 1564, 1580, 1597, 1613, 1629,
    1645, 1662, 1678, 1694, 1711, 1727, 1744, 1760, 1776, 1793, 1809, 1826, 1842,
    1859, 1875, 1892, 1908, 1925, 1941, 1958, 1974, 1991, 2008, 2024, 2041, 2058,
    2074, 2091, 2108, 2124, 2140, 2157, 2174, 2191, 2208, 2225, 2241, 2258, 2275,
    2292, 2308, 2326, 2342, 2359, 2376, 2393, 2410, 2427, 2444, 2461, 2478, 2495,
    2512, 2529, 2545, 2563, 2580, 2597, 2614, 2631, 2649, 2666, 2683, 2700, 2718,
    2735, 2752, 2769, 2787, 2804, 2821, 2839, 2856, 2874, 2891, 2908, 2927, 2944,
    2962, 2979, 2997, 3015, 3032, 3050, 3067, 3086, 0, 2, 3, 4, 6, 7, 9, 11, 14,
    17, 21, 26, 32, 40, 50, 61, 74, 88, 102, 117, 131, 146, 162, 177, 192, 207,
    223, 238, 254, 269, 285, 300, 316, 331, 347, 362, 378, 393, 409, 425, 440,
    456, 471, 487, 503, 518, 534, 550, 565, 581, 596, 612, 628, 644, 659, 676,
    691, 707, 722, 738, 755, 770, 786, 802, 818, 834, 850, 865, 882, 898, 914,
    929, 945, 961, 977, 993, 1009, 1025, 1041, 1056, 1073, 1088, 1105, 1121,
    1137, 1152, 1168, 1184, 1200, 1216, 1232, 1248, 1264, 1280, 1296, 1312, 1328,
    1343, 1359, 1375, 1392, 1408, 1424, 1440, 1456, 1472, 1488, 1504, 1520, 1537,
    1553, 1569, 1585, 1601, 1617, 1633, 1650, 1666, 1682, 1698, 1714, 1731, 1747,
    1764, 1780, 1796, 1813, 1829, 1845, 1861, 1878, 1894, 1911, 1927, 1944, 1960,
    1977, 1993, 2010, 2026, 2043, 2059, 2076, 2092, 2109, 2125, 2142, 2158, 2176,
    2192, 2209, 2225, 2242, 2259, 2276, 2292, 2309, 2326, 2343, 2359, 2376, 2393,
    2410, 2427, 2443, 2460, 2477, 2494, 2511, 2527, 2545, 2562, 2579, 2596, 2613,
    2630, 2647, 2664, 2681, 2698, 2716, 2733, 2750, 2767, 2784, 2801, 2819, 2836,
    2854, 2871, 2888, 2906, 2923, 2941, 2958, 2976, 2994, 3011, 3029, 3046, 3064,
    0, 2, 3, 4, 6, 7, 9, 11, 14, 17, 21, 26, 32, 39, 49, 60, 73, 86, 100, 115,
    129, 145, 160, 175, 189, 205, 220, 236, 251, 266, 282, 297, 313, 328, 344,
    359, 374, 390, 405, 421, 436, 452, 467, 483, 498, 514, 529, 545, 560, 576,
    591, 607, 623, 639, 654, 670, 686, 701, 717, 732, 749, 764, 780, 796, 812,
    827, 843, 859, 875, 891, 907, 922, 938, 953, 970, 986, 1001, 1017, 1033,
    1048, 1065, 1080, 1096, 1112, 1128, 1144, 1160, 1175, 1191, 1207, 1223, 1238,
    1255, 1270, 1286, 1302, 1318, 1334, 1349, 1365, 1382, 1398, 1414, 1430, 1446,
    1461, 1477, 1493, 1509, 1525, 1541, 1557, 1574, 1590, 1606, 1621, 1638, 1654,
    1670, 1686, 1702, 1719, 1735, 1751, 1767, 1783, 1800, 1816, 1832, 1848, 1865,
    1880, 1897, 1914, 1930, 1946, 1963, 1979, 1995, 2012, 2028, 2045, 2061, 2078,
    2094, 2110, 2127, 2143, 2160, 2177, 2193, 2210, 2226, 2243, 2259, 2276, 2293,
    2309, 2326, 2343, 2359, 2376, 2393, 2409, 2426, 2443, 2460, 2477, 2493, 2510,
    2527, 2544, 2561, 2578, 2595, 2611, 2628, 2645, 2662, 2679, 2697, 2714, 2731,
    2748, 2765, 2782, 2799, 2817, 2834, 2851, 2868, 2886, 2903, 2920, 2938, 2955,
    2973, 2990, 3008, 3025, 3043, 0, 2, 3, 4, 5, 7, 9, 11, 14, 17, 21, 25, 31,
    39, 48, 59, 71, 85, 99, 113, 127, 143, 158, 173, 187, 203, 218, 233, 248,
    264, 279, 294, 310, 325, 340, 356, 371, 386, 402, 417, 432, 448, 463, 479,
    494, 509, 525, 540, 555, 571, 586, 602, 618, 634, 649, 665, 680, 696, 711,
    727, 743, 758, 774, 790, 805, 821, 837, 852, 868, 884, 900, 915, 931, 946,
    962, 978, 994, 1010, 1025, 1040, 1057, 1072, 1088, 1104, 1120, 1135, 1151,
    1166, 1182, 1198, 1214, 1229, 1245, 1261, 1277, 1293, 1309, 1324, 1340, 1355,
    1372, 1388, 1403, 1419, 1435, 1451, 1467, 1483, 1498, 1514, 1530, 1546, 1562,
    1578, 1594, 1610, 1626, 1642, 1658, 1674, 1690, 1706, 1722, 1738, 1754, 1771,
    1787, 1803, 1819, 1835, 1851, 1867, 1884, 1900, 1916, 1932, 1949, 1965, 1981,
    1998, 2014, 2030, 2047, 2063, 2079, 2095, 2111, 2128, 2145, 2161, 2178, 2194,
    2211, 2227, 2244, 2260, 2277, 2293, 2310, 2326, 2343, 2359, 2376, 2393, 2409,
    2426, 2442, 2459, 2476, 2492, 2509, 2526, 2543, 2560, 2576, 2593, 2610, 2627,
    2644, 2661, 2678, 2695, 2712, 2729, 2746, 2762, 2780, 2797, 2814, 2831, 2848,
    2866, 2883, 2900, 2917, 2935, 2952, 2969, 2987, 3004, 3022, 0, 1, 3, 4, 5, 7,
    9, 11, 14, 17, 21, 25, 31, 38, 47, 58, 70, 83, 97, 112, 126, 141, 156, 170,
    185, 201, 216, 231, 246, 261, 276, 291, 307, 322, 337, 352, 368, 383, 398,
    413, 429, 444, 459, 475, 490, 505, 521, 536, 551, 567, 582, 598, 613, 628,
    643, 659, 675, 690, 705, 721, 737, 752, 768, 784, 799, 814, 830, 845, 861,
    877, 893, 908, 924, 939, 955, 971, 986, 1002, 1018, 1033, 1049, 1064, 1080,
    1096, 1111, 1127, 1143, 1158, 1174, 1189, 1205, 1220, 1236, 1251, 1268, 1283,
    1299, 1314, 1330, 1345, 1362, 1377, 1393, 1409, 1425, 1440, 1456, 1472, 1488,
    1503, 1519, 1535, 1551, 1567, 1583, 1598, 1614, 1630, 1646, 1662, 1678, 1694,
    1710, 1726, 1742, 1758, 1774, 1790, 1806, 1822, 1838, 1854, 1870, 1887, 1903,
    1919, 1935, 1951, 1967, 1983, 2000, 2016, 2032, 2048, 2065, 2080, 2097, 2113,
    2130, 2146, 2162, 2179, 2195, 2211, 2228, 2244, 2261, 2277, 2293, 2310, 2326,
    2343, 2359, 2376, 2392, 2409, 2425, 2442, 2459, 2475, 2492, 2508, 2525, 2542,
    2559, 2575, 2592, 2609, 2626, 2642, 2659, 2676, 2693, 2710, 2727, 2743, 2761,
    2778, 2795, 2812, 2828, 2846, 2863, 2880, 2897, 2914, 2932, 2949, 2966, 2983,
    3001, 0, 1, 3, 4, 5, 7, 9, 11, 14, 17, 20, 25, 30, 38, 46, 57, 69, 82, 96,
    110, 124, 139, 154, 168, 183, 198, 213, 228, 243, 258, 273, 289, 304, 319,
    334, 349, 364, 379, 395, 410, 425, 440, 455, 470, 486, 501, 516, 531, 546,
    562, 577, 593, 608, 623, 638, 654, 669, 685, 700, 715, 731, 746, 762, 777,
    793, 808, 824, 839, 855, 870, 886, 901, 917, 932, 948, 963, 979, 994, 1010,
    1025, 1041, 1056, 1072, 1088, 1103, 1119, 1134, 1149, 1165, 1181, 1196, 1211,
    1227, 1242, 1258, 1274, 1290, 1305, 1320, 1336, 1352, 1367, 1383, 1399, 1414,
    1430, 1446, 1461, 1477, 1493, 1508, 1524, 1540, 1556, 1571, 1587, 1603, 1619,
    1634, 1650, 1666, 1682, 1698, 1714, 1730, 1746, 1761, 1777, 1793, 1809, 1825,
    1841, 1857, 1873, 1889, 1905, 1921, 1937, 1953, 1970, 1986, 2002, 2018, 2034,
    2050, 2066, 2082, 2098, 2115, 2131, 2147, 2164, 2180, 2196, 2212, 2228, 2245,
    2261, 2278, 2294, 2310, 2327, 2343, 2359, 2376, 2392, 2409, 2425, 2442, 2458,
    2475, 2491, 2508, 2524, 2541, 2557, 2574, 2591, 2607, 2624, 2641, 2658, 2674,
    2691, 2708, 2724, 2742, 2758, 2775, 2792, 2809, 2826, 2843, 2860, 2877, 2894,
    2912, 2929, 2946, 2962, 2980, 0, 1, 3, 4, 5, 7, 9, 11, 13, 16, 20, 24, 30,
    37, 46, 56, 68, 81, 94, 108, 122, 137, 152, 166, 181, 196, 211, 226, 241,
    256, 271, 286, 301, 316, 331, 346, 361, 376, 391, 406, 421, 436, 451, 466,
    482, 497, 512, 527, 542, 557, 572, 588, 603, 618, 633, 649, 664, 679, 694,
    710, 725, 740, 756, 771, 787, 802, 818, 832, 848, 864, 879, 895, 910, 925,
    941, 956, 972, 987, 1002, 1017, 1033, 1048, 1064, 1079, 1095, 1110, 1126,
    1141, 1157, 1172, 1187, 1202, 1218, 1233, 1249, 1265, 1280, 1295, 1311, 1326,
    1342, 1358, 1373, 1389, 1404, 1420, 1435, 1451, 1466, 1482, 1498, 1513, 1529,
    1545, 1560, 1575, 1591, 1607, 1622, 1639, 1654, 1670, 1686, 1702, 1717, 1733,
    1749, 1765, 1781, 1796, 1812, 1828, 1844, 1860, 1876, 1892, 1908, 1924, 1940,
    1956, 1972, 1988, 2004, 2020, 2036, 2051, 2067, 2084, 2100, 2116, 2132, 2149,
    2165, 2181, 2197, 2213, 2229, 2246, 2262, 2278, 2294, 2310, 2327, 2343, 2359,
    2376, 2392, 2408, 2425, 2441, 2458, 2474, 2490, 2507, 2523, 2540, 2556, 2573,
    2589, 2606, 2623, 2639, 2656, 2673, 2689, 2705, 2723, 2740, 2756, 2773, 2789,
    2807, 2824, 2841, 2858, 2875, 2892, 2909, 2926, 2942, 2960, 0, 1, 3, 4, 5, 7,
    9, 11, 13, 16, 20, 24, 30, 36, 45, 55, 67, 79, 93, 107, 120, 135, 150, 164,
    179, 194, 209, 223, 238, 253, 268, 283, 298, 313, 328, 343, 358, 373, 388,
    402, 417, 432, 447, 462, 478, 493, 508, 523, 537, 553, 567, 583, 598, 613,
    628, 644, 659, 674, 689, 704, 720, 735, 750, 766, 781, 796, 811, 826, 842,
    857, 873, 888, 903, 918, 934, 949, 964, 980, 995, 1010, 1026, 1040, 1056,
    1072, 1087, 1102, 1118, 1132, 1148, 1164, 1179, 1194, 1210, 1224, 1240, 1256,
    1271, 1286, 1301, 1317, 1333, 1348, 1363, 1379, 1394, 1410, 1425, 1441, 1456,
    1471, 1487, 1503, 1518, 1534, 1549, 1564, 1580, 1596, 1611, 1627, 1642, 1658,
    1674, 1690, 1705, 1721, 1737, 1752, 1768, 1783, 1800, 1815, 1831, 1847, 1863,
    1879, 1895, 1910, 1926, 1942, 1958, 1974, 1990, 2006, 2022, 2037, 2053, 2069,
    2086, 2102, 2118, 2134, 2150, 2166, 2182, 2197, 2214, 2230, 2246, 2262, 2278,
    2295, 2311, 2327, 2343, 2359, 2376, 2392, 2408, 2424, 2441, 2457, 2473, 2490,
    2506, 2522, 2539, 2555, 2572, 2588, 2605, 2621, 2638, 2654, 2671, 2687, 2704,
    2721, 2738, 2754, 2770, 2788, 2805, 2821, 2838, 2855, 2872, 2889, 2906, 2922,
    2940, 0, 1, 3, 4, 5, 7, 9, 11, 13, 16, 20, 24, 29, 36, 44, 54, 65, 78, 91,
    105, 119, 133, 148, 162, 176, 192, 206, 221, 236, 251, 265, 280, 295, 310,
    325, 339, 354, 369, 384, 399, 414, 429, 444, 459, 473, 488, 503, 518, 533,
    548, 563, 578, 593, 608, 623, 639, 654, 669, 683, 699, 714, 729, 745, 760,
    775, 790, 805, 820, 836, 851, 866, 881, 896, 911, 927, 942, 957, 972, 988,
    1002, 1018, 1033, 1049, 1064, 1079, 1094, 1109, 1124, 1140, 1155, 1170, 1185,
    1201, 1216, 1231, 1247, 1262, 1277, 1292, 1307, 1323, 1338, 1354, 1369, 1384,
    1400, 1415, 1430, 1446, 1461, 1476, 1492, 1507, 1523, 1538, 1553, 1569, 1585,
    1600, 1616, 1631, 1647, 1662, 1678, 1693, 1709, 1725, 1740, 1756, 1771, 1787,
    1802, 1819, 1834, 1850, 1866, 1881, 1897, 1913, 1929, 1945, 1960, 1976, 1992,
    2008, 2023, 2039, 2055, 2071, 2087, 2103, 2119, 2135, 2151, 2167, 2182, 2199,
    2215, 2231, 2247, 2263, 2279, 2295, 2311, 2327, 2343, 2359, 2375, 2392, 2407,
    2424, 2440, 2456, 2473, 2489, 2505, 2522, 2538, 2554, 2571, 2587, 2603, 2620,
    2636, 2653, 2669, 2686, 2702, 2719, 2736, 2752, 2769, 2786, 2802, 2819, 2836,
    2853, 2869, 2886, 2902, 2920, 0, 1, 3, 4, 5, 7, 9, 11, 13, 16, 19, 24, 29,
    35, 43, 53, 64, 77, 90, 104, 117, 132, 146, 160, 174, 189, 204, 219, 233,
    248, 263, 277, 292, 307, 322, 336, 351, 366, 381, 395, 410, 425, 440, 455,
    470, 484, 499, 514, 529, 544, 558, 574, 589, 604, 618, 634, 649, 664, 678,
    693, 709, 723, 739, 754, 769, 784, 799, 814, 829, 844, 859, 875, 890, 904,
    920, 935, 950, 965, 980, 995, 1011, 1025, 1041, 1056, 1071, 1086, 1101, 1116,
    1132, 1147, 1162, 1177, 1192, 1207, 1223, 1238, 1253, 1268, 1283, 1298, 1314,
    1329, 1344, 1359, 1374, 1390, 1405, 1420, 1436, 1451, 1466, 1481, 1497, 1512,
    1527, 1542, 1558, 1574, 1588, 1604, 1619, 1635, 1651, 1666, 1682, 1697, 1713,
    1728, 1744, 1759, 1775, 1790, 1806, 1822, 1837, 1853, 1868, 1884, 1900, 1915,
    1931, 1947, 1963, 1978, 1994, 2009, 2025, 2041, 2057, 2073, 2089, 2104, 2120,
    2136, 2152, 2167, 2184, 2200, 2216, 2231, 2247, 2263, 2279, 2295, 2311, 2327,
    2343, 2359, 2375, 2391, 2408, 2424, 2440, 2456, 2472, 2488, 2505, 2521, 2537,
    2553, 2570, 2586, 2602, 2619, 2635, 2651, 2668, 2684, 2701, 2717, 2733, 2750,
    2767, 2783, 2800, 2817, 2833, 2850, 2867, 2883, 2900, 0, 1, 3, 4, 5, 7, 9,
    11, 13, 16, 19, 23, 28, 35, 43, 52, 63, 75, 88, 102, 115, 130, 144, 158, 172,
    187, 202, 216, 231, 246, 260, 275, 289, 304, 319, 333, 348, 363, 377, 392,
    407, 421, 436, 451, 466, 480, 495, 510, 524, 540, 554, 569, 584, 599, 613,
    629, 644, 658, 673, 688, 703, 718, 733, 748, 763, 778, 793, 808, 823, 838,
    853, 868, 883, 898, 913, 928, 943, 958, 973, 988, 1003, 1018, 1033, 1048,
    1063, 1078, 1093, 1108, 1123, 1139, 1154, 1168, 1184, 1198, 1214, 1229, 1244,
    1259, 1274, 1289, 1304, 1319, 1335, 1350, 1365, 1380, 1395, 1410, 1425, 1441,
    1456, 1471, 1486, 1502, 1517, 1532, 1547, 1563, 1577, 1593, 1608, 1624, 1639,
    1655, 1670, 1685, 1701, 1716, 1732, 1747, 1763, 1778, 1794, 1809, 1825, 1840,
    1856, 1871, 1887, 1902, 1918, 1934, 1949, 1965, 1980, 1995, 2011, 2027, 2043,
    2059, 2074, 2090, 2106, 2122, 2137, 2153, 2169, 2185, 2200, 2216, 2232, 2248,
    2264, 2280, 2296, 2312, 2327, 2343, 2359, 2375, 2391, 2407, 2423, 2439, 2455,
    2472, 2488, 2504, 2520, 2536, 2552, 2568, 2585, 2601, 2617, 2633, 2650, 2666,
    2683, 2699, 2715, 2732, 2748, 2765, 2781, 2798, 2814, 2831, 2848, 2864, 2881,
    0, 1, 3, 4, 5, 7, 8, 11, 13, 16, 19, 23, 28, 34, 42, 51, 62, 74, 87, 101,
    114, 128, 142, 157, 170, 185, 200, 214, 229, 243, 258, 272, 287, 301, 316,
    330, 345, 359, 374, 389, 403, 418, 432, 447, 462, 476, 491, 506, 520, 535,
    549, 565, 579, 594, 608, 624, 639, 653, 668, 683, 698, 712, 728, 742, 757,
    772, 787, 802, 817, 832, 847, 862, 877, 891, 906, 921, 936, 951, 966, 981,
    996, 1010, 1026, 1041, 1056, 1071, 1086, 1100, 1115, 1130, 1145, 1160, 1175,
    1190, 1205, 1220, 1235, 1250, 1265, 1280, 1295, 1310, 1325, 1340, 1355, 1370,
    1385, 1400, 1415, 1431, 1446, 1461, 1476, 1491, 1506, 1521, 1537, 1552, 1566,
    1582, 1597, 1613, 1628, 1643, 1658, 1674, 1689, 1704, 1720, 1735, 1750, 1765,
    1781, 1797, 1812, 1828, 1843, 1858, 1874, 1889, 1905, 1920, 1936, 1951, 1967,
    1982, 1997, 2013, 2029, 2045, 2060, 2076, 2092, 2107, 2123, 2138, 2154, 2170,
    2186, 2201, 2217, 2233, 2249, 2264, 2280, 2296, 2312, 2328, 2343, 2359, 2375,
    2391, 2407, 2423, 2439, 2455, 2471, 2487, 2503, 2519, 2535, 2551, 2567, 2583,
    2600, 2615, 2632, 2648, 2665, 2681, 2697, 2714, 2730, 2746, 2763, 2779, 2796,
    2812, 2829, 2845, 2862, 0, 1, 3, 4, 5, 7, 8, 10, 13, 15, 19, 23, 28, 34, 41,
    50, 61, 73, 86, 99, 112, 127, 141, 155, 168, 183, 197, 212, 226, 241, 255,
    269, 284, 298, 313, 327, 342, 356, 371, 385, 400, 414, 429, 443, 458, 472,
    487, 502, 516, 531, 545, 560, 575, 589, 604, 619, 634, 648, 663, 677, 693,
    707, 722, 737, 752, 766, 781, 796, 811, 826, 840, 855, 870, 884, 900, 915,
    929, 944, 959, 973, 989, 1003, 1018, 1033, 1048, 1063, 1078, 1092, 1108,
    1122, 1137, 1152, 1167, 1181, 1197, 1212, 1226, 1241, 1256, 1271, 1286, 1301,
    1316, 1331, 1346, 1361, 1376, 1391, 1406, 1421, 1436, 1451, 1466, 1481, 1496,
    1510, 1526, 1541, 1556, 1571, 1586, 1602, 1617, 1632, 1647, 1662, 1678, 1693,
    1708, 1723, 1739, 1753, 1769, 1784, 1800, 1815, 1830, 1846, 1861, 1876, 1892,
    1907, 1923, 1938, 1954, 1968, 1984, 1999, 2015, 2031, 2046, 2062, 2077, 2093,
    2109, 2124, 2140, 2155, 2171, 2187, 2202, 2218, 2233, 2249, 2265, 2281, 2296,
    2312, 2328, 2343, 2359, 2375, 2391, 2407, 2423, 2439, 2454, 2470, 2486, 2502,
    2518, 2534, 2550, 2566, 2582, 2598, 2615, 2631, 2647, 2663, 2679, 2695, 2712,
    2728, 2744, 2761, 2777, 2793, 2810, 2826, 2843, 0, 1, 3, 4, 5, 7, 8, 10, 13,
    15, 19, 22, 27, 33, 40, 49, 60, 72, 84, 98, 111, 125, 139, 153, 166, 181,
    195, 210, 224, 238, 252, 267, 281, 296, 310, 324, 339, 353, 367, 382, 396,
    411, 425, 440, 454, 469, 483, 498, 512, 527, 541, 556, 570, 585, 599, 614,
    629, 643, 657, 672, 687, 701, 717, 731, 746, 760, 775, 790, 805, 820, 834,
    849, 864, 878, 893, 908, 923, 937, 952, 966, 982, 996, 1011, 1026, 1041,
    1055, 1070, 1084, 1100, 1114, 1129, 1143, 1159, 1173, 1188, 1203, 1218, 1232,
    1247, 1262, 1277, 1292, 1307, 1322, 1336, 1351, 1366, 1381, 1396, 1411, 1426,
    1441, 1456, 1471, 1486, 1500, 1515, 1530, 1545, 1561, 1575, 1591, 1606, 1621,
    1636, 1651, 1666, 1681, 1696, 1711, 1727, 1741, 1757, 1772, 1788, 1803, 1818,
    1833, 1849, 1864, 1879, 1894, 1910, 1925, 1940, 1955, 1971, 1986, 2002, 2017,
    2033, 2048, 2064, 2079, 2094, 2109, 2125, 2141, 2156, 2172, 2187, 2203, 2219,
    2234, 2250, 2265, 2281, 2297, 2312, 2327, 2344, 2359, 2375, 2391, 2407, 2422,
    2438, 2454, 2470, 2486, 2501, 2517, 2533, 2549, 2565, 2581, 2597, 2613, 2629,
    2645, 2661, 2678, 2694, 2710, 2726, 2742, 2759, 2775, 2791, 2807, 2824, 0, 1,
    3, 4, 5, 7, 8, 10, 13, 15, 18, 22, 27, 33, 40, 49, 59, 71, 83, 96, 109, 123,
    137, 151, 165, 179, 193, 207, 222, 236, 250, 264, 279, 293, 307, 321, 336,
    350, 364, 379, 393, 407, 422, 436, 450, 465, 479, 494, 508, 522, 536, 551,
    566, 580, 594, 609, 624, 638, 652, 667, 682, 696, 711, 726, 740, 755, 770,
    784, 799, 814, 828, 843, 857, 872, 887, 901, 916, 931, 945, 959, 975, 989,
    1004, 1019, 1033, 1048, 1063, 1077, 1092, 1107, 1121, 1135, 1151, 1165, 1180,
    1195, 1209, 1224, 1238, 1253, 1268, 1283, 1298, 1312, 1327, 1342, 1357, 1371,
    1386, 1401, 1416, 1431, 1446, 1460, 1475, 1490, 1505, 1520, 1534, 1550, 1564,
    1580, 1595, 1610, 1625, 1640, 1655, 1670, 1685, 1699, 1715, 1730, 1745, 1760,
    1775, 1791, 1806, 1821, 1836, 1851, 1866, 1882, 1897, 1912, 1927, 1942, 1957,
    1973, 1988, 2004, 2019, 2034, 2050, 2065, 2080, 2095, 2111, 2127, 2142, 2157,
    2173, 2188, 2204, 2219, 2235, 2250, 2266, 2281, 2297, 2312, 2328, 2344, 2359,
    2375, 2391, 2406, 2422, 2438, 2453, 2469, 2485, 2501, 2517, 2532, 2548, 2564,
    2580, 2596, 2612, 2628, 2643, 2660, 2676, 2692, 2708, 2724, 2741, 2757, 2773,
    2789, 2805, 0, 1, 2, 4, 5, 7, 8, 10, 12, 15, 18, 22, 26, 32, 39, 48, 58, 69,
    82, 95, 108, 122, 135, 149, 163, 177, 191, 205, 219, 233, 248, 262, 276, 290,
    304, 318, 333, 347, 361, 375, 390, 404, 418, 432, 447, 461, 475, 490, 503,
    518, 532, 547, 561, 576, 590, 605, 619, 634, 647, 662, 677, 691, 706, 720,
    735, 749, 764, 778, 793, 808, 822, 837, 851, 865, 880, 895, 909, 924, 939,
    953, 968, 982, 997, 1011, 1026, 1040, 1055, 1069, 1084, 1099, 1113, 1127,
    1143, 1157, 1172, 1186, 1201, 1215, 1230, 1244, 1259, 1274, 1289, 1303, 1318,
    1333, 1347, 1362, 1377, 1391, 1406, 1421, 1436, 1450, 1465, 1480, 1495, 1510,
    1524, 1539, 1554, 1569, 1584, 1599, 1614, 1629, 1644, 1659, 1674, 1688, 1703,
    1718, 1733, 1748, 1764, 1779, 1794, 1809, 1824, 1839, 1854, 1869, 1884, 1899,
    1914, 1929, 1944, 1959, 1975, 1990, 2006, 2021, 2036, 2051, 2067, 2081, 2097,
    2112, 2128, 2143, 2158, 2174, 2189, 2205, 2220, 2235, 2251, 2266, 2282, 2297,
    2313, 2328, 2344, 2359, 2375, 2390, 2406, 2422, 2437, 2453, 2469, 2484, 2500,
    2516, 2532, 2547, 2563, 2579, 2595, 2611, 2626, 2643, 2659, 2675, 2691, 2707,
    2723, 2739, 2755, 2770, 2787, 0, 1, 2, 4, 5, 7, 8, 10, 12, 15, 18, 22, 26,
    32, 39, 47, 57, 68, 80, 93, 106, 120, 134, 147, 161, 175, 189, 203, 217, 231,
    245, 259, 273, 287, 301, 316, 330, 344, 358, 372, 386, 400, 415, 429, 443,
    457, 471, 486, 499, 514, 528, 543, 557, 571, 585, 600, 614, 629, 643, 657,
    672, 686, 701, 715, 729, 743, 758, 772, 787, 802, 816, 831, 845, 859, 874,
    888, 903, 917, 932, 946, 961, 975, 990, 1004, 1019, 1033, 1048, 1062, 1077,
    1091, 1106, 1120, 1135, 1149, 1164, 1178, 1193, 1207, 1221, 1236, 1251, 1265,
    1280, 1294, 1309, 1324, 1338, 1353, 1367, 1382, 1397, 1411, 1426, 1441, 1455,
    1470, 1485, 1499, 1514, 1529, 1543, 1558, 1573, 1588, 1603, 1618, 1633, 1647,
    1662, 1677, 1692, 1706, 1722, 1737, 1752, 1767, 1782, 1797, 1812, 1827, 1842,
    1857, 1872, 1887, 1902, 1916, 1931, 1946, 1962, 1977, 1992, 2007, 2023, 2038,
    2053, 2068, 2083, 2099, 2114, 2129, 2144, 2159, 2175, 2190, 2205, 2221, 2236,
    2251, 2267, 2282, 2297, 2313, 2328, 2344, 2359, 2375, 2390, 2406, 2421, 2437,
    2452, 2468, 2484, 2499, 2515, 2530, 2546, 2562, 2578, 2594, 2609, 2625, 2641,
    2657, 2673, 2689, 2705, 2721, 2737, 2752, 2769, 0, 1, 2, 4, 5, 6, 8, 10, 12,
    15, 18, 21, 26, 31, 38, 46, 56, 67, 79, 92, 105, 118, 132, 146, 159, 173,
    187, 201, 215, 229, 243, 257, 271, 285, 299, 313, 327, 341, 355, 369, 383,
    397, 411, 425, 439, 453, 468, 482, 495, 510, 524, 538, 553, 567, 581, 595,
    610, 624, 638, 652, 667, 681, 695, 710, 724, 738, 753, 767, 782, 796, 810,
    825, 839, 853, 868, 882, 896, 911, 925, 939, 954, 968, 983, 997, 1012, 1026,
    1040, 1054, 1069, 1083, 1098, 1112, 1127, 1141, 1156, 1170, 1184, 1198, 1213,
    1227, 1242, 1257, 1271, 1286, 1300, 1315, 1329, 1344, 1358, 1373, 1387, 1402,
    1416, 1431, 1445, 1460, 1475, 1489, 1503, 1519, 1533, 1548, 1563, 1577, 1592,
    1607, 1622, 1636, 1651, 1665, 1681, 1695, 1710, 1725, 1740, 1755, 1770, 1785,
    1800, 1814, 1829, 1844, 1859, 1874, 1889, 1904, 1919, 1934, 1949, 1964, 1979,
    1994, 2009, 2024, 2039, 2054, 2070, 2085, 2100, 2115, 2130, 2145, 2160, 2176,
    2191, 2206, 2221, 2237, 2252, 2267, 2282, 2298, 2313, 2328, 2344, 2359, 2375,
    2390, 2406, 2421, 2436, 2452, 2468, 2483, 2499, 2514, 2530, 2546, 2561, 2577,
    2592, 2608, 2624, 2640, 2656, 2671, 2687, 2703, 2719, 2735, 2751, 0, 1, 2, 4,
    5, 6, 8, 10, 12, 15, 18, 21, 25, 31, 37, 45, 55, 66, 78, 90, 103, 117, 130,
    144, 157, 171, 185, 199, 213, 226, 240, 254, 268, 282, 296, 310, 324, 338,
    352, 366, 380, 394, 408, 422, 436, 450, 464, 478, 492, 506, 520, 534, 548,
    563, 576, 591, 605, 619, 633, 647, 662, 676, 690, 705, 719, 733, 747, 761,
    776, 790, 804, 819, 833, 847, 862, 876, 890, 904, 919, 932, 947, 961, 976,
    990, 1004, 1019, 1033, 1047, 1062, 1076, 1090, 1104, 1119, 1133, 1148, 1162,
    1176, 1190, 1204, 1219, 1234, 1248, 1262, 1277, 1291, 1306, 1320, 1334, 1349,
    1363, 1378, 1392, 1407, 1421, 1436, 1450, 1465, 1479, 1493, 1508, 1523, 1538,
    1552, 1567, 1581, 1596, 1611, 1625, 1640, 1654, 1670, 1684, 1699, 1714, 1728,
    1743, 1758, 1773, 1788, 1802, 1817, 1832, 1847, 1862, 1877, 1891, 1906, 1921,
    1936, 1951, 1966, 1981, 1996, 2011, 2026, 2041, 2056, 2071, 2086, 2101, 2116,
    2131, 2146, 2161, 2177, 2192, 2207, 2222, 2237, 2252, 2268, 2283, 2298, 2313,
    2329, 2344, 2359, 2375, 2390, 2405, 2421, 2436, 2452, 2467, 2482, 2497, 2513,
    2529, 2545, 2560, 2575, 2591, 2607, 2623, 2639, 2654, 2670, 2686, 2702, 2717,
    2733, 0, 1, 2, 4, 5, 6, 8, 10, 12, 14, 17, 21, 25, 30, 37, 45, 54, 65, 77,
    89, 102, 115, 129, 142, 155, 169, 183, 197, 210, 224, 238, 252, 266, 279,
    293, 307, 321, 335, 349, 363, 377, 390, 404, 418, 432, 446, 460, 474, 488,
    502, 516, 530, 544, 558, 572, 586, 600, 615, 628, 642, 657, 671, 685, 699,
    714, 727, 742, 756, 770, 784, 799, 813, 827, 841, 855, 870, 884, 898, 912,
    926, 941, 954, 969, 983, 997, 1012, 1026, 1040, 1054, 1069, 1083, 1097, 1111,
    1125, 1140, 1154, 1168, 1182, 1196, 1210, 1225, 1240, 1254, 1268, 1282, 1297,
    1311, 1325, 1340, 1354, 1369, 1383, 1397, 1412, 1426, 1440, 1455, 1469, 1483,
    1498, 1512, 1527, 1542, 1556, 1571, 1586, 1600, 1615, 1629, 1643, 1658, 1673,
    1688, 1702, 1717, 1732, 1746, 1761, 1776, 1791, 1805, 1820, 1835, 1850, 1864,
    1879, 1893, 1908, 1924, 1938, 1953, 1968, 1983, 1998, 2013, 2027, 2043, 2058,
    2072, 2087, 2102, 2117, 2132, 2147, 2162, 2177, 2193, 2208, 2223, 2237, 2253,
    2268, 2283, 2298, 2314, 2329, 2344, 2359, 2374, 2390, 2405, 2420, 2436, 2451,
    2466, 2481, 2497, 2513, 2528, 2544, 2559, 2575, 2590, 2606, 2622, 2637, 2653,
    2669, 2684, 2699, 2716, 0, 1, 2, 4, 5, 6, 8, 10, 12, 14, 17, 21, 25, 30, 36,
    44, 53, 64, 75, 88, 100, 114, 127, 140, 153, 167, 181, 195, 208, 222, 236,
    249, 263, 277, 291, 304, 318, 332, 346, 360, 373, 387, 401, 415, 429, 443,
    456, 470, 484, 498, 512, 526, 540, 554, 567, 582, 596, 610, 624, 638, 652,
    666, 680, 694, 708, 722, 736, 750, 765, 779, 793, 807, 821, 835, 849, 863,
    878, 892, 906, 919, 934, 948, 962, 976, 991, 1005, 1019, 1032, 1047, 1061,
    1075, 1089, 1104, 1117, 1132, 1146, 1160, 1174, 1188, 1202, 1217, 1231, 1245,
    1260, 1274, 1288, 1302, 1317, 1331, 1345, 1359, 1374, 1388, 1402, 1417, 1430,
    1445, 1460, 1474, 1488, 1502, 1517, 1532, 1546, 1561, 1575, 1590, 1604, 1619,
    1633, 1648, 1662, 1677, 1691, 1706, 1720, 1735, 1750, 1764, 1779, 1793, 1808,
    1823, 1837, 1852, 1866, 1881, 1896, 1911, 1926, 1941, 1955, 1970, 1985, 2000,
    2014, 2029, 2044, 2059, 2074, 2089, 2104, 2119, 2134, 2148, 2163, 2178, 2193,
    2208, 2223, 2238, 2253, 2268, 2284, 2299, 2314, 2329, 2344, 2359, 2374, 2390,
    2405, 2420, 2435, 2451, 2465, 2481, 2497, 2512, 2527, 2542, 2558, 2574, 2589,
    2605, 2620, 2636, 2651, 2667, 2682, 2698, 0, 1, 2, 4, 5, 6, 8, 10, 12, 14,
    17, 20, 25, 30, 36, 43, 52, 63, 74, 86, 99, 112, 125, 139, 152, 165, 179,
    193, 206, 220, 233, 247, 261, 274, 288, 302, 315, 329, 343, 357, 370, 384,
    398, 412, 425, 439, 453, 467, 480, 494, 508, 522, 536, 550, 563, 578, 591,
    605, 619, 633, 647, 661, 675, 689, 703, 717, 731, 745, 759, 773, 787, 801,
    815, 829, 843, 857, 871, 885, 899, 913, 928, 941, 956, 970, 984, 998, 1012,
    1025, 1040, 1054, 1068, 1082, 1096, 1110, 1124, 1138, 1152, 1166, 1180, 1194,
    1209, 1223, 1237, 1251, 1265, 1279, 1294, 1308, 1322, 1336, 1350, 1364, 1379,
    1393, 1407, 1421, 1436, 1450, 1464, 1479, 1492, 1507, 1522, 1536, 1550, 1565,
    1579, 1593, 1608, 1622, 1637, 1651, 1666, 1680, 1695, 1709, 1724, 1738, 1753,
    1767, 1782, 1796, 1811, 1826, 1840, 1854, 1869, 1884, 1899, 1913, 1928, 1943,
    1957, 1972, 1987, 2001, 2016, 2031, 2046, 2061, 2075, 2090, 2105, 2120, 2135,
    2149, 2164, 2179, 2194, 2209, 2224, 2239, 2254, 2269, 2284, 2299, 2314, 2329,
    2344, 2359, 2374, 2389, 2405, 2420, 2435, 2450, 2465, 2481, 2496, 2511, 2526,
    2542, 2557, 2573, 2588, 2604, 2619, 2635, 2650, 2665, 2681, 0, 1, 2, 4, 5, 6,
    8, 10, 12, 14, 17, 20, 24, 29, 35, 43, 52, 62, 73, 85, 97, 111, 124, 137,
    150, 164, 177, 190, 204, 218, 231, 245, 258, 272, 285, 299, 313, 326, 340,
    354, 367, 381, 394, 408, 422, 436, 449, 463, 476, 490, 504, 518, 532, 546,
    559, 573, 587, 601, 614, 628, 642, 656, 670, 684, 698, 711, 726, 739, 754,
    768, 782, 796, 809, 823, 837, 851, 865, 879, 893, 907, 921, 935, 949, 963,
    977, 991, 1005, 1018, 1033, 1047, 1061, 1074, 1089, 1102, 1117, 1131, 1145,
    1158, 1172, 1186, 1201, 1215, 1229, 1243, 1257, 1271, 1285, 1299, 1313, 1327,
    1341, 1355, 1370, 1384, 1398, 1412, 1426, 1440, 1454, 1469, 1483, 1497, 1512,
    1526, 1540, 1554, 1569, 1583, 1597, 1611, 1626, 1640, 1655, 1669, 1684, 1698,
    1712, 1727, 1741, 1756, 1770, 1785, 1799, 1814, 1828, 1842, 1857, 1871, 1886,
    1901, 1916, 1930, 1945, 1959, 1974, 1988, 2003, 2018, 2033, 2047, 2062, 2077,
    2091, 2106, 2121, 2136, 2150, 2165, 2180, 2194, 2210, 2225, 2239, 2254, 2269,
    2284, 2299, 2314, 2329, 2344, 2359, 2374, 2389, 2404, 2419, 2434, 2450, 2465,
    2480, 2495, 2510, 2526, 2541, 2556, 2572, 2587, 2603, 2618, 2633, 2648, 2664,
    0, 1, 2, 4, 5, 6, 8, 10, 12, 14, 17, 20, 24, 29, 35, 42, 51, 61, 72, 84, 96,
    109, 122, 135, 148, 162, 175, 188, 202, 215, 229, 242, 256, 269, 283, 296,
    310, 323, 337, 351, 364, 378, 391, 405, 418, 432, 446, 459, 473, 487, 500,
    514, 528, 541, 555, 569, 583, 596, 610, 624, 638, 651, 665, 679, 693, 706,
    721, 734, 748, 762, 776, 790, 804, 817, 831, 845, 859, 873, 887, 900, 915,
    928, 942, 956, 970, 984, 998, 1011, 1026, 1040, 1053, 1067, 1081, 1095, 1109,
    1123, 1137, 1150, 1164, 1178, 1193, 1207, 1220, 1234, 1248, 1262, 1276, 1290,
    1304, 1318, 1332, 1346, 1360, 1375, 1389, 1402, 1417, 1431, 1445, 1459, 1473,
    1487, 1502, 1516, 1530, 1544, 1558, 1573, 1587, 1601, 1615, 1629, 1644, 1658,
    1673, 1687, 1701, 1716, 1730, 1744, 1759, 1773, 1788, 1802, 1816, 1830, 1845,
    1859, 1874, 1889, 1903, 1918, 1932, 1947, 1961, 1975, 1990, 2005, 2020, 2034,
    2049, 2063, 2078, 2093, 2107, 2122, 2137, 2151, 2166, 2180, 2196, 2210, 2225,
    2240, 2255, 2270, 2285, 2299, 2314, 2329, 2344, 2359, 2374, 2389, 2404, 2419,
    2434, 2449, 2464, 2480, 2494, 2510, 2525, 2540, 2556, 2571, 2586, 2601, 2617,
    2632, 2648, 0, 1, 2, 4, 5, 6, 8, 9, 12, 14, 17, 20, 24, 28, 34, 41, 50, 60,
    71, 83, 95, 108, 121, 134, 146, 160, 173, 187, 200, 213, 227, 240, 253, 267,
    280, 294, 307, 321, 334, 348, 361, 375, 388, 402, 415, 429, 442, 456, 469,
    483, 496, 510, 524, 537, 551, 565, 578, 592, 605, 619, 633, 646, 661, 674,
    688, 701, 716, 729, 743, 757, 771, 784, 798, 811, 826, 839, 853, 867, 881,
    894, 908, 922, 936, 950, 964, 977, 991, 1004, 1019, 1033, 1046, 1060, 1074,
    1087, 1102, 1115, 1129, 1143, 1156, 1170, 1185, 1198, 1212, 1226, 1240, 1254,
    1268, 1282, 1296, 1310, 1324, 1338, 1352, 1366, 1379, 1393, 1408, 1422, 1435,
    1450, 1463, 1478, 1492, 1506, 1520, 1534, 1548, 1562, 1577, 1590, 1605, 1619,
    1633, 1648, 1662, 1676, 1690, 1705, 1719, 1733, 1747, 1762, 1776, 1790, 1805,
    1819, 1833, 1847, 1862, 1877, 1891, 1905, 1920, 1934, 1949, 1963, 1978, 1992,
    2007, 2021, 2036, 2050, 2065, 2079, 2094, 2109, 2123, 2138, 2152, 2167, 2182,
    2196, 2211, 2226, 2241, 2255, 2270, 2285, 2300, 2315, 2329, 2344, 2359, 2374,
    2389, 2403, 2419, 2434, 2449, 2464, 2478, 2494, 2509, 2524, 2539, 2555, 2570,
    2585, 2600, 2615, 2631, 0, 1, 2, 4, 5, 6, 8, 9, 11, 14, 16, 20, 23, 28, 34,
    41, 49, 59, 70, 81, 93, 106, 119, 132, 145, 158, 171, 185, 198, 211, 224,
    238, 251, 264, 278, 291, 305, 318, 331, 345, 358, 371, 385, 398, 412, 425,
    439, 452, 465, 479, 492, 506, 520, 533, 546, 560, 574, 588, 601, 614, 628,
    642, 656, 669, 683, 696, 710, 724, 738, 751, 765, 779, 793, 806, 820, 834,
    847, 861, 875, 888, 902, 915, 930, 943, 957, 971, 984, 998, 1012, 1026, 1039,
    1053, 1067, 1080, 1094, 1108, 1122, 1135, 1149, 1163, 1177, 1191, 1204, 1218,
    1232, 1246, 1260, 1273, 1287, 1301, 1315, 1329, 1343, 1357, 1370, 1384, 1398,
    1412, 1426, 1440, 1454, 1468, 1482, 1496, 1510, 1524, 1538, 1552, 1566, 1580,
    1595, 1608, 1623, 1637, 1651, 1665, 1679, 1694, 1708, 1722, 1736, 1750, 1765,
    1779, 1793, 1807, 1821, 1836, 1850, 1865, 1879, 1893, 1908, 1922, 1936, 1950,
    1965, 1980, 1994, 2008, 2023, 2037, 2052, 2066, 2081, 2095, 2110, 2124, 2139,
    2153, 2168, 2183, 2197, 2212, 2226, 2241, 2256, 2271, 2285, 2300, 2315, 2330,
    2344, 2359, 2374, 2388, 2404, 2419, 2434, 2448, 2463, 2478, 2493, 2509, 2524,
    2539, 2554, 2569, 2584, 2599, 2614, 0, 1, 2, 4, 5, 6, 8, 9, 11, 14, 16, 19,
    23, 28, 33, 40, 48, 58, 69, 80, 92, 105, 117, 130, 143, 156, 169, 183, 196,
    209, 222, 235, 249, 262, 275, 289, 302, 315, 328, 342, 355, 368, 382, 395,
    409, 422, 435, 449, 462, 475, 488, 502, 516, 529, 542, 556, 570, 583, 596,
    610, 624, 637, 651, 665, 678, 691, 705, 719, 733, 746, 760, 773, 787, 800,
    814, 828, 841, 855, 869, 882, 896, 909, 923, 937, 950, 964, 978, 991, 1005,
    1019, 1032, 1045, 1060, 1073, 1087, 1101, 1114, 1127, 1141, 1155, 1169, 1183,
    1196, 1210, 1224, 1238, 1251, 1265, 1279, 1293, 1306, 1320, 1334, 1348, 1362,
    1375, 1389, 1403, 1416, 1431, 1444, 1459, 1473, 1486, 1500, 1514, 1528, 1542,
    1556, 1570, 1584, 1598, 1612, 1626, 1641, 1655, 1669, 1683, 1697, 1711, 1725,
    1739, 1753, 1768, 1782, 1795, 1810, 1824, 1839, 1853, 1867, 1881, 1896, 1910,
    1924, 1938, 1953, 1967, 1981, 1996, 2010, 2024, 2039, 2053, 2068, 2082, 2096,
    2111, 2125, 2139, 2154, 2169, 2183, 2198, 2212, 2227, 2242, 2256, 2271, 2286,
    2300, 2315, 2330, 2344, 2359, 2373, 2389, 2404, 2418, 2433, 2448, 2463, 2478,
    2493, 2508, 2523, 2538, 2553, 2568, 2583, 2598, 0, 1, 2, 4, 5, 6, 8, 9, 11,
    13, 16, 19, 23, 27, 33, 39, 48, 57, 68, 79, 91, 103, 116, 129, 141, 155, 168,
    181, 194, 207, 220, 233, 246, 260, 273, 286, 299, 312, 326, 339, 352, 365,
    379, 392, 405, 419, 432, 445, 458, 472, 485, 499, 512, 525, 538, 552, 566,
    579, 592, 605, 619, 632, 646, 660, 673, 686, 700, 713, 727, 741, 754, 768,
    782, 795, 809, 822, 836, 849, 863, 876, 890, 903, 917, 931, 944, 958, 971,
    984, 998, 1012, 1025, 1039, 1053, 1066, 1080, 1093, 1107, 1120, 1134, 1147,
    1161, 1175, 1188, 1202, 1216, 1229, 1243, 1257, 1270, 1284, 1298, 1312, 1325,
    1339, 1353, 1366, 1380, 1394, 1407, 1422, 1435, 1449, 1463, 1477, 1491, 1505,
    1519, 1532, 1546, 1560, 1574, 1588, 1602, 1616, 1630, 1644, 1658, 1672, 1686,
    1700, 1714, 1728, 1742, 1756, 1770, 1784, 1798, 1812, 1827, 1841, 1855, 1869,
    1884, 1898, 1912, 1926, 1940, 1955, 1969, 1983, 1997, 2012, 2026, 2040, 2055,
    2069, 2083, 2098, 2112, 2126, 2141, 2155, 2170, 2184, 2199, 2213, 2228, 2242,
    2257, 2271, 2286, 2300, 2315, 2330, 2344, 2359, 2374, 2389, 2403, 2418, 2432,
    2448, 2463, 2477, 2492, 2507, 2522, 2537, 2552, 2567, 2582, 0, 1, 2, 3, 5, 6,
    8, 9, 11, 13, 16, 19, 23, 27, 32, 39, 47, 56, 66, 78, 89, 102, 114, 127, 140,
    153, 166, 179, 192, 205, 218, 231, 244, 257, 270, 283, 297, 310, 323, 336,
    349, 362, 376, 389, 402, 415, 428, 442, 455, 468, 481, 495, 508, 521, 534,
    548, 561, 575, 588, 601, 615, 628, 642, 655, 669, 682, 695, 708, 722, 736,
    749, 763, 776, 789, 803, 816, 830, 843, 857, 870, 884, 897, 911, 924, 938,
    951, 965, 978, 992, 1005, 1019, 1032, 1046, 1059, 1073, 1086, 1100, 1113,
    1126, 1140, 1154, 1167, 1181, 1194, 1208, 1221, 1235, 1249, 1262, 1276, 1289,
    1303, 1317, 1330, 1344, 1357, 1371, 1385, 1398, 1412, 1426, 1440, 1454, 1467,
    1481, 1495, 1509, 1523, 1536, 1550, 1564, 1578, 1592, 1606, 1620, 1634, 1648,
    1661, 1675, 1689, 1703, 1717, 1731, 1745, 1759, 1773, 1787, 1801, 1815, 1829,
    1844, 1858, 1872, 1886, 1900, 1914, 1928, 1942, 1957, 1971, 1985, 1999, 2013,
    2028, 2042, 2056, 2070, 2085, 2099, 2113, 2127, 2142, 2156, 2171, 2185, 2199,
    2214, 2228, 2243, 2257, 2272, 2286, 2301, 2315, 2330, 2344, 2359, 2374, 2388,
    2403, 2417, 2433, 2447, 2462, 2477, 2492, 2507, 2521, 2536, 2551, 2566, 0, 1,
    2, 3, 5, 6, 7, 9, 11, 13, 16, 19, 22, 27, 32, 38, 46, 55, 65, 77, 88, 101,
    113, 126, 138, 151, 164, 177, 190, 203, 216, 229, 242, 255, 268, 281, 294,
    307, 320, 333, 346, 359, 373, 386, 399, 412, 425, 438, 451, 465, 477, 491,
    504, 518, 530, 544, 557, 571, 583, 597, 611, 623, 637, 650, 664, 677, 691,
    703, 717, 731, 744, 757, 771, 784, 798, 811, 824, 838, 851, 864, 878, 891,
    905, 918, 931, 945, 958, 971, 985, 998, 1012, 1025, 1039, 1052, 1065, 1079,
    1092, 1105, 1119, 1132, 1146, 1160, 1173, 1186, 1200, 1213, 1227, 1240, 1254,
    1268, 1281, 1295, 1308, 1322, 1335, 1348, 1363, 1376, 1389, 1403, 1417, 1431,
    1444, 1458, 1472, 1485, 1499, 1513, 1527, 1540, 1554, 1568, 1582, 1596, 1609,
    1623, 1637, 1651, 1665, 1679, 1693, 1706, 1720, 1734, 1748, 1762, 1776, 1790,
    1804, 1818, 1832, 1846, 1860, 1874, 1888, 1902, 1916, 1930, 1944, 1958, 1973,
    1987, 2001, 2015, 2029, 2043, 2057, 2072, 2086, 2100, 2114, 2129, 2143, 2157,
    2171, 2186, 2200, 2214, 2229, 2243, 2258, 2272, 2287, 2301, 2316, 2330, 2345,
    2359, 2374, 2388, 2402, 2418, 2432, 2447, 2462, 2476, 2491, 2506, 2521, 2535,
    2550, 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 16, 19, 22, 26, 31, 38, 45, 54, 64, 75,
    87, 99, 112, 124, 136, 149, 162, 175, 188, 201, 214, 227, 240, 253, 266, 279,
    292, 305, 318, 331, 344, 357, 370, 383, 396, 409, 422, 435, 448, 461, 474,
    487, 501, 514, 526, 540, 553, 566, 579, 592, 606, 619, 633, 646, 659, 672,
    686, 699, 712, 726, 739, 752, 765, 778, 792, 805, 819, 832, 845, 858, 872,
    885, 899, 912, 925, 939, 952, 965, 978, 992, 1005, 1018, 1032, 1045, 1058,
    1072, 1085, 1098, 1111, 1125, 1139, 1152, 1165, 1179, 1192, 1206, 1219, 1232,
    1246, 1259, 1273, 1286, 1300, 1313, 1327, 1340, 1354, 1367, 1380, 1394, 1408,
    1422, 1435, 1449, 1462, 1476, 1490, 1503, 1517, 1530, 1544, 1558, 1572, 1586,
    1599, 1613, 1627, 1641, 1654, 1668, 1682, 1696, 1710, 1723, 1737, 1751, 1765,
    1778, 1793, 1807, 1821, 1834, 1848, 1862, 1876, 1890, 1904, 1918, 1932, 1946,
    1960, 1974, 1988, 2002, 2016, 2031, 2045, 2059, 2073, 2087, 2101, 2115, 2130,
    2144, 2158, 2172, 2187, 2201, 2215, 2229, 2244, 2258, 2272, 2287, 2301, 2315,
    2330, 2345, 2359, 2374, 2388, 2403, 2417, 2432, 2447, 2461, 2476, 2491, 2505,
    2520, 2535, 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 16, 18, 22, 26, 31, 37, 45, 53,
    63, 74, 85, 98, 110, 123, 135, 148, 160, 173, 186, 199, 212, 225, 237, 250,
    263, 276, 289, 302, 315, 328, 341, 354, 367, 380, 393, 406, 419, 432, 444,
    458, 470, 484, 497, 510, 523, 536, 549, 562, 575, 588, 602, 615, 628, 641,
    655, 667, 681, 694, 707, 721, 734, 747, 760, 773, 787, 800, 813, 826, 840,
    852, 866, 879, 893, 906, 919, 932, 946, 958, 972, 985, 998, 1011, 1025, 1038,
    1052, 1065, 1078, 1091, 1104, 1117, 1131, 1145, 1158, 1171, 1184, 1198, 1211,
    1225, 1238, 1251, 1265, 1278, 1291, 1305, 1318, 1331, 1345, 1359, 1372, 1386,
    1399, 1413, 1426, 1440, 1453, 1467, 1480, 1494, 1507, 1521, 1535, 1548, 1562,
    1576, 1589, 1603, 1617, 1630, 1644, 1658, 1671, 1685, 1699, 1713, 1726, 1740,
    1754, 1767, 1782, 1795, 1809, 1823, 1837, 1851, 1865, 1878, 1892, 1906, 1920,
    1934, 1948, 1962, 1976, 1990, 2004, 2018, 2032, 2046, 2060, 2074, 2088, 2102,
    2116, 2131, 2145, 2159, 2173, 2187, 2201, 2216, 2230, 2244, 2259, 2273, 2287,
    2301, 2316, 2330, 2345, 2359, 2373, 2388, 2403, 2417, 2432, 2446, 2461, 2475,
    2490, 2504, 2519, 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 15, 18, 22, 26, 31, 37, 44,
    53, 62, 73, 84, 97, 109, 121, 133, 146, 159, 171, 184, 197, 210, 222, 235,
    248, 261, 274, 287, 299, 312, 325, 338, 351, 364, 377, 390, 402, 415, 428,
    441, 454, 467, 480, 493, 506, 519, 532, 545, 558, 571, 584, 598, 610, 624,
    637, 650, 663, 676, 689, 702, 716, 729, 742, 755, 768, 781, 794, 808, 821,
    834, 847, 860, 873, 887, 900, 913, 926, 939, 952, 966, 979, 992, 1005, 1018,
    1031, 1045, 1058, 1071, 1084, 1097, 1110, 1124, 1137, 1150, 1164, 1177, 1190,
    1203, 1217, 1230, 1243, 1257, 1270, 1283, 1297, 1310, 1323, 1337, 1350, 1363,
    1377, 1390, 1404, 1417, 1431, 1444, 1457, 1471, 1484, 1498, 1511, 1525, 1538,
    1552, 1566, 1579, 1593, 1606, 1620, 1634, 1647, 1661, 1675, 1688, 1702, 1716,
    1729, 1743, 1756, 1771, 1784, 1798, 1812, 1826, 1839, 1853, 1866, 1881, 1895,
    1908, 1922, 1936, 1950, 1964, 1978, 1992, 2006, 2020, 2034, 2047, 2061, 2075,
    2089, 2104, 2118, 2132, 2146, 2160, 2174, 2188, 2202, 2216, 2231, 2245, 2259,
    2273, 2287, 2302, 2316, 2330, 2345, 2359, 2374, 2388, 2402, 2417, 2431, 2446,
    2460, 2475, 2489, 2504, 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 15, 18, 21, 25, 30,
    36, 43, 52, 61, 72, 83, 95, 107, 120, 132, 144, 157, 170, 182, 195, 208, 220,
    233, 246, 259, 271, 284, 297, 310, 322, 335, 348, 361, 374, 386, 399, 412,
    425, 437, 451, 463, 477, 489, 502, 515, 528, 541, 554, 567, 580, 593, 606,
    619, 632, 645, 658, 671, 684, 698, 711, 724, 737, 750, 763, 776, 789, 802,
    815, 828, 841, 855, 867, 881, 894, 907, 920, 933, 946, 959, 972, 985, 998,
    1012, 1024, 1038, 1051, 1064, 1077, 1090, 1103, 1117, 1130, 1143, 1156, 1169,
    1182, 1196, 1209, 1222, 1235, 1249, 1262, 1275, 1288, 1302, 1314, 1328, 1341,
    1354, 1368, 1381, 1395, 1408, 1422, 1435, 1448, 1462, 1475, 1489, 1502, 1515,
    1529, 1542, 1556, 1569, 1583, 1596, 1610, 1624, 1637, 1651, 1664, 1678, 1691,
    1705, 1718, 1732, 1745, 1760, 1773, 1787, 1801, 1814, 1828, 1842, 1855, 1869,
    1883, 1897, 1910, 1924, 1938, 1952, 1966, 1979, 1993, 2007, 2021, 2035, 2048,
    2063, 2077, 2091, 2105, 2119, 2133, 2147, 2161, 2175, 2189, 2203, 2217, 2231,
    2245, 2259, 2273, 2288, 2302, 2316, 2331, 2344, 2359, 2374, 2388, 2402, 2417,
    2431, 2446, 2460, 2474, 2489, 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 15, 18, 21, 25,
    30, 36, 43, 51, 60, 71, 82, 94, 106, 118, 130, 143, 155, 168, 180, 193, 206,
    218, 231, 244, 256, 269, 282, 294, 307, 320, 332, 345, 358, 371, 383, 396,
    409, 422, 434, 447, 460, 473, 486, 499, 511, 524, 537, 550, 563, 576, 589,
    602, 615, 628, 641, 653, 667, 679, 693, 706, 719, 732, 745, 757, 771, 784,
    797, 810, 823, 835, 849, 861, 875, 888, 901, 914, 927, 940, 953, 966, 979,
    992, 1005, 1018, 1031, 1044, 1057, 1070, 1083, 1096, 1109, 1123, 1136, 1149,
    1162, 1175, 1188, 1201, 1214, 1227, 1241, 1254, 1267, 1280, 1293, 1306, 1320,
    1333, 1346, 1359, 1372, 1386, 1399, 1413, 1426, 1439, 1453, 1466, 1479, 1492,
    1506, 1519, 1533, 1546, 1560, 1573, 1587, 1600, 1613, 1627, 1640, 1654, 1667,
    1681, 1695, 1708, 1721, 1735, 1749, 1762, 1776, 1790, 1803, 1817, 1830, 1844,
    1858, 1871, 1885, 1899, 1912, 1926, 1940, 1954, 1967, 1981, 1995, 2009, 2023,
    2036, 2050, 2064, 2078, 2092, 2106, 2120, 2134, 2148, 2162, 2176, 2190, 2204,
    2218, 2232, 2246, 2259, 2274, 2288, 2302, 2316, 2330, 2345, 2359, 2373, 2388,
    2402, 2416, 2431, 2445, 2459, 2474, 0, 1, 2, 3, 5, 6, 7, 9, 11, 13, 15, 18,
    21, 25, 30, 35, 42, 50, 60, 70, 81, 93, 105, 117, 128, 141, 154, 166, 179,
    191, 204, 216, 229, 241, 254, 267, 279, 292, 304, 317, 330, 342, 355, 368,
    380, 393, 406, 419, 431, 444, 456, 470, 482, 495, 507, 521, 534, 546, 559,
    572, 585, 597, 611, 624, 636, 649, 662, 675, 688, 701, 714, 727, 740, 752,
    766, 779, 791, 804, 817, 830, 843, 856, 869, 882, 895, 908, 921, 933, 947,
    960, 973, 985, 999, 1011, 1024, 1037, 1050, 1063, 1076, 1089, 1102, 1115,
    1128, 1141, 1154, 1167, 1181, 1194, 1207, 1220, 1233, 1246, 1259, 1272, 1285,
    1298, 1311, 1325, 1337, 1351, 1364, 1377, 1391, 1404, 1417, 1430, 1444, 1457,
    1470, 1483, 1497, 1510, 1523, 1537, 1550, 1563, 1577, 1590, 1604, 1617, 1630,
    1644, 1657, 1671, 1684, 1697, 1711, 1724, 1738, 1752, 1765, 1779, 1792, 1806,
    1819, 1832, 1846, 1860, 1874, 1887, 1901, 1914, 1928, 1942, 1955, 1969, 1983,
    1997, 2010, 2024, 2038, 2052, 2065, 2079, 2093, 2107, 2121, 2135, 2148, 2162,
    2176, 2190, 2204, 2218, 2232, 2246, 2260, 2274, 2288, 2303, 2316, 2331, 2345,
    2359, 2373, 2388, 2402, 2416, 2431, 2444, 2459, 0, 1, 2, 3, 5, 6, 7, 9, 11,
    13, 15, 18, 21, 25, 29, 35, 41, 49, 59, 69, 79, 91, 103, 115, 127, 140, 152,
    164, 177, 189, 202, 214, 227, 239, 252, 264, 277, 289, 302, 314, 327, 340,
    352, 365, 377, 390, 403, 415, 428, 441, 453, 466, 479, 491, 504, 517, 530,
    542, 555, 568, 581, 593, 606, 619, 632, 644, 658, 670, 683, 696, 709, 722,
    735, 747, 760, 773, 786, 799, 812, 824, 838, 850, 863, 876, 889, 902, 915,
    927, 941, 953, 966, 979, 992, 1005, 1018, 1031, 1044, 1056, 1069, 1082, 1095,
    1108, 1121, 1134, 1147, 1160, 1173, 1186, 1199, 1212, 1225, 1238, 1251, 1264,
    1277, 1290, 1303, 1316, 1329, 1343, 1355, 1369, 1382, 1395, 1408, 1421, 1435,
    1448, 1461, 1474, 1487, 1500, 1514, 1527, 1540, 1554, 1567, 1580, 1594, 1607,
    1620, 1634, 1647, 1660, 1674, 1687, 1700, 1714, 1727, 1741, 1754, 1768, 1781,
    1795, 1808, 1821, 1835, 1849, 1862, 1876, 1889, 1903, 1916, 1930, 1944, 1957,
    1971, 1985, 1998, 2011, 2026, 2039, 2053, 2067, 2080, 2094, 2108, 2122, 2136,
    2149, 2163, 2177, 2191, 2205, 2219, 2232, 2247, 2261, 2275, 2289, 2302, 2317,
    2331, 2345, 2359, 2373, 2388, 2402, 2416, 2430, 2445, 0, 1, 2, 3, 4, 6, 7, 9,
    10, 12, 15, 17, 21, 24, 29, 34, 41, 49, 58, 68, 78, 90, 102, 114, 125, 138,
    150, 163, 175, 187, 200, 212, 225, 237, 249, 262, 274, 287, 299, 312, 324,
    337, 349, 362, 375, 387, 400, 412, 424, 437, 450, 463, 475, 488, 500, 513,
    526, 539, 551, 564, 577, 589, 602, 615, 628, 640, 653, 666, 679, 691, 704,
    717, 730, 742, 755, 768, 781, 794, 807, 819, 832, 845, 858, 871, 883, 896,
    909, 921, 935, 947, 960, 973, 986, 998, 1011, 1024, 1037, 1049, 1062, 1075,
    1088, 1101, 1114, 1127, 1140, 1153, 1166, 1179, 1191, 1204, 1217, 1230, 1243,
    1256, 1269, 1282, 1295, 1308, 1321, 1334, 1347, 1360, 1373, 1386, 1400, 1413,
    1426, 1439, 1452, 1465, 1478, 1491, 1505, 1518, 1531, 1544, 1557, 1571, 1584,
    1597, 1610, 1624, 1637, 1650, 1664, 1676, 1690, 1703, 1717, 1730, 1744, 1757,
    1770, 1784, 1797, 1810, 1824, 1838, 1851, 1864, 1878, 1891, 1905, 1918, 1932,
    1945, 1959, 1973, 1986, 1999, 2013, 2027, 2041, 2054, 2068, 2082, 2095, 2109,
    2123, 2137, 2150, 2164, 2178, 2192, 2206, 2219, 2233, 2247, 2261, 2275, 2289,
    2303, 2317, 2331, 2345, 2359, 2373, 2388, 2402, 2415, 2430, 0, 1, 2, 3, 4, 6,
    7, 9, 10, 12, 15, 17, 20, 24, 28, 34, 40, 48, 57, 67, 77, 89, 100, 112, 124,
    136, 149, 161, 173, 185, 198, 210, 223, 235, 247, 260, 272, 285, 297, 309,
    322, 334, 347, 359, 372, 384, 397, 409, 421, 434, 446, 459, 472, 484, 497,
    510, 522, 535, 547, 560, 573, 585, 598, 611, 623, 636, 649, 661, 674, 687,
    700, 712, 725, 737, 750, 763, 776, 789, 801, 814, 827, 839, 852, 865, 878,
    890, 903, 915, 928, 941, 954, 966, 979, 992, 1005, 1018, 1030, 1043, 1056,
    1068, 1082, 1094, 1107, 1120, 1133, 1146, 1158, 1171, 1184, 1197, 1210, 1223,
    1235, 1248, 1261, 1274, 1287, 1300, 1313, 1326, 1338, 1352, 1365, 1378, 1391,
    1404, 1417, 1430, 1443, 1456, 1469, 1482, 1495, 1509, 1522, 1535, 1548, 1561,
    1574, 1587, 1601, 1614, 1627, 1640, 1654, 1666, 1680, 1693, 1707, 1720, 1733,
    1746, 1760, 1773, 1786, 1799, 1813, 1826, 1840, 1853, 1867, 1880, 1893, 1907,
    1920, 1934, 1947, 1961, 1974, 1987, 2001, 2015, 2028, 2042, 2056, 2069, 2083,
    2096, 2110, 2124, 2137, 2151, 2165, 2179, 2192, 2206, 2220, 2234, 2248, 2262,
    2275, 2289, 2303, 2317, 2331, 2345, 2359, 2373, 2387, 2401, 2416, 0, 1, 2, 3,
    4, 6, 7, 9, 10, 12, 14, 17, 20, 24, 28, 33, 40, 47, 56, 66, 76, 88, 99, 111,
    122, 135, 147, 159, 171, 184, 196, 208, 220, 233, 245, 257, 270, 282, 294,
    307, 319, 332, 344, 356, 369, 381, 394, 406, 418, 431, 443, 456, 468, 481,
    493, 506, 518, 531, 543, 556, 569, 581, 594, 606, 619, 631, 644, 656, 670,
    682, 695, 707, 720, 732, 745, 758, 771, 783, 796, 808, 821, 834, 847, 859,
    872, 885, 897, 909, 923, 935, 948, 960, 973, 985, 999, 1011, 1024, 1036,
    1049, 1062, 1075, 1087, 1100, 1113, 1126, 1138, 1151, 1164, 1177, 1189, 1202,
    1215, 1228, 1241, 1253, 1266, 1279, 1292, 1304, 1318, 1330, 1344, 1356, 1369,
    1382, 1395, 1408, 1421, 1434, 1447, 1460, 1473, 1486, 1499, 1512, 1525, 1539,
    1552, 1565, 1578, 1591, 1604, 1617, 1630, 1644, 1656, 1669, 1683, 1696, 1709,
    1723, 1736, 1749, 1762, 1776, 1788, 1802, 1815, 1829, 1842, 1855, 1869, 1882,
    1895, 1909, 1922, 1936, 1949, 1962, 1975, 1989, 2003, 2016, 2030, 2043, 2057,
    2070, 2084, 2098, 2111, 2125, 2138, 2152, 2166, 2179, 2193, 2207, 2221, 2234,
    2248, 2262, 2276, 2290, 2304, 2318, 2331, 2345, 2359, 2373, 2387, 2401, 0, 1,
    2, 3, 4, 6, 7, 9, 10, 12, 14, 17, 20, 23, 28, 33, 39, 47, 55, 65, 75, 86, 98,
    110, 121, 133, 145, 157, 170, 182, 194, 206, 218, 231, 243, 255, 267, 280,
    292, 304, 317, 329, 341, 354, 366, 378, 391, 403, 415, 428, 440, 453, 465,
    477, 489, 502, 515, 527, 539, 552, 565, 577, 590, 602, 615, 627, 640, 652,
    665, 678, 690, 703, 715, 727, 740, 753, 766, 778, 791, 803, 816, 828, 841,
    854, 866, 879, 891, 904, 917, 929, 942, 954, 967, 979, 992, 1005, 1017, 1030,
    1042, 1055, 1068, 1081, 1093, 1106, 1119, 1131, 1144, 1157, 1169, 1182, 1195,
    1207, 1220, 1233, 1246, 1258, 1271, 1284, 1296, 1310, 1322, 1335, 1348, 1361,
    1374, 1387, 1400, 1413, 1425, 1438, 1451, 1464, 1477, 1490, 1503, 1516, 1529,
    1542, 1555, 1568, 1581, 1594, 1607, 1621, 1634, 1646, 1659, 1672, 1686, 1699,
    1712, 1725, 1739, 1752, 1765, 1778, 1791, 1805, 1818, 1831, 1844, 1858, 1871,
    1884, 1897, 1911, 1924, 1937, 1951, 1964, 1978, 1991, 2004, 2018, 2031, 2045,
    2058, 2072, 2085, 2099, 2112, 2126, 2139, 2153, 2167, 2180, 2194, 2208, 2221,
    2235, 2248, 2262, 2276, 2290, 2304, 2318, 2332, 2346, 2359, 2373, 2387, 0, 1,
    2, 3, 4, 6, 7, 8, 10, 12, 14, 17, 20, 23, 27, 32, 39, 46, 54, 64, 74, 85, 97,
    108, 120, 132, 144, 156, 168, 180, 192, 204, 216, 229, 241, 253, 265, 277,
    290, 302, 314, 326, 339, 351, 363, 375, 388, 400, 412, 425, 436, 449, 462,
    474, 486, 499, 511, 524, 536, 548, 561, 573, 586, 598, 611, 623, 636, 648,
    661, 673, 686, 698, 711, 723, 736, 748, 761, 773, 786, 798, 811, 823, 836,
    848, 861, 873, 886, 898, 911, 923, 936, 948, 961, 973, 986, 998, 1011, 1023,
    1036, 1048, 1061, 1074, 1086, 1099, 1112, 1124, 1137, 1149, 1162, 1175, 1187,
    1200, 1213, 1225, 1238, 1250, 1263, 1276, 1288, 1302, 1314, 1327, 1340, 1353,
    1366, 1378, 1391, 1404, 1417, 1429, 1443, 1455, 1468, 1481, 1494, 1507, 1520,
    1533, 1546, 1559, 1572, 1585, 1598, 1611, 1624, 1636, 1649, 1662, 1676, 1689,
    1702, 1715, 1728, 1741, 1754, 1767, 1781, 1794, 1807, 1820, 1833, 1847, 1860,
    1873, 1886, 1899, 1913, 1926, 1939, 1952, 1966, 1979, 1992, 2006, 2019, 2033,
    2046, 2059, 2073, 2086, 2100, 2113, 2127, 2140, 2154, 2167, 2181, 2195, 2208,
    2222, 2235, 2249, 2263, 2277, 2290, 2304, 2318, 2332, 2346, 2359, 2373, 0, 1,
    2, 3, 4, 6, 7, 8, 10, 12, 14, 17, 20, 23, 27, 32, 38, 45, 53, 63, 73, 84, 95,
    107, 118, 130, 142, 154, 166, 178, 190, 202, 214, 227, 239, 251, 263, 275,
    287, 299, 312, 324, 336, 348, 360, 373, 385, 397, 409, 421, 433, 446, 458,
    471, 482, 495, 508, 520, 532, 544, 557, 569, 582, 594, 606, 618, 631, 643,
    656, 669, 681, 693, 706, 718, 731, 743, 756, 768, 780, 793, 805, 817, 830,
    843, 855, 868, 880, 892, 905, 917, 930, 942, 955, 967, 980, 992, 1005, 1017,
    1029, 1042, 1055, 1067, 1080, 1092, 1105, 1117, 1130, 1142, 1155, 1167, 1180,
    1193, 1205, 1218, 1230, 1243, 1256, 1268, 1281, 1294, 1306, 1319, 1332, 1344,
    1357, 1370, 1383, 1395, 1408, 1421, 1434, 1446, 1459, 1472, 1485, 1498, 1511,
    1524, 1537, 1549, 1562, 1575, 1588, 1601, 1614, 1627, 1640, 1653, 1666, 1679,
    1692, 1705, 1718, 1731, 1744, 1757, 1770, 1783, 1796, 1809, 1822, 1836, 1849,
    1862, 1875, 1888, 1901, 1915, 1928, 1941, 1954, 1967, 1981, 1994, 2007, 2021,
    2034, 2047, 2061, 2074, 2087, 2101, 2114, 2128, 2141, 2154, 2168, 2182, 2195,
    2209, 2222, 2236, 2250, 2263, 2277, 2291, 2304, 2318, 2332, 2345, 2360, 0, 1,
    2, 3, 4, 6, 7, 8, 10, 12, 14, 16, 19, 23, 27, 32, 38, 44, 53, 62, 72, 83, 94,
    105, 117, 129, 141, 153, 164, 176, 188, 200, 212, 225, 237, 249, 261, 273,
    285, 297, 309, 321, 333, 345, 358, 370, 382, 394, 406, 418, 430, 443, 455,
    467, 479, 492, 504, 516, 528, 540, 553, 565, 578, 590, 602, 614, 627, 639,
    652, 664, 676, 689, 701, 713, 726, 738, 751, 763, 775, 787, 800, 812, 825,
    837, 850, 862, 874, 886, 899, 912, 924, 936, 949, 961, 974, 986, 998, 1010,
    1023, 1035, 1048, 1061, 1073, 1085, 1098, 1110, 1123, 1135, 1148, 1160, 1173,
    1185, 1198, 1210, 1223, 1235, 1248, 1261, 1273, 1286, 1298, 1311, 1324, 1336,
    1349, 1362, 1374, 1387, 1400, 1412, 1425, 1438, 1451, 1463, 1476, 1489, 1502,
    1515, 1527, 1540, 1553, 1566, 1579, 1592, 1604, 1617, 1630, 1643, 1656, 1669,
    1682, 1695, 1708, 1721, 1734, 1746, 1760, 1773, 1786, 1799, 1812, 1825, 1838,
    1851, 1864, 1877, 1890, 1903, 1916, 1929, 1943, 1956, 1969, 1982, 1996, 2009,
    2022, 2035, 2049, 2062, 2075, 2089, 2102, 2115, 2129, 2142, 2156, 2169, 2182,
    2196, 2209, 2223, 2237, 2250, 2264, 2277, 2291, 2305, 2318, 2332, 2346, 0, 1,
    2, 3, 4, 6, 7, 8, 10, 12, 14, 16, 19, 23, 26, 31, 37, 44, 52, 61, 71, 82, 93,
    104, 115, 127, 139, 151, 163, 175, 187, 199, 211, 223, 235, 247, 259, 271,
    283, 295, 307, 319, 331, 343, 355, 367, 379, 391, 403, 415, 427, 440, 452,
    464, 476, 488, 500, 513, 524, 537, 549, 561, 574, 586, 598, 610, 623, 635,
    647, 660, 672, 684, 697, 708, 721, 733, 746, 758, 770, 782, 795, 807, 820,
    832, 844, 857, 869, 881, 894, 906, 918, 930, 943, 955, 967, 980, 992, 1004,
    1016, 1029, 1042, 1054, 1066, 1079, 1091, 1103, 1116, 1128, 1141, 1153, 1166,
    1178, 1191, 1203, 1215, 1228, 1240, 1253, 1265, 1278, 1290, 1303, 1316, 1328,
    1341, 1353, 1366, 1379, 1391, 1404, 1417, 1429, 1442, 1455, 1467, 1480, 1493,
    1506, 1518, 1531, 1544, 1557, 1569, 1582, 1595, 1607, 1620, 1633, 1646, 1659,
    1672, 1685, 1698, 1711, 1723, 1736, 1749, 1762, 1775, 1788, 1801, 1814, 1827,
    1840, 1853, 1866, 1879, 1892, 1905, 1918, 1931, 1944, 1958, 1971, 1984, 1997,
    2010, 2023, 2037, 2050, 2063, 2076, 2090, 2103, 2116, 2129, 2143, 2156, 2170,
    2183, 2196, 2210, 2224, 2237, 2251, 2264, 2278, 2291, 2305, 2318, 2332, 0, 1,
    2, 3, 4, 5, 7, 8, 10, 12, 14, 16, 19, 22, 26, 31, 37, 43, 51, 60, 70, 81, 92,
    103, 114, 126, 138, 149, 161, 173, 185, 197, 209, 221, 232, 244, 256, 268,
    280, 292, 304, 316, 328, 340, 352, 364, 376, 388, 400, 412, 424, 436, 448,
    461, 472, 485, 497, 509, 521, 533, 545, 557, 570, 582, 594, 606, 619, 630,
    643, 655, 668, 680, 692, 704, 716, 729, 741, 753, 765, 777, 790, 802, 814,
    827, 839, 851, 863, 875, 888, 900, 912, 924, 937, 949, 961, 974, 986, 998,
    1010, 1022, 1035, 1047, 1060, 1072, 1084, 1097, 1109, 1121, 1134, 1146, 1159,
    1171, 1183, 1196, 1208, 1220, 1233, 1245, 1257, 1270, 1282, 1295, 1308, 1320,
    1333, 1345, 1358, 1370, 1383, 1395, 1408, 1420, 1433, 1446, 1459, 1471, 1484,
    1497, 1509, 1522, 1535, 1547, 1560, 1573, 1585, 1598, 1611, 1623, 1636, 1649,
    1662, 1675, 1688, 1700, 1713, 1726, 1739, 1752, 1765, 1778, 1791, 1803, 1816,
    1829, 1842, 1855, 1868, 1881, 1894, 1907, 1920, 1933, 1946, 1959, 1972, 1985,
    1999, 2012, 2025, 2038, 2051, 2064, 2078, 2091, 2104, 2117, 2131, 2144, 2157,
    2171, 2184, 2197, 2211, 2224, 2238, 2251, 2265, 2278, 2292, 2305, 2319, 0, 1,
    2, 3, 4, 5, 7, 8, 10, 12, 14, 16, 19, 22, 26, 30, 36, 43, 50, 59, 69, 79, 90,
    102, 113, 124, 136, 148, 160, 171, 183, 195, 207, 219, 230, 242, 254, 266,
    278, 290, 302, 314, 326, 337, 349, 361, 373, 385, 397, 409, 421, 433, 445,
    457, 469, 481, 493, 505, 517, 529, 542, 553, 566, 578, 590, 602, 615, 626,
    639, 651, 663, 675, 687, 699, 712, 724, 736, 748, 761, 772, 785, 797, 809,
    821, 834, 846, 858, 870, 882, 895, 907, 918, 931, 943, 955, 968, 980, 992,
    1004, 1016, 1029, 1041, 1053, 1066, 1078, 1090, 1102, 1115, 1127, 1139, 1151,
    1164, 1176, 1188, 1201, 1213, 1226, 1238, 1250, 1263, 1275, 1288, 1300, 1312,
    1325, 1337, 1350, 1362, 1375, 1387, 1400, 1412, 1425, 1437, 1450, 1463, 1475,
    1488, 1500, 1513, 1526, 1538, 1551, 1563, 1576, 1588, 1601, 1614, 1627, 1640,
    1652, 1665, 1678, 1691, 1703, 1716, 1729, 1742, 1754, 1767, 1780, 1793, 1806,
    1819, 1831, 1844, 1857, 1870, 1883, 1896, 1909, 1922, 1935, 1948, 1961, 1974,
    1987, 2000, 2013, 2026, 2039, 2052, 2066, 2079, 2092, 2105, 2118, 2132, 2145,
    2158, 2171, 2185, 2198, 2211, 2225, 2238, 2252, 2265, 2279, 2292, 2306, 0, 1,
    2, 3, 4, 5, 7, 8, 10, 12, 14, 16, 19, 22, 26, 30, 36, 42, 50, 58, 68, 78, 89,
    100, 111, 123, 135, 146, 158, 170, 181, 193, 205, 217, 228, 240, 252, 264,
    276, 287, 299, 311, 323, 335, 347, 359, 371, 382, 394, 406, 418, 430, 442,
    454, 466, 478, 490, 502, 514, 526, 538, 550, 562, 574, 586, 598, 610, 622,
    635, 647, 659, 671, 683, 695, 707, 719, 731, 744, 756, 767, 780, 792, 804,
    816, 828, 840, 853, 864, 877, 889, 901, 913, 925, 937, 950, 962, 974, 986,
    998, 1010, 1022, 1035, 1047, 1059, 1071, 1083, 1096, 1108, 1120, 1132, 1144,
    1157, 1169, 1181, 1194, 1205, 1218, 1230, 1242, 1255, 1267, 1280, 1292, 1305,
    1317, 1329, 1342, 1354, 1367, 1379, 1391, 1404, 1416, 1429, 1441, 1454, 1466,
    1479, 1491, 1504, 1517, 1529, 1542, 1554, 1567, 1579, 1592, 1604, 1617, 1630,
    1643, 1655, 1668, 1681, 1693, 1706, 1719, 1731, 1744, 1757, 1770, 1782, 1795,
    1808, 1821, 1834, 1846, 1859, 1872, 1885, 1898, 1911, 1924, 1937, 1950, 1963,
    1976, 1989, 2002, 2015, 2028, 2041, 2054, 2067, 2080, 2093, 2106, 2119, 2133,
    2146, 2159, 2172, 2185, 2199, 2212, 2225, 2239, 2252, 2266, 2278, 2292, 0, 1,
    2, 3, 4, 5, 7, 8, 10, 11, 13, 16, 18, 22, 25, 30, 35, 41, 49, 57, 67, 77, 88,
    99, 110, 122, 133, 145, 156, 168, 180, 191, 203, 215, 226, 238, 250, 262,
    273, 285, 297, 309, 321, 332, 344, 356, 368, 380, 391, 403, 415, 427, 439,
    451, 462, 475, 487, 499, 510, 522, 534, 546, 558, 570, 582, 594, 606, 618,
    630, 642, 654, 667, 679, 690, 703, 715, 727, 739, 751, 762, 775, 787, 799,
    811, 823, 835, 847, 859, 871, 883, 895, 907, 920, 931, 944, 956, 968, 980,
    992, 1004, 1016, 1028, 1040, 1053, 1065, 1077, 1089, 1101, 1113, 1125, 1138,
    1150, 1162, 1174, 1186, 1198, 1211, 1223, 1235, 1248, 1259, 1272, 1284, 1297,
    1309, 1321, 1334, 1346, 1358, 1370, 1383, 1395, 1408, 1420, 1433, 1445, 1458,
    1470, 1483, 1495, 1508, 1520, 1533, 1545, 1558, 1570, 1582, 1595, 1608, 1620,
    1633, 1646, 1658, 1671, 1683, 1696, 1709, 1721, 1734, 1747, 1759, 1772, 1785,
    1798, 1810, 1823, 1836, 1849, 1861, 1874, 1887, 1900, 1913, 1926, 1938, 1951,
    1964, 1977, 1990, 2003, 2016, 2029, 2042, 2055, 2068, 2081, 2094, 2107, 2120,
    2133, 2146, 2160, 2173, 2186, 2199, 2213, 2226, 2239, 2253, 2266, 2279, 0, 1,
    2, 3, 4, 5, 7, 8, 10, 11, 13, 16, 18, 21, 25, 29, 35, 41, 48, 57, 66, 76, 87,
    98, 109, 120, 132, 143, 155, 166, 178, 189, 201, 213, 224, 236, 248, 259,
    271, 283, 295, 306, 318, 330, 341, 353, 365, 377, 388, 400, 412, 424, 436,
    448, 459, 471, 483, 495, 507, 518, 531, 542, 555, 567, 578, 590, 602, 614,
    626, 638, 650, 662, 674, 686, 698, 710, 722, 734, 746, 758, 770, 782, 794,
    806, 818, 830, 842, 854, 866, 878, 890, 901, 914, 925, 938, 950, 962, 974,
    986, 998, 1010, 1022, 1034, 1046, 1058, 1070, 1082, 1094, 1107, 1119, 1131,
    1143, 1155, 1167, 1179, 1191, 1204, 1216, 1228, 1240, 1252, 1265, 1277, 1289,
    1301, 1314, 1326, 1338, 1350, 1362, 1375, 1387, 1400, 1412, 1424, 1437, 1449,
    1462, 1474, 1486, 1499, 1511, 1524, 1536, 1549, 1561, 1573, 1586, 1599, 1611,
    1624, 1636, 1649, 1661, 1674, 1686, 1699, 1711, 1724, 1737, 1749, 1762, 1775,
    1787, 1800, 1813, 1825, 1838, 1851, 1863, 1876, 1889, 1902, 1914, 1927, 1940,
    1953, 1966, 1979, 1991, 2004, 2017, 2030, 2043, 2056, 2069, 2082, 2095, 2108,
    2121, 2134, 2147, 2161, 2174, 2187, 2200, 2213, 2227, 2240, 2253, 2266, 0, 1,
    2, 3, 4, 5, 7, 8, 10, 11, 13, 16, 18, 21, 25, 29, 34, 40, 48, 56, 65, 75, 86,
    97, 107, 119, 130, 142, 153, 165, 176, 188, 199, 211, 222, 234, 246, 257,
    269, 281, 292, 304, 316, 327, 339, 351, 362, 374, 385, 397, 409, 421, 433,
    444, 456, 468, 480, 492, 503, 515, 527, 539, 551, 563, 575, 586, 598, 610,
    622, 634, 646, 658, 670, 681, 694, 706, 717, 729, 741, 753, 765, 777, 789,
    801, 813, 825, 837, 848, 861, 872, 884, 896, 908, 920, 932, 944, 956, 968,
    980, 992, 1004, 1016, 1028, 1040, 1052, 1064, 1076, 1088, 1100, 1112, 1124,
    1136, 1148, 1160, 1172, 1184, 1196, 1209, 1220, 1233, 1245, 1257, 1269, 1281,
    1294, 1306, 1318, 1330, 1342, 1354, 1367, 1379, 1392, 1404, 1416, 1428, 1441,
    1453, 1465, 1478, 1490, 1502, 1515, 1527, 1540, 1552, 1564, 1576, 1589, 1602,
    1614, 1627, 1639, 1652, 1664, 1676, 1689, 1702, 1714, 1727, 1739, 1752, 1764,
    1777, 1790, 1802, 1815, 1827, 1840, 1852, 1865, 1878, 1891, 1904, 1916, 1929,
    1942, 1955, 1967, 1980, 1993, 2006, 2019, 2032, 2044, 2057, 2070, 2083, 2096,
    2109, 2122, 2135, 2148, 2161, 2175, 2188, 2201, 2214, 2227, 2240, 2254, 0, 1,
    2, 3, 4, 5, 7, 8, 9, 11, 13, 15, 18, 21, 25, 29, 34, 40, 47, 55, 64, 74, 84,
    95, 106, 117, 129, 140, 152, 163, 174, 186, 197, 209, 221, 232, 244, 255,
    267, 278, 290, 302, 313, 325, 336, 348, 360, 371, 383, 395, 406, 418, 430,
    441, 453, 465, 477, 488, 500, 511, 524, 535, 547, 559, 571, 582, 594, 606,
    618, 630, 642, 654, 665, 677, 689, 701, 713, 725, 737, 748, 760, 772, 784,
    796, 808, 820, 831, 843, 855, 867, 879, 890, 903, 914, 926, 938, 950, 962,
    974, 985, 998, 1010, 1022, 1034, 1045, 1057, 1069, 1081, 1093, 1105, 1117,
    1129, 1141, 1153, 1165, 1177, 1189, 1201, 1213, 1226, 1237, 1250, 1262, 1274,
    1286, 1298, 1310, 1322, 1335, 1346, 1359, 1371, 1383, 1396, 1408, 1420, 1432,
    1445, 1457, 1469, 1481, 1494, 1506, 1518, 1531, 1543, 1555, 1567, 1580, 1592,
    1605, 1617, 1630, 1642, 1654, 1667, 1679, 1692, 1704, 1717, 1729, 1742, 1754,
    1767, 1779, 1792, 1804, 1817, 1830, 1842, 1855, 1867, 1880, 1893, 1905, 1918,
    1931, 1943, 1956, 1969, 1982, 1994, 2007, 2020, 2033, 2045, 2059, 2071, 2084,
    2097, 2110, 2123, 2136, 2149, 2162, 2175, 2188, 2201, 2215, 2227, 2241, 0, 1,
    2, 3, 4, 5, 7, 8, 9, 11, 13, 15, 18, 21, 24, 28, 33, 39, 46, 54, 63, 73, 83,
    94, 105, 116, 127, 139, 150, 161, 173, 184, 196, 207, 219, 230, 242, 253,
    265, 276, 288, 299, 311, 322, 334, 345, 357, 369, 380, 392, 403, 415, 427,
    438, 450, 462, 473, 485, 496, 508, 520, 531, 544, 555, 567, 578, 591, 602,
    614, 626, 638, 649, 661, 673, 685, 697, 708, 720, 732, 743, 756, 767, 779,
    791, 803, 815, 826, 838, 850, 862, 874, 885, 897, 909, 921, 933, 944, 956,
    968, 980, 992, 1004, 1015, 1027, 1039, 1051, 1063, 1075, 1087, 1099, 1111,
    1123, 1134, 1146, 1158, 1170, 1182, 1194, 1206, 1218, 1230, 1242, 1254, 1266,
    1278, 1291, 1303, 1315, 1327, 1339, 1351, 1363, 1375, 1387, 1400, 1412, 1424,
    1436, 1448, 1461, 1473, 1485, 1497, 1510, 1522, 1534, 1546, 1558, 1571, 1583,
    1596, 1608, 1620, 1633, 1645, 1657, 1670, 1682, 1695, 1707, 1719, 1732, 1744,
    1757, 1769, 1782, 1794, 1807, 1819, 1831, 1844, 1857, 1869, 1882, 1895, 1907,
    1920, 1932, 1945, 1958, 1970, 1983, 1996, 2009, 2021, 2034, 2047, 2060, 2073,
    2086, 2098, 2111, 2124, 2137, 2150, 2163, 2176, 2189, 2202, 2215, 2228, 0, 1,
    2, 3, 4, 5, 6, 8, 9, 11, 13, 15, 18, 21, 24, 28, 33, 39, 46, 53, 62, 72, 82,
    93, 103, 115, 126, 137, 148, 160, 171, 182, 194, 205, 217, 228, 240, 251,
    262, 274, 285, 297, 308, 320, 331, 343, 354, 366, 377, 389, 400, 412, 424,
    435, 446, 458, 470, 482, 493, 505, 517, 528, 540, 552, 563, 575, 587, 598,
    610, 622, 634, 645, 657, 668, 680, 692, 704, 716, 727, 739, 751, 762, 774,
    786, 798, 809, 821, 833, 845, 856, 868, 880, 892, 903, 915, 927, 939, 950,
    962, 974, 986, 998, 1009, 1021, 1033, 1045, 1057, 1068, 1080, 1092, 1104,
    1116, 1128, 1140, 1152, 1163, 1175, 1187, 1199, 1211, 1223, 1235, 1247, 1259,
    1271, 1283, 1295, 1307, 1319, 1331, 1343, 1355, 1367, 1379, 1392, 1404, 1416,
    1428, 1440, 1452, 1464, 1477, 1489, 1501, 1513, 1525, 1537, 1549, 1562, 1574,
    1586, 1599, 1611, 1623, 1636, 1647, 1660, 1673, 1685, 1697, 1710, 1722, 1734,
    1747, 1759, 1772, 1784, 1796, 1809, 1821, 1834, 1846, 1859, 1871, 1884, 1896,
    1909, 1922, 1934, 1947, 1959, 1972, 1985, 1997, 2010, 2022, 2035, 2048, 2061,
    2074, 2086, 2099, 2112, 2125, 2138, 2151, 2164, 2177, 2190, 2202, 2216, 0, 1,
    2, 3, 4, 5, 6, 8, 9, 11, 13, 15, 18, 20, 24, 28, 33, 38, 45, 53, 61, 71, 81,
    92, 102, 113, 125, 136, 147, 158, 169, 181, 192, 203, 215, 226, 238, 249,
    260, 272, 283, 295, 306, 317, 329, 340, 352, 363, 374, 386, 397, 409, 421,
    432, 443, 455, 467, 478, 490, 501, 513, 524, 536, 548, 560, 571, 583, 594,
    606, 618, 629, 641, 653, 664, 676, 688, 699, 711, 723, 734, 746, 757, 769,
    781, 793, 804, 816, 827, 840, 851, 863, 874, 886, 898, 910, 921, 933, 944,
    956, 968, 980, 992, 1003, 1015, 1027, 1039, 1050, 1062, 1074, 1086, 1097,
    1109, 1121, 1133, 1145, 1156, 1168, 1180, 1192, 1204, 1216, 1228, 1240, 1252,
    1264, 1276, 1287, 1299, 1311, 1323, 1335, 1347, 1359, 1371, 1383, 1396, 1408,
    1420, 1432, 1444, 1456, 1468, 1480, 1492, 1504, 1516, 1528, 1540, 1553, 1565,
    1577, 1590, 1602, 1614, 1626, 1638, 1651, 1663, 1675, 1688, 1700, 1712, 1724,
    1737, 1749, 1761, 1774, 1786, 1799, 1811, 1823, 1836, 1848, 1861, 1873, 1886,
    1898, 1911, 1923, 1936, 1948, 1961, 1974, 1986, 1999, 2011, 2024, 2037, 2049,
    2062, 2075, 2088, 2100, 2113, 2126, 2139, 2152, 2165, 2178, 2190, 2204, 0, 1,
    2, 3, 4, 5, 6, 8, 9, 11, 13, 15, 17, 20, 24, 27, 32, 38, 44, 52, 60, 70, 80,
    91, 101, 112, 123, 134, 145, 157, 168, 179, 190, 202, 213, 224, 236, 247,
    258, 270, 281, 292, 304, 315, 326, 338, 349, 361, 372, 383, 394, 406, 418,
    429, 440, 452, 464, 475, 486, 498, 510, 521, 533, 544, 556, 567, 579, 590,
    602, 614, 625, 637, 649, 660, 672, 683, 695, 707, 718, 729, 741, 753, 765,
    776, 788, 800, 811, 822, 834, 846, 858, 869, 881, 892, 904, 916, 927, 939,
    950, 962, 974, 986, 997, 1009, 1021, 1032, 1044, 1056, 1068, 1079, 1091,
    1103, 1114, 1126, 1138, 1149, 1162, 1173, 1185, 1197, 1208, 1221, 1233, 1244,
    1256, 1268, 1280, 1292, 1304, 1315, 1328, 1339, 1352, 1364, 1376, 1388, 1400,
    1412, 1424, 1436, 1448, 1460, 1472, 1484, 1496, 1507, 1520, 1532, 1544, 1556,
    1568, 1581, 1593, 1605, 1617, 1629, 1641, 1654, 1666, 1678, 1690, 1702, 1715,
    1727, 1739, 1752, 1764, 1776, 1788, 1800, 1813, 1826, 1838, 1850, 1863, 1875,
    1888, 1900, 1912, 1925, 1937, 1950, 1962, 1975, 1988, 2000, 2013, 2025, 2038,
    2051, 2063, 2076, 2089, 2102, 2114, 2127, 2140, 2153, 2166, 2178, 2191, 0, 1,
    2, 3, 4, 5, 6, 8, 9, 11, 13, 15, 17, 20, 23, 27, 32, 37, 44, 51, 59, 69, 79,
    89, 100, 111, 122, 133, 144, 155, 166, 177, 189, 200, 211, 222, 234, 245,
    256, 267, 279, 290, 301, 313, 324, 335, 347, 358, 369, 381, 392, 403, 415,
    426, 437, 449, 460, 472, 483, 494, 506, 517, 529, 541, 552, 563, 575, 586,
    598, 610, 621, 633, 644, 656, 668, 679, 691, 702, 714, 725, 737, 748, 760,
    772, 783, 795, 806, 817, 829, 841, 852, 864, 876, 887, 899, 910, 922, 933,
    945, 956, 968, 980, 991, 1003, 1015, 1026, 1038, 1050, 1061, 1073, 1085,
    1096, 1108, 1120, 1131, 1143, 1155, 1167, 1178, 1190, 1201, 1214, 1225, 1237,
    1249, 1261, 1273, 1284, 1296, 1308, 1320, 1332, 1344, 1356, 1368, 1380, 1391,
    1403, 1415, 1427, 1439, 1451, 1463, 1475, 1487, 1499, 1511, 1523, 1535, 1547,
    1560, 1572, 1584, 1596, 1608, 1620, 1632, 1644, 1656, 1669, 1681, 1693, 1705,
    1717, 1729, 1742, 1754, 1766, 1778, 1790, 1803, 1815, 1828, 1840, 1852, 1865,
    1877, 1889, 1902, 1914, 1927, 1939, 1952, 1964, 1977, 1989, 2002, 2014, 2027,
    2039, 2051, 2065, 2077, 2090, 2103, 2115, 2128, 2141, 2154, 2166, 2179, 0, 1,
    2, 3, 4, 5, 6, 8, 9, 11, 13, 15, 17, 20, 23, 27, 31, 37, 43, 50, 59, 68, 78,
    88, 98, 110, 120, 131, 142, 153, 165, 176, 187, 198, 209, 220, 232, 243, 254,
    265, 276, 288, 299, 310, 321, 333, 344, 355, 366, 378, 389, 401, 412, 423,
    434, 446, 457, 469, 480, 491, 503, 514, 526, 537, 549, 560, 572, 583, 594,
    606, 617, 629, 640, 651, 663, 675, 686, 698, 709, 720, 732, 743, 755, 767,
    778, 790, 801, 812, 824, 836, 847, 858, 870, 881, 893, 905, 916, 927, 939,
    951, 962, 974, 986, 997, 1009, 1020, 1032, 1043, 1055, 1067, 1078, 1090,
    1101, 1113, 1125, 1136, 1148, 1160, 1171, 1183, 1194, 1207, 1218, 1230, 1242,
    1254, 1265, 1277, 1289, 1300, 1312, 1324, 1336, 1348, 1360, 1372, 1384, 1395,
    1407, 1419, 1431, 1443, 1455, 1467, 1479, 1490, 1502, 1514, 1527, 1539, 1551,
    1563, 1575, 1587, 1599, 1610, 1623, 1635, 1647, 1659, 1671, 1683, 1696, 1708,
    1720, 1732, 1744, 1756, 1768, 1780, 1793, 1805, 1817, 1830, 1842, 1854, 1867,
    1879, 1891, 1904, 1916, 1928, 1941, 1953, 1966, 1978, 1990, 2003, 2015, 2028,
    2040, 2053, 2066, 2078, 2091, 2104, 2116, 2129, 2142, 2154, 2167, 0, 1, 2, 3,
    4, 5, 6, 8, 9, 11, 13, 15, 17, 20, 23, 27, 31, 36, 43, 50, 58, 67, 77, 87,
    97, 108, 119, 130, 141, 152, 163, 174, 185, 196, 207, 219, 230, 241, 252,
    263, 274, 285, 297, 308, 319, 330, 341, 353, 364, 375, 386, 398, 409, 420,
    431, 443, 454, 466, 476, 488, 500, 511, 522, 534, 545, 556, 568, 579, 591,
    602, 613, 625, 636, 647, 659, 671, 682, 693, 705, 716, 728, 739, 751, 762,
    774, 785, 796, 807, 819, 831, 842, 853, 865, 876, 888, 899, 911, 922, 933,
    945, 957, 968, 980, 991, 1003, 1014, 1026, 1037, 1049, 1060, 1072, 1083,
    1095, 1107, 1118, 1129, 1141, 1153, 1164, 1176, 1188, 1200, 1211, 1223, 1235,
    1246, 1258, 1270, 1281, 1293, 1305, 1316, 1328, 1340, 1352, 1364, 1376, 1387,
    1399, 1411, 1423, 1435, 1447, 1459, 1470, 1482, 1494, 1506, 1518, 1530, 1542,
    1554, 1566, 1578, 1590, 1601, 1614, 1626, 1638, 1650, 1662, 1674, 1686, 1698,
    1710, 1722, 1734, 1747, 1759, 1770, 1783, 1795, 1807, 1819, 1832, 1844, 1856,
    1868, 1881, 1893, 1905, 1918, 1930, 1942, 1955, 1967, 1979, 1992, 2004, 2017,
    2029, 2042, 2054, 2067, 2079, 2092, 2105, 2117, 2130, 2142, 2155, 0, 1, 2, 3,
    4, 5, 6, 8, 9, 11, 12, 14, 17, 19, 23, 26, 31, 36, 42, 49, 57, 66, 76, 86,
    96, 107, 118, 129, 139, 150, 161, 172, 183, 195, 206, 217, 228, 239, 250,
    261, 272, 283, 294, 306, 317, 328, 339, 350, 361, 373, 383, 395, 406, 417,
    428, 440, 451, 462, 473, 485, 496, 507, 519, 530, 542, 552, 564, 575, 587,
    598, 610, 621, 632, 643, 655, 666, 678, 689, 701, 712, 723, 734, 746, 757,
    769, 780, 792, 803, 814, 826, 837, 848, 860, 871, 883, 894, 905, 916, 928,
    939, 951, 963, 974, 985, 997, 1008, 1020, 1031, 1043, 1054, 1066, 1077, 1089,
    1100, 1112, 1123, 1135, 1146, 1158, 1169, 1181, 1193, 1204, 1216, 1228, 1239,
    1251, 1262, 1274, 1285, 1298, 1309, 1321, 1333, 1344, 1356, 1368, 1380, 1391,
    1403, 1415, 1427, 1439, 1450, 1462, 1474, 1485, 1497, 1510, 1521, 1533, 1545,
    1557, 1569, 1581, 1592, 1605, 1617, 1629, 1641, 1653, 1665, 1677, 1689, 1701,
    1713, 1725, 1737, 1749, 1761, 1773, 1785, 1797, 1809, 1822, 1834, 1846, 1858,
    1870, 1883, 1895, 1907, 1919, 1932, 1944, 1956, 1969, 1981, 1993, 2006, 2018,
    2031, 2043, 2056, 2068, 2081, 2093, 2106, 2118, 2130, 2143, 0, 1, 2, 3, 4, 5,
    6, 8, 9, 11, 12, 14, 17, 19, 22, 26, 30, 35, 41, 48, 56, 65, 75, 85, 95, 106,
    116, 127, 138, 149, 160, 171, 182, 193, 204, 215, 226, 237, 248, 259, 270,
    281, 292, 303, 314, 325, 337, 348, 358, 370, 381, 392, 403, 414, 425, 437,
    448, 459, 470, 481, 493, 504, 515, 527, 538, 549, 561, 571, 583, 594, 606,
    617, 628, 639, 651, 662, 674, 685, 696, 707, 719, 730, 741, 753, 764, 775,
    787, 798, 809, 821, 832, 843, 855, 866, 877, 889, 900, 911, 922, 934, 945,
    957, 968, 980, 991, 1002, 1014, 1025, 1037, 1048, 1059, 1071, 1082, 1094,
    1105, 1116, 1128, 1140, 1151, 1163, 1174, 1186, 1197, 1209, 1221, 1232, 1244,
    1255, 1267, 1278, 1290, 1301, 1313, 1325, 1337, 1348, 1360, 1372, 1384, 1395,
    1407, 1419, 1430, 1442, 1454, 1465, 1477, 1489, 1501, 1513, 1525, 1537, 1548,
    1560, 1572, 1584, 1596, 1608, 1620, 1632, 1643, 1655, 1667, 1679, 1691, 1703,
    1715, 1727, 1739, 1751, 1763, 1775, 1787, 1799, 1812, 1824, 1836, 1848, 1860,
    1872, 1884, 1897, 1909, 1921, 1933, 1945, 1958, 1970, 1982, 1995, 2007, 2019,
    2032, 2044, 2057, 2069, 2082, 2094, 2107, 2119, 2132, 0, 1, 2, 3, 4, 5, 6, 8,
    9, 11, 12, 14, 17, 19, 22, 26, 30, 35, 41, 48, 55, 64, 74, 84, 94, 104, 115,
    126, 137, 147, 158, 169, 180, 191, 202, 213, 224, 235, 246, 257, 268, 279,
    290, 301, 312, 323, 334, 345, 356, 367, 378, 389, 401, 412, 422, 434, 445,
    456, 467, 478, 490, 501, 512, 523, 535, 545, 557, 568, 579, 591, 602, 613,
    624, 635, 647, 658, 669, 681, 692, 703, 714, 725, 737, 748, 759, 771, 782,
    793, 805, 816, 827, 838, 850, 861, 872, 883, 895, 906, 917, 928, 940, 951,
    963, 974, 985, 997, 1008, 1019, 1031, 1042, 1053, 1065, 1076, 1088, 1099,
    1110, 1122, 1133, 1144, 1156, 1167, 1179, 1191, 1202, 1214, 1225, 1237, 1248,
    1260, 1271, 1283, 1294, 1306, 1318, 1329, 1341, 1352, 1364, 1376, 1387, 1399,
    1411, 1422, 1434, 1446, 1457, 1469, 1481, 1493, 1504, 1516, 1528, 1540, 1552,
    1563, 1575, 1587, 1599, 1611, 1622, 1634, 1646, 1658, 1670, 1682, 1694, 1706,
    1718, 1730, 1741, 1754, 1766, 1777, 1790, 1802, 1814, 1826, 1838, 1850, 1862,
    1874, 1886, 1898, 1910, 1923, 1934, 1947, 1959, 1972, 1984, 1996, 2008, 2021,
    2033, 2045, 2058, 2070, 2083, 2095, 2107, 2120, 0, 1, 2, 3, 4, 5, 6, 7, 9,
    10, 12, 14, 16, 19, 22, 25, 30, 35, 40, 47, 55, 63, 73, 83, 93, 103, 114,
    124, 135, 146, 157, 168, 178, 189, 200, 211, 222, 233, 244, 255, 266, 277,
    288, 299, 310, 321, 332, 343, 353, 365, 375, 387, 398, 409, 420, 431, 442,
    453, 464, 475, 486, 497, 509, 520, 531, 542, 553, 564, 576, 587, 598, 609,
    620, 631, 643, 654, 665, 676, 688, 698, 710, 721, 732, 744, 755, 766, 777,
    788, 800, 811, 822, 833, 845, 855, 867, 878, 889, 900, 912, 923, 934, 946,
    957, 968, 979, 991, 1002, 1013, 1025, 1036, 1047, 1059, 1070, 1081, 1093,
    1104, 1115, 1127, 1138, 1149, 1161, 1172, 1184, 1195, 1207, 1218, 1230, 1241,
    1253, 1264, 1276, 1287, 1299, 1310, 1322, 1333, 1345, 1356, 1368, 1380, 1391,
    1403, 1414, 1426, 1438, 1449, 1461, 1472, 1484, 1496, 1508, 1520, 1531, 1543,
    1555, 1566, 1578, 1590, 1602, 1614, 1625, 1637, 1649, 1661, 1673, 1684, 1696,
    1708, 1720, 1732, 1744, 1756, 1768, 1780, 1792, 1804, 1816, 1828, 1840, 1852,
    1864, 1876, 1888, 1900, 1912, 1924, 1936, 1949, 1961, 1973, 1985, 1998, 2010,
    2022, 2034, 2047, 2059, 2071, 2084, 2096, 2109, 0, 1, 2, 3, 4, 5, 6, 7, 9,
    10, 12, 14, 16, 19, 22, 25, 29, 34, 40, 46, 54, 63, 72, 82, 91, 102, 113,
    123, 134, 145, 155, 166, 177, 188, 199, 209, 220, 231, 242, 253, 264, 275,
    286, 296, 307, 318, 329, 340, 351, 362, 373, 384, 395, 406, 417, 428, 439,
    450, 461, 472, 483, 494, 505, 517, 528, 538, 550, 561, 572, 583, 594, 605,
    617, 627, 639, 650, 661, 672, 683, 694, 706, 717, 728, 739, 750, 761, 773,
    783, 795, 806, 817, 828, 840, 850, 862, 873, 884, 895, 906, 917, 929, 940,
    951, 963, 974, 985, 996, 1007, 1019, 1030, 1041, 1052, 1064, 1075, 1086,
    1097, 1109, 1120, 1131, 1143, 1154, 1166, 1177, 1188, 1200, 1211, 1223, 1234,
    1245, 1257, 1268, 1279, 1291, 1303, 1314, 1326, 1337, 1349, 1360, 1372, 1383,
    1395, 1407, 1418, 1430, 1441, 1453, 1464, 1476, 1488, 1499, 1511, 1523, 1534,
    1546, 1557, 1569, 1581, 1593, 1605, 1616, 1628, 1640, 1652, 1663, 1675, 1687,
    1699, 1711, 1722, 1734, 1746, 1758, 1770, 1782, 1794, 1806, 1818, 1830, 1842,
    1854, 1866, 1878, 1890, 1902, 1913, 1926, 1938, 1950, 1962, 1974, 1987, 1999,
    2011, 2023, 2036, 2048, 2060, 2073, 2084, 2097, 0, 1, 2, 3, 4, 5, 6, 7, 9,
    10, 12, 14, 16, 19, 22, 25, 29, 34, 39, 46, 53, 62, 71, 81, 90, 101, 111,
    122, 132, 143, 154, 165, 175, 186, 197, 208, 218, 229, 240, 251, 262, 273,
    283, 294, 305, 316, 327, 338, 348, 360, 370, 381, 392, 403, 414, 425, 436,
    447, 458, 469, 480, 491, 502, 513, 524, 535, 546, 557, 568, 580, 591, 602,
    613, 623, 635, 646, 657, 668, 679, 690, 701, 712, 724, 735, 746, 757, 768,
    779, 790, 801, 812, 823, 835, 845, 857, 868, 879, 890, 901, 912, 924, 935,
    946, 957, 968, 979, 990, 1002, 1013, 1024, 1035, 1046, 1058, 1069, 1080,
    1091, 1103, 1114, 1125, 1136, 1147, 1159, 1170, 1182, 1193, 1204, 1216, 1227,
    1238, 1249, 1261, 1272, 1284, 1295, 1307, 1318, 1330, 1341, 1353, 1364, 1376,
    1387, 1399, 1410, 1422, 1433, 1444, 1456, 1468, 1480, 1491, 1503, 1514, 1526,
    1538, 1549, 1561, 1572, 1584, 1596, 1607, 1619, 1631, 1643, 1654, 1666, 1678,
    1690, 1701, 1713, 1725, 1737, 1748, 1760, 1772, 1784, 1796, 1808, 1820, 1832,
    1844, 1855, 1867, 1879, 1891, 1903, 1915, 1928, 1940, 1952, 1963, 1976, 1988,
    2000, 2012, 2025, 2037, 2049, 2061, 2073, 2086, 0, 1, 2, 3, 4, 5, 6, 7, 9,
    10, 12, 14, 16, 18, 21, 25, 29, 33, 39, 45, 52, 61, 70, 79, 89, 100, 110,
    120, 131, 142, 152, 163, 174, 184, 195, 206, 217, 227, 238, 249, 260, 270,
    281, 292, 303, 314, 324, 335, 346, 357, 367, 379, 390, 400, 411, 422, 433,
    444, 455, 466, 477, 488, 499, 510, 521, 532, 543, 554, 565, 576, 587, 598,
    609, 620, 631, 642, 653, 664, 675, 686, 697, 708, 719, 730, 741, 752, 763,
    774, 786, 797, 808, 818, 830, 840, 852, 863, 874, 885, 896, 907, 918, 929,
    940, 951, 963, 974, 985, 996, 1007, 1018, 1029, 1040, 1052, 1063, 1074, 1085,
    1096, 1108, 1118, 1130, 1141, 1153, 1164, 1175, 1186, 1198, 1209, 1220, 1232,
    1242, 1254, 1265, 1277, 1288, 1300, 1311, 1322, 1334, 1345, 1357, 1368, 1380,
    1391, 1402, 1414, 1425, 1437, 1448, 1460, 1471, 1483, 1494, 1506, 1518, 1529,
    1540, 1552, 1564, 1575, 1587, 1599, 1610, 1622, 1634, 1645, 1657, 1669, 1680,
    1692, 1703, 1715, 1727, 1739, 1751, 1763, 1774, 1786, 1798, 1810, 1822, 1834,
    1845, 1857, 1869, 1881, 1893, 1905, 1917, 1929, 1941, 1953, 1965, 1977, 1989,
    2002, 2014, 2026, 2038, 2050, 2062, 2075, 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12,
    14, 16, 18, 21, 24, 28, 33, 38, 45, 52, 60, 69, 78, 88, 98, 109, 119, 130,
    140, 151, 161, 172, 183, 193, 204, 215, 225, 236, 247, 258, 268, 279, 290,
    301, 311, 322, 333, 343, 354, 365, 376, 387, 398, 408, 419, 430, 441, 452,
    463, 474, 484, 496, 507, 518, 528, 539, 550, 561, 572, 583, 594, 605, 616,
    627, 638, 649, 660, 671, 682, 693, 704, 715, 726, 737, 748, 759, 770, 781,
    792, 803, 813, 825, 835, 847, 858, 869, 879, 890, 901, 913, 924, 935, 946,
    957, 968, 979, 990, 1001, 1012, 1023, 1035, 1046, 1057, 1068, 1079, 1090,
    1101, 1112, 1124, 1134, 1146, 1157, 1168, 1180, 1191, 1202, 1213, 1225, 1236,
    1247, 1258, 1270, 1281, 1292, 1304, 1315, 1326, 1338, 1349, 1361, 1372, 1383,
    1395, 1406, 1417, 1429, 1440, 1452, 1463, 1475, 1486, 1498, 1509, 1521, 1532,
    1544, 1555, 1567, 1578, 1590, 1602, 1613, 1625, 1636, 1648, 1660, 1671, 1683,
    1694, 1706, 1718, 1730, 1741, 1753, 1765, 1776, 1788, 1800, 1812, 1824, 1836,
    1847, 1859, 1871, 1883, 1895, 1907, 1919, 1931, 1942, 1955, 1967, 1979, 1991,
    2003, 2015, 2027, 2039, 2051, 2064, 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 14,
    16, 18, 21, 24, 28, 33, 38, 44, 51, 59, 68, 77, 87, 97, 108, 118, 128, 139,
    149, 160, 170, 181, 192, 202, 213, 224, 234, 245, 256, 266, 277, 288, 298,
    309, 320, 330, 341, 352, 362, 373, 384, 395, 405, 417, 427, 438, 449, 460,
    471, 481, 492, 503, 514, 525, 536, 547, 558, 569, 580, 591, 601, 612, 623,
    634, 645, 656, 667, 678, 689, 699, 711, 722, 733, 743, 754, 765, 776, 787,
    798, 809, 820, 831, 842, 853, 864, 874, 885, 896, 908, 919, 930, 941, 952,
    963, 974, 985, 996, 1007, 1018, 1029, 1040, 1051, 1062, 1073, 1084, 1095,
    1106, 1117, 1128, 1140, 1151, 1162, 1173, 1184, 1195, 1207, 1218, 1229, 1240,
    1251, 1263, 1274, 1285, 1297, 1308, 1319, 1330, 1342, 1353, 1364, 1376, 1387,
    1398, 1409, 1421, 1432, 1444, 1455, 1467, 1478, 1490, 1501, 1512, 1524, 1535,
    1547, 1558, 1570, 1581, 1593, 1604, 1616, 1627, 1639, 1651, 1662, 1674, 1685,
    1697, 1709, 1720, 1732, 1744, 1755, 1767, 1779, 1790, 1802, 1814, 1826, 1837,
    1849, 1861, 1872, 1885, 1897, 1909, 1920, 1932, 1944, 1956, 1968, 1980, 1992,
    2004, 2016, 2028, 2040, 2053, 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 14, 16, 18,
    21, 24, 28, 32, 37, 43, 50, 58, 67, 76, 86, 96, 106, 117, 127, 137, 148, 158,
    169, 179, 190, 201, 211, 222, 232, 243, 254, 264, 275, 286, 296, 307, 317,
    328, 338, 349, 360, 371, 382, 392, 403, 414, 425, 435, 446, 457, 468, 478,
    489, 500, 511, 521, 533, 543, 554, 565, 576, 587, 598, 608, 619, 630, 641,
    652, 663, 673, 685, 695, 706, 717, 728, 739, 750, 760, 772, 783, 793, 804,
    815, 826, 837, 848, 859, 869, 880, 891, 902, 913, 924, 935, 946, 957, 968,
    979, 990, 1001, 1012, 1023, 1034, 1045, 1056, 1067, 1078, 1089, 1100, 1111,
    1122, 1133, 1144, 1155, 1166, 1178, 1189, 1200, 1211, 1222, 1233, 1244, 1256,
    1267, 1278, 1289, 1301, 1312, 1323, 1334, 1346, 1357, 1368, 1379, 1391, 1402,
    1413, 1424, 1436, 1447, 1459, 1470, 1481, 1493, 1504, 1515, 1527, 1538, 1550,
    1561, 1573, 1584, 1596, 1607, 1619, 1630, 1642, 1653, 1665, 1676, 1688, 1699,
    1711, 1723, 1734, 1746, 1757, 1769, 1781, 1792, 1804, 1816, 1828, 1839, 1851,
    1862, 1875, 1886, 1898, 1910, 1922, 1934, 1946, 1958, 1970, 1982, 1994, 2006,
    2018, 2029, 2042, 0, 1, 2, 3, 4, 5, 6, 7, 9, 10, 12, 13, 16, 18, 21, 24, 27,
    32, 37, 43, 50, 58, 66, 75, 85, 95, 105, 115, 126, 136, 146, 157, 167, 178,
    188, 199, 209, 220, 231, 241, 252, 262, 273, 283, 294, 305, 315, 326, 336,
    347, 357, 368, 379, 390, 400, 411, 422, 432, 443, 454, 465, 475, 486, 497,
    508, 518, 529, 540, 551, 562, 572, 583, 594, 604, 616, 626, 637, 648, 659,
    669, 681, 691, 702, 713, 724, 735, 745, 756, 767, 778, 789, 799, 810, 821,
    832, 843, 854, 864, 875, 886, 897, 908, 919, 930, 941, 952, 962, 973, 984,
    995, 1006, 1017, 1028, 1039, 1050, 1061, 1072, 1083, 1093, 1105, 1115, 1127,
    1138, 1149, 1160, 1171, 1182, 1193, 1204, 1215, 1227, 1237, 1249, 1260, 1271,
    1282, 1293, 1305, 1316, 1327, 1338, 1349, 1361, 1372, 1383, 1394, 1405, 1417,
    1428, 1439, 1451, 1462, 1473, 1485, 1496, 1507, 1519, 1530, 1541, 1553, 1564,
    1576, 1587, 1598, 1610, 1621, 1633, 1644, 1656, 1667, 1679, 1690, 1702, 1713,
    1725, 1736, 1748, 1760, 1771, 1783, 1794, 1806, 1818, 1830, 1841, 1853, 1865,
    1876, 1888, 1900, 1911, 1924, 1935, 1947, 1959, 1971, 1983, 1995, 2007, 2019,
    2031, 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 13, 15, 18, 20, 24, 27, 31, 36, 42,
    49, 57, 65, 74, 84, 94, 104, 114, 124, 135, 145, 155, 166, 176, 187, 197,
    208, 218, 229, 239, 250, 260, 271, 281, 292, 302, 313, 323, 334, 345, 355,
    366, 376, 387, 397, 408, 419, 430, 440, 451, 462, 472, 483, 494, 504, 515,
    526, 536, 547, 558, 569, 580, 590, 601, 612, 623, 633, 644, 655, 665, 676,
    687, 698, 709, 720, 730, 741, 751, 763, 773, 784, 795, 806, 816, 827, 838,
    849, 859, 870, 881, 892, 903, 914, 925, 935, 946, 957, 968, 979, 990, 1000,
    1011, 1022, 1033, 1044, 1055, 1066, 1077, 1087, 1099, 1109, 1121, 1132, 1143,
    1154, 1165, 1176, 1187, 1198, 1208, 1220, 1230, 1242, 1253, 1264, 1275, 1286,
    1297, 1309, 1320, 1331, 1342, 1353, 1364, 1376, 1386, 1398, 1409, 1420, 1432,
    1443, 1454, 1465, 1477, 1488, 1499, 1511, 1522, 1533, 1544, 1556, 1567, 1579,
    1590, 1601, 1613, 1624, 1635, 1647, 1658, 1670, 1681, 1693, 1704, 1716, 1727,
    1739, 1750, 1762, 1773, 1785, 1797, 1808, 1820, 1831, 1843, 1855, 1866, 1878,
    1890, 1901, 1913, 1925, 1937, 1949, 1961, 1973, 1984, 1996, 2008, 2020, 0, 1,
    2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, 18, 20, 23, 27, 31, 36, 42, 48, 56, 64,
    73, 83, 93, 103, 113, 123, 133, 144, 154, 164, 175, 185, 196, 206, 216, 227,
    237, 248, 258, 269, 279, 290, 300, 311, 321, 331, 342, 352, 363, 374, 384,
    395, 406, 416, 427, 437, 448, 459, 469, 480, 491, 501, 512, 523, 533, 544,
    555, 565, 576, 587, 597, 608, 619, 630, 640, 651, 661, 672, 683, 694, 705,
    715, 726, 737, 747, 758, 769, 780, 790, 801, 811, 823, 833, 844, 854, 865,
    876, 887, 898, 908, 919, 930, 941, 952, 962, 973, 984, 995, 1006, 1016, 1027,
    1038, 1049, 1060, 1071, 1081, 1093, 1103, 1114, 1125, 1136, 1147, 1158, 1169,
    1180, 1191, 1202, 1213, 1224, 1235, 1246, 1257, 1268, 1279, 1290, 1301, 1313,
    1324, 1335, 1346, 1357, 1368, 1379, 1390, 1401, 1413, 1424, 1435, 1446, 1457,
    1469, 1480, 1491, 1502, 1514, 1525, 1536, 1547, 1559, 1570, 1581, 1593, 1604,
    1615, 1627, 1638, 1649, 1661, 1672, 1684, 1695, 1707, 1718, 1729, 1741, 1752,
    1764, 1775, 1787, 1799, 1810, 1822, 1833, 1845, 1857, 1868, 1880, 1891, 1903,
    1915, 1927, 1939, 1950, 1962, 1974, 1986, 1997, 2010, 0, 1, 2, 3, 4, 5, 6, 7,
    8, 10, 11, 13, 15, 17, 20, 23, 27, 31, 36, 41, 48, 55, 64, 73, 82, 92, 102,
    112, 122, 132, 142, 152, 163, 173, 184, 194, 204, 215, 225, 235, 246, 256,
    267, 277, 288, 298, 308, 319, 329, 340, 350, 361, 371, 382, 392, 403, 413,
    424, 434, 445, 456, 466, 477, 488, 498, 508, 519, 530, 541, 551, 562, 573,
    583, 593, 604, 615, 626, 636, 647, 657, 668, 679, 690, 700, 711, 722, 732,
    743, 754, 764, 775, 785, 796, 807, 818, 828, 839, 849, 860, 871, 882, 893,
    903, 914, 925, 935, 946, 957, 968, 978, 989, 1000, 1011, 1022, 1032, 1043,
    1054, 1065, 1075, 1086, 1097, 1108, 1119, 1130, 1141, 1152, 1163, 1174, 1185,
    1195, 1206, 1217, 1228, 1239, 1250, 1261, 1272, 1283, 1294, 1305, 1316, 1327,
    1339, 1350, 1361, 1371, 1383, 1394, 1405, 1416, 1427, 1439, 1450, 1461, 1472,
    1483, 1494, 1506, 1517, 1528, 1539, 1550, 1562, 1573, 1584, 1595, 1607, 1618,
    1629, 1640, 1652, 1663, 1675, 1686, 1697, 1709, 1720, 1732, 1743, 1755, 1766,
    1778, 1789, 1801, 1812, 1823, 1835, 1847, 1858, 1870, 1881, 1893, 1905, 1917,
    1928, 1940, 1952, 1964, 1975, 1987, 1999, 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11,
    13, 15, 17, 20, 23, 26, 30, 35, 41, 47, 55, 63, 72, 81, 91, 100, 110, 120,
    131, 141, 151, 161, 172, 182, 192, 203, 213, 223, 234, 244, 254, 265, 275,
    285, 296, 306, 317, 327, 337, 348, 358, 369, 379, 389, 400, 411, 421, 431,
    442, 453, 463, 474, 484, 495, 505, 516, 526, 537, 548, 558, 569, 580, 590,
    601, 611, 622, 633, 643, 653, 664, 675, 686, 696, 707, 718, 728, 738, 749,
    760, 771, 781, 792, 802, 813, 824, 834, 845, 855, 866, 877, 888, 898, 909,
    920, 930, 941, 952, 962, 973, 984, 994, 1005, 1016, 1027, 1037, 1048, 1059,
    1069, 1080, 1091, 1102, 1113, 1124, 1135, 1145, 1156, 1167, 1178, 1189, 1200,
    1210, 1222, 1233, 1243, 1254, 1265, 1276, 1287, 1298, 1309, 1320, 1331, 1342,
    1353, 1364, 1375, 1386, 1398, 1409, 1420, 1431, 1442, 1453, 1464, 1475, 1486,
    1497, 1509, 1520, 1531, 1542, 1553, 1565, 1576, 1587, 1598, 1609, 1621, 1632,
    1643, 1655, 1666, 1677, 1688, 1700, 1711, 1723, 1734, 1745, 1757, 1768, 1780,
    1791, 1803, 1814, 1826, 1837, 1849, 1860, 1871, 1883, 1895, 1907, 1918, 1930,
    1942, 1953, 1965, 1976, 1989, 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, 17,
    20, 23, 26, 30, 35, 40, 46, 54, 62, 71, 80, 89, 99, 109, 119, 129, 139, 150,
    160, 170, 180, 191, 201, 211, 221, 232, 242, 252, 263, 273, 283, 294, 304,
    314, 324, 335, 345, 356, 366, 377, 387, 398, 408, 418, 429, 439, 450, 460,
    471, 481, 492, 502, 513, 523, 534, 544, 555, 566, 576, 586, 597, 608, 618,
    629, 639, 650, 661, 671, 682, 692, 703, 713, 724, 734, 745, 756, 766, 776,
    787, 797, 808, 819, 830, 840, 850, 861, 872, 883, 893, 904, 914, 925, 936,
    946, 957, 968, 978, 989, 1000, 1010, 1021, 1031, 1042, 1053, 1063, 1075,
    1085, 1096, 1107, 1118, 1128, 1139, 1150, 1161, 1172, 1182, 1193, 1204, 1215,
    1226, 1237, 1248, 1259, 1269, 1280, 1291, 1302, 1313, 1324, 1335, 1346, 1357,
    1368, 1379, 1390, 1401, 1412, 1423, 1434, 1445, 1456, 1467, 1478, 1489, 1501,
    1512, 1523, 1534, 1545, 1556, 1567, 1579, 1590, 1601, 1612, 1623, 1635, 1646,
    1657, 1668, 1680, 1691, 1702, 1713, 1725, 1736, 1747, 1759, 1770, 1782, 1793,
    1804, 1816, 1827, 1839, 1850, 1862, 1874, 1885, 1897, 1908, 1920, 1932, 1943,
    1955, 1966, 1978, 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, 17, 20, 22, 26,
    30, 34, 40, 46, 53, 61, 70, 79, 88, 98, 108, 118, 128, 138, 148, 158, 169,
    179, 189, 199, 209, 220, 230, 240, 250, 261, 271, 281, 292, 302, 312, 322,
    333, 343, 353, 364, 374, 384, 395, 405, 416, 426, 436, 447, 457, 468, 478,
    489, 499, 510, 520, 531, 541, 552, 562, 573, 583, 594, 604, 615, 625, 636,
    646, 657, 667, 678, 688, 699, 709, 720, 730, 741, 751, 762, 772, 783, 793,
    804, 814, 825, 835, 846, 856, 867, 878, 888, 899, 909, 920, 930, 941, 952,
    962, 973, 983, 994, 1005, 1015, 1026, 1037, 1047, 1058, 1069, 1079, 1090,
    1101, 1111, 1122, 1133, 1144, 1154, 1165, 1176, 1187, 1197, 1208, 1219, 1230,
    1241, 1252, 1263, 1273, 1284, 1295, 1306, 1317, 1328, 1339, 1349, 1360, 1371,
    1383, 1394, 1405, 1416, 1426, 1437, 1448, 1459, 1471, 1482, 1493, 1504, 1515,
    1526, 1537, 1548, 1559, 1570, 1581, 1592, 1604, 1614, 1626, 1637, 1648, 1659,
    1671, 1682, 1693, 1704, 1716, 1727, 1738, 1750, 1761, 1772, 1784, 1795, 1806,
    1818, 1829, 1841, 1852, 1864, 1875, 1887, 1898, 1910, 1921, 1933, 1945, 1956,
    1968, 0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 11, 13, 15, 17, 19, 22, 26, 29, 34, 39,
    45, 52, 60, 69, 78, 87, 97, 107, 117, 127, 137, 147, 157, 167, 177, 187, 198,
    208, 218, 228, 238, 249, 259, 269, 279, 289, 300, 310, 320, 330, 340, 351,
    361, 372, 382, 392, 403, 413, 423, 433, 444, 454, 465, 475, 486, 496, 507,
    517, 527, 538, 548, 559, 569, 579, 590, 600, 611, 621, 632, 642, 653, 663,
    674, 684, 695, 705, 715, 726, 736, 747, 757, 767, 778, 788, 799, 810, 820,
    830, 841, 851, 862, 873, 883, 894, 904, 915, 925, 936, 946, 957, 967, 978,
    989, 999, 1010, 1020, 1031, 1041, 1052, 1063, 1073, 1084, 1095, 1105, 1116,
    1127, 1137, 1148, 1159, 1169, 1180, 1191, 1202, 1213, 1223, 1234, 1245, 1256,
    1267, 1277, 1288, 1299, 1310, 1321, 1332, 1342, 1353, 1364, 1375, 1386, 1397,
    1408, 1419, 1430, 1441, 1451, 1463, 1474, 1485, 1496, 1507, 1518, 1529, 1540,
    1551, 1562, 1573, 1584, 1595, 1606, 1617, 1628, 1640, 1651, 1662, 1673, 1684,
    1696, 1707, 1718, 1729, 1740, 1752, 1763, 1774, 1785, 1797, 1808, 1820, 1831,
    1842, 1854, 1865, 1877, 1888, 1900, 1911, 1923, 1935, 1946, 1958, 0, 1, 2, 3,
    4, 5, 6, 7, 8, 10, 11, 13, 15, 17, 19, 22, 25, 29, 34, 39, 45, 52, 59, 68,
    77, 86, 96, 106, 115, 125, 135, 145, 155, 166, 176, 186, 196, 206, 216, 226,
    236, 247, 257, 267, 277, 287, 298, 308, 318, 328, 338, 349, 359, 369, 379,
    390, 400, 410, 420, 431, 441, 451, 462, 472, 483, 493, 503, 513, 524, 535,
    545, 555, 566, 576, 586, 597, 607, 618, 628, 638, 649, 659, 670, 680, 691,
    701, 711, 721, 732, 743, 753, 763, 774, 784, 795, 805, 816, 826, 836, 846,
    857, 868, 878, 889, 899, 910, 920, 931, 941, 952, 962, 973, 983, 994, 1004,
    1014, 1025, 1036, 1046, 1057, 1067, 1078, 1089, 1099, 1110, 1121, 1131, 1142,
    1153, 1163, 1174, 1184, 1195, 1206, 1217, 1228, 1238, 1249, 1260, 1271, 1281,
    1292, 1303, 1314, 1325, 1335, 1346, 1357, 1368, 1379, 1390, 1400, 1411, 1422,
    1433, 1444, 1455, 1466, 1477, 1488, 1499, 1510, 1521, 1532, 1543, 1554, 1565,
    1576, 1587, 1597, 1609, 1620, 1631, 1642, 1653, 1664, 1676, 1687, 1698, 1709,
    1720, 1731, 1743, 1754, 1765, 1776, 1788, 1799, 1810, 1822, 1833, 1844, 1856,
    1867, 1879, 1890, 1902, 1913, 1925, 1936, 1948, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
    11, 13, 15, 17, 19, 22, 25, 29, 33, 38, 44, 51, 59, 67, 76, 85, 95, 104, 114,
    124, 134, 144, 154, 164, 174, 184, 194, 204, 214, 225, 235, 245, 255, 265,
    275, 285, 295, 306, 315, 326, 336, 346, 356, 367, 377, 387, 397, 408, 418,
    428, 438, 448, 459, 469, 480, 490, 500, 510, 521, 531, 542, 552, 562, 572,
    583, 593, 604, 614, 624, 634, 645, 655, 666, 676, 687, 697, 707, 717, 728,
    738, 749, 759, 769, 779, 790, 801, 811, 821, 831, 842, 852, 863, 873, 884,
    894, 904, 915, 925, 936, 946, 957, 967, 978, 988, 999, 1009, 1020, 1030,
    1040, 1051, 1061, 1072, 1083, 1093, 1104, 1115, 1125, 1136, 1146, 1157, 1168,
    1178, 1189, 1200, 1210, 1221, 1232, 1242, 1253, 1264, 1275, 1285, 1296, 1307,
    1318, 1328, 1339, 1349, 1361, 1371, 1382, 1393, 1404, 1415, 1426, 1436, 1447,
    1458, 1469, 1480, 1491, 1502, 1513, 1524, 1535, 1546, 1556, 1567, 1578, 1589,
    1600, 1611, 1623, 1634, 1645, 1656, 1667, 1678, 1689, 1700, 1711, 1722, 1734,
    1745, 1756, 1767, 1778, 1790, 1801, 1812, 1823, 1835, 1846, 1858, 1869, 1880,
    1892, 1903, 1915, 1926, 1938, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 13, 14, 17,
    19, 22, 25, 28, 33, 38, 43, 50, 58, 66, 75, 84, 94, 103, 113, 123, 133, 143,
    153, 163, 173, 183, 193, 203, 213, 223, 233, 243, 253, 263, 273, 283, 293,
    303, 313, 324, 333, 344, 354, 364, 374, 385, 395, 405, 415, 425, 436, 446,
    456, 466, 477, 487, 497, 507, 518, 528, 538, 549, 559, 569, 579, 590, 600,
    610, 621, 631, 641, 651, 662, 672, 683, 693, 703, 713, 724, 734, 744, 754,
    765, 775, 786, 796, 806, 816, 827, 837, 848, 858, 868, 879, 889, 899, 910,
    920, 931, 941, 951, 962, 972, 983, 993, 1003, 1014, 1024, 1035, 1045, 1056,
    1066, 1077, 1087, 1098, 1109, 1119, 1130, 1140, 1150, 1161, 1172, 1183, 1193,
    1204, 1214, 1225, 1236, 1246, 1257, 1268, 1278, 1289, 1300, 1311, 1321, 1332,
    1342, 1353, 1364, 1375, 1386, 1396, 1407, 1418, 1429, 1440, 1451, 1461, 1472,
    1483, 1494, 1505, 1516, 1527, 1537, 1548, 1559, 1570, 1581, 1592, 1603, 1614,
    1625, 1636, 1647, 1658, 1669, 1680, 1691, 1702, 1714, 1725, 1736, 1747, 1758,
    1769, 1780, 1792, 1803, 1814, 1825, 1837, 1848, 1859, 1871, 1882, 1893, 1905,
    1916, 1928, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 16, 19, 21, 25, 28, 32,
    37, 43, 50, 57, 65, 74, 83, 93, 102, 112, 122, 131, 141, 151, 161, 171, 181,
    191, 201, 211, 221, 231, 241, 251, 261, 271, 281, 291, 301, 311, 321, 331,
    342, 352, 362, 372, 382, 392, 402, 412, 422, 433, 443, 453, 463, 474, 484,
    494, 504, 515, 525, 535, 545, 556, 565, 576, 586, 597, 607, 617, 627, 638,
    647, 658, 668, 679, 689, 699, 709, 720, 730, 740, 750, 761, 771, 781, 792,
    802, 812, 822, 832, 843, 853, 864, 874, 884, 895, 905, 915, 926, 936, 946,
    957, 967, 977, 988, 998, 1008, 1019, 1029, 1040, 1050, 1061, 1071, 1082,
    1092, 1103, 1113, 1124, 1134, 1144, 1155, 1165, 1176, 1187, 1197, 1208, 1219,
    1229, 1240, 1250, 1261, 1272, 1282, 1293, 1304, 1314, 1325, 1335, 1346, 1357,
    1368, 1378, 1389, 1400, 1411, 1421, 1432, 1443, 1454, 1464, 1475, 1486, 1497,
    1508, 1519, 1529, 1540, 1551, 1562, 1573, 1584, 1595, 1606, 1617, 1628, 1639,
    1650, 1661, 1672, 1683, 1694, 1705, 1716, 1727, 1738, 1749, 1760, 1771, 1782,
    1794, 1804, 1816, 1827, 1839, 1850, 1861, 1872, 1884, 1895, 1906, 1918, 0, 1,
    2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 16, 19, 21, 24, 28, 32, 37, 42, 49, 56,
    64, 73, 82, 91, 101, 111, 120, 130, 140, 150, 160, 170, 180, 189, 199, 209,
    219, 229, 239, 249, 259, 269, 279, 289, 299, 309, 319, 329, 339, 349, 359,
    369, 380, 390, 400, 410, 420, 430, 440, 450, 461, 471, 481, 491, 501, 511,
    522, 532, 542, 552, 562, 573, 583, 593, 603, 613, 623, 634, 644, 654, 664,
    675, 685, 695, 705, 716, 726, 736, 746, 756, 766, 777, 787, 797, 807, 817,
    828, 838, 849, 859, 869, 879, 890, 900, 910, 920, 931, 941, 951, 962, 972,
    982, 992, 1003, 1013, 1023, 1034, 1044, 1055, 1065, 1076, 1086, 1097, 1107,
    1118, 1128, 1138, 1149, 1159, 1170, 1180, 1191, 1202, 1212, 1223, 1233, 1244,
    1254, 1265, 1275, 1286, 1297, 1307, 1318, 1328, 1339, 1350, 1361, 1371, 1382,
    1393, 1403, 1414, 1425, 1435, 1446, 1457, 1468, 1478, 1489, 1500, 1511, 1521,
    1532, 1543, 1554, 1564, 1576, 1586, 1597, 1608, 1619, 1630, 1641, 1652, 1663,
    1674, 1685, 1696, 1707, 1718, 1729, 1740, 1751, 1762, 1773, 1784, 1795, 1807,
    1818, 1829, 1840, 1852, 1863, 1874, 1885, 1896, 1908, 0, 1, 2, 3, 4, 5, 6, 7,
    8, 9, 11, 12, 14, 16, 18, 21, 24, 28, 32, 37, 42, 48, 56, 64, 72, 81, 90,
    100, 109, 119, 129, 139, 148, 158, 168, 178, 188, 198, 208, 218, 227, 237,
    247, 257, 267, 277, 287, 297, 307, 317, 327, 337, 347, 357, 367, 377, 387,
    397, 407, 417, 427, 437, 448, 458, 468, 478, 488, 498, 508, 518, 529, 539,
    549, 559, 569, 579, 589, 600, 610, 620, 630, 640, 650, 661, 671, 681, 691,
    701, 711, 722, 732, 742, 752, 762, 772, 783, 793, 803, 813, 823, 834, 844,
    854, 864, 874, 885, 895, 905, 915, 926, 936, 946, 956, 967, 977, 987, 998,
    1008, 1018, 1029, 1039, 1049, 1060, 1070, 1080, 1091, 1101, 1112, 1122, 1132,
    1143, 1153, 1164, 1174, 1185, 1195, 1206, 1216, 1227, 1237, 1248, 1258, 1269,
    1279, 1290, 1300, 1311, 1321, 1332, 1343, 1353, 1364, 1375, 1385, 1396, 1406,
    1417, 1428, 1439, 1449, 1460, 1471, 1481, 1492, 1503, 1514, 1524, 1535, 1546,
    1556, 1567, 1578, 1589, 1600, 1611, 1622, 1633, 1643, 1654, 1665, 1676, 1687,
    1698, 1709, 1720, 1731, 1742, 1753, 1764, 1775, 1786, 1798, 1809, 1820, 1831,
    1842, 1853, 1865, 1876, 1887, 1898, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14,
    16, 18, 21, 24, 27, 31, 36, 41, 48, 55, 63, 71, 80, 89, 99, 108, 118, 128,
    137, 147, 157, 167, 176, 186, 196, 206, 216, 226, 236, 245, 255, 265, 275,
    285, 295, 305, 315, 324, 335, 345, 355, 364, 375, 385, 395, 404, 414, 425,
    434, 445, 455, 465, 475, 485, 495, 505, 515, 525, 536, 546, 555, 566, 576,
    586, 596, 606, 616, 626, 636, 647, 657, 667, 677, 687, 697, 707, 718, 728,
    737, 748, 758, 768, 778, 788, 798, 808, 819, 829, 839, 849, 859, 870, 880,
    890, 900, 910, 921, 931, 941, 951, 961, 972, 982, 992, 1002, 1012, 1023,
    1033, 1044, 1054, 1064, 1075, 1085, 1095, 1106, 1116, 1126, 1137, 1147, 1158,
    1168, 1178, 1189, 1199, 1210, 1220, 1231, 1241, 1252, 1262, 1273, 1283, 1293,
    1304, 1314, 1325, 1336, 1346, 1357, 1367, 1378, 1389, 1399, 1410, 1420, 1431,
    1442, 1452, 1463, 1474, 1484, 1495, 1506, 1516, 1527, 1538, 1548, 1559, 1570,
    1581, 1592, 1603, 1613, 1624, 1635, 1646, 1657, 1668, 1679, 1689, 1700, 1711,
    1722, 1733, 1744, 1755, 1766, 1777, 1788, 1799, 1811, 1822, 1833, 1844, 1855,
    1866, 1877, 1889, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 16, 18, 21, 24,
    27, 31, 36, 41, 47, 54, 62, 70, 79, 88, 98, 107, 117, 126, 136, 146, 155,
    165, 175, 185, 195, 204, 214, 224, 234, 244, 253, 263, 273, 283, 293, 302,
    313, 322, 332, 342, 352, 362, 372, 382, 392, 402, 412, 422, 432, 442, 452,
    462, 472, 482, 492, 502, 512, 522, 532, 542, 552, 562, 573, 583, 593, 603,
    612, 623, 633, 643, 653, 663, 673, 683, 693, 703, 713, 724, 733, 744, 753,
    764, 774, 784, 794, 804, 814, 824, 835, 845, 855, 865, 875, 885, 895, 905,
    916, 926, 936, 946, 956, 966, 976, 987, 997, 1007, 1018, 1027, 1038, 1048,
    1059, 1069, 1079, 1089, 1100, 1110, 1120, 1131, 1141, 1151, 1162, 1172, 1183,
    1193, 1203, 1214, 1224, 1235, 1245, 1255, 1266, 1276, 1286, 1297, 1307, 1318,
    1329, 1339, 1350, 1360, 1371, 1381, 1392, 1403, 1413, 1424, 1434, 1445, 1455,
    1466, 1477, 1487, 1498, 1509, 1519, 1530, 1540, 1551, 1562, 1573, 1584, 1594,
    1605, 1616, 1627, 1637, 1648, 1659, 1670, 1681, 1692, 1703, 1713, 1724, 1735,
    1746, 1757, 1768, 1779, 1790, 1801, 1812, 1823, 1835, 1846, 1857, 1868, 1879,
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 16, 18, 21, 23, 27, 31, 35, 40, 47,
    54, 61, 69, 78, 87, 96, 106, 115, 125, 135, 144, 154, 164, 173, 183, 193,
    203, 212, 222, 232, 242, 252, 261, 271, 281, 291, 300, 311, 320, 330, 340,
    350, 360, 370, 380, 390, 399, 409, 419, 429, 439, 449, 459, 469, 479, 489,
    499, 509, 519, 529, 539, 549, 559, 569, 579, 589, 599, 609, 619, 629, 639,
    649, 659, 669, 679, 689, 699, 709, 719, 729, 740, 749, 760, 770, 780, 789,
    799, 809, 820, 830, 840, 850, 860, 870, 880, 890, 900, 911, 921, 931, 941,
    951, 961, 971, 982, 992, 1002, 1012, 1022, 1032, 1043, 1053, 1063, 1073,
    1084, 1094, 1104, 1114, 1125, 1135, 1145, 1156, 1166, 1176, 1187, 1197, 1207,
    1218, 1228, 1238, 1249, 1259, 1270, 1280, 1290, 1301, 1311, 1322, 1332, 1343,
    1353, 1364, 1374, 1384, 1395, 1406, 1416, 1427, 1437, 1448, 1459, 1469, 1480,
    1490, 1501, 1512, 1522, 1532, 1543, 1554, 1565, 1575, 1586, 1597, 1608, 1618,
    1629, 1640, 1651, 1661, 1672, 1683, 1694, 1704, 1716, 1727, 1737, 1748, 1759,
    1770, 1781, 1792, 1803, 1814, 1825, 1836, 1847, 1858, 1870, 0, 1, 2, 3, 4, 5,
    6, 7, 8, 9, 10, 12, 14, 16, 18, 20, 23, 27, 30, 35, 40, 46, 53, 60, 68, 77,
    86, 95, 105, 114, 124, 133, 143, 153, 162, 172, 182, 191, 201, 211, 221, 230,
    240, 250, 260, 269, 279, 289, 298, 308, 318, 328, 338, 348, 357, 367, 377,
    387, 397, 406, 417, 426, 437, 446, 456, 466, 476, 486, 496, 506, 516, 526,
    536, 546, 556, 566, 576, 586, 596, 605, 616, 625, 636, 646, 656, 666, 676,
    685, 695, 705, 715, 725, 735, 745, 755, 765, 775, 785, 795, 805, 815, 825,
    835, 845, 855, 865, 875, 886, 896, 906, 916, 926, 936, 946, 956, 966, 976,
    986, 996, 1007, 1016, 1027, 1037, 1047, 1057, 1068, 1078, 1088, 1098, 1108,
    1119, 1129, 1139, 1150, 1160, 1170, 1180, 1191, 1201, 1211, 1222, 1232, 1242,
    1253, 1263, 1273, 1283, 1294, 1305, 1315, 1325, 1336, 1346, 1357, 1367, 1377,
    1388, 1399, 1409, 1420, 1430, 1441, 1451, 1462, 1472, 1483, 1493, 1504, 1514,
    1525, 1536, 1546, 1557, 1567, 1578, 1589, 1599, 1610, 1621, 1632, 1642, 1653,
    1664, 1675, 1685, 1696, 1707, 1718, 1729, 1740, 1750, 1761, 1772, 1783, 1794,
    1805, 1816, 1827, 1838, 1849, 1860, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 14,
    16, 18, 20, 23, 26, 30, 34, 39, 45, 52, 60, 67, 76, 85, 94, 104, 113, 123,
    132, 142, 151, 161, 171, 180, 190, 199, 209, 219, 229, 238, 248, 258, 267,
    277, 287, 296, 306, 316, 326, 336, 345, 355, 365, 375, 385, 394, 404, 414,
    424, 434, 444, 454, 463, 473, 483, 493, 503, 513, 523, 533, 542, 553, 562,
    572, 582, 592, 602, 612, 622, 632, 642, 652, 662, 672, 681, 692, 701, 711,
    721, 731, 741, 751, 761, 771, 781, 791, 801, 811, 821, 831, 841, 851, 861,
    871, 881, 891, 901, 911, 921, 931, 941, 951, 961, 971, 981, 991, 1001, 1011,
    1021, 1032, 1042, 1052, 1062, 1072, 1082, 1093, 1102, 1113, 1123, 1133, 1144,
    1154, 1164, 1174, 1184, 1195, 1205, 1215, 1226, 1236, 1246, 1257, 1266, 1277,
    1287, 1298, 1308, 1319, 1329, 1339, 1350, 1360, 1370, 1381, 1391, 1402, 1412,
    1423, 1433, 1444, 1454, 1465, 1475, 1486, 1496, 1507, 1517, 1528, 1538, 1549,
    1559, 1570, 1581, 1591, 1602, 1613, 1623, 1634, 1645, 1655, 1666, 1677, 1687,
    1698, 1709, 1720, 1731, 1741, 1752, 1763, 1774, 1785, 1796, 1807, 1818, 1829,
    1840, 1851, 0, 1, 2, 3, 4, 4, 5, 7, 8, 9, 10, 12, 14, 15, 18, 20, 23, 26, 30,
    34, 39, 45, 52, 59, 67, 75, 84, 93, 103, 112, 121, 131, 140, 150, 159, 169,
    179, 188, 198, 208, 217, 227, 236, 246, 256, 265, 275, 285, 294, 304, 314,
    324, 333, 343, 353, 363, 372, 382, 392, 401, 411, 421, 431, 441, 451, 460,
    470, 480, 490, 500, 510, 520, 530, 539, 549, 559, 569, 579, 589, 598, 609,
    618, 628, 638, 648, 658, 668, 677, 688, 698, 707, 717, 727, 737, 747, 757,
    767, 776, 786, 796, 806, 816, 826, 836, 846, 856, 866, 876, 886, 896, 906,
    916, 926, 936, 946, 955, 966, 976, 986, 996, 1006, 1016, 1026, 1036, 1046,
    1056, 1067, 1077, 1087, 1097, 1107, 1117, 1127, 1138, 1148, 1158, 1168, 1178,
    1189, 1199, 1209, 1219, 1229, 1240, 1250, 1260, 1270, 1281, 1291, 1301, 1312,
    1322, 1332, 1343, 1353, 1363, 1374, 1384, 1395, 1405, 1415, 1426, 1436, 1447,
    1457, 1468, 1478, 1488, 1499, 1509, 1520, 1530, 1541, 1552, 1562, 1573, 1583,
    1594, 1604, 1615, 1626, 1636, 1647, 1658, 1668, 1679, 1690, 1701, 1711, 1722,
    1732, 1744, 1754, 1765, 1776, 1787, 1798, 1809, 1820, 1830, 1842, 0, 1, 2, 3,
    4, 4, 5, 7, 8, 9, 10, 12, 13, 15, 17, 20, 23, 26, 29, 34, 38, 44, 51, 58, 66,
    74, 83, 92, 101, 111, 120, 130, 139, 149, 158, 168, 177, 187, 196, 206, 215,
    225, 235, 244, 254, 264, 273, 283, 292, 302, 311, 321, 331, 341, 350, 360,
    370, 380, 389, 399, 409, 418, 428, 438, 448, 457, 468, 477, 487, 497, 507,
    517, 526, 536, 546, 556, 566, 576, 585, 595, 605, 615, 625, 635, 644, 654,
    664, 674, 684, 694, 703, 713, 723, 733, 743, 753, 763, 772, 782, 792, 802,
    812, 822, 832, 842, 851, 861, 871, 881, 891, 901, 911, 921, 931, 941, 950,
    961, 971, 980, 991, 1000, 1011, 1021, 1031, 1041, 1051, 1061, 1071, 1081,
    1091, 1101, 1111, 1121, 1132, 1142, 1152, 1162, 1172, 1182, 1193, 1203, 1213,
    1223, 1233, 1244, 1253, 1264, 1274, 1285, 1295, 1305, 1315, 1326, 1336, 1346,
    1356, 1367, 1377, 1387, 1398, 1408, 1419, 1429, 1439, 1450, 1460, 1471, 1481,
    1491, 1501, 1512, 1523, 1533, 1544, 1554, 1565, 1575, 1586, 1596, 1607, 1617,
    1628, 1639, 1649, 1660, 1670, 1681, 1692, 1703, 1713, 1724, 1735, 1746, 1756,
    1767, 1778, 1789, 1800, 1811, 1821, 1832, 0, 1, 2, 3, 4, 4, 5, 6, 8, 9, 10,
    12, 13, 15, 17, 20, 22, 26, 29, 33, 38, 44, 50, 57, 65, 74, 82, 91, 100, 110,
    119, 128, 138, 147, 157, 166, 176, 185, 195, 204, 214, 223, 233, 242, 252,
    262, 271, 281, 290, 300, 309, 319, 329, 339, 348, 358, 368, 377, 387, 396,
    406, 416, 426, 436, 445, 455, 465, 474, 484, 494, 504, 514, 523, 533, 543,
    553, 562, 572, 582, 592, 602, 611, 621, 631, 641, 651, 660, 670, 680, 690,
    700, 709, 719, 729, 739, 749, 758, 768, 778, 788, 798, 807, 817, 827, 837,
    847, 857, 867, 876, 886, 896, 906, 916, 926, 936, 945, 956, 966, 975, 985,
    995, 1005, 1015, 1025, 1035, 1045, 1055, 1065, 1075, 1085, 1095, 1105, 1116,
    1126, 1136, 1146, 1156, 1166, 1176, 1186, 1196, 1207, 1217, 1227, 1237, 1247,
    1257, 1267, 1278, 1288, 1298, 1309, 1319, 1329, 1339, 1349, 1360, 1370, 1380,
    1391, 1401, 1411, 1422, 1432, 1442, 1453, 1463, 1473, 1484, 1494, 1505, 1515,
    1525, 1536, 1546, 1557, 1567, 1578, 1588, 1599, 1609, 1620, 1630, 1641, 1652,
    1662, 1673, 1684, 1694, 1705, 1715, 1726, 1737, 1748, 1758, 1769, 1780, 1791,
    1802, 1812, 1823, 0, 1, 2, 3, 4, 4, 5, 6, 8, 9, 10, 12, 13, 15, 17, 20, 22,
    25, 29, 33, 38, 43, 50, 57, 64, 73, 81, 90, 99, 108, 118, 127, 136, 146, 155,
    165, 174, 184, 193, 203, 212, 222, 231, 241, 250, 260, 269, 279, 288, 298,
    307, 317, 327, 336, 346, 356, 365, 375, 384, 394, 404, 413, 423, 433, 443,
    452, 462, 471, 481, 491, 501, 511, 520, 530, 540, 549, 559, 569, 579, 588,
    598, 608, 618, 627, 637, 647, 657, 666, 676, 686, 696, 705, 715, 725, 735,
    744, 754, 764, 773, 783, 793, 803, 813, 823, 832, 842, 852, 862, 872, 882,
    891, 901, 911, 921, 931, 940, 951, 960, 970, 980, 990, 1000, 1010, 1020,
    1030, 1040, 1050, 1060, 1070, 1079, 1090, 1099, 1110, 1120, 1130, 1140, 1150,
    1160, 1170, 1180, 1190, 1200, 1211, 1221, 1231, 1241, 1251, 1261, 1271, 1282,
    1292, 1302, 1312, 1322, 1332, 1342, 1353, 1363, 1373, 1384, 1394, 1404, 1414,
    1425, 1435, 1445, 1456, 1466, 1476, 1486, 1497, 1507, 1518, 1528, 1539, 1549,
    1559, 1570, 1580, 1591, 1601, 1612, 1622, 1633, 1643, 1654, 1665, 1675, 1686,
    1696, 1707, 1718, 1728, 1739, 1750, 1760, 1771, 1782, 1793, 1803, 1814, 0, 1,
    2, 3, 4, 4, 5, 6, 8, 9, 10, 12, 13, 15, 17, 19, 22, 25, 29, 33, 37, 43, 49,
    56, 63, 72, 80, 89, 98, 107, 117, 126, 135, 145, 154, 163, 173, 182, 192,
    201, 211, 220, 229, 239, 248, 258, 267, 277, 286, 296, 305, 315, 325, 334,
    343, 353, 363, 372, 382, 391, 401, 411, 421, 430, 440, 449, 459, 468, 478,
    488, 498, 508, 517, 527, 537, 546, 556, 566, 575, 585, 595, 604, 614, 624,
    634, 643, 653, 662, 672, 682, 692, 701, 711, 721, 731, 740, 750, 759, 769,
    779, 789, 799, 808, 818, 828, 838, 848, 857, 867, 877, 887, 896, 906, 916,
    926, 935, 946, 955, 965, 975, 985, 995, 1005, 1015, 1024, 1034, 1044, 1054,
    1064, 1074, 1084, 1094, 1104, 1114, 1124, 1134, 1144, 1154, 1164, 1174, 1184,
    1194, 1204, 1214, 1224, 1234, 1244, 1254, 1265, 1275, 1285, 1295, 1305, 1316,
    1326, 1336, 1346, 1356, 1366, 1377, 1387, 1397, 1407, 1418, 1428, 1438, 1448,
    1459, 1469, 1479, 1489, 1500, 1510, 1521, 1531, 1541, 1552, 1562, 1572, 1583,
    1593, 1604, 1614, 1625, 1635, 1645, 1656, 1667, 1677, 1688, 1698, 1709, 1720,
    1730, 1741, 1752, 1762, 1773, 1784, 1794, 1805, 0, 1, 2, 3, 4, 4, 5, 6, 8, 9,
    10, 12, 13, 15, 17, 19, 22, 25, 28, 32, 37, 42, 48, 55, 62, 71, 79, 88, 97,
    106, 115, 125, 134, 143, 153, 162, 171, 181, 190, 200, 209, 218, 228, 237,
    247, 256, 266, 275, 284, 294, 303, 313, 322, 332, 341, 351, 361, 370, 379,
    389, 399, 408, 418, 428, 437, 446, 456, 466, 476, 485, 495, 505, 514, 524,
    533, 543, 553, 562, 572, 581, 591, 601, 611, 620, 630, 640, 649, 659, 669,
    678, 688, 697, 707, 717, 727, 736, 746, 755, 765, 775, 785, 794, 804, 814,
    824, 833, 843, 853, 862, 872, 882, 892, 901, 911, 921, 930, 941, 950, 960,
    970, 979, 990, 999, 1009, 1019, 1029, 1039, 1049, 1059, 1068, 1078, 1088,
    1098, 1108, 1118, 1128, 1138, 1148, 1158, 1168, 1178, 1188, 1198, 1208, 1218,
    1228, 1238, 1248, 1258, 1268, 1279, 1289, 1299, 1309, 1319, 1329, 1339, 1349,
    1360, 1370, 1380, 1390, 1400, 1410, 1421, 1431, 1441, 1451, 1462, 1471, 1482,
    1492, 1503, 1513, 1523, 1534, 1544, 1554, 1565, 1575, 1585, 1596, 1606, 1617,
    1627, 1637, 1648, 1658, 1669, 1680, 1690, 1701, 1711, 1722, 1732, 1743, 1754,
    1764, 1775, 1785, 1796, 0, 1, 2, 3, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, 17, 19,
    22, 25, 28, 32, 36, 42, 48, 55, 62, 70, 78, 87, 96, 105, 114, 123, 133, 142,
    151, 161, 170, 179, 189, 198, 207, 217, 226, 235, 245, 254, 264, 273, 282,
    292, 301, 311, 320, 330, 339, 349, 358, 368, 377, 386, 396, 406, 415, 425,
    435, 444, 454, 463, 473, 482, 492, 502, 511, 520, 530, 540, 550, 559, 569,
    578, 588, 597, 607, 617, 626, 636, 646, 655, 665, 675, 684, 693, 703, 713,
    723, 732, 742, 751, 761, 771, 780, 790, 800, 809, 819, 829, 838, 848, 858,
    868, 877, 887, 897, 906, 916, 925, 936, 945, 955, 965, 974, 984, 994, 1004,
    1014, 1024, 1033, 1043, 1053, 1063, 1073, 1082, 1093, 1103, 1112, 1122, 1132,
    1142, 1152, 1162, 1172, 1182, 1192, 1202, 1212, 1222, 1232, 1242, 1252, 1262,
    1272, 1282, 1292, 1302, 1312, 1322, 1332, 1343, 1353, 1363, 1373, 1383, 1393,
    1403, 1413, 1424, 1434, 1444, 1454, 1464, 1475, 1485, 1495, 1505, 1516, 1526,
    1536, 1546, 1557, 1567, 1577, 1588, 1598, 1609, 1619, 1629, 1640, 1650, 1661,
    1671, 1681, 1692, 1703, 1713, 1724, 1734, 1745, 1756, 1766, 1777, 1788, 0, 1,
    2, 3, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, 17, 19, 22, 24, 28, 32, 36, 41, 47,
    54, 61, 69, 77, 86, 95, 104, 113, 122, 131, 141, 150, 159, 168, 178, 187,
    196, 206, 215, 224, 234, 243, 252, 262, 271, 280, 290, 299, 309, 318, 328,
    337, 346, 356, 365, 375, 384, 394, 403, 413, 422, 432, 441, 451, 460, 470,
    480, 489, 499, 508, 517, 527, 537, 546, 556, 566, 575, 585, 594, 604, 613,
    623, 633, 642, 651, 661, 671, 680, 690, 700, 709, 719, 728, 738, 747, 757,
    766, 776, 786, 796, 805, 815, 824, 834, 844, 853, 863, 873, 882, 892, 902,
    911, 921, 931, 940, 950, 960, 969, 979, 989, 999, 1009, 1018, 1028, 1038,
    1048, 1057, 1067, 1077, 1087, 1097, 1107, 1117, 1126, 1136, 1146, 1156, 1166,
    1176, 1186, 1196, 1206, 1215, 1225, 1235, 1246, 1256, 1266, 1276, 1286, 1296,
    1306, 1315, 1326, 1336, 1346, 1356, 1366, 1376, 1386, 1396, 1406, 1416, 1427,
    1437, 1447, 1457, 1467, 1477, 1488, 1498, 1508, 1518, 1529, 1539, 1549, 1559,
    1570, 1580, 1590, 1601, 1611, 1621, 1632, 1642, 1653, 1663, 1673, 1684, 1694,
    1705, 1715, 1726, 1736, 1747, 1758, 1768, 1779, 0, 1, 2, 3, 3, 4, 5, 6, 7, 9,
    10, 11, 13, 15, 17, 19, 21, 24, 27, 31, 36, 41, 47, 53, 60, 68, 77, 85, 94,
    103, 112, 121, 130, 139, 149, 158, 167, 176, 186, 195, 204, 213, 223, 232,
    241, 251, 260, 269, 278, 288, 297, 307, 316, 325, 335, 344, 354, 363, 372,
    382, 391, 401, 410, 420, 429, 438, 448, 457, 467, 477, 486, 496, 505, 514,
    524, 534, 543, 553, 562, 572, 581, 591, 600, 610, 619, 629, 639, 648, 658,
    667, 677, 686, 696, 705, 715, 724, 734, 743, 753, 762, 772, 782, 791, 801,
    810, 820, 830, 839, 849, 858, 868, 878, 887, 897, 906, 916, 926, 935, 945,
    955, 964, 974, 984, 994, 1003, 1013, 1023, 1033, 1042, 1052, 1062, 1071,
    1081, 1091, 1101, 1111, 1121, 1131, 1140, 1150, 1160, 1170, 1180, 1190, 1200,
    1209, 1219, 1229, 1239, 1249, 1259, 1269, 1279, 1289, 1299, 1309, 1319, 1329,
    1339, 1349, 1359, 1369, 1379, 1389, 1399, 1409, 1420, 1430, 1440, 1449, 1460,
    1470, 1480, 1490, 1501, 1511, 1521, 1531, 1541, 1552, 1562, 1572, 1582, 1593,
    1603, 1613, 1624, 1634, 1644, 1655, 1665, 1676, 1686, 1696, 1707, 1717, 1728,
    1738, 1749, 1759, 1770, 0, 1, 2, 3, 3, 4, 5, 6, 7, 9, 10, 11, 13, 15, 17, 19,
    21, 24, 27, 31, 35, 40, 46, 53, 59, 67, 76, 84, 93, 102, 111, 120, 129, 138,
    147, 156, 166, 175, 184, 193, 203, 212, 221, 230, 240, 249, 258, 267, 276,
    286, 295, 305, 314, 323, 332, 342, 351, 361, 370, 379, 389, 398, 408, 417,
    427, 436, 446, 455, 464, 474, 483, 493, 502, 511, 521, 531, 540, 550, 559,
    568, 578, 587, 597, 607, 616, 626, 635, 644, 654, 663, 673, 682, 692, 701,
    711, 720, 730, 739, 749, 758, 768, 777, 787, 797, 806, 816, 825, 835, 844,
    854, 863, 873, 883, 892, 902, 911, 921, 931, 940, 950, 959, 969, 979, 988,
    998, 1008, 1018, 1027, 1037, 1046, 1056, 1066, 1076, 1086, 1095, 1105, 1115,
    1125, 1135, 1144, 1154, 1164, 1174, 1184, 1194, 1203, 1213, 1223, 1233, 1243,
    1253, 1263, 1273, 1283, 1293, 1302, 1312, 1322, 1332, 1342, 1352, 1362, 1372,
    1382, 1392, 1402, 1412, 1422, 1433, 1442, 1453, 1463, 1473, 1483, 1493, 1503,
    1513, 1524, 1534, 1544, 1554, 1564, 1575, 1585, 1595, 1605, 1616, 1626, 1636,
    1647, 1657, 1667, 1678, 1688, 1699, 1709, 1719, 1730, 1740, 1751, 1761, 0, 1,
    2, 3, 3, 4, 5, 6, 7, 9, 10, 11, 13, 14, 16, 19, 21, 24, 27, 31, 35, 40, 45,
    52, 59, 67, 75, 83, 92, 101, 110, 119, 128, 137, 146, 155, 164, 173, 183,
    192, 201, 210, 219, 229, 238, 247, 256, 266, 275, 284, 293, 303, 312, 321,
    330, 340, 349, 359, 368, 377, 387, 396, 405, 415, 424, 433, 443, 452, 462,
    471, 481, 490, 499, 508, 518, 528, 537, 547, 556, 565, 575, 584, 594, 603,
    613, 622, 631, 641, 650, 660, 669, 678, 688, 697, 707, 717, 726, 735, 745,
    754, 764, 773, 783, 792, 802, 811, 821, 830, 840, 849, 859, 868, 878, 887,
    897, 906, 916, 926, 935, 945, 954, 964, 974, 983, 993, 1003, 1012, 1022,
    1032, 1041, 1051, 1060, 1070, 1080, 1090, 1100, 1109, 1119, 1129, 1139, 1148,
    1158, 1168, 1178, 1188, 1197, 1207, 1217, 1227, 1237, 1247, 1256, 1266, 1276,
    1286, 1296, 1306, 1316, 1326, 1336, 1346, 1356, 1366, 1375, 1385, 1395, 1405,
    1415, 1425, 1435, 1446, 1456, 1466, 1476, 1486, 1496, 1506, 1516, 1526, 1536,
    1547, 1557, 1567, 1577, 1587, 1597, 1608, 1618, 1628, 1639, 1649, 1659, 1670,
    1680, 1690, 1701, 1711, 1722, 1732, 1742, 1753, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8,
    10, 11, 13, 14, 16, 18, 21, 24, 27, 30, 34, 39, 45, 51, 58, 66, 74, 82, 91,
    100, 108, 117, 126, 136, 145, 154, 163, 172, 181, 190, 199, 209, 218, 227,
    236, 245, 255, 264, 273, 282, 291, 301, 310, 319, 328, 338, 347, 356, 365,
    375, 384, 393, 403, 412, 422, 431, 440, 449, 459, 468, 478, 487, 496, 506,
    515, 525, 534, 543, 553, 562, 572, 581, 590, 600, 609, 619, 628, 637, 647,
    656, 666, 675, 684, 694, 703, 713, 722, 731, 741, 750, 760, 769, 779, 788,
    798, 807, 816, 826, 835, 845, 854, 864, 873, 883, 892, 902, 911, 921, 930,
    940, 949, 959, 969, 978, 988, 997, 1007, 1017, 1026, 1036, 1046, 1055, 1065,
    1075, 1084, 1094, 1104, 1113, 1123, 1133, 1143, 1152, 1162, 1172, 1182, 1191,
    1201, 1211, 1221, 1231, 1240, 1250, 1260, 1270, 1280, 1289, 1299, 1309, 1319,
    1329, 1339, 1349, 1359, 1369, 1379, 1389, 1398, 1408, 1418, 1428, 1438, 1448,
    1458, 1468, 1478, 1488, 1499, 1509, 1519, 1529, 1539, 1549, 1559, 1569, 1580,
    1589, 1600, 1610, 1620, 1631, 1641, 1651, 1661, 1672, 1682, 1692, 1703, 1713,
    1724, 1734, 1744, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 16, 18, 21,
    23, 26, 30, 34, 39, 44, 51, 57, 65, 73, 81, 90, 99, 107, 116, 125, 134, 143,
    152, 161, 171, 180, 189, 198, 207, 216, 225, 234, 244, 253, 262, 271, 280,
    289, 299, 308, 317, 326, 336, 345, 354, 363, 372, 382, 391, 400, 410, 419,
    428, 438, 447, 456, 466, 475, 484, 494, 503, 512, 522, 531, 540, 550, 559,
    568, 577, 587, 596, 606, 615, 625, 634, 643, 653, 662, 671, 681, 690, 699,
    709, 718, 727, 737, 746, 756, 765, 775, 784, 793, 803, 812, 822, 831, 840,
    850, 859, 869, 878, 888, 897, 907, 916, 925, 935, 944, 954, 964, 973, 983,
    992, 1002, 1012, 1021, 1030, 1040, 1050, 1060, 1069, 1079, 1088, 1098, 1108,
    1117, 1127, 1137, 1147, 1156, 1166, 1176, 1185, 1195, 1205, 1215, 1224, 1234,
    1244, 1254, 1263, 1273, 1283, 1293, 1303, 1313, 1322, 1332, 1342, 1352, 1362,
    1372, 1382, 1392, 1401, 1411, 1421, 1431, 1441, 1451, 1461, 1471, 1481, 1491,
    1501, 1511, 1521, 1531, 1541, 1552, 1562, 1572, 1582, 1592, 1602, 1612, 1623,
    1633, 1643, 1653, 1664, 1674, 1684, 1694, 1705, 1715, 1725, 1736, 0, 1, 2, 3,
    3, 4, 5, 6, 7, 8, 10, 11, 13, 14, 16, 18, 20, 23, 26, 30, 34, 39, 44, 50, 57,
    64, 72, 80, 89, 97, 106, 115, 124, 133, 142, 151, 160, 169, 178, 187, 196,
    205, 215, 224, 233, 242, 251, 260, 269, 278, 287, 297, 306, 315, 324, 333,
    343, 352, 361, 370, 379, 388, 398, 407, 416, 425, 435, 444, 454, 463, 472,
    481, 491, 500, 509, 519, 528, 537, 547, 556, 565, 574, 584, 593, 602, 612,
    621, 630, 640, 649, 658, 667, 677, 686, 696, 705, 714, 723, 733, 742, 752,
    761, 770, 780, 789, 798, 808, 817, 827, 836, 845, 855, 864, 874, 883, 892,
    902, 911, 921, 930, 939, 949, 959, 968, 978, 987, 997, 1006, 1016, 1025,
    1035, 1044, 1054, 1064, 1073, 1083, 1093, 1102, 1112, 1121, 1131, 1141, 1150,
    1160, 1170, 1179, 1189, 1199, 1209, 1218, 1228, 1238, 1247, 1257, 1267, 1276,
    1286, 1296, 1306, 1316, 1326, 1335, 1345, 1355, 1365, 1375, 1385, 1395, 1404,
    1414, 1424, 1434, 1444, 1454, 1464, 1474, 1484, 1494, 1504, 1514, 1524, 1534,
    1544, 1554, 1564, 1574, 1584, 1594, 1605, 1615, 1625, 1635, 1645, 1656, 1666,
    1676, 1686, 1697, 1707, 1717, 1728, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 10, 11, 12,
    14, 16, 18, 20, 23, 26, 29, 33, 38, 43, 49, 56, 63, 71, 79, 88, 96, 105, 114,
    123, 132, 141, 150, 159, 168, 177, 186, 195, 204, 213, 222, 231, 240, 249,
    258, 267, 276, 285, 295, 304, 313, 322, 331, 340, 350, 358, 368, 377, 386,
    395, 405, 414, 423, 432, 441, 451, 460, 469, 479, 488, 497, 506, 516, 525,
    534, 543, 552, 562, 571, 581, 590, 599, 608, 618, 627, 636, 646, 655, 664,
    673, 682, 692, 701, 711, 719, 729, 738, 748, 757, 766, 776, 785, 794, 804,
    813, 822, 832, 841, 850, 860, 869, 878, 888, 897, 907, 916, 926, 935, 944,
    954, 963, 973, 982, 992, 1001, 1011, 1020, 1030, 1039, 1049, 1058, 1068,
    1077, 1087, 1097, 1106, 1116, 1125, 1135, 1145, 1154, 1164, 1173, 1183, 1193,
    1203, 1212, 1222, 1232, 1241, 1251, 1261, 1270, 1280, 1290, 1300, 1309, 1319,
    1329, 1339, 1348, 1358, 1368, 1378, 1388, 1398, 1407, 1417, 1427, 1437, 1447,
    1457, 1467, 1477, 1487, 1497, 1507, 1516, 1526, 1536, 1547, 1557, 1566, 1577,
    1587, 1597, 1607, 1617, 1627, 1637, 1648, 1658, 1668, 1678, 1688, 1699, 1709,
    1719, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 10, 11, 12, 14, 16, 18, 20, 23, 26, 29,
    33, 38, 43, 49, 55, 63, 70, 78, 87, 95, 104, 113, 122, 131, 140, 148, 157,
    166, 175, 184, 193, 202, 211, 220, 229, 238, 247, 256, 265, 275, 283, 293,
    302, 311, 320, 329, 338, 347, 356, 365, 375, 384, 393, 402, 411, 420, 430,
    439, 448, 457, 467, 476, 485, 494, 504, 513, 522, 531, 540, 549, 559, 568,
    577, 587, 596, 605, 614, 623, 633, 642, 651, 660, 670, 679, 688, 697, 707,
    716, 725, 734, 744, 753, 762, 772, 781, 790, 799, 809, 818, 827, 837, 846,
    855, 865, 874, 883, 893, 902, 911, 921, 930, 940, 949, 958, 968, 977, 987,
    996, 1006, 1015, 1025, 1034, 1043, 1053, 1063, 1072, 1082, 1091, 1101, 1110,
    1120, 1129, 1139, 1148, 1158, 1167, 1177, 1187, 1197, 1206, 1216, 1225, 1235,
    1245, 1254, 1264, 1274, 1283, 1293, 1303, 1313, 1322, 1332, 1342, 1352, 1361,
    1371, 1381, 1391, 1400, 1410, 1420, 1430, 1440, 1450, 1460, 1469, 1479, 1489,
    1499, 1509, 1519, 1529, 1539, 1549, 1559, 1569, 1579, 1589, 1599, 1609, 1619,
    1629, 1640, 1650, 1660, 1670, 1680, 1690, 1700, 1711, 0, 1, 2, 3, 3, 4, 5, 6,
    7, 8, 9, 11, 12, 14, 16, 18, 20, 23, 26, 29, 33, 37, 42, 48, 54, 62, 69, 78,
    86, 94, 103, 112, 121, 129, 138, 147, 156, 165, 174, 183, 192, 201, 210, 219,
    228, 237, 246, 255, 263, 273, 281, 291, 300, 309, 318, 327, 336, 345, 354,
    363, 372, 381, 391, 400, 409, 418, 427, 436, 446, 455, 464, 473, 482, 491,
    501, 510, 519, 528, 537, 546, 556, 565, 574, 583, 593, 602, 611, 620, 629,
    639, 648, 657, 666, 675, 685, 694, 703, 712, 721, 730, 740, 749, 758, 767,
    777, 786, 795, 804, 814, 823, 832, 842, 851, 860, 869, 878, 888, 897, 906,
    916, 925, 935, 944, 953, 963, 972, 982, 991, 1000, 1010, 1019, 1028, 1038,
    1048, 1057, 1067, 1076, 1086, 1095, 1105, 1114, 1124, 1133, 1143, 1152, 1162,
    1171, 1181, 1191, 1200, 1210, 1219, 1229, 1239, 1248, 1258, 1268, 1277, 1287,
    1296, 1306, 1316, 1326, 1335, 1345, 1355, 1364, 1374, 1384, 1393, 1403, 1413,
    1423, 1433, 1443, 1452, 1462, 1472, 1482, 1492, 1502, 1512, 1522, 1532, 1541,
    1551, 1561, 1571, 1581, 1591, 1601, 1612, 1622, 1632, 1642, 1652, 1662, 1672,
    1682, 1692, 1703, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 16, 18, 20,
    22, 25, 29, 32, 37, 42, 48, 54, 61, 69, 77, 85, 93, 102, 111, 119, 128, 137,
    146, 155, 164, 173, 181, 190, 199, 208, 217, 226, 235, 244, 253, 262, 271,
    280, 289, 298, 307, 316, 325, 334, 343, 352, 361, 370, 379, 388, 397, 407,
    415, 425, 434, 443, 452, 461, 470, 479, 488, 498, 507, 516, 525, 534, 543,
    553, 562, 571, 580, 589, 598, 608, 616, 626, 635, 644, 653, 663, 671, 681,
    690, 699, 708, 717, 726, 736, 745, 754, 763, 773, 782, 791, 800, 809, 819,
    828, 837, 846, 856, 865, 874, 883, 893, 902, 911, 920, 930, 939, 949, 958,
    967, 977, 986, 995, 1004, 1014, 1023, 1033, 1042, 1052, 1061, 1071, 1080,
    1090, 1099, 1109, 1118, 1128, 1137, 1147, 1156, 1165, 1175, 1185, 1194, 1204,
    1213, 1223, 1232, 1242, 1251, 1261, 1271, 1281, 1290, 1300, 1309, 1319, 1329,
    1338, 1348, 1358, 1367, 1377, 1387, 1397, 1406, 1416, 1426, 1436, 1445, 1455,
    1465, 1475, 1485, 1495, 1504, 1514, 1524, 1534, 1544, 1554, 1564, 1574, 1584,
    1593, 1604, 1614, 1624, 1634, 1644, 1654, 1664, 1674, 1684, 1695, 0, 1, 2, 3,
    3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 16, 17, 20, 22, 25, 28, 32, 36, 41, 47, 53,
    60, 68, 76, 84, 92, 101, 110, 118, 127, 136, 145, 153, 162, 171, 180, 189,
    198, 207, 216, 224, 233, 242, 251, 260, 269, 278, 287, 296, 305, 314, 323,
    332, 341, 350, 359, 368, 377, 386, 395, 404, 413, 422, 431, 440, 449, 459,
    468, 477, 486, 495, 504, 513, 522, 531, 540, 550, 558, 568, 577, 586, 595,
    604, 613, 622, 632, 641, 650, 659, 668, 677, 686, 695, 704, 713, 723, 732,
    741, 750, 759, 769, 778, 787, 796, 805, 814, 824, 833, 842, 851, 860, 869,
    879, 888, 897, 907, 916, 925, 935, 944, 953, 962, 972, 981, 990, 999, 1009,
    1018, 1028, 1037, 1047, 1056, 1065, 1075, 1084, 1094, 1103, 1113, 1122, 1131,
    1141, 1150, 1160, 1169, 1179, 1188, 1198, 1207, 1217, 1226, 1236, 1245, 1255,
    1265, 1274, 1284, 1293, 1303, 1313, 1322, 1332, 1342, 1351, 1361, 1370, 1380,
    1390, 1400, 1409, 1419, 1429, 1438, 1448, 1458, 1468, 1478, 1487, 1497, 1507,
    1517, 1527, 1536, 1546, 1556, 1566, 1576, 1586, 1596, 1606, 1616, 1626, 1636,
    1646, 1656, 1666, 1676, 1687, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14,
    15, 17, 20, 22, 25, 28, 32, 36, 41, 46, 53, 60, 67, 75, 83, 91, 100, 108,
    117, 126, 135, 143, 152, 161, 170, 179, 187, 196, 205, 214, 223, 232, 241,
    249, 258, 267, 276, 285, 294, 303, 312, 321, 330, 339, 347, 356, 366, 374,
    384, 393, 402, 410, 420, 428, 438, 447, 456, 465, 474, 483, 492, 501, 510,
    519, 528, 537, 547, 555, 565, 574, 583, 592, 601, 610, 619, 628, 637, 646,
    655, 664, 674, 683, 692, 701, 710, 719, 728, 737, 746, 755, 765, 774, 783,
    792, 801, 810, 819, 829, 838, 847, 856, 865, 874, 884, 893, 902, 911, 921,
    930, 939, 948, 958, 967, 976, 985, 994, 1004, 1013, 1023, 1032, 1041, 1051,
    1060, 1069, 1079, 1088, 1098, 1107, 1116, 1126, 1135, 1144, 1154, 1163, 1173,
    1182, 1192, 1201, 1211, 1220, 1230, 1239, 1249, 1258, 1268, 1278, 1287, 1297,
    1306, 1316, 1325, 1335, 1345, 1354, 1364, 1373, 1383, 1393, 1402, 1412, 1422,
    1431, 1441, 1451, 1461, 1470, 1480, 1490, 1500, 1510, 1519, 1529, 1539, 1549,
    1559, 1569, 1578, 1588, 1598, 1608, 1618, 1628, 1638, 1648, 1658, 1668, 1678,
    0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 9, 11, 12, 14, 15, 17, 19, 22, 25, 28, 31, 36,
    40, 46, 52, 59, 66, 74, 82, 90, 99, 107, 116, 125, 133, 142, 151, 160, 168,
    177, 186, 195, 204, 212, 221, 230, 239, 248, 256, 265, 274, 283, 292, 301,
    310, 319, 328, 337, 345, 354, 363, 372, 381, 390, 399, 408, 417, 426, 435,
    444, 453, 462, 471, 480, 489, 498, 507, 516, 525, 534, 543, 552, 562, 571,
    580, 589, 598, 606, 616, 625, 634, 643, 652, 661, 670, 679, 688, 697, 706,
    715, 724, 733, 742, 752, 761, 770, 779, 788, 797, 806, 815, 824, 833, 843,
    852, 861, 870, 879, 888, 897, 906, 916, 925, 934, 943, 953, 962, 971, 980,
    989, 999, 1008, 1018, 1027, 1036, 1045, 1055, 1064, 1073, 1083, 1092, 1102,
    1111, 1120, 1130, 1139, 1148, 1158, 1167, 1177, 1186, 1196, 1205, 1214, 1224,
    1233, 1243, 1252, 1262, 1271, 1281, 1290, 1300, 1309, 1319, 1329, 1338, 1348,
    1357, 1366, 1376, 1386, 1396, 1405, 1415, 1425, 1434, 1444, 1454, 1463, 1473,
    1483, 1493, 1502, 1512, 1522, 1532, 1541, 1551, 1561, 1571, 1581, 1591, 1601,
    1611, 1621, 1631, 1640, 1650, 1660, 1671, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 9,
    11, 12, 14, 15, 17, 19, 22, 24, 28, 31, 35, 40, 45, 51, 58, 65, 73, 81, 89,
    98, 106, 115, 124, 132, 141, 150, 158, 167, 176, 185, 193, 202, 211, 220,
    228, 237, 246, 255, 264, 272, 281, 290, 299, 308, 317, 326, 334, 343, 352,
    361, 370, 379, 388, 397, 406, 415, 423, 433, 442, 451, 460, 469, 477, 487,
    496, 505, 514, 522, 531, 540, 549, 558, 567, 576, 585, 594, 603, 612, 621,
    630, 639, 648, 657, 666, 675, 684, 693, 702, 711, 721, 730, 739, 748, 757,
    766, 775, 784, 793, 802, 811, 820, 829, 838, 847, 856, 866, 875, 883, 893,
    902, 911, 920, 930, 939, 948, 957, 966, 976, 984, 994, 1003, 1012, 1022,
    1031, 1040, 1050, 1059, 1068, 1077, 1087, 1096, 1105, 1115, 1124, 1133, 1142,
    1152, 1162, 1171, 1180, 1190, 1199, 1208, 1218, 1227, 1237, 1246, 1256, 1265,
    1275, 1284, 1294, 1303, 1313, 1322, 1332, 1341, 1351, 1360, 1370, 1379, 1389,
    1399, 1408, 1418, 1427, 1437, 1447, 1456, 1466, 1476, 1485, 1495, 1505, 1514,
    1524, 1534, 1544, 1554, 1563, 1573, 1583, 1593, 1603, 1613, 1623, 1633, 1643,
    1652, 1663, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, 17, 19, 22, 24,
    27, 31, 35, 40, 45, 51, 57, 65, 72, 80, 88, 97, 105, 114, 122, 131, 140, 148,
    157, 166, 174, 183, 192, 201, 209, 218, 227, 236, 244, 253, 262, 270, 279,
    288, 297, 306, 315, 324, 332, 341, 350, 359, 368, 377, 386, 395, 403, 412,
    421, 430, 439, 448, 457, 466, 475, 484, 493, 502, 511, 520, 528, 537, 546,
    555, 564, 573, 582, 591, 600, 609, 618, 627, 636, 645, 654, 663, 672, 681,
    690, 699, 707, 717, 726, 735, 744, 753, 762, 771, 780, 789, 798, 807, 816,
    825, 834, 843, 852, 861, 870, 879, 888, 897, 907, 916, 925, 934, 943, 952,
    961, 971, 980, 989, 998, 1007, 1017, 1026, 1035, 1044, 1054, 1063, 1072,
    1081, 1091, 1100, 1109, 1119, 1128, 1137, 1146, 1156, 1165, 1175, 1184, 1193,
    1203, 1212, 1221, 1231, 1240, 1250, 1259, 1268, 1278, 1287, 1297, 1306, 1316,
    1325, 1335, 1344, 1353, 1363, 1373, 1382, 1392, 1401, 1411, 1421, 1430, 1440,
    1449, 1459, 1469, 1478, 1488, 1498, 1507, 1517, 1527, 1537, 1546, 1556, 1566,
    1576, 1586, 1595, 1605, 1615, 1625, 1635, 1644, 1655, 0, 1, 2, 3, 3, 4, 5, 6,
    7, 8, 9, 10, 12, 13, 15, 17, 19, 21, 24, 27, 30, 35, 39, 44, 50, 57, 64, 71,
    79, 87, 96, 104, 113, 121, 130, 138, 147, 156, 164, 173, 182, 190, 199, 208,
    216, 225, 234, 243, 251, 260, 269, 278, 286, 295, 304, 313, 322, 330, 339,
    348, 357, 365, 374, 383, 392, 401, 410, 419, 428, 437, 445, 454, 463, 472,
    481, 490, 499, 508, 517, 525, 535, 543, 552, 561, 570, 579, 588, 597, 606,
    615, 624, 632, 642, 650, 659, 668, 677, 686, 695, 704, 713, 722, 731, 740,
    749, 758, 767, 776, 785, 794, 803, 812, 821, 830, 839, 847, 857, 866, 875,
    884, 893, 902, 911, 920, 929, 938, 948, 957, 966, 975, 984, 993, 1002, 1012,
    1021, 1030, 1039, 1048, 1058, 1067, 1076, 1085, 1095, 1104, 1113, 1122, 1131,
    1141, 1150, 1159, 1169, 1178, 1187, 1197, 1206, 1215, 1225, 1234, 1244, 1253,
    1262, 1272, 1281, 1290, 1300, 1309, 1319, 1328, 1338, 1347, 1357, 1366, 1376,
    1385, 1395, 1404, 1414, 1423, 1433, 1442, 1452, 1462, 1471, 1481, 1491, 1500,
    1510, 1520, 1529, 1539, 1548, 1558, 1568, 1578, 1588, 1598, 1607, 1617, 1627,
    1637, 1647, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, 17, 19, 21, 24,
    27, 30, 34, 39, 44, 49, 56, 63, 71, 78, 86, 95, 103, 112, 120, 129, 137, 146,
    154, 163, 172, 180, 189, 198, 206, 215, 224, 232, 241, 249, 258, 267, 276,
    285, 293, 302, 311, 319, 328, 337, 346, 355, 363, 372, 381, 390, 398, 408,
    416, 425, 434, 443, 452, 461, 469, 478, 487, 496, 505, 514, 522, 532, 540,
    549, 558, 567, 576, 585, 593, 603, 611, 620, 629, 638, 647, 656, 665, 674,
    682, 691, 700, 709, 718, 727, 736, 745, 754, 763, 772, 781, 790, 799, 808,
    817, 825, 834, 843, 852, 861, 870, 879, 888, 898, 907, 916, 925, 934, 943,
    952, 961, 970, 979, 988, 997, 1007, 1016, 1025, 1034, 1043, 1052, 1062, 1071,
    1080, 1089, 1098, 1108, 1117, 1126, 1135, 1145, 1154, 1163, 1172, 1182, 1191,
    1200, 1209, 1219, 1228, 1238, 1247, 1256, 1266, 1275, 1284, 1294, 1303, 1312,
    1322, 1331, 1340, 1350, 1360, 1369, 1379, 1388, 1397, 1407, 1417, 1426, 1436,
    1445, 1455, 1464, 1474, 1483, 1493, 1503, 1512, 1522, 1532, 1541, 1551, 1561,
    1571, 1580, 1590, 1600, 1610, 1620, 1629, 1639, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8,
    9, 10, 12, 13, 15, 17, 19, 21, 24, 27, 30, 34, 38, 43, 49, 55, 62, 70, 78,
    86, 94, 102, 110, 119, 127, 136, 145, 153, 162, 170, 179, 187, 196, 205, 213,
    222, 231, 239, 248, 257, 265, 274, 283, 291, 300, 309, 317, 326, 335, 343,
    352, 361, 370, 379, 388, 396, 405, 414, 423, 432, 440, 449, 458, 467, 476,
    484, 493, 502, 511, 520, 529, 537, 546, 555, 564, 573, 582, 590, 599, 608,
    617, 626, 635, 643, 652, 661, 670, 679, 688, 696, 706, 714, 723, 732, 741,
    750, 759, 768, 777, 786, 795, 803, 812, 821, 830, 839, 848, 857, 866, 875,
    884, 893, 902, 911, 920, 929, 938, 947, 956, 965, 974, 983, 993, 1002, 1011,
    1020, 1029, 1038, 1047, 1056, 1065, 1075, 1084, 1093, 1102, 1111, 1120, 1129,
    1139, 1148, 1157, 1167, 1176, 1185, 1194, 1203, 1213, 1222, 1232, 1241, 1250,
    1259, 1269, 1278, 1287, 1297, 1306, 1316, 1325, 1334, 1344, 1353, 1363, 1372,
    1381, 1391, 1400, 1410, 1419, 1429, 1438, 1448, 1457, 1467, 1476, 1486, 1496,
    1505, 1515, 1524, 1534, 1544, 1553, 1563, 1573, 1583, 1592, 1602, 1612, 1621,
    1632, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, 17, 19, 21, 23, 26,
    30, 33, 38, 43, 48, 55, 62, 69, 77, 85, 93, 101, 109, 118, 126, 135, 143,
    152, 160, 169, 177, 186, 195, 203, 212, 220, 229, 238, 246, 255, 263, 272,
    281, 289, 298, 307, 315, 324, 333, 341, 350, 359, 368, 376, 385, 394, 403,
    411, 420, 429, 438, 447, 455, 464, 473, 482, 491, 499, 508, 517, 526, 534,
    543, 552, 561, 570, 579, 587, 596, 605, 614, 622, 631, 640, 649, 658, 667,
    675, 684, 693, 702, 711, 720, 728, 737, 746, 755, 764, 773, 782, 790, 799,
    808, 817, 826, 835, 844, 853, 861, 871, 879, 889, 898, 906, 915, 924, 933,
    942, 951, 960, 970, 978, 988, 997, 1006, 1015, 1024, 1033, 1042, 1051, 1060,
    1069, 1079, 1088, 1097, 1106, 1115, 1124, 1133, 1143, 1152, 1161, 1170, 1179,
    1189, 1198, 1207, 1216, 1226, 1235, 1244, 1253, 1263, 1272, 1281, 1291, 1300,
    1309, 1319, 1328, 1337, 1347, 1356, 1365, 1375, 1384, 1394, 1403, 1413, 1422,
    1431, 1441, 1450, 1460, 1469, 1479, 1489, 1498, 1508, 1517, 1527, 1537, 1546,
    1556, 1565, 1575, 1585, 1595, 1604, 1614, 1624, 0, 1, 2, 3, 3, 4, 5, 6, 7, 8,
    9, 10, 12, 13, 15, 16, 18, 21, 23, 26, 29, 33, 37, 42, 48, 54, 61, 68, 76,
    84, 92, 100, 108, 117, 125, 134, 142, 151, 159, 168, 176, 185, 193, 202, 210,
    219, 227, 236, 244, 253, 261, 270, 279, 288, 296, 305, 314, 322, 331, 339,
    348, 357, 366, 374, 383, 391, 400, 409, 418, 427, 435, 444, 453, 461, 470,
    479, 488, 497, 505, 514, 523, 531, 540, 549, 558, 567, 575, 584, 593, 602,
    611, 619, 628, 637, 646, 654, 663, 672, 680, 689, 698, 707, 716, 725, 733,
    742, 751, 760, 769, 778, 786, 795, 804, 813, 822, 830, 840, 848, 857, 866,
    875, 884, 893, 902, 911, 920, 929, 938, 947, 955, 965, 973, 983, 992, 1001,
    1010, 1019, 1028, 1037, 1046, 1055, 1064, 1073, 1082, 1091, 1100, 1109, 1119,
    1128, 1137, 1146, 1155, 1165, 1174, 1183, 1192, 1201, 1211, 1220, 1229, 1238,
    1247, 1257, 1266, 1275, 1284, 1294, 1303, 1312, 1321, 1331, 1340, 1350, 1359,
    1368, 1378, 1387, 1396, 1406, 1415, 1425, 1434, 1444, 1453, 1463, 1472, 1482,
    1491, 1501, 1510, 1519, 1529, 1539, 1549, 1558, 1568, 1577, 1587, 1597, 1606,
    1616, 0, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 12, 13, 15, 16, 18, 21, 23, 26,
    29, 33, 37, 42, 47, 53, 60, 67, 75, 83, 91, 99, 107, 116, 124, 132, 141, 149,
    158, 166, 175, 183, 192, 200, 209, 217, 226, 234, 243, 251, 260, 269, 277,
    286, 294, 303, 312, 320, 328, 337, 346, 354, 363, 372, 381, 389, 398, 406,
    415, 424, 433, 442, 450, 459, 468, 476, 485, 494, 503, 511, 520, 528, 537,
    546, 555, 564, 572, 581, 590, 599, 607, 616, 625, 633, 642, 651, 660, 668,
    677, 686, 695, 703, 712, 721, 730, 739, 747, 756, 765, 774, 782, 791, 800,
    809, 818, 826, 835, 844, 853, 862, 871, 880, 889, 897, 906, 915, 924, 933,
    942, 951, 960, 969, 978, 987, 996, 1005, 1014, 1023, 1032, 1041, 1050, 1059,
    1068, 1077, 1086, 1095, 1104, 1113, 1123, 1132, 1141, 1150, 1159, 1168, 1177,
    1186, 1196, 1205, 1214, 1223, 1232, 1241, 1251, 1260, 1269, 1278, 1288, 1297,
    1306, 1315, 1325, 1334, 1343, 1352, 1362, 1371, 1380, 1390, 1399, 1409, 1418,
    1427, 1437, 1446, 1456, 1465, 1475, 1484, 1494, 1503, 1512, 1522, 1532, 1541,
    1551, 1560, 1570, 1580, 1589, 1599, 1609, 0, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9,
    10, 11, 13, 14, 16, 18, 20, 23, 26, 29, 32, 37, 41, 47, 53, 59, 67, 74, 82,
    90, 98, 106, 115, 123, 131, 140, 148, 156, 165, 173, 182, 190, 199, 207, 216,
    224, 233, 241, 250, 258, 267, 275, 284, 292, 301, 310, 318, 326, 335, 344,
    352, 361, 370, 378, 387, 396, 404, 413, 422, 430, 439, 448, 456, 465, 474,
    482, 491, 500, 508, 517, 526, 535, 543, 552, 561, 569, 578, 587, 595, 604,
    612, 621, 630, 639, 648, 656, 665, 673, 682, 691, 700, 709, 717, 726, 735,
    743, 752, 761, 770, 778, 787, 796, 805, 814, 822, 831, 840, 849, 858, 866,
    875, 884, 893, 902, 911, 920, 929, 937, 946, 955, 964, 973, 982, 991, 1000,
    1009, 1018, 1027, 1036, 1045, 1054, 1063, 1072, 1081, 1090, 1099, 1108, 1117,
    1126, 1135, 1144, 1153, 1162, 1172, 1180, 1190, 1199, 1208, 1217, 1226, 1236,
    1245, 1254, 1263, 1272, 1281, 1291, 1300, 1309, 1318, 1328, 1337, 1346, 1355,
    1365, 1374, 1383, 1393, 1402, 1411, 1421, 1430, 1439, 1449, 1458, 1468, 1477,
    1487, 1496, 1505, 1515, 1525, 1534, 1544, 1553, 1563, 1572, 1582, 1591, 1601,
    0, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 16, 18, 20, 23, 25, 28, 32,
    36, 41, 46, 52, 59, 66, 73, 81, 89, 97, 105, 113, 122, 130, 138, 147, 155,
    164, 172, 180, 189, 197, 206, 214, 223, 231, 239, 248, 256, 265, 274, 282,
    290, 299, 308, 316, 324, 333, 342, 350, 359, 368, 376, 385, 393, 402, 411,
    419, 428, 437, 445, 453, 462, 471, 480, 488, 497, 505, 514, 523, 532, 540,
    549, 558, 566, 575, 584, 592, 601, 609, 618, 627, 636, 644, 653, 661, 670,
    679, 688, 696, 705, 714, 722, 731, 740, 748, 757, 766, 775, 783, 792, 801,
    809, 818, 827, 836, 844, 853, 862, 871, 880, 889, 897, 906, 915, 924, 933,
    941, 951, 959, 968, 977, 986, 995, 1004, 1013, 1022, 1031, 1040, 1049, 1058,
    1067, 1076, 1084, 1093, 1102, 1112, 1121, 1130, 1139, 1148, 1157, 1166, 1175,
    1184, 1193, 1202, 1211, 1221, 1230, 1239, 1248, 1257, 1266, 1275, 1285, 1294,
    1303, 1312, 1321, 1331, 1340, 1349, 1358, 1368, 1377, 1386, 1395, 1405, 1414,
    1423, 1433, 1442, 1451, 1461, 1470, 1480, 1489, 1498, 1508, 1517, 1527, 1536,
    1546, 1556, 1565, 1575, 1584, 1594, 0, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
    13, 14, 16, 18, 20, 22, 25, 28, 32, 36, 41, 46, 52, 58, 65, 72, 80, 88, 96,
    104, 112, 121, 129, 137, 146, 154, 162, 171, 179, 187, 196, 204, 213, 221,
    230, 238, 246, 255, 263, 272, 280, 288, 297, 306, 314, 322, 331, 340, 348,
    357, 365, 374, 382, 391, 399, 408, 417, 425, 434, 443, 451, 460, 468, 477,
    486, 494, 503, 512, 520, 529, 537, 546, 555, 563, 572, 580, 589, 598, 606,
    615, 623, 632, 641, 649, 658, 666, 675, 684, 693, 701, 710, 719, 727, 736,
    745, 753, 762, 771, 779, 788, 797, 805, 814, 823, 832, 840, 849, 858, 867,
    875, 884, 893, 902, 911, 919, 928, 937, 946, 954, 964, 972, 981, 990, 999,
    1008, 1017, 1026, 1035, 1044, 1053, 1062, 1070, 1079, 1088, 1097, 1106, 1115,
    1124, 1133, 1142, 1151, 1160, 1169, 1178, 1188, 1197, 1206, 1215, 1224, 1233,
    1242, 1251, 1260, 1269, 1278, 1288, 1296, 1306, 1315, 1324, 1333, 1343, 1352,
    1361, 1370, 1380, 1389, 1398, 1407, 1417, 1426, 1435, 1444, 1454, 1463, 1473,
    1482, 1491, 1501, 1510, 1520, 1529, 1539, 1548, 1558, 1567, 1577, 1586, 0, 1,
    2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 16, 18, 20, 22, 25, 28, 32, 36,
    40, 45, 51, 57, 64, 72, 79, 87, 95, 103, 111, 119, 128, 136, 144, 153, 161,
    169, 178, 186, 194, 203, 211, 220, 228, 236, 245, 253, 262, 270, 278, 287,
    295, 304, 312, 320, 329, 338, 346, 355, 363, 372, 380, 389, 397, 406, 414,
    423, 432, 440, 448, 457, 466, 474, 483, 492, 500, 509, 517, 526, 534, 543,
    552, 560, 568, 577, 586, 595, 603, 612, 620, 629, 637, 646, 654, 663, 672,
    680, 689, 698, 706, 715, 724, 732, 741, 749, 758, 767, 775, 784, 793, 801,
    810, 819, 827, 836, 845, 853, 862, 871, 880, 889, 897, 906, 915, 924, 932,
    941, 950, 959, 968, 977, 985, 994, 1003, 1012, 1021, 1030, 1039, 1048, 1056,
    1065, 1074, 1083, 1092, 1101, 1110, 1119, 1128, 1137, 1146, 1155, 1164, 1173,
    1182, 1191, 1200, 1209, 1218, 1227, 1236, 1245, 1254, 1263, 1272, 1282, 1290,
    1300, 1309, 1318, 1327, 1336, 1346, 1355, 1364, 1373, 1382, 1392, 1401, 1410,
    1419, 1429, 1438, 1447, 1457, 1466, 1475, 1484, 1494, 1503, 1513, 1522, 1532,
    1541, 1551, 1560, 1569, 1579, 0, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 13,
    14, 16, 18, 20, 22, 25, 28, 31, 35, 40, 45, 50, 57, 64, 71, 78, 86, 94, 102,
    110, 118, 127, 135, 143, 151, 160, 168, 176, 185, 193, 201, 210, 218, 226,
    234, 243, 251, 260, 268, 277, 285, 293, 302, 310, 318, 327, 336, 344, 353,
    361, 370, 378, 387, 395, 404, 412, 421, 429, 438, 446, 455, 463, 472, 480,
    489, 497, 506, 514, 523, 532, 540, 549, 557, 565, 574, 583, 591, 600, 609,
    617, 626, 634, 643, 651, 660, 668, 677, 686, 694, 703, 711, 720, 728, 737,
    746, 754, 763, 771, 780, 789, 797, 806, 815, 823, 832, 841, 849, 858, 867,
    875, 884, 893, 902, 910, 919, 928, 937, 945, 954, 963, 972, 981, 989, 998,
    1007, 1016, 1025, 1034, 1042, 1051, 1060, 1069, 1078, 1087, 1096, 1105, 1114,
    1123, 1131, 1140, 1149, 1158, 1167, 1176, 1185, 1194, 1203, 1212, 1221, 1230,
    1239, 1248, 1257, 1266, 1275, 1284, 1294, 1303, 1312, 1321, 1330, 1339, 1348,
    1358, 1367, 1376, 1385, 1394, 1403, 1413, 1422, 1431, 1440, 1450, 1459, 1468,
    1477, 1487, 1496, 1506, 1515, 1525, 1534, 1543, 1553, 1562, 1572, 0, 1, 2, 2,
    3, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 16, 18, 20, 22, 25, 27, 31, 35, 39, 44,
    50, 56, 63, 70, 77, 85, 93, 101, 109, 117, 125, 134, 142, 150, 158, 167, 175,
    183, 192, 200, 208, 216, 225, 233, 241, 250, 258, 267, 275, 283, 292, 300,
    308, 317, 325, 334, 342, 351, 359, 367, 376, 384, 393, 401, 410, 418, 427,
    435, 443, 452, 461, 469, 478, 486, 494, 503, 511, 520, 529, 537, 546, 554,
    562, 571, 580, 588, 597, 605, 614, 622, 631, 639, 648, 656, 665, 674, 682,
    691, 699, 708, 716, 725, 733, 742, 750, 759, 768, 776, 785, 793, 802, 811,
    819, 828, 837, 845, 854, 863, 871, 880, 889, 897, 906, 915, 923, 932, 941,
    950, 958, 967, 976, 985, 993, 1002, 1011, 1020, 1029, 1037, 1046, 1055, 1064,
    1073, 1081, 1091, 1099, 1108, 1117, 1126, 1135, 1144, 1153, 1162, 1171, 1180,
    1189, 1198, 1207, 1215, 1224, 1233, 1242, 1251, 1260, 1269, 1278, 1288, 1297,
    1306, 1315, 1324, 1333, 1342, 1351, 1360, 1369, 1379, 1388, 1397, 1406, 1415,
    1424, 1434, 1443, 1452, 1462, 1471, 1480, 1490, 1499, 1508, 1518, 1527, 1536,
    1546, 1555, 1565, 0, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 16, 17,
    19, 22, 24, 27, 31, 34, 39, 44, 49, 55, 62, 69, 77, 84, 92, 100, 108, 116,
    124, 133, 141, 149, 157, 165, 174, 182, 190, 198, 207, 215, 223, 231, 240,
    248, 256, 265, 273, 281, 290, 298, 307, 315, 323, 332, 340, 348, 357, 365,
    373, 382, 390, 399, 407, 416, 424, 433, 441, 450, 458, 467, 475, 484, 492,
    500, 509, 517, 526, 534, 543, 551, 559, 568, 577, 585, 593, 602, 610, 619,
    628, 636, 644, 653, 661, 670, 679, 687, 696, 704, 713, 721, 730, 738, 747,
    755, 764, 772, 781, 789, 798, 807, 815, 824, 832, 841, 850, 858, 867, 876,
    884, 893, 902, 910, 919, 928, 936, 945, 954, 962, 971, 980, 989, 997, 1006,
    1015, 1024, 1032, 1041, 1050, 1059, 1067, 1076, 1085, 1094, 1103, 1112, 1121,
    1130, 1138, 1147, 1156, 1165, 1174, 1183, 1192, 1201, 1210, 1219, 1228, 1237,
    1246, 1255, 1264, 1272, 1282, 1291, 1300, 1309, 1318, 1327, 1336, 1345, 1354,
    1363, 1372, 1381, 1390, 1400, 1409, 1418, 1427, 1436, 1446, 1455, 1464, 1473,
    1483, 1492, 1501, 1511, 1520, 1529, 1539, 1548, 1557, 0, 1, 2, 2, 3, 4, 5, 6,
    7, 8, 9, 10, 11, 12, 14, 16, 17, 19, 22, 24, 27, 30, 34, 38, 43, 49, 55, 61,
    68, 76, 83, 91, 99, 107, 115, 123, 131, 140, 148, 156, 164, 172, 181, 189,
    197, 205, 213, 222, 230, 238, 246, 255, 263, 271, 279, 288, 296, 305, 313,
    321, 330, 338, 346, 355, 363, 371, 380, 388, 397, 405, 413, 422, 430, 438,
    447, 456, 464, 472, 481, 489, 498, 506, 515, 523, 531, 540, 548, 557, 565,
    574, 582, 590, 599, 607, 616, 624, 633, 641, 649, 658, 667, 675, 684, 692,
    701, 709, 717, 726, 734, 743, 751, 760, 768, 777, 786, 794, 803, 811, 819,
    828, 837, 845, 854, 863, 871, 880, 888, 897, 906, 914, 923, 931, 940, 949,
    958, 966, 975, 984, 993, 1001, 1010, 1019, 1028, 1036, 1045, 1054, 1062,
    1071, 1080, 1089, 1098, 1107, 1115, 1124, 1133, 1142, 1151, 1160, 1169, 1177,
    1186, 1195, 1204, 1213, 1222, 1231, 1240, 1249, 1258, 1266, 1276, 1285, 1294,
    1303, 1312, 1321, 1330, 1339, 1348, 1357, 1366, 1375, 1384, 1393, 1402, 1411,
    1421, 1430, 1439, 1448, 1457, 1467, 1476, 1485, 1494, 1504, 1513, 1522, 1532,
    1541, 1550, 0, 1, 2, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, 17, 19, 21,
    24, 27, 30, 34, 38, 43, 48, 54, 61, 68, 75, 82, 90, 98, 106, 114, 122, 130,
    138, 146, 155, 163, 171, 179, 187, 196, 204, 212, 220, 228, 237, 245, 253,
    261, 270, 278, 286, 294, 303, 311, 319, 328, 336, 344, 353, 361, 369, 378,
    386, 394, 403, 411, 420, 428, 436, 445, 453, 461, 470, 478, 486, 495, 503,
    512, 520, 529, 537, 545, 554, 562, 571, 579, 587, 596, 604, 613, 621, 630,
    638, 646, 655, 663, 672, 680, 689, 697, 705, 714, 722, 731, 739, 748, 756,
    765, 773, 782, 790, 799, 807, 815, 824, 832, 841, 850, 858, 867, 876, 884,
    893, 901, 910, 919, 927, 936, 944, 953, 962, 970, 979, 988, 997, 1005, 1014,
    1023, 1031, 1040, 1049, 1057, 1066, 1075, 1084, 1093, 1101, 1110, 1119, 1128,
    1136, 1145, 1154, 1163, 1172, 1181, 1190, 1198, 1207, 1216, 1225, 1234, 1243,
    1252, 1260, 1270, 1279, 1288, 1296, 1305, 1314, 1323, 1332, 1341, 1350, 1360,
    1369, 1378, 1387, 1396, 1405, 1414, 1423, 1432, 1441, 1450, 1460, 1469, 1478,
    1488, 1497, 1506, 1515, 1525, 1534, 1543, 0, 1, 2, 2, 3, 4, 5, 6, 6, 7, 9,
    10, 11, 12, 14, 15, 17, 19, 21, 24, 26, 30, 33, 38, 42, 48, 53, 60, 67, 74,
    82, 89, 97, 105, 113, 121, 129, 137, 145, 153, 162, 170, 178, 186, 194, 202,
    210, 219, 227, 235, 243, 251, 260, 268, 276, 284, 293, 301, 309, 317, 326,
    334, 342, 351, 359, 367, 375, 384, 392, 400, 409, 417, 425, 434, 442, 451,
    459, 467, 476, 484, 492, 500, 509, 517, 526, 534, 543, 551, 559, 568, 576,
    584, 593, 601, 610, 618, 626, 634, 643, 651, 660, 668, 677, 685, 693, 702,
    710, 719, 727, 736, 744, 752, 761, 769, 778, 786, 795, 803, 811, 820, 828,
    837, 846, 854, 863, 871, 880, 888, 897, 905, 914, 922, 931, 940, 949, 957,
    966, 974, 983, 992, 1000, 1009, 1018, 1026, 1035, 1044, 1052, 1061, 1070,
    1079, 1087, 1096, 1105, 1114, 1122, 1131, 1140, 1149, 1158, 1166, 1175, 1184,
    1193, 1202, 1210, 1219, 1228, 1237, 1246, 1255, 1264, 1273, 1282, 1290, 1299,
    1308, 1317, 1326, 1335, 1344, 1353, 1362, 1371, 1380, 1389, 1398, 1408, 1417,
    1426, 1435, 1444, 1453, 1462, 1472, 1481, 1490, 1499, 1508, 1518, 1527, 1536,
    0, 1, 2, 2, 3, 4, 5, 6, 6, 7, 8, 10, 11, 12, 14, 15, 17, 19, 21, 24, 26, 30,
    33, 37, 42, 47, 53, 59, 66, 73, 81, 88, 96, 104, 112, 120, 128, 136, 144,
    152, 160, 168, 176, 185, 193, 201, 209, 217, 225, 233, 241, 250, 258, 266,
    274, 283, 291, 299, 307, 315, 324, 332, 340, 348, 357, 365, 373, 381, 390,
    398, 406, 415, 423, 431, 440, 448, 456, 465, 473, 481, 490, 498, 506, 515,
    523, 531, 540, 548, 556, 565, 573, 581, 590, 598, 606, 615, 623, 631, 640,
    648, 656, 665, 673, 682, 690, 698, 707, 715, 723, 732, 740, 749, 757, 765,
    774, 782, 791, 799, 807, 816, 824, 833, 842, 850, 859, 867, 876, 884, 893,
    901, 910, 918, 927, 935, 944, 953, 961, 970, 978, 987, 996, 1004, 1013, 1022,
    1030, 1039, 1047, 1056, 1065, 1074, 1082, 1091, 1100, 1108, 1117, 1126, 1135,
    1143, 1152, 1161, 1170, 1178, 1187, 1196, 1205, 1214, 1222, 1231, 1240, 1249,
    1258, 1267, 1276, 1284, 1293, 1302, 1311, 1320, 1329, 1338, 1347, 1356, 1365,
    1374, 1383, 1392, 1401, 1410, 1419, 1428, 1437, 1447, 1456, 1465, 1474, 1483,
    1492, 1502, 1511, 1520, 1529, 0, 1, 2, 2, 3, 4, 5, 6, 6, 7, 8, 10, 11, 12,
    14, 15, 17, 19, 21, 23, 26, 29, 33, 37, 41, 47, 52, 59, 65, 72, 80, 87, 95,
    103, 111, 119, 127, 135, 143, 151, 159, 167, 175, 183, 191, 199, 208, 216,
    223, 232, 240, 248, 256, 264, 272, 281, 289, 297, 305, 313, 322, 330, 338,
    346, 355, 363, 371, 379, 388, 396, 404, 412, 421, 429, 437, 446, 454, 462,
    470, 478, 487, 495, 504, 512, 520, 529, 537, 545, 553, 562, 570, 578, 587,
    595, 603, 612, 620, 628, 636, 645, 653, 661, 670, 678, 686, 695, 703, 712,
    720, 728, 737, 745, 753, 762, 770, 778, 787, 795, 803, 812, 820, 829, 837,
    846, 854, 863, 871, 880, 888, 897, 905, 914, 922, 931, 939, 948, 957, 965,
    974, 982, 991, 999, 1008, 1017, 1025, 1034, 1042, 1051, 1060, 1069, 1077,
    1086, 1095, 1103, 1112, 1120, 1129, 1138, 1147, 1155, 1164, 1173, 1182, 1190,
    1199, 1208, 1217, 1226, 1234, 1243, 1252, 1261, 1270, 1279, 1287, 1296, 1305,
    1314, 1323, 1332, 1341, 1350, 1359, 1368, 1377, 1385, 1395, 1404, 1413, 1422,
    1431, 1440, 1449, 1458, 1467, 1476, 1486, 1495, 1504, 1513, 1522, 0, 1, 2, 2,
    3, 4, 5, 6, 6, 7, 8, 10, 11, 12, 13, 15, 17, 19, 21, 23, 26, 29, 32, 36, 41,
    46, 52, 58, 65, 72, 79, 86, 94, 102, 110, 118, 126, 134, 142, 150, 158, 166,
    174, 182, 190, 198, 206, 214, 222, 230, 238, 247, 255, 263, 271, 279, 287,
    295, 303, 311, 320, 328, 336, 344, 353, 361, 369, 377, 385, 394, 402, 410,
    418, 426, 435, 443, 451, 460, 468, 476, 484, 492, 501, 509, 517, 526, 534,
    542, 551, 559, 567, 575, 584, 592, 600, 608, 617, 625, 633, 641, 650, 658,
    666, 675, 683, 691, 700, 708, 716, 725, 733, 741, 750, 758, 766, 774, 783,
    791, 800, 808, 816, 825, 833, 842, 850, 859, 867, 876, 884, 892, 901, 909,
    918, 926, 935, 943, 952, 961, 969, 978, 986, 995, 1003, 1012, 1020, 1029,
    1037, 1046, 1055, 1063, 1072, 1081, 1089, 1098, 1107, 1115, 1124, 1133, 1141,
    1150, 1159, 1167, 1176, 1185, 1194, 1202, 1211, 1220, 1229, 1237, 1246, 1255,
    1264, 1273, 1281, 1290, 1299, 1308, 1317, 1326, 1335, 1344, 1352, 1361, 1370,
    1379, 1388, 1397, 1406, 1415, 1424, 1433, 1442, 1451, 1461, 1470, 1479, 1488,
    1497, 1506, 1515, 0, 1, 2, 2, 3, 4, 5, 5, 6, 7, 8, 9, 11, 12, 13, 15, 17, 19,
    21, 23, 26, 29, 32, 36, 40, 46, 51, 57, 64, 71, 78, 86, 93, 101, 109, 117,
    125, 133, 141, 149, 157, 165, 173, 181, 189, 197, 205, 213, 220, 229, 237,
    245, 253, 261, 269, 277, 285, 293, 301, 309, 318, 326, 334, 342, 351, 358,
    367, 375, 383, 391, 400, 408, 416, 424, 432, 441, 449, 457, 465, 473, 482,
    490, 498, 507, 515, 523, 531, 539, 548, 556, 564, 572, 581, 589, 597, 605,
    614, 622, 630, 638, 647, 655, 663, 671, 680, 688, 696, 704, 713, 721, 729,
    738, 746, 754, 763, 771, 779, 788, 796, 804, 812, 821, 829, 838, 846, 855,
    863, 871, 880, 888, 897, 905, 914, 922, 930, 939, 947, 956, 964, 973, 981,
    990, 999, 1007, 1016, 1024, 1032, 1041, 1050, 1058, 1067, 1076, 1084, 1093,
    1102, 1110, 1119, 1127, 1136, 1145, 1153, 1162, 1171, 1179, 1188, 1197, 1206,
    1214, 1223, 1231, 1240, 1249, 1258, 1267, 1276, 1284, 1293, 1302, 1311, 1320,
    1329, 1337, 1346, 1355, 1364, 1373, 1382, 1391, 1400, 1409, 1418, 1427, 1436,
    1445, 1454, 1463, 1472, 1481, 1490, 1499, 1509, 0, 1, 2, 2, 3, 4, 5, 5, 6, 7,
    8, 9, 11, 12, 13, 15, 17, 18, 21, 23, 25, 28, 32, 36, 40, 45, 50, 57, 63, 70,
    77, 85, 92, 100, 108, 116, 124, 131, 139, 147, 155, 163, 171, 179, 187, 195,
    203, 211, 219, 227, 235, 243, 251, 259, 267, 276, 284, 292, 299, 308, 316,
    324, 332, 340, 348, 356, 365, 373, 381, 389, 397, 406, 414, 422, 430, 438,
    446, 455, 463, 471, 479, 487, 496, 504, 512, 520, 528, 536, 545, 553, 561,
    569, 578, 586, 594, 602, 610, 618, 627, 635, 643, 652, 660, 668, 676, 684,
    693, 701, 709, 717, 726, 734, 742, 750, 759, 767, 775, 784, 792, 800, 808,
    817, 825, 834, 842, 850, 859, 867, 876, 884, 892, 900, 909, 918, 926, 934,
    943, 951, 960, 968, 977, 985, 994, 1002, 1011, 1019, 1028, 1036, 1045, 1054,
    1062, 1071, 1079, 1088, 1096, 1105, 1114, 1122, 1131, 1139, 1148, 1157, 1165,
    1174, 1183, 1191, 1200, 1209, 1217, 1226, 1235, 1244, 1252, 1261, 1270, 1278,
    1287, 1296, 1305, 1314, 1322, 1331, 1340, 1349, 1358, 1366, 1376, 1385, 1394,
    1402, 1411, 1420, 1429, 1438, 1447, 1456, 1465, 1474, 1484, 1492, 1502, 0, 1,
    2, 2, 3, 4, 5, 5, 6, 7, 8, 9, 11, 12, 13, 15, 16, 18, 20, 23, 25, 28, 32, 35,
    39, 45, 50, 56, 62, 69, 76, 84, 91, 99, 107, 115, 122, 130, 138, 146, 154,
    162, 170, 178, 186, 194, 202, 210, 217, 226, 233, 242, 250, 258, 265, 274,
    282, 290, 298, 306, 314, 322, 330, 338, 346, 354, 363, 370, 379, 387, 395,
    403, 411, 419, 428, 436, 444, 452, 460, 468, 477, 485, 493, 501, 509, 517,
    526, 534, 542, 550, 558, 566, 575, 583, 591, 599, 607, 615, 623, 632, 640,
    648, 656, 665, 673, 681, 689, 697, 706, 714, 722, 730, 739, 747, 755, 763,
    772, 780, 788, 796, 804, 813, 821, 830, 838, 846, 855, 863, 871, 879, 888,
    896, 905, 913, 922, 930, 938, 947, 955, 964, 972, 981, 989, 998, 1006, 1014,
    1023, 1031, 1040, 1049, 1057, 1066, 1074, 1083, 1091, 1100, 1108, 1117, 1126,
    1134, 1143, 1151, 1160, 1169, 1177, 1186, 1194, 1203, 1212, 1220, 1229, 1238,
    1246, 1255, 1264, 1273, 1281, 1290, 1299, 1308, 1316, 1325, 1334, 1343, 1352,
    1360, 1369, 1378, 1387, 1396, 1405, 1414, 1423, 1432, 1441, 1450, 1459, 1468,
    1477, 1486, 1495, 0, 1, 2, 2, 3, 4, 5, 5, 6, 7, 8, 9, 11, 12, 13, 15, 16, 18,
    20, 23, 25, 28, 31, 35, 39, 44, 49, 55, 62, 68, 76, 83, 90, 98, 106, 114,
    121, 129, 137, 145, 153, 161, 169, 177, 184, 192, 200, 208, 216, 224, 232,
    240, 248, 256, 264, 272, 280, 288, 296, 304, 312, 320, 328, 336, 344, 352,
    361, 368, 377, 385, 393, 401, 409, 417, 425, 433, 442, 450, 458, 466, 474,
    482, 490, 498, 507, 515, 523, 531, 539, 547, 555, 563, 572, 580, 588, 596,
    604, 612, 620, 628, 637, 645, 653, 661, 669, 678, 686, 694, 702, 710, 719,
    727, 735, 743, 751, 759, 768, 776, 784, 793, 801, 809, 817, 826, 834, 842,
    851, 859, 867, 875, 884, 892, 901, 909, 917, 926, 934, 942, 951, 959, 968,
    976, 984, 993, 1001, 1010, 1018, 1026, 1035, 1044, 1052, 1061, 1069, 1078,
    1086, 1094, 1103, 1112, 1120, 1129, 1137, 1146, 1155, 1163, 1172, 1180, 1189,
    1198, 1206, 1215, 1223, 1232, 1241, 1249, 1258, 1267, 1276, 1284, 1293, 1302,
    1310, 1319, 1328, 1337, 1346, 1354, 1363, 1372, 1381, 1390, 1398, 1408, 1416,
    1425, 1434, 1443, 1452, 1461, 1470, 1479, 1488, 0, 1, 2, 2, 3, 4, 5, 5, 6, 7,
    8, 9, 10, 12, 13, 15, 16, 18, 20, 22, 25, 28, 31, 35, 39, 44, 49, 55, 61, 68,
    75, 82, 89, 97, 105, 112, 120, 128, 136, 144, 152, 159, 167, 175, 183, 191,
    199, 207, 214, 223, 230, 239, 246, 254, 262, 270, 278, 286, 294, 302, 310,
    318, 326, 334, 342, 350, 358, 366, 375, 383, 391, 399, 407, 415, 423, 431,
    439, 447, 455, 463, 472, 479, 488, 496, 504, 512, 520, 528, 536, 544, 553,
    560, 569, 577, 585, 593, 601, 609, 617, 625, 634, 642, 650, 658, 666, 674,
    682, 691, 699, 707, 715, 723, 731, 739, 748, 756, 764, 772, 780, 789, 797,
    805, 813, 822, 830, 838, 846, 855, 863, 871, 880, 888, 896, 905, 913, 921,
    930, 938, 946, 955, 963, 971, 980, 988, 997, 1005, 1013, 1022, 1030, 1039,
    1047, 1056, 1064, 1073, 1081, 1089, 1098, 1107, 1115, 1124, 1132, 1141, 1149,
    1158, 1166, 1175, 1183, 1192, 1201, 1209, 1218, 1226, 1235, 1244, 1252, 1261,
    1270, 1278, 1287, 1296, 1305, 1313, 1322, 1331, 1339, 1348, 1357, 1366, 1375,
    1383, 1392, 1401, 1410, 1419, 1428, 1437, 1446, 1455, 1464, 1472, 1482, 0, 1,
    2, 2, 3, 4, 5, 5, 6, 7, 8, 9, 10, 12, 13, 15, 16, 18, 20, 22, 25, 27, 31, 34,
    38, 43, 48, 54, 60, 67, 74, 81, 89, 96, 104, 111, 119, 127, 135, 143, 150,
    158, 166, 174, 182, 190, 198, 205, 213, 221, 229, 237, 245, 253, 260, 269,
    277, 285, 292, 300, 308, 316, 324, 332, 340, 348, 356, 364, 372, 380, 389,
    397, 405, 412, 421, 429, 437, 445, 453, 461, 469, 477, 485, 493, 501, 509,
    517, 525, 534, 542, 550, 557, 566, 574, 582, 590, 598, 606, 614, 622, 630,
    638, 647, 655, 663, 671, 679, 687, 695, 703, 711, 720, 728, 736, 744, 752,
    760, 769, 776, 785, 793, 801, 809, 818, 826, 834, 842, 851, 859, 867, 875,
    883, 892, 900, 909, 917, 925, 933, 942, 950, 959, 967, 975, 984, 992, 1000,
    1008, 1017, 1026, 1034, 1042, 1051, 1059, 1068, 1076, 1084, 1093, 1101, 1110,
    1118, 1127, 1135, 1144, 1152, 1161, 1170, 1178, 1187, 1195, 1203, 1212, 1221,
    1229, 1238, 1247, 1255, 1264, 1273, 1281, 1290, 1299, 1307, 1316, 1325, 1333,
    1342, 1351, 1360, 1368, 1377, 1386, 1395, 1404, 1413, 1421, 1430, 1439, 1448,
    1457, 1466, 1475, 0, 1, 2, 2, 3, 4, 5, 5, 6, 7, 8, 9, 10, 12, 13, 14, 16, 18,
    20, 22, 24, 27, 30, 34, 38, 43, 48, 53, 60, 66, 73, 80, 88, 95, 103, 110,
    118, 126, 134, 141, 149, 157, 165, 173, 180, 188, 196, 204, 212, 220, 227,
    235, 243, 251, 259, 267, 275, 283, 290, 298, 307, 314, 322, 330, 338, 346,
    354, 362, 370, 378, 386, 394, 402, 410, 418, 426, 434, 442, 450, 458, 466,
    474, 483, 491, 499, 507, 515, 522, 531, 539, 547, 555, 563, 571, 579, 587,
    595, 603, 611, 619, 627, 635, 643, 651, 659, 668, 676, 684, 692, 700, 708,
    716, 724, 732, 740, 748, 757, 765, 773, 781, 789, 797, 806, 814, 822, 830,
    838, 847, 855, 863, 871, 879, 888, 896, 904, 913, 921, 929, 937, 946, 954,
    962, 971, 979, 987, 995, 1004, 1012, 1021, 1029, 1038, 1046, 1054, 1063,
    1071, 1079, 1088, 1096, 1105, 1113, 1122, 1130, 1139, 1147, 1156, 1164, 1173,
    1181, 1190, 1198, 1207, 1215, 1224, 1232, 1241, 1250, 1258, 1267, 1275, 1284,
    1293, 1301, 1310, 1319, 1327, 1336, 1345, 1354, 1362, 1371, 1380, 1389, 1397,
    1406, 1415, 1424, 1433, 1442, 1451, 1459, 1468, 0, 1, 2, 2, 3, 4, 5, 5, 6, 7,
    8, 9, 10, 12, 13, 14, 16, 18, 20, 22, 24, 27, 30, 34, 37, 42, 47, 53, 59, 65,
    72, 79, 87, 94, 102, 109, 117, 125, 132, 140, 148, 156, 164, 171, 179, 187,
    195, 203, 210, 218, 226, 234, 242, 250, 257, 265, 273, 281, 289, 297, 305,
    312, 321, 328, 336, 344, 352, 360, 368, 376, 384, 392, 400, 408, 416, 424,
    432, 440, 448, 456, 464, 472, 480, 488, 496, 504, 512, 520, 528, 536, 544,
    552, 560, 568, 576, 584, 592, 600, 608, 616, 624, 632, 640, 648, 656, 664,
    672, 680, 688, 696, 704, 713, 721, 729, 737, 745, 753, 761, 769, 777, 785,
    794, 802, 810, 818, 826, 834, 843, 851, 859, 867, 875, 884, 892, 900, 908,
    916, 925, 933, 941, 950, 958, 966, 974, 983, 991, 999, 1007, 1016, 1024,
    1033, 1041, 1049, 1058, 1066, 1074, 1083, 1091, 1100, 1108, 1117, 1125, 1133,
    1142, 1150, 1159, 1167, 1176, 1184, 1192, 1201, 1210, 1218, 1227, 1235, 1244,
    1252, 1261, 1270, 1278, 1287, 1295, 1304, 1313, 1321, 1330, 1339, 1347, 1356,
    1365, 1373, 1382, 1391, 1400, 1409, 1418, 1426, 1435, 1444, 1453, 1462, 0, 1,
    2, 2, 3, 4, 5, 5, 6, 7, 8, 9, 10, 12, 13, 14, 16, 18, 20, 22, 24, 27, 30, 33,
    37, 42, 47, 52, 58, 65, 71, 79, 86, 93, 101, 108, 116, 124, 131, 139, 147,
    155, 162, 170, 178, 186, 193, 201, 209, 217, 224, 232, 240, 248, 256, 264,
    271, 279, 287, 295, 303, 311, 319, 327, 334, 342, 350, 358, 366, 374, 382,
    390, 398, 406, 414, 422, 430, 438, 446, 453, 462, 469, 477, 485, 493, 501,
    509, 517, 525, 533, 541, 549, 557, 565, 573, 581, 589, 597, 605, 613, 621,
    629, 637, 645, 653, 661, 669, 677, 685, 693, 701, 709, 717, 725, 733, 741,
    749, 757, 765, 774, 781, 790, 798, 806, 814, 822, 830, 838, 847, 855, 863,
    871, 879, 888, 896, 904, 912, 920, 929, 937, 945, 953, 962, 970, 978, 986,
    994, 1003, 1011, 1020, 1028, 1036, 1045, 1053, 1061, 1069, 1078, 1086, 1095,
    1103, 1111, 1120, 1128, 1137, 1145, 1154, 1162, 1170, 1179, 1187, 1196, 1204,
    1213, 1221, 1230, 1238, 1247, 1255, 1264, 1272, 1281, 1290, 1298, 1307, 1315,
    1324, 1333, 1341, 1350, 1359, 1367, 1376, 1385, 1394, 1402, 1411, 1420, 1429,
    1438, 1446, 1455, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 16, 17,
    19, 22, 24, 27, 30, 33, 37, 41, 46, 52, 58, 64, 71, 78, 85, 92, 100, 107,
    115, 123, 130, 138, 146, 153, 161, 169, 177, 184, 192, 200, 207, 215, 223,
    231, 239, 246, 254, 262, 270, 278, 285, 293, 301, 309, 317, 325, 333, 340,
    348, 356, 364, 372, 380, 388, 396, 403, 412, 419, 427, 435, 443, 451, 459,
    467, 475, 483, 491, 499, 507, 514, 523, 530, 538, 546, 554, 562, 570, 578,
    586, 594, 602, 610, 618, 626, 634, 642, 650, 658, 666, 674, 682, 690, 698,
    706, 714, 722, 730, 737, 746, 754, 761, 770, 778, 786, 794, 802, 810, 818,
    826, 834, 843, 850, 859, 867, 875, 883, 891, 900, 908, 916, 924, 932, 941,
    949, 957, 965, 974, 982, 990, 998, 1007, 1015, 1023, 1031, 1040, 1048, 1056,
    1064, 1073, 1081, 1090, 1098, 1106, 1115, 1123, 1131, 1140, 1148, 1157, 1165,
    1174, 1182, 1190, 1199, 1207, 1216, 1224, 1233, 1241, 1250, 1258, 1267, 1275,
    1284, 1292, 1301, 1310, 1318, 1327, 1335, 1344, 1353, 1361, 1370, 1379, 1387,
    1396, 1405, 1414, 1422, 1431, 1440, 1449, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9,
    10, 11, 13, 14, 16, 17, 19, 21, 24, 26, 29, 33, 36, 41, 46, 51, 57, 63, 70,
    77, 84, 91, 99, 106, 114, 122, 129, 137, 144, 152, 160, 168, 175, 183, 191,
    198, 206, 214, 221, 229, 237, 245, 252, 260, 268, 276, 283, 291, 299, 307,
    315, 323, 331, 338, 346, 354, 362, 370, 378, 386, 393, 401, 409, 417, 425,
    433, 441, 448, 457, 464, 472, 480, 488, 496, 504, 512, 520, 528, 536, 543,
    551, 559, 567, 575, 583, 591, 599, 607, 615, 623, 631, 639, 646, 654, 662,
    670, 678, 686, 694, 702, 710, 718, 726, 734, 742, 750, 758, 766, 774, 782,
    790, 798, 806, 814, 822, 831, 839, 846, 855, 863, 871, 879, 887, 895, 904,
    912, 920, 928, 936, 944, 953, 961, 969, 977, 985, 993, 1002, 1010, 1018,
    1027, 1035, 1043, 1052, 1060, 1068, 1076, 1085, 1093, 1101, 1110, 1118, 1126,
    1135, 1143, 1151, 1160, 1168, 1176, 1185, 1193, 1202, 1210, 1219, 1227, 1236,
    1244, 1253, 1261, 1270, 1278, 1287, 1295, 1304, 1312, 1321, 1329, 1338, 1347,
    1355, 1364, 1373, 1381, 1390, 1399, 1407, 1416, 1425, 1433, 1442, 0, 1, 2, 2,
    3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 16, 17, 19, 21, 23, 26, 29, 32, 36,
    40, 45, 50, 56, 63, 69, 76, 83, 90, 98, 105, 113, 120, 128, 136, 143, 151,
    159, 166, 174, 182, 189, 197, 204, 212, 220, 228, 235, 243, 251, 259, 266,
    274, 282, 289, 297, 305, 313, 321, 329, 336, 344, 352, 360, 368, 376, 383,
    391, 399, 407, 415, 423, 431, 438, 446, 454, 462, 470, 478, 486, 494, 501,
    509, 517, 525, 533, 540, 549, 556, 564, 572, 580, 588, 596, 604, 612, 620,
    627, 635, 643, 651, 659, 667, 675, 683, 691, 699, 707, 715, 723, 730, 738,
    746, 754, 762, 770, 778, 786, 794, 802, 811, 819, 827, 835, 842, 851, 859,
    867, 875, 883, 891, 899, 907, 916, 924, 932, 940, 948, 956, 965, 972, 981,
    989, 997, 1006, 1014, 1022, 1030, 1038, 1047, 1055, 1063, 1071, 1080, 1088,
    1096, 1105, 1113, 1121, 1130, 1138, 1146, 1155, 1163, 1171, 1180, 1188, 1196,
    1205, 1213, 1222, 1230, 1238, 1247, 1255, 1264, 1272, 1281, 1289, 1298, 1306,
    1315, 1323, 1332, 1341, 1349, 1358, 1366, 1375, 1384, 1392, 1401, 1410, 1419,
    1427, 1436, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 13, 14, 15, 17, 19,
    21, 23, 26, 29, 32, 36, 40, 45, 50, 56, 62, 68, 75, 82, 90, 97, 104, 112,
    119, 127, 135, 142, 150, 157, 165, 173, 180, 188, 196, 203, 211, 218, 226,
    234, 242, 249, 257, 265, 272, 280, 288, 296, 303, 311, 319, 327, 334, 342,
    350, 358, 366, 374, 381, 389, 397, 405, 413, 420, 428, 436, 444, 452, 459,
    467, 475, 483, 491, 499, 506, 514, 522, 530, 538, 546, 553, 562, 569, 577,
    585, 593, 601, 609, 617, 624, 632, 640, 648, 656, 664, 672, 680, 687, 695,
    703, 711, 719, 727, 735, 743, 751, 759, 766, 775, 783, 791, 799, 807, 815,
    823, 831, 838, 847, 855, 863, 871, 879, 887, 895, 903, 911, 919, 928, 936,
    944, 952, 960, 968, 976, 984, 993, 1001, 1009, 1017, 1025, 1034, 1042, 1050,
    1058, 1067, 1075, 1083, 1091, 1100, 1108, 1116, 1124, 1133, 1141, 1149, 1158,
    1166, 1174, 1183, 1191, 1199, 1208, 1216, 1225, 1233, 1241, 1250, 1258, 1267,
    1275, 1284, 1292, 1300, 1309, 1318, 1326, 1335, 1343, 1352, 1360, 1369, 1378,
    1386, 1395, 1404, 1412, 1421, 1430, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10,
    11, 13, 14, 15, 17, 19, 21, 23, 26, 29, 32, 35, 40, 44, 49, 55, 61, 68, 74,
    81, 89, 96, 103, 111, 118, 126, 133, 141, 149, 156, 164, 171, 179, 187, 194,
    202, 209, 217, 225, 232, 240, 247, 255, 263, 271, 278, 286, 294, 301, 309,
    317, 325, 332, 340, 348, 356, 364, 371, 379, 387, 395, 403, 410, 418, 426,
    434, 441, 449, 457, 465, 473, 481, 488, 496, 504, 512, 520, 527, 535, 543,
    551, 559, 567, 574, 582, 590, 598, 606, 613, 621, 629, 637, 645, 653, 660,
    668, 676, 684, 692, 700, 708, 716, 723, 731, 739, 747, 755, 763, 771, 779,
    787, 795, 803, 811, 819, 827, 835, 843, 851, 859, 867, 875, 883, 891, 899,
    907, 915, 923, 931, 939, 948, 956, 963, 972, 980, 988, 996, 1004, 1013, 1021,
    1029, 1037, 1045, 1053, 1062, 1070, 1078, 1086, 1095, 1103, 1111, 1119, 1128,
    1136, 1144, 1152, 1160, 1169, 1177, 1186, 1194, 1202, 1211, 1219, 1227, 1236,
    1244, 1253, 1261, 1269, 1278, 1286, 1294, 1303, 1312, 1320, 1329, 1337, 1346,
    1354, 1363, 1372, 1380, 1389, 1397, 1406, 1414, 1423, 0, 1, 2, 2, 3, 4, 4, 5,
    6, 7, 8, 9, 10, 11, 12, 14, 15, 17, 19, 21, 23, 26, 28, 32, 35, 39, 44, 49,
    54, 60, 67, 74, 81, 88, 95, 102, 110, 117, 125, 132, 140, 147, 155, 163, 170,
    178, 185, 193, 200, 208, 215, 223, 231, 239, 246, 254, 261, 269, 277, 284,
    292, 300, 308, 315, 323, 330, 338, 346, 354, 362, 369, 377, 385, 392, 400,
    408, 416, 424, 431, 439, 447, 454, 463, 470, 478, 486, 494, 501, 509, 517,
    525, 532, 540, 548, 556, 564, 571, 579, 587, 595, 603, 610, 618, 626, 634,
    642, 649, 657, 665, 673, 681, 689, 696, 704, 712, 720, 728, 736, 743, 752,
    759, 767, 775, 783, 791, 799, 807, 815, 823, 831, 839, 847, 855, 863, 871,
    879, 887, 895, 903, 911, 919, 927, 935, 943, 951, 959, 967, 975, 984, 992,
    1000, 1008, 1016, 1024, 1032, 1040, 1049, 1057, 1065, 1073, 1081, 1090, 1098,
    1106, 1114, 1122, 1131, 1139, 1147, 1155, 1164, 1172, 1180, 1189, 1197, 1205,
    1214, 1222, 1230, 1239, 1247, 1255, 1264, 1272, 1281, 1289, 1297, 1306, 1314,
    1323, 1331, 1340, 1348, 1357, 1365, 1374, 1383, 1391, 1400, 1408, 1417, 0, 1,
    2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, 17, 19, 21, 23, 25, 28, 31,
    35, 39, 43, 48, 54, 60, 66, 73, 80, 87, 94, 101, 109, 116, 124, 131, 139,
    146, 154, 161, 169, 176, 184, 191, 199, 207, 214, 222, 229, 237, 244, 252,
    260, 267, 275, 282, 290, 298, 306, 313, 321, 328, 336, 344, 352, 360, 367,
    375, 383, 390, 398, 406, 414, 421, 429, 437, 445, 452, 460, 468, 476, 483,
    491, 499, 507, 514, 522, 530, 538, 545, 553, 561, 569, 576, 584, 592, 600,
    607, 615, 623, 631, 638, 646, 654, 662, 670, 677, 685, 693, 701, 709, 716,
    724, 732, 740, 748, 756, 764, 772, 779, 787, 795, 803, 811, 819, 827, 835,
    843, 851, 859, 867, 875, 883, 891, 899, 907, 915, 923, 931, 939, 947, 955,
    963, 971, 979, 987, 995, 1003, 1011, 1020, 1028, 1036, 1044, 1052, 1060,
    1068, 1077, 1085, 1093, 1101, 1109, 1117, 1126, 1134, 1142, 1150, 1159, 1167,
    1175, 1183, 1192, 1200, 1208, 1216, 1225, 1233, 1241, 1250, 1258, 1266, 1275,
    1283, 1292, 1300, 1309, 1317, 1325, 1334, 1342, 1351, 1359, 1368, 1377, 1385,
    1394, 1402, 1411, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 12, 14, 15, 17,
    19, 20, 23, 25, 28, 31, 34, 38, 43, 48, 53, 59, 65, 72, 79, 86, 93, 100, 108,
    115, 123, 130, 138, 145, 153, 160, 168, 175, 183, 190, 197, 205, 213, 220,
    228, 235, 243, 251, 258, 266, 273, 281, 289, 296, 304, 312, 319, 327, 335,
    342, 350, 358, 365, 373, 381, 388, 396, 404, 411, 419, 427, 434, 442, 450,
    458, 465, 473, 481, 489, 496, 504, 512, 519, 527, 535, 542, 550, 558, 566,
    573, 581, 589, 597, 604, 612, 620, 628, 635, 643, 651, 659, 666, 674, 682,
    690, 698, 705, 713, 721, 729, 736, 744, 752, 760, 768, 776, 784, 791, 799,
    807, 815, 823, 831, 839, 847, 855, 863, 871, 879, 887, 895, 902, 910, 918,
    926, 934, 942, 950, 958, 966, 975, 983, 991, 999, 1007, 1015, 1023, 1031,
    1039, 1047, 1055, 1064, 1072, 1080, 1088, 1096, 1104, 1112, 1121, 1129, 1137,
    1145, 1153, 1162, 1170, 1178, 1186, 1194, 1203, 1211, 1219, 1228, 1236, 1244,
    1253, 1261, 1269, 1277, 1286, 1294, 1303, 1311, 1319, 1328, 1336, 1345, 1353,
    1362, 1370, 1379, 1388, 1396, 1405, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10,
    11, 12, 14, 15, 17, 18, 20, 22, 25, 28, 31, 34, 38, 42, 47, 53, 58, 65, 71,
    78, 85, 92, 100, 107, 114, 122, 129, 136, 144, 151, 159, 166, 174, 181, 189,
    196, 204, 211, 219, 226, 234, 241, 249, 257, 264, 271, 279, 287, 294, 302,
    310, 317, 325, 333, 340, 348, 356, 363, 371, 378, 386, 394, 401, 409, 417,
    425, 432, 440, 447, 455, 463, 471, 478, 486, 493, 501, 509, 517, 524, 532,
    540, 547, 555, 563, 570, 578, 586, 594, 601, 609, 617, 625, 632, 640, 648,
    655, 663, 671, 679, 686, 694, 702, 709, 717, 725, 733, 741, 748, 756, 764,
    772, 780, 788, 796, 803, 811, 819, 827, 835, 843, 851, 859, 867, 874, 882,
    890, 898, 906, 914, 922, 930, 938, 946, 954, 962, 970, 978, 986, 994, 1002,
    1010, 1018, 1026, 1034, 1043, 1051, 1059, 1067, 1075, 1083, 1091, 1099, 1107,
    1116, 1124, 1132, 1140, 1148, 1156, 1165, 1173, 1181, 1189, 1197, 1206, 1214,
    1222, 1230, 1239, 1247, 1255, 1264, 1272, 1280, 1289, 1297, 1305, 1313, 1322,
    1331, 1339, 1347, 1356, 1364, 1373, 1381, 1390, 1399, 0, 1, 2, 2, 3, 4, 4, 5,
    6, 7, 8, 9, 10, 11, 12, 14, 15, 17, 18, 20, 22, 25, 27, 30, 34, 38, 42, 47,
    52, 58, 64, 70, 77, 84, 91, 99, 106, 113, 121, 128, 135, 143, 150, 158, 165,
    173, 180, 187, 195, 202, 210, 217, 225, 232, 240, 247, 255, 263, 270, 277,
    285, 293, 300, 308, 316, 323, 331, 338, 346, 354, 361, 369, 376, 384, 392,
    399, 407, 415, 422, 430, 438, 445, 453, 460, 468, 476, 483, 491, 499, 506,
    514, 521, 529, 537, 545, 552, 560, 567, 575, 583, 591, 598, 606, 614, 621,
    629, 637, 645, 652, 660, 668, 675, 683, 691, 699, 706, 714, 722, 729, 737,
    745, 753, 761, 768, 776, 784, 792, 800, 808, 815, 823, 831, 839, 847, 855,
    863, 870, 878, 886, 894, 902, 910, 918, 926, 934, 942, 949, 957, 966, 974,
    982, 990, 998, 1006, 1014, 1021, 1030, 1038, 1046, 1054, 1062, 1070, 1078,
    1086, 1094, 1102, 1110, 1119, 1127, 1135, 1143, 1151, 1159, 1167, 1176, 1184,
    1192, 1200, 1208, 1217, 1225, 1233, 1241, 1250, 1258, 1266, 1275, 1283, 1291,
    1300, 1308, 1316, 1325, 1333, 1342, 1350, 1358, 1367, 1375, 1384, 1392, 0, 1,
    2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, 16, 18, 20, 22, 25, 27, 30,
    33, 37, 41, 46, 51, 57, 63, 70, 76, 83, 90, 98, 105, 112, 120, 127, 134, 142,
    149, 156, 164, 171, 179, 186, 193, 201, 208, 216, 223, 231, 238, 246, 253,
    261, 268, 276, 283, 291, 299, 306, 314, 321, 329, 336, 344, 352, 359, 367,
    374, 382, 390, 397, 405, 412, 420, 427, 435, 443, 450, 458, 466, 473, 481,
    488, 496, 504, 511, 519, 527, 534, 542, 550, 557, 565, 572, 580, 588, 595,
    603, 611, 618, 626, 634, 641, 649, 657, 664, 672, 680, 687, 695, 703, 711,
    718, 726, 734, 741, 749, 757, 765, 773, 780, 788, 796, 804, 811, 819, 827,
    835, 843, 851, 859, 866, 874, 882, 890, 898, 906, 914, 922, 930, 937, 945,
    953, 961, 969, 977, 985, 993, 1001, 1009, 1017, 1025, 1033, 1041, 1049, 1057,
    1065, 1073, 1081, 1089, 1097, 1106, 1114, 1122, 1130, 1138, 1146, 1154, 1162,
    1170, 1179, 1187, 1195, 1203, 1211, 1220, 1228, 1236, 1244, 1252, 1260, 1269,
    1277, 1286, 1294, 1302, 1311, 1319, 1327, 1336, 1344, 1353, 1361, 1369, 1378,
    1386, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, 16, 18, 20, 22,
    24, 27, 30, 33, 37, 41, 46, 51, 57, 63, 69, 76, 83, 90, 97, 104, 111, 118,
    126, 133, 141, 148, 155, 163, 170, 177, 185, 192, 200, 207, 215, 222, 229,
    237, 244, 252, 259, 267, 274, 282, 289, 297, 304, 312, 319, 327, 334, 342,
    350, 357, 365, 372, 380, 387, 395, 403, 410, 418, 425, 433, 440, 448, 456,
    463, 471, 478, 486, 494, 501, 509, 516, 524, 531, 539, 547, 554, 562, 569,
    577, 585, 593, 600, 608, 615, 623, 631, 638, 646, 654, 661, 669, 677, 684,
    692, 699, 707, 715, 722, 730, 738, 746, 753, 761, 769, 777, 784, 792, 800,
    807, 816, 823, 831, 839, 847, 855, 862, 870, 878, 886, 894, 902, 910, 917,
    925, 933, 941, 949, 957, 965, 973, 981, 989, 997, 1005, 1012, 1020, 1028,
    1036, 1044, 1052, 1060, 1068, 1076, 1084, 1093, 1101, 1109, 1117, 1124, 1133,
    1141, 1149, 1157, 1165, 1173, 1181, 1190, 1198, 1206, 1214, 1222, 1231, 1239,
    1247, 1255, 1263, 1272, 1280, 1288, 1296, 1305, 1313, 1322, 1330, 1338, 1347,
    1355, 1363, 1372, 1380, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13,
    15, 16, 18, 20, 22, 24, 27, 30, 33, 37, 41, 45, 50, 56, 62, 68, 75, 82, 89,
    96, 103, 110, 117, 125, 132, 139, 147, 154, 161, 169, 176, 184, 191, 198,
    205, 213, 221, 228, 235, 243, 250, 258, 265, 272, 280, 287, 295, 303, 310,
    317, 325, 332, 340, 348, 355, 363, 370, 378, 385, 393, 400, 408, 416, 423,
    431, 438, 446, 453, 461, 468, 476, 483, 491, 499, 506, 514, 521, 529, 537,
    544, 552, 559, 567, 574, 582, 590, 597, 605, 612, 620, 628, 635, 643, 650,
    658, 666, 673, 681, 689, 696, 704, 712, 719, 727, 734, 742, 750, 758, 765,
    773, 781, 788, 796, 804, 812, 819, 827, 835, 843, 851, 858, 866, 874, 882,
    890, 898, 905, 913, 921, 929, 937, 944, 953, 960, 968, 976, 984, 992, 1000,
    1008, 1016, 1024, 1032, 1040, 1048, 1056, 1064, 1072, 1080, 1088, 1096, 1104,
    1112, 1119, 1128, 1136, 1144, 1152, 1160, 1168, 1176, 1184, 1192, 1201, 1209,
    1217, 1225, 1233, 1241, 1249, 1258, 1266, 1274, 1283, 1291, 1299, 1307, 1316,
    1324, 1332, 1341, 1349, 1357, 1366, 1374, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9,
    10, 11, 12, 13, 15, 16, 18, 20, 22, 24, 26, 29, 32, 36, 40, 45, 50, 55, 61,
    67, 74, 81, 88, 95, 102, 109, 116, 124, 131, 138, 146, 153, 160, 168, 175,
    182, 189, 197, 204, 212, 219, 226, 234, 241, 249, 256, 263, 271, 278, 286,
    293, 301, 308, 315, 323, 330, 338, 346, 353, 361, 368, 375, 383, 391, 398,
    406, 413, 421, 428, 436, 443, 451, 458, 466, 474, 481, 489, 496, 504, 511,
    519, 526, 534, 541, 549, 556, 564, 571, 579, 587, 594, 602, 609, 617, 625,
    632, 640, 647, 655, 662, 670, 678, 685, 693, 701, 708, 716, 723, 731, 739,
    746, 754, 762, 769, 777, 785, 793, 800, 808, 815, 823, 831, 839, 847, 854,
    862, 870, 878, 886, 893, 901, 909, 917, 924, 932, 940, 948, 956, 964, 972,
    980, 988, 995, 1003, 1011, 1019, 1027, 1035, 1043, 1051, 1059, 1067, 1075,
    1083, 1091, 1099, 1107, 1114, 1123, 1131, 1139, 1147, 1155, 1163, 1171, 1179,
    1187, 1195, 1203, 1212, 1220, 1228, 1236, 1244, 1252, 1261, 1269, 1277, 1285,
    1293, 1302, 1310, 1318, 1327, 1335, 1343, 1352, 1360, 1368, 0, 1, 2, 2, 3, 4,
    4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 15, 16, 18, 20, 21, 24, 26, 29, 32, 36, 40,
    44, 49, 55, 61, 67, 73, 80, 87, 94, 101, 108, 115, 123, 130, 137, 144, 152,
    159, 166, 174, 181, 188, 196, 203, 210, 218, 225, 232, 240, 247, 255, 262,
    269, 277, 284, 292, 299, 306, 314, 321, 329, 336, 344, 351, 359, 366, 373,
    381, 389, 396, 404, 411, 418, 426, 433, 441, 449, 456, 464, 471, 478, 486,
    494, 501, 508, 516, 523, 531, 539, 546, 553, 561, 569, 576, 584, 591, 599,
    606, 614, 621, 629, 637, 644, 652, 659, 667, 674, 682, 689, 697, 705, 712,
    720, 727, 735, 743, 751, 758, 766, 773, 781, 789, 796, 804, 812, 820, 827,
    835, 843, 851, 858, 866, 874, 882, 889, 897, 905, 913, 920, 928, 936, 944,
    952, 960, 967, 975, 983, 991, 999, 1007, 1015, 1023, 1030, 1038, 1046, 1054,
    1062, 1070, 1078, 1086, 1094, 1102, 1110, 1118, 1126, 1134, 1142, 1150, 1158,
    1166, 1174, 1182, 1190, 1198, 1206, 1214, 1222, 1231, 1238, 1247, 1255, 1263,
    1271, 1279, 1288, 1296, 1304, 1313, 1321, 1329, 1337, 1346, 1354, 1362, 0, 1,
    2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 16, 18, 19, 21, 24, 26, 29,
    32, 35, 39, 44, 49, 54, 60, 66, 73, 79, 86, 93, 100, 107, 114, 122, 129, 136,
    143, 151, 158, 165, 172, 180, 187, 194, 201, 209, 216, 224, 231, 238, 246,
    253, 260, 267, 275, 282, 290, 297, 305, 312, 319, 327, 334, 342, 349, 357,
    364, 371, 379, 386, 394, 401, 409, 416, 424, 431, 439, 446, 454, 461, 469,
    476, 484, 491, 499, 506, 514, 521, 529, 536, 543, 551, 558, 566, 573, 581,
    588, 596, 603, 611, 618, 626, 634, 641, 649, 656, 664, 671, 679, 686, 694,
    701, 709, 717, 724, 732, 739, 747, 755, 762, 770, 778, 785, 793, 800, 808,
    816, 824, 831, 839, 847, 854, 862, 870, 878, 885, 893, 901, 909, 916, 924,
    932, 940, 947, 955, 963, 971, 979, 987, 994, 1002, 1010, 1018, 1026, 1034,
    1042, 1049, 1057, 1065, 1073, 1081, 1089, 1097, 1105, 1113, 1121, 1129, 1137,
    1145, 1153, 1161, 1169, 1177, 1185, 1193, 1201, 1209, 1217, 1225, 1233, 1241,
    1250, 1258, 1266, 1274, 1282, 1290, 1299, 1307, 1315, 1323, 1332, 1340, 1348,
    1356, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 16, 18, 19, 21,
    23, 26, 29, 32, 35, 39, 43, 48, 53, 59, 65, 72, 78, 85, 92, 99, 106, 113,
    121, 128, 135, 142, 149, 157, 164, 171, 178, 185, 193, 200, 208, 215, 222,
    229, 237, 244, 251, 258, 266, 273, 281, 288, 296, 303, 310, 318, 325, 332,
    340, 347, 355, 362, 369, 377, 384, 392, 399, 407, 414, 422, 429, 436, 444,
    451, 459, 466, 473, 481, 489, 496, 503, 511, 518, 526, 533, 541, 548, 555,
    563, 571, 578, 586, 593, 601, 608, 615, 623, 630, 638, 645, 653, 660, 668,
    676, 683, 691, 698, 705, 713, 721, 728, 736, 743, 751, 759, 766, 774, 782,
    789, 797, 804, 812, 820, 827, 835, 843, 850, 858, 866, 873, 881, 889, 897,
    904, 912, 920, 927, 935, 943, 951, 959, 967, 974, 982, 990, 998, 1006, 1013,
    1021, 1029, 1037, 1045, 1053, 1061, 1068, 1076, 1084, 1092, 1100, 1108, 1116,
    1124, 1132, 1140, 1148, 1156, 1164, 1172, 1180, 1188, 1196, 1204, 1212, 1220,
    1228, 1236, 1244, 1252, 1260, 1268, 1277, 1285, 1293, 1301, 1309, 1318, 1326,
    1334, 1342, 1351, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 8, 8, 9, 11, 12, 13, 14, 16,
    17, 19, 21, 23, 26, 28, 31, 35, 39, 43, 48, 53, 59, 65, 71, 78, 84, 91, 98,
    105, 112, 120, 127, 134, 141, 148, 156, 163, 170, 177, 184, 192, 199, 206,
    213, 221, 228, 235, 243, 250, 257, 264, 272, 279, 286, 294, 301, 308, 316,
    323, 331, 338, 345, 353, 360, 367, 375, 382, 390, 397, 405, 412, 419, 427,
    434, 442, 449, 456, 464, 471, 479, 486, 494, 501, 508, 516, 523, 531, 538,
    545, 553, 560, 568, 575, 583, 590, 598, 605, 612, 620, 627, 635, 642, 650,
    657, 665, 672, 680, 687, 695, 702, 710, 717, 725, 732, 740, 748, 755, 763,
    770, 778, 785, 793, 800, 808, 816, 824, 831, 839, 846, 854, 862, 870, 877,
    885, 893, 900, 908, 915, 923, 931, 939, 947, 954, 962, 970, 978, 985, 993,
    1001, 1009, 1017, 1024, 1032, 1040, 1048, 1056, 1064, 1072, 1079, 1087, 1095,
    1103, 1111, 1119, 1127, 1135, 1143, 1151, 1159, 1166, 1174, 1182, 1190, 1198,
    1206, 1215, 1222, 1231, 1239, 1247, 1255, 1263, 1271, 1279, 1287, 1296, 1304,
    1312, 1320, 1328, 1336, 1345, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 7, 8, 9, 11, 12,
    13, 14, 16, 17, 19, 21, 23, 25, 28, 31, 34, 38, 42, 47, 52, 58, 64, 70, 77,
    83, 90, 97, 104, 111, 119, 126, 133, 140, 147, 154, 162, 169, 176, 183, 190,
    197, 205, 212, 219, 226, 234, 241, 248, 255, 263, 270, 277, 285, 292, 299,
    306, 314, 321, 329, 336, 343, 351, 358, 365, 373, 380, 388, 395, 402, 410,
    417, 424, 432, 439, 447, 454, 461, 469, 476, 484, 491, 498, 506, 513, 521,
    528, 535, 543, 550, 557, 565, 572, 580, 587, 595, 602, 610, 617, 624, 632,
    639, 647, 654, 662, 669, 676, 684, 692, 699, 707, 714, 722, 729, 737, 744,
    752, 759, 767, 774, 782, 789, 797, 805, 812, 820, 827, 835, 843, 850, 858,
    866, 873, 881, 889, 896, 904, 911, 919, 927, 935, 942, 950, 958, 966, 973,
    981, 989, 997, 1004, 1012, 1020, 1028, 1035, 1043, 1051, 1059, 1067, 1075,
    1082, 1090, 1098, 1106, 1114, 1122, 1130, 1138, 1146, 1153, 1161, 1169, 1177,
    1185, 1193, 1201, 1209, 1217, 1225, 1233, 1241, 1249, 1257, 1266, 1274, 1282,
    1290, 1298, 1306, 1314, 1323, 1330, 1339, 0, 1, 2, 2, 3, 4, 4, 5, 6, 7, 7, 8,
    9, 10, 12, 13, 14, 16, 17, 19, 21, 23, 25, 28, 31, 34, 38, 42, 47, 52, 57,
    63, 69, 76, 83, 89, 96, 103, 110, 118, 125, 132, 139, 146, 153, 160, 167,
    175, 182, 189, 196, 203, 211, 218, 225, 232, 240, 247, 254, 261, 269, 276,
    283, 290, 298, 305, 312, 319, 327, 334, 342, 349, 356, 363, 371, 378, 386,
    393, 400, 407, 415, 422, 430, 437, 444, 452, 459, 466, 474, 481, 489, 496,
    503, 510, 518, 525, 533, 540, 547, 555, 562, 570, 577, 584, 592, 599, 607,
    614, 621, 629, 636, 644, 651, 658, 666, 673, 681, 688, 695, 703, 710, 718,
    726, 733, 741, 748, 756, 763, 771, 778, 786, 793, 801, 808, 816, 824, 831,
    839, 846, 854, 862, 869, 877, 884, 892, 900, 907, 915, 923, 930, 938, 946,
    954, 961, 969, 976, 984, 992, 1000, 1008, 1015, 1023, 1031, 1039, 1046, 1054,
    1062, 1070, 1078, 1085, 1093, 1101, 1109, 1117, 1125, 1133, 1140, 1148, 1156,
    1164, 1172, 1180, 1188, 1196, 1204, 1212, 1220, 1228, 1236, 1244, 1252, 1260,
    1268, 1276, 1284, 1292, 1301, 1309, 1317, 1325, 1333, 0, 1, 2, 2, 3, 4, 4, 5,
    6, 7, 7, 8, 9, 10, 12, 13, 14, 16, 17, 19, 21, 23, 25, 28, 30, 34, 38, 42,
    46, 51, 57, 63, 69, 75, 82, 89, 95, 102, 109, 117, 124, 131, 138, 145, 152,
    159, 166, 173, 180, 188, 195, 202, 209, 216, 223, 231, 238, 245, 252, 259,
    267, 274, 281, 289, 296, 303, 310, 318, 325, 332, 340, 347, 354, 361, 369,
    376, 383, 391, 398, 405, 413, 420, 427, 435, 442, 449, 457, 464, 471, 479,
    486, 493, 501, 508, 515, 523, 530, 537, 545, 552, 560, 567, 574, 582, 589,
    596, 604, 611, 618, 626, 633, 641, 648, 655, 663, 670, 678, 685, 692, 700,
    707, 715, 722, 730, 737, 745, 752, 760, 767, 774, 782, 789, 797, 805, 812,
    820, 827, 835, 843, 850, 858, 865, 873, 880, 888, 895, 903, 911, 919, 926,
    934, 942, 949, 957, 965, 972, 980, 988, 995, 1003, 1011, 1019, 1026, 1034,
    1042, 1050, 1057, 1065, 1073, 1080, 1089, 1096, 1104, 1112, 1120, 1128, 1135,
    1143, 1151, 1159, 1167, 1175, 1183, 1191, 1199, 1206, 1215, 1223, 1231, 1239,
    1246, 1255, 1263, 1271, 1279, 1287, 1295, 1303, 1311, 1319, 1327, 0, 1, 2, 2,
    3, 4, 4, 5, 6, 7, 7, 8, 9, 10, 11, 13, 14, 15, 17, 19, 20, 23, 25, 27, 30,
    34, 37, 41, 46, 51, 56, 62, 68, 74, 81, 88, 95, 102, 108, 116, 123, 130, 137,
    144, 151, 158, 165, 172, 179, 186, 193, 201, 208, 215, 222, 229, 237, 244,
    251, 258, 265, 272, 280, 287, 294, 301, 309, 316, 323, 330, 338, 345, 352,
    359, 367, 374, 381, 389, 396, 403, 411, 418, 425, 432, 440, 447, 454, 461,
    469, 476, 484, 491, 498, 505, 513, 520, 527, 535, 542, 549, 557, 564, 571,
    579, 586, 593, 601, 608, 615, 623, 630, 638, 645, 652, 660, 667, 674, 682,
    689, 697, 704, 711, 719, 726, 734, 741, 749, 756, 764, 771, 779, 786, 794,
    801, 809, 816, 824, 831, 839, 846, 854, 861, 869, 876, 884, 891, 899, 907,
    914, 922, 930, 937, 945, 953, 960, 968, 976, 983, 991, 999, 1006, 1014, 1022,
    1029, 1037, 1045, 1053, 1060, 1068, 1076, 1084, 1092, 1099, 1107, 1115, 1123,
    1131, 1138, 1146, 1154, 1162, 1170, 1178, 1186, 1194, 1201, 1209, 1217, 1225,
    1233, 1241, 1249, 1257, 1265, 1273, 1281, 1289, 1297, 1306, 1313, 1322, 0, 1,
    2, 2, 3, 3, 4, 5, 6, 7, 7, 8, 9, 10, 11, 13, 14, 15, 17, 19, 20, 22, 25, 27,
    30, 33, 37, 41, 45, 50, 55, 61, 67, 74, 80, 87, 94, 101, 108, 115, 122, 129,
    136, 143, 150, 157, 164, 171, 178, 185, 192, 199, 207, 214, 221, 228, 235,
    242, 249, 256, 264, 271, 278, 285, 292, 299, 307, 314, 321, 329, 336, 343,
    350, 357, 365, 372, 379, 387, 394, 401, 408, 415, 423, 430, 438, 445, 452,
    459, 467, 474, 481, 488, 496, 503, 510, 518, 525, 532, 539, 547, 554, 561,
    569, 576, 583, 591, 598, 605, 612, 620, 627, 634, 642, 649, 656, 664, 671,
    679, 686, 693, 700, 708, 715, 723, 730, 738, 745, 753, 760, 767, 775, 782,
    790, 797, 805, 812, 820, 827, 835, 842, 850, 857, 865, 873, 880, 887, 895,
    903, 910, 918, 926, 933, 941, 948, 956, 963, 971, 979, 987, 994, 1002, 1010,
    1017, 1025, 1033, 1040, 1048, 1056, 1063, 1071, 1079, 1087, 1094, 1102, 1110,
    1118, 1126, 1133, 1141, 1149, 1157, 1165, 1173, 1180, 1188, 1196, 1204, 1212,
    1220, 1228, 1236, 1244, 1252, 1260, 1268, 1276, 1284, 1292, 1300, 1308, 1316,
    0, 1, 2, 2, 3, 3, 4, 5, 6, 6, 7, 8, 9, 10, 11, 13, 14, 15, 17, 18, 20, 22,
    25, 27, 30, 33, 36, 40, 45, 50, 55, 61, 67, 73, 79, 86, 93, 100, 107, 114,
    120, 127, 135, 142, 149, 156, 163, 170, 177, 184, 191, 198, 205, 212, 219,
    226, 234, 241, 248, 255, 262, 269, 276, 284, 291, 298, 305, 312, 320, 327,
    334, 341, 348, 355, 363, 370, 377, 385, 392, 399, 406, 413, 421, 428, 435,
    443, 450, 457, 464, 472, 479, 486, 493, 500, 508, 515, 522, 529, 537, 544,
    551, 559, 566, 573, 580, 588, 595, 602, 610, 617, 624, 631, 639, 646, 653,
    660, 668, 675, 682, 690, 697, 705, 712, 720, 727, 734, 742, 749, 757, 764,
    771, 779, 786, 794, 801, 809, 816, 824, 831, 839, 846, 854, 861, 869, 876,
    883, 891, 898, 906, 914, 921, 929, 937, 944, 952, 959, 967, 975, 982, 990,
    997, 1005, 1013, 1020, 1028, 1036, 1043, 1051, 1059, 1066, 1074, 1082, 1090,
    1097, 1105, 1113, 1121, 1128, 1136, 1144, 1152, 1160, 1167, 1175, 1183, 1191,
    1199, 1207, 1215, 1223, 1230, 1238, 1246, 1254, 1262, 1270, 1278, 1286, 1294,
    1302, 1310, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6, 7, 8, 9, 10, 11, 13, 14, 15, 17,
    18, 20, 22, 24, 27, 29, 33, 36, 40, 44, 49, 54, 60, 66, 72, 79, 85, 92, 99,
    106, 113, 119, 126, 133, 140, 147, 154, 161, 169, 175, 183, 189, 197, 204,
    211, 218, 225, 232, 239, 246, 253, 261, 267, 275, 282, 289, 296, 303, 310,
    318, 325, 332, 339, 346, 353, 361, 368, 375, 383, 390, 397, 404, 411, 419,
    426, 433, 440, 447, 454, 462, 469, 476, 483, 491, 498, 505, 512, 520, 527,
    534, 541, 549, 556, 563, 570, 578, 585, 592, 599, 607, 614, 621, 628, 636,
    643, 650, 657, 665, 672, 679, 687, 694, 702, 709, 716, 724, 731, 738, 746,
    753, 760, 768, 775, 783, 790, 798, 805, 812, 820, 827, 835, 842, 850, 857,
    865, 872, 879, 887, 894, 902, 910, 917, 925, 932, 940, 948, 955, 963, 970,
    978, 985, 993, 1001, 1008, 1016, 1024, 1031, 1039, 1047, 1054, 1062, 1070,
    1077, 1085, 1093, 1100, 1108, 1116, 1124, 1131, 1139, 1147, 1155, 1162, 1170,
    1178, 1186, 1194, 1202, 1209, 1217, 1225, 1233, 1241, 1249, 1257, 1265, 1273,
    1281, 1289, 1297, 1305, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6, 7, 8, 9, 10, 11, 12,
    14, 15, 17, 18, 20, 22, 24, 27, 29, 32, 36, 40, 44, 49, 54, 59, 65, 71, 78,
    84, 91, 98, 105, 112, 118, 125, 132, 139, 146, 153, 160, 167, 174, 181, 188,
    195, 202, 209, 216, 224, 231, 238, 245, 252, 259, 266, 273, 280, 287, 294,
    302, 309, 316, 323, 330, 337, 345, 352, 359, 366, 373, 380, 388, 395, 402,
    409, 416, 424, 431, 438, 445, 452, 460, 467, 474, 481, 488, 495, 503, 510,
    517, 524, 531, 539, 546, 553, 560, 568, 575, 582, 589, 596, 604, 611, 618,
    625, 633, 640, 647, 654, 662, 669, 676, 684, 691, 698, 706, 713, 720, 728,
    735, 742, 750, 757, 764, 771, 779, 786, 794, 801, 809, 816, 824, 831, 838,
    846, 853, 861, 868, 875, 883, 890, 898, 906, 913, 921, 928, 936, 943, 951,
    958, 966, 974, 981, 989, 996, 1004, 1011, 1019, 1027, 1034, 1042, 1050, 1057,
    1065, 1073, 1080, 1088, 1096, 1103, 1111, 1119, 1126, 1134, 1142, 1150, 1157,
    1165, 1173, 1181, 1189, 1196, 1204, 1212, 1220, 1228, 1236, 1244, 1252, 1259,
    1267, 1275, 1283, 1291, 1299, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6, 7, 8, 9, 10, 11,
    12, 14, 15, 16, 18, 20, 22, 24, 26, 29, 32, 35, 39, 43, 48, 53, 59, 64, 71,
    77, 83, 90, 97, 104, 111, 117, 124, 131, 138, 145, 152, 159, 166, 173, 180,
    187, 194, 201, 208, 215, 222, 229, 236, 243, 250, 257, 264, 272, 279, 286,
    293, 300, 307, 314, 321, 328, 336, 343, 350, 357, 364, 371, 378, 386, 393,
    400, 407, 414, 421, 429, 436, 443, 450, 457, 464, 472, 478, 486, 493, 500,
    507, 515, 522, 529, 536, 543, 550, 558, 565, 572, 579, 586, 594, 601, 608,
    615, 622, 630, 637, 644, 651, 659, 666, 673, 680, 687, 695, 702, 710, 717,
    724, 731, 739, 746, 753, 761, 768, 776, 783, 790, 798, 805, 812, 820, 827,
    835, 842, 849, 857, 864, 872, 879, 886, 894, 902, 909, 917, 924, 932, 939,
    946, 954, 962, 969, 977, 984, 992, 999, 1007, 1015, 1022, 1030, 1037, 1045,
    1052, 1060, 1068, 1075, 1083, 1091, 1098, 1106, 1114, 1122, 1129, 1137, 1145,
    1152, 1160, 1168, 1175, 1183, 1191, 1199, 1207, 1214, 1223, 1230, 1238, 1246,
    1254, 1262, 1270, 1278, 1286, 1294, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6, 7, 8, 9,
    10, 11, 12, 14, 15, 16, 18, 20, 22, 24, 26, 29, 32, 35, 39, 43, 48, 53, 58,
    64, 70, 76, 83, 89, 96, 103, 110, 116, 123, 130, 137, 144, 151, 158, 165,
    172, 179, 186, 193, 200, 207, 214, 221, 228, 235, 242, 249, 256, 263, 270,
    277, 284, 291, 298, 305, 312, 320, 327, 334, 341, 348, 355, 362, 369, 376,
    384, 390, 398, 405, 412, 419, 426, 433, 441, 448, 455, 462, 469, 476, 483,
    490, 498, 505, 512, 519, 526, 533, 541, 548, 555, 562, 569, 576, 584, 591,
    598, 605, 612, 620, 627, 634, 641, 648, 656, 663, 670, 677, 684, 692, 699,
    706, 714, 721, 728, 735, 743, 750, 757, 764, 772, 779, 787, 794, 801, 809,
    816, 823, 831, 838, 846, 853, 860, 868, 875, 883, 890, 898, 905, 913, 920,
    927, 935, 942, 950, 957, 965, 972, 980, 988, 995, 1003, 1010, 1018, 1025,
    1033, 1040, 1048, 1056, 1063, 1071, 1078, 1086, 1094, 1101, 1109, 1117, 1124,
    1132, 1140, 1147, 1155, 1163, 1170, 1178, 1186, 1194, 1202, 1209, 1217, 1225,
    1233, 1241, 1249, 1257, 1265, 1272, 1280, 1288, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6,
    7, 8, 9, 10, 11, 12, 14, 15, 16, 18, 20, 22, 24, 26, 28, 31, 35, 38, 42, 47,
    52, 57, 63, 69, 75, 82, 88, 95, 102, 109, 115, 122, 129, 136, 143, 150, 157,
    164, 170, 178, 184, 191, 198, 205, 212, 219, 226, 233, 240, 247, 254, 261,
    268, 275, 282, 289, 297, 303, 311, 318, 325, 332, 339, 346, 353, 360, 367,
    374, 382, 388, 396, 403, 410, 417, 424, 431, 438, 445, 453, 460, 467, 474,
    481, 488, 495, 502, 510, 516, 524, 531, 538, 545, 552, 559, 567, 574, 581,
    588, 595, 602, 609, 617, 624, 631, 638, 645, 652, 660, 667, 674, 681, 689,
    696, 703, 710, 718, 725, 732, 739, 746, 754, 761, 768, 776, 783, 790, 798,
    805, 812, 820, 827, 834, 842, 849, 857, 864, 871, 879, 886, 894, 901, 909,
    916, 923, 931, 938, 946, 953, 961, 968, 976, 983, 991, 998, 1006, 1013, 1021,
    1028, 1036, 1043, 1051, 1059, 1066, 1074, 1081, 1089, 1097, 1104, 1112, 1119,
    1127, 1135, 1143, 1150, 1158, 1165, 1173, 1181, 1189, 1197, 1204, 1212, 1220,
    1228, 1236, 1243, 1251, 1259, 1267, 1275, 1283, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6,
    7, 8, 9, 10, 11, 12, 13, 15, 16, 18, 19, 21, 23, 26, 28, 31, 34, 38, 42, 47,
    51, 57, 62, 68, 75, 81, 88, 94, 101, 108, 114, 121, 128, 135, 142, 149, 156,
    163, 169, 176, 183, 190, 197, 204, 211, 218, 225, 232, 239, 246, 253, 260,
    267, 274, 281, 288, 295, 302, 309, 316, 323, 330, 337, 344, 351, 358, 365,
    372, 379, 386, 394, 400, 408, 415, 422, 429, 436, 443, 450, 457, 464, 471,
    479, 485, 493, 500, 507, 514, 521, 528, 535, 542, 550, 557, 564, 571, 578,
    585, 592, 599, 607, 614, 621, 628, 635, 642, 649, 657, 664, 671, 678, 685,
    693, 700, 707, 714, 721, 729, 736, 743, 750, 757, 765, 772, 780, 787, 794,
    801, 809, 816, 823, 831, 838, 845, 853, 860, 867, 875, 882, 890, 897, 904,
    912, 919, 927, 934, 942, 949, 956, 964, 971, 979, 986, 994, 1001, 1009, 1016,
    1024, 1031, 1039, 1046, 1054, 1062, 1069, 1077, 1084, 1092, 1099, 1107, 1115,
    1122, 1130, 1138, 1145, 1153, 1160, 1168, 1176, 1184, 1191, 1199, 1207, 1215,
    1223, 1230, 1238, 1246, 1254, 1262, 1269, 1277, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6,
    7, 8, 9, 10, 11, 12, 13, 15, 16, 18, 19, 21, 23, 26, 28, 31, 34, 38, 42, 46,
    51, 56, 62, 68, 74, 80, 87, 93, 100, 107, 114, 120, 127, 134, 141, 148, 155,
    161, 168, 175, 182, 189, 196, 203, 209, 217, 223, 230, 237, 244, 251, 258,
    265, 272, 279, 286, 293, 300, 307, 314, 321, 328, 335, 342, 349, 356, 363,
    370, 377, 384, 392, 398, 406, 413, 420, 427, 434, 441, 448, 455, 462, 469,
    476, 483, 490, 497, 505, 511, 518, 525, 533, 540, 547, 554, 561, 568, 575,
    582, 589, 597, 604, 611, 618, 625, 632, 639, 646, 654, 660, 668, 675, 682,
    689, 697, 704, 711, 718, 725, 733, 740, 747, 754, 761, 769, 776, 783, 791,
    798, 805, 812, 820, 827, 834, 842, 849, 856, 863, 871, 878, 886, 893, 900,
    908, 915, 923, 930, 937, 945, 952, 960, 967, 975, 982, 989, 997, 1004, 1012,
    1019, 1027, 1034, 1042, 1049, 1057, 1064, 1072, 1080, 1087, 1095, 1102, 1110,
    1117, 1125, 1133, 1140, 1148, 1155, 1163, 1171, 1179, 1186, 1194, 1202, 1210,
    1217, 1225, 1233, 1241, 1248, 1256, 1264, 1272, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6,
    7, 8, 9, 10, 11, 12, 13, 15, 16, 18, 19, 21, 23, 25, 28, 31, 34, 37, 41, 46,
    50, 56, 61, 67, 73, 79, 86, 92, 99, 106, 113, 119, 126, 133, 140, 147, 153,
    160, 167, 174, 181, 188, 195, 201, 208, 215, 222, 229, 236, 243, 250, 256,
    264, 271, 278, 284, 291, 298, 305, 312, 319, 326, 333, 340, 347, 354, 361,
    368, 375, 382, 390, 396, 404, 411, 418, 425, 432, 438, 446, 453, 460, 467,
    474, 481, 488, 495, 502, 509, 516, 523, 530, 537, 544, 551, 558, 565, 573,
    580, 587, 594, 601, 608, 615, 622, 629, 636, 643, 650, 657, 665, 672, 679,
    686, 693, 700, 708, 715, 722, 729, 736, 744, 751, 758, 765, 772, 780, 787,
    794, 801, 809, 816, 823, 831, 838, 845, 852, 860, 867, 874, 882, 889, 896,
    904, 911, 919, 926, 933, 941, 948, 955, 963, 970, 978, 985, 993, 1000, 1007,
    1015, 1022, 1030, 1037, 1045, 1052, 1060, 1067, 1075, 1082, 1090, 1098, 1105,
    1113, 1120, 1128, 1135, 1143, 1150, 1158, 1166, 1174, 1181, 1189, 1197, 1204,
    1212, 1220, 1228, 1235, 1243, 1251, 1259, 1267, 0, 1, 2, 2, 3, 3, 4, 5, 6, 6,
    7, 8, 9, 10, 11, 12, 13, 15, 16, 17, 19, 21, 23, 25, 28, 30, 34, 37, 41, 45,
    50, 55, 60, 66, 72, 79, 85, 92, 98, 105, 112, 118, 125, 132, 139, 145, 152,
    159, 166, 173, 179, 186, 193, 200, 207, 214, 221, 228, 234, 241, 248, 255,
    262, 269, 276, 283, 290, 297, 304, 311, 318, 325, 332, 338, 346, 353, 359,
    366, 373, 380, 387, 394, 401, 408, 415, 422, 429, 436, 443, 450, 457, 464,
    472, 478, 486, 493, 500, 506, 513, 520, 528, 535, 542, 549, 556, 563, 570,
    577, 584, 591, 598, 605, 612, 619, 626, 633, 640, 647, 654, 662, 669, 676,
    683, 690, 697, 704, 712, 719, 726, 733, 740, 747, 755, 762, 769, 776, 783,
    791, 798, 805, 812, 820, 827, 834, 841, 848, 856, 863, 871, 878, 885, 893,
    900, 907, 915, 922, 929, 937, 944, 951, 959, 966, 973, 981, 988, 996, 1003,
    1011, 1018, 1025, 1033, 1040, 1048, 1055, 1063, 1070, 1078, 1085, 1093, 1100,
    1108, 1115, 1123, 1131, 1138, 1146, 1153, 1161, 1169, 1176, 1184, 1192, 1199,
    1207, 1215, 1222, 1230, 1238, 1246, 1253, 1261, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 23, 25, 27, 30, 33, 37, 40, 45,
    49, 54, 60, 66, 72, 78, 84, 91, 97, 104, 111, 117, 124, 131, 138, 144, 151,
    158, 164, 171, 178, 185, 192, 199, 205, 212, 219, 226, 233, 240, 247, 253,
    261, 267, 274, 281, 288, 295, 302, 309, 316, 323, 330, 336, 344, 351, 358,
    365, 371, 378, 385, 392, 399, 406, 413, 420, 427, 434, 441, 448, 455, 462,
    469, 476, 483, 490, 497, 504, 511, 518, 525, 532, 539, 546, 553, 560, 567,
    574, 581, 588, 595, 602, 609, 616, 623, 630, 637, 644, 651, 659, 665, 673,
    680, 687, 694, 701, 708, 715, 723, 729, 737, 744, 751, 758, 765, 773, 780,
    787, 794, 801, 809, 816, 823, 830, 838, 845, 852, 859, 867, 874, 881, 889,
    896, 903, 910, 918, 925, 932, 940, 947, 954, 962, 969, 977, 984, 991, 999,
    1006, 1014, 1021, 1028, 1036, 1043, 1051, 1058, 1066, 1073, 1081, 1088, 1096,
    1103, 1111, 1118, 1126, 1133, 1141, 1148, 1156, 1164, 1171, 1179, 1187, 1194,
    1202, 1210, 1217, 1225, 1233, 1240, 1248, 1256, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 23, 25, 27, 30, 33, 36, 40, 44,
    49, 54, 59, 65, 71, 77, 83, 90, 96, 103, 110, 116, 123, 130, 136, 143, 150,
    157, 163, 170, 177, 184, 191, 197, 204, 211, 218, 225, 231, 238, 245, 252,
    259, 266, 273, 279, 286, 293, 300, 307, 314, 321, 328, 335, 342, 349, 356,
    363, 370, 376, 383, 390, 397, 404, 411, 418, 425, 432, 439, 446, 453, 460,
    467, 474, 481, 488, 495, 501, 508, 515, 523, 529, 536, 543, 550, 557, 564,
    571, 578, 585, 592, 599, 606, 613, 620, 627, 634, 641, 648, 656, 662, 670,
    677, 684, 691, 698, 705, 712, 719, 726, 733, 740, 748, 755, 762, 769, 776,
    784, 791, 798, 805, 812, 819, 827, 834, 841, 848, 855, 863, 870, 877, 885,
    892, 899, 906, 914, 921, 928, 936, 943, 950, 958, 965, 972, 980, 987, 994,
    1002, 1009, 1016, 1024, 1031, 1039, 1046, 1054, 1061, 1069, 1076, 1083, 1091,
    1098, 1106, 1113, 1121, 1128, 1136, 1144, 1151, 1159, 1166, 1174, 1182, 1189,
    1197, 1204, 1212, 1220, 1227, 1235, 1243, 1251, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 21, 22, 25, 27, 30, 33, 36, 40, 44,
    48, 53, 58, 64, 70, 76, 83, 89, 95, 102, 109, 115, 122, 129, 135, 142, 149,
    156, 162, 169, 176, 183, 189, 196, 203, 210, 217, 223, 230, 237, 244, 250,
    257, 264, 271, 278, 285, 291, 299, 305, 312, 319, 326, 333, 340, 347, 354,
    361, 368, 374, 381, 388, 395, 402, 409, 416, 423, 430, 437, 444, 451, 457,
    464, 471, 478, 485, 492, 499, 506, 513, 520, 527, 534, 541, 548, 555, 562,
    569, 576, 583, 590, 596, 603, 610, 617, 624, 631, 638, 645, 652, 659, 667,
    674, 681, 688, 695, 702, 709, 716, 723, 730, 737, 744, 751, 759, 766, 773,
    780, 787, 794, 801, 809, 816, 823, 830, 837, 844, 852, 859, 866, 873, 881,
    888, 895, 902, 910, 917, 924, 932, 939, 946, 953, 961, 968, 975, 983, 990,
    997, 1005, 1012, 1020, 1027, 1034, 1042, 1049, 1057, 1064, 1071, 1079, 1086,
    1094, 1101, 1109, 1116, 1124, 1131, 1139, 1146, 1154, 1161, 1169, 1176, 1184,
    1192, 1199, 1207, 1215, 1222, 1230, 1237, 1245, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 20, 22, 24, 27, 29, 32, 36, 39, 43,
    48, 53, 58, 63, 69, 75, 82, 88, 95, 101, 108, 114, 121, 128, 134, 141, 148,
    154, 161, 168, 174, 181, 188, 195, 201, 208, 215, 222, 228, 235, 242, 249,
    256, 263, 270, 276, 283, 290, 297, 304, 311, 317, 324, 331, 338, 345, 352,
    359, 366, 372, 379, 386, 393, 400, 407, 414, 421, 427, 435, 441, 448, 455,
    462, 469, 476, 483, 490, 496, 503, 510, 517, 524, 531, 538, 545, 552, 559,
    566, 573, 580, 587, 594, 601, 608, 615, 621, 629, 635, 642, 649, 656, 663,
    670, 677, 685, 692, 699, 706, 713, 720, 727, 734, 741, 748, 755, 762, 769,
    776, 784, 791, 798, 805, 812, 819, 827, 833, 841, 848, 855, 862, 870, 877,
    884, 891, 899, 906, 913, 920, 928, 935, 942, 949, 957, 964, 971, 978, 986,
    993, 1000, 1008, 1015, 1022, 1030, 1037, 1045, 1052, 1059, 1067, 1074, 1082,
    1089, 1096, 1104, 1111, 1119, 1126, 1134, 1141, 1149, 1156, 1164, 1172, 1179,
    1187, 1194, 1202, 1209, 1217, 1225, 1232, 1240, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 18, 20, 22, 24, 27, 29, 32, 35, 39, 43,
    47, 52, 57, 63, 69, 75, 81, 87, 94, 100, 107, 113, 120, 127, 133, 140, 147,
    153, 160, 167, 173, 180, 187, 194, 200, 207, 214, 221, 227, 234, 241, 247,
    254, 261, 268, 275, 282, 288, 295, 302, 309, 316, 323, 329, 336, 343, 350,
    357, 364, 370, 377, 384, 391, 398, 405, 412, 419, 425, 432, 439, 446, 453,
    460, 467, 474, 481, 487, 494, 501, 508, 515, 522, 529, 536, 543, 549, 556,
    563, 570, 577, 584, 591, 598, 605, 612, 618, 626, 633, 639, 646, 653, 660,
    667, 674, 681, 688, 695, 702, 709, 716, 724, 730, 738, 745, 752, 759, 766,
    773, 780, 787, 794, 801, 809, 816, 823, 830, 837, 844, 851, 859, 866, 873,
    880, 887, 895, 902, 909, 916, 923, 931, 938, 945, 952, 960, 967, 974, 982,
    989, 996, 1003, 1011, 1018, 1025, 1033, 1040, 1047, 1055, 1062, 1070, 1077,
    1084, 1092, 1099, 1107, 1114, 1121, 1129, 1137, 1144, 1152, 1159, 1167, 1174,
    1182, 1189, 1197, 1204, 1212, 1220, 1227, 1235, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 15, 17, 18, 20, 22, 24, 26, 29, 32, 35, 38, 42,
    47, 52, 57, 62, 68, 74, 80, 86, 93, 99, 106, 112, 119, 126, 132, 139, 146,
    152, 159, 166, 172, 179, 186, 192, 199, 206, 212, 219, 226, 232, 239, 246,
    253, 260, 266, 273, 280, 287, 294, 300, 307, 314, 321, 327, 334, 341, 348,
    355, 362, 368, 375, 382, 389, 396, 403, 410, 416, 423, 430, 437, 444, 451,
    458, 464, 471, 478, 485, 492, 498, 505, 512, 519, 526, 533, 540, 547, 554,
    561, 567, 574, 581, 588, 595, 602, 609, 616, 623, 630, 636, 643, 650, 657,
    664, 671, 678, 685, 692, 699, 706, 713, 720, 727, 734, 741, 748, 755, 762,
    770, 777, 784, 791, 798, 805, 812, 819, 826, 833, 840, 848, 855, 862, 869,
    876, 883, 891, 898, 905, 912, 919, 927, 934, 941, 948, 956, 963, 970, 977,
    985, 992, 999, 1006, 1014, 1021, 1028, 1036, 1043, 1050, 1058, 1065, 1072,
    1080, 1087, 1095, 1102, 1109, 1117, 1124, 1132, 1139, 1147, 1154, 1162, 1169,
    1177, 1184, 1192, 1199, 1207, 1215, 1222, 1230, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 15, 17, 18, 20, 22, 24, 26, 29, 32, 35, 38, 42,
    46, 51, 56, 61, 67, 73, 79, 86, 92, 98, 105, 111, 118, 125, 131, 138, 144,
    151, 158, 164, 171, 178, 184, 191, 197, 204, 211, 218, 224, 231, 238, 244,
    251, 258, 265, 271, 278, 285, 292, 299, 305, 312, 319, 326, 333, 339, 346,
    353, 360, 366, 373, 380, 387, 394, 401, 408, 414, 421, 428, 435, 442, 448,
    455, 462, 469, 476, 483, 489, 496, 503, 510, 517, 524, 530, 537, 544, 551,
    558, 565, 572, 578, 585, 592, 599, 606, 613, 620, 627, 633, 640, 647, 654,
    661, 668, 675, 682, 689, 696, 703, 710, 717, 724, 731, 738, 745, 752, 759,
    766, 773, 780, 787, 794, 801, 808, 816, 822, 830, 837, 844, 851, 858, 865,
    872, 880, 887, 894, 901, 908, 915, 923, 930, 937, 944, 951, 959, 966, 973,
    980, 988, 995, 1002, 1009, 1017, 1024, 1031, 1039, 1046, 1053, 1060, 1068,
    1075, 1083, 1090, 1097, 1105, 1112, 1119, 1127, 1134, 1142, 1149, 1157, 1164,
    1172, 1179, 1187, 1194, 1202, 1209, 1217, 1225, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 15, 17, 18, 20, 22, 24, 26, 28, 31, 34, 38, 42,
    46, 50, 55, 61, 66, 72, 79, 85, 91, 98, 104, 111, 117, 124, 130, 137, 143,
    150, 156, 163, 170, 176, 183, 190, 196, 203, 210, 216, 223, 230, 236, 243,
    250, 257, 263, 270, 277, 283, 290, 297, 304, 310, 317, 324, 331, 338, 344,
    351, 358, 364, 371, 378, 385, 392, 399, 405, 412, 419, 426, 433, 439, 446,
    453, 460, 467, 473, 480, 487, 494, 500, 507, 514, 521, 528, 535, 542, 548,
    555, 562, 569, 576, 583, 589, 596, 603, 610, 617, 624, 630, 638, 644, 651,
    658, 665, 672, 679, 686, 693, 700, 707, 714, 720, 728, 735, 742, 749, 756,
    763, 770, 777, 784, 791, 798, 805, 812, 819, 826, 833, 840, 847, 854, 862,
    869, 876, 883, 890, 897, 904, 911, 919, 926, 933, 940, 947, 954, 962, 969,
    976, 983, 990, 998, 1005, 1012, 1020, 1027, 1034, 1041, 1049, 1056, 1063,
    1071, 1078, 1085, 1093, 1100, 1107, 1115, 1122, 1130, 1137, 1144, 1152, 1159,
    1167, 1174, 1182, 1189, 1197, 1204, 1212, 1219, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 10, 11, 12, 13, 14, 15, 17, 18, 20, 22, 24, 26, 28, 31, 34, 37, 41,
    45, 50, 55, 60, 66, 72, 78, 84, 90, 97, 103, 110, 116, 123, 129, 136, 142,
    149, 155, 162, 168, 175, 182, 188, 195, 202, 208, 215, 221, 228, 235, 241,
    248, 255, 262, 268, 275, 282, 289, 295, 302, 309, 316, 322, 329, 336, 343,
    349, 356, 363, 370, 376, 383, 390, 397, 403, 410, 417, 424, 430, 437, 444,
    451, 457, 464, 471, 478, 484, 491, 498, 505, 512, 519, 525, 532, 539, 546,
    553, 559, 566, 573, 580, 587, 593, 600, 607, 614, 621, 627, 635, 641, 648,
    655, 662, 669, 676, 683, 690, 697, 703, 711, 717, 724, 731, 738, 745, 752,
    759, 766, 773, 780, 787, 794, 801, 808, 815, 822, 829, 837, 844, 851, 858,
    865, 872, 879, 886, 893, 900, 907, 915, 922, 929, 936, 943, 950, 958, 965,
    972, 979, 986, 993, 1001, 1008, 1015, 1022, 1030, 1037, 1044, 1051, 1059,
    1066, 1073, 1081, 1088, 1095, 1102, 1110, 1117, 1125, 1132, 1139, 1147, 1154,
    1162, 1169, 1177, 1184, 1192, 1199, 1207, 1214, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 9, 10, 12, 13, 14, 15, 16, 18, 20, 21, 23, 26, 28, 31, 34, 37, 41,
    45, 49, 54, 60, 65, 71, 77, 83, 89, 96, 102, 109, 115, 122, 128, 135, 141,
    148, 154, 161, 167, 174, 181, 187, 194, 200, 207, 214, 220, 227, 234, 240,
    247, 254, 260, 267, 274, 280, 287, 294, 300, 307, 314, 320, 327, 334, 341,
    347, 354, 361, 368, 374, 381, 388, 395, 401, 408, 415, 422, 428, 435, 442,
    449, 455, 462, 469, 476, 482, 489, 496, 503, 509, 516, 523, 530, 536, 543,
    550, 557, 564, 570, 577, 584, 591, 598, 604, 611, 618, 625, 632, 638, 645,
    652, 659, 666, 673, 680, 687, 693, 700, 707, 714, 721, 728, 735, 742, 749,
    756, 763, 770, 777, 784, 791, 798, 805, 812, 819, 826, 833, 840, 847, 854,
    861, 868, 875, 882, 889, 896, 904, 911, 918, 925, 932, 939, 946, 953, 961,
    968, 975, 982, 989, 996, 1004, 1011, 1018, 1025, 1032, 1040, 1047, 1054,
    1062, 1069, 1076, 1083, 1091, 1098, 1105, 1113, 1120, 1127, 1135, 1142, 1150,
    1157, 1164, 1172, 1179, 1187, 1194, 1202, 1209, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6,
    7, 8, 9, 9, 10, 11, 13, 14, 15, 16, 18, 20, 21, 23, 25, 28, 30, 33, 37, 40,
    44, 49, 54, 59, 64, 70, 76, 82, 89, 95, 101, 108, 114, 121, 127, 134, 140,
    147, 153, 160, 166, 173, 179, 186, 192, 199, 206, 212, 219, 225, 232, 239,
    245, 252, 259, 265, 272, 278, 285, 292, 299, 305, 312, 319, 325, 332, 339,
    346, 352, 359, 366, 372, 379, 386, 393, 399, 406, 412, 419, 426, 433, 439,
    446, 453, 460, 466, 473, 480, 486, 493, 500, 507, 514, 520, 527, 534, 541,
    547, 554, 561, 568, 574, 581, 588, 595, 601, 608, 615, 622, 629, 635, 642,
    649, 656, 663, 670, 677, 683, 690, 697, 704, 711, 718, 725, 732, 739, 746,
    752, 759, 766, 773, 780, 787, 794, 801, 808, 815, 822, 829, 836, 843, 850,
    857, 864, 871, 878, 885, 893, 900, 907, 914, 921, 928, 935, 942, 949, 956,
    964, 971, 978, 985, 992, 999, 1007, 1014, 1021, 1028, 1035, 1043, 1050, 1057,
    1064, 1072, 1079, 1086, 1093, 1101, 1108, 1115, 1123, 1130, 1137, 1145, 1152,
    1160, 1167, 1174, 1182, 1189, 1197, 1204, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 8,
    8, 9, 10, 11, 13, 14, 15, 16, 18, 19, 21, 23, 25, 28, 30, 33, 36, 40, 44, 48,
    53, 58, 64, 69, 75, 81, 88, 94, 100, 107, 113, 120, 126, 133, 139, 146, 152,
    159, 165, 172, 178, 185, 191, 198, 204, 211, 217, 224, 231, 237, 244, 251,
    257, 264, 270, 277, 284, 290, 297, 304, 310, 317, 324, 330, 337, 344, 350,
    357, 364, 370, 377, 384, 391, 397, 404, 410, 417, 424, 431, 437, 444, 451,
    457, 464, 471, 477, 484, 491, 498, 504, 511, 518, 525, 531, 538, 545, 551,
    558, 565, 572, 578, 585, 592, 599, 606, 612, 619, 626, 632, 639, 646, 653,
    660, 667, 674, 680, 687, 694, 701, 708, 715, 722, 728, 735, 742, 749, 756,
    763, 770, 777, 784, 791, 798, 804, 811, 818, 826, 833, 840, 847, 854, 861,
    868, 874, 882, 889, 896, 903, 910, 917, 924, 931, 938, 945, 952, 959, 967,
    973, 981, 988, 995, 1002, 1009, 1017, 1024, 1031, 1038, 1045, 1053, 1060,
    1067, 1074, 1081, 1089, 1096, 1103, 1111, 1118, 1125, 1133, 1140, 1147, 1155,
    1162, 1169, 1177, 1184, 1192, 1199, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 8, 8, 9,
    10, 11, 12, 14, 15, 16, 18, 19, 21, 23, 25, 27, 30, 33, 36, 40, 44, 48, 53,
    58, 63, 69, 75, 81, 87, 93, 99, 106, 112, 119, 125, 132, 138, 145, 151, 157,
    164, 170, 177, 184, 190, 197, 203, 210, 216, 223, 229, 236, 242, 249, 256,
    262, 269, 275, 282, 289, 295, 302, 309, 315, 322, 329, 335, 342, 349, 355,
    362, 368, 375, 382, 389, 395, 402, 408, 415, 422, 429, 435, 442, 448, 455,
    462, 469, 475, 482, 488, 495, 502, 509, 515, 522, 529, 535, 542, 549, 556,
    562, 569, 576, 582, 589, 596, 603, 609, 616, 623, 630, 637, 643, 650, 657,
    664, 670, 677, 684, 691, 698, 704, 711, 718, 725, 732, 739, 746, 753, 760,
    766, 773, 780, 787, 794, 801, 808, 815, 822, 829, 836, 843, 850, 857, 864,
    871, 878, 885, 892, 899, 906, 913, 920, 927, 934, 941, 948, 955, 962, 969,
    977, 984, 991, 998, 1005, 1012, 1019, 1027, 1034, 1041, 1048, 1055, 1062,
    1070, 1077, 1084, 1091, 1099, 1106, 1113, 1120, 1128, 1135, 1143, 1150, 1157,
    1165, 1172, 1179, 1187, 1194, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 8, 8, 9, 10,
    11, 12, 14, 15, 16, 18, 19, 21, 23, 25, 27, 30, 33, 36, 39, 43, 47, 52, 57,
    62, 68, 74, 80, 86, 92, 99, 105, 111, 118, 124, 131, 137, 143, 150, 156, 163,
    169, 176, 182, 189, 195, 202, 208, 215, 221, 228, 234, 241, 248, 254, 261,
    267, 274, 280, 287, 294, 300, 307, 313, 320, 327, 333, 340, 347, 353, 360,
    366, 373, 380, 387, 393, 400, 406, 413, 420, 426, 433, 440, 446, 453, 460,
    466, 473, 479, 486, 493, 500, 506, 513, 520, 526, 533, 540, 546, 553, 560,
    566, 573, 580, 586, 593, 600, 607, 613, 620, 627, 634, 640, 647, 654, 661,
    667, 674, 681, 688, 695, 701, 708, 715, 722, 729, 736, 742, 749, 756, 763,
    770, 777, 784, 791, 797, 804, 811, 818, 825, 832, 839, 846, 853, 860, 867,
    874, 881, 888, 895, 902, 909, 916, 923, 930, 937, 944, 951, 958, 965, 972,
    979, 987, 994, 1001, 1008, 1015, 1022, 1029, 1036, 1044, 1051, 1058, 1065,
    1072, 1079, 1087, 1094, 1101, 1109, 1116, 1123, 1130, 1138, 1145, 1152, 1160,
    1167, 1174, 1182, 1189, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 8, 8, 9, 10, 11, 12,
    13, 15, 16, 17, 19, 21, 23, 25, 27, 29, 32, 35, 39, 43, 47, 52, 56, 62, 67,
    73, 79, 85, 91, 98, 104, 110, 117, 123, 130, 136, 142, 149, 155, 161, 168,
    175, 181, 187, 194, 201, 207, 213, 220, 227, 233, 240, 246, 253, 259, 266,
    272, 279, 285, 292, 299, 305, 312, 318, 325, 332, 338, 345, 351, 358, 364,
    371, 378, 385, 391, 398, 404, 411, 418, 424, 431, 438, 444, 451, 457, 464,
    470, 477, 484, 491, 497, 504, 510, 517, 524, 530, 537, 544, 550, 557, 564,
    570, 577, 584, 590, 597, 604, 610, 617, 624, 631, 637, 644, 651, 658, 664,
    671, 678, 684, 691, 698, 705, 712, 719, 725, 732, 739, 746, 753, 760, 767,
    773, 780, 787, 794, 801, 808, 815, 822, 829, 835, 842, 849, 856, 863, 870,
    877, 884, 891, 898, 905, 912, 919, 926, 933, 940, 947, 954, 961, 968, 975,
    982, 989, 997, 1004, 1011, 1018, 1025, 1032, 1039, 1046, 1054, 1061, 1068,
    1075, 1082, 1089, 1097, 1104, 1111, 1118, 1126, 1133, 1140, 1148, 1155, 1162,
    1170, 1177, 1184, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 8, 8, 9, 10, 11, 12, 13,
    15, 16, 17, 19, 21, 22, 24, 27, 29, 32, 35, 38, 42, 46, 51, 56, 61, 67, 72,
    78, 84, 91, 97, 103, 109, 116, 122, 128, 135, 141, 147, 154, 160, 167, 173,
    180, 186, 193, 199, 206, 212, 218, 225, 231, 238, 245, 251, 257, 264, 271,
    277, 284, 290, 297, 304, 310, 317, 323, 330, 336, 343, 349, 356, 363, 369,
    376, 383, 389, 396, 402, 409, 416, 422, 429, 435, 442, 449, 455, 462, 468,
    475, 481, 488, 495, 501, 508, 515, 521, 528, 535, 541, 548, 554, 561, 568,
    574, 581, 587, 594, 601, 607, 614, 621, 628, 634, 641, 648, 655, 661, 668,
    675, 681, 688, 695, 702, 709, 715, 722, 729, 736, 743, 749, 756, 763, 770,
    777, 784, 790, 797, 804, 811, 818, 825, 832, 839, 846, 853, 859, 866, 873,
    880, 887, 894, 901, 908, 915, 922, 929, 936, 943, 950, 957, 964, 971, 978,
    985, 992, 999, 1006, 1014, 1021, 1028, 1035, 1042, 1049, 1056, 1063, 1070,
    1078, 1085, 1092, 1099, 1106, 1114, 1121, 1128, 1136, 1143, 1150, 1157, 1165,
    1172, 1179, 0, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 7, 8, 9, 10, 11, 12, 13, 15, 16,
    17, 19, 20, 22, 24, 27, 29, 32, 35, 38, 42, 46, 50, 55, 60, 66, 72, 78, 84,
    90, 96, 102, 108, 115, 121, 127, 134, 140, 146, 153, 159, 166, 172, 179, 185,
    192, 198, 204, 211, 217, 224, 230, 237, 243, 250, 256, 263, 269, 276, 282,
    289, 295, 302, 308, 315, 322, 328, 335, 341, 348, 354, 361, 367, 374, 381,
    387, 394, 400, 407, 413, 420, 426, 433, 440, 446, 453, 459, 466, 472, 479,
    486, 492, 499, 506, 512, 519, 525, 532, 539, 545, 552, 558, 565, 572, 578,
    585, 592, 598, 605, 612, 618, 625, 632, 638, 645, 652, 658, 665, 672, 678,
    685, 692, 699, 706, 712, 719, 726, 733, 739, 746, 753, 760, 767, 773, 780,
    787, 794, 801, 808, 814, 821, 828, 835, 842, 849, 856, 863, 870, 876, 883,
    890, 897, 904, 911, 918, 925, 932, 939, 946, 953, 960, 967, 974, 981, 988,
    995, 1002, 1009, 1016, 1023, 1030, 1038, 1045, 1052, 1059, 1066, 1073, 1080,
    1088, 1095, 1102, 1109, 1116, 1124, 1131, 1138, 1145, 1153, 1160, 1167, 1175,
    0, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 7, 8, 9, 10, 11, 12, 13, 15, 16, 17, 19, 20,
    22, 24, 26, 29, 31, 34, 38, 41, 45, 50, 55, 60, 65, 71, 77, 83, 89, 95, 101,
    108, 114, 120, 126, 133, 139, 145, 152, 158, 165, 171, 177, 184, 190, 197,
    203, 209, 216, 222, 229, 235, 242, 248, 254, 261, 267, 274, 281, 287, 294,
    300, 306, 313, 320, 326, 333, 339, 346, 352, 359, 366, 372, 379, 385, 392,
    398, 405, 411, 418, 424, 431, 437, 444, 451, 457, 464, 470, 477, 483, 490,
    497, 503, 510, 516, 523, 529, 536, 543, 549, 556, 562, 569, 576, 582, 589,
    596, 602, 609, 615, 622, 629, 635, 642, 649, 655, 662, 669, 675, 682, 689,
    696, 702, 709, 716, 723, 729, 736, 743, 750, 756, 763, 770, 777, 783, 790,
    797, 804, 811, 818, 825, 831, 838, 845, 852, 859, 866, 873, 880, 887, 893,
    900, 907, 914, 921, 928, 935, 942, 949, 956, 963, 970, 977, 984, 991, 998,
    1005, 1012, 1019, 1026, 1033, 1040, 1047, 1054, 1061, 1069, 1076, 1083, 1090,
    1097, 1104, 1112, 1119, 1126, 1133, 1141, 1148, 1155, 1162, 1170, 0, 1, 1, 2,
    3, 3, 4, 4, 5, 6, 7, 7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 19, 20, 22, 24, 26,
    29, 31, 34, 37, 41, 45, 49, 54, 59, 65, 70, 76, 82, 88, 94, 100, 107, 113,
    119, 125, 132, 138, 144, 151, 157, 164, 170, 176, 182, 189, 195, 202, 208,
    214, 221, 227, 234, 240, 247, 253, 260, 266, 273, 279, 286, 292, 299, 305,
    312, 318, 325, 331, 338, 344, 351, 357, 364, 370, 377, 383, 390, 396, 403,
    409, 416, 422, 429, 435, 442, 448, 455, 461, 468, 474, 481, 488, 494, 501,
    507, 514, 520, 527, 534, 540, 547, 553, 560, 566, 573, 579, 586, 593, 599,
    606, 612, 619, 626, 632, 639, 646, 652, 659, 666, 672, 679, 686, 693, 699,
    706, 713, 719, 726, 733, 740, 746, 753, 760, 767, 773, 780, 787, 794, 801,
    807, 814, 821, 828, 835, 842, 848, 855, 862, 869, 876, 883, 890, 896, 903,
    910, 917, 924, 931, 938, 945, 952, 959, 966, 973, 980, 987, 994, 1001, 1008,
    1015, 1022, 1029, 1036, 1043, 1050, 1057, 1064, 1071, 1078, 1086, 1092, 1100,
    1107, 1114, 1121, 1129, 1136, 1143, 1150, 1157, 1165, 0, 1, 1, 2, 3, 3, 4, 4,
    5, 6, 7, 7, 8, 9, 10, 11, 12, 13, 14, 16, 17, 18, 20, 22, 24, 26, 28, 31, 34,
    37, 41, 45, 49, 54, 59, 64, 69, 75, 81, 87, 93, 99, 106, 112, 118, 124, 131,
    137, 143, 150, 156, 162, 169, 175, 181, 188, 194, 201, 207, 213, 220, 226,
    232, 239, 245, 252, 258, 264, 271, 277, 284, 290, 297, 303, 310, 316, 323,
    329, 336, 342, 349, 355, 362, 368, 375, 381, 388, 394, 401, 407, 414, 420,
    427, 433, 440, 446, 453, 459, 466, 472, 479, 485, 492, 498, 505, 511, 518,
    524, 531, 538, 544, 551, 557, 564, 570, 577, 583, 590, 596, 603, 610, 616,
    623, 630, 636, 643, 649, 656, 663, 669, 676, 683, 689, 696, 703, 709, 716,
    723, 730, 736, 743, 750, 756, 763, 770, 777, 783, 790, 797, 804, 811, 817,
    824, 831, 838, 844, 852, 858, 865, 872, 879, 886, 893, 900, 906, 913, 920,
    927, 934, 941, 948, 955, 962, 969, 976, 983, 990, 997, 1004, 1011, 1018,
    1025, 1032, 1039, 1046, 1052, 1060, 1067, 1074, 1081, 1088, 1095, 1102, 1110,
    1117, 1124, 1131, 1138, 1146, 1153, 1160, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 7, 7,
    8, 9, 10, 11, 12, 13, 14, 16, 17, 18, 20, 22, 24, 26, 28, 31, 34, 37, 40, 44,
    48, 53, 58, 63, 69, 74, 80, 86, 92, 99, 105, 111, 117, 123, 130, 136, 142,
    149, 155, 161, 168, 174, 180, 187, 193, 199, 205, 212, 218, 224, 231, 237,
    244, 250, 257, 263, 270, 276, 282, 289, 295, 301, 308, 315, 321, 328, 334,
    340, 347, 353, 360, 366, 373, 379, 386, 392, 399, 405, 412, 418, 425, 431,
    438, 444, 451, 457, 463, 470, 477, 483, 489, 496, 502, 509, 515, 522, 529,
    535, 542, 548, 555, 561, 568, 574, 581, 587, 594, 600, 607, 614, 620, 627,
    633, 640, 647, 653, 660, 666, 673, 679, 686, 693, 700, 706, 713, 720, 726,
    733, 740, 746, 753, 760, 767, 773, 780, 787, 794, 800, 807, 814, 821, 827,
    834, 841, 848, 855, 861, 868, 875, 882, 889, 896, 903, 909, 916, 923, 930,
    937, 944, 951, 958, 965, 971, 978, 985, 992, 999, 1006, 1013, 1020, 1027,
    1034, 1041, 1048, 1055, 1062, 1069, 1077, 1083, 1091, 1098, 1105, 1112, 1119,
    1126, 1134, 1141, 1148, 1155, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 7, 7, 8, 9, 10,
    11, 12, 13, 14, 15, 17, 18, 20, 22, 23, 26, 28, 30, 33, 36, 40, 44, 48, 52,
    57, 63, 68, 74, 80, 86, 92, 98, 104, 110, 116, 122, 129, 135, 141, 148, 154,
    160, 166, 173, 179, 185, 192, 198, 204, 210, 217, 223, 230, 236, 242, 249,
    255, 261, 268, 274, 281, 287, 294, 300, 306, 313, 319, 326, 332, 338, 345,
    351, 358, 364, 371, 377, 384, 390, 397, 403, 410, 416, 423, 429, 435, 442,
    448, 455, 461, 468, 474, 481, 487, 494, 500, 507, 513, 520, 526, 533, 539,
    546, 552, 559, 565, 571, 578, 585, 591, 598, 604, 611, 617, 624, 630, 637,
    644, 650, 657, 663, 670, 676, 683, 690, 697, 703, 710, 716, 723, 730, 736,
    743, 750, 757, 763, 770, 776, 783, 790, 797, 804, 810, 817, 824, 831, 837,
    844, 851, 858, 865, 871, 878, 885, 892, 899, 905, 912, 919, 926, 933, 940,
    947, 954, 960, 967, 974, 981, 988, 995, 1002, 1009, 1016, 1023, 1030, 1037,
    1044, 1051, 1058, 1065, 1072, 1079, 1086, 1093, 1100, 1108, 1115, 1122, 1129,
    1136, 1143, 1150, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 17, 18, 20, 21, 23, 25, 28, 30, 33, 36, 39, 43, 47, 52, 57, 62, 67,
    73, 79, 85, 91, 97, 103, 109, 115, 122, 128, 134, 140, 146, 152, 159, 165,
    172, 178, 184, 190, 197, 203, 209, 216, 222, 228, 235, 241, 247, 254, 260,
    266, 273, 279, 286, 292, 298, 305, 311, 318, 324, 330, 337, 343, 349, 356,
    363, 369, 375, 382, 388, 395, 401, 408, 414, 420, 427, 433, 440, 446, 452,
    459, 465, 472, 478, 485, 491, 498, 504, 511, 517, 524, 530, 536, 543, 549,
    556, 562, 569, 575, 582, 588, 595, 601, 608, 615, 621, 628, 634, 641, 647,
    654, 660, 667, 673, 680, 687, 693, 700, 707, 713, 720, 727, 733, 740, 747,
    753, 760, 766, 773, 780, 787, 793, 800, 807, 814, 820, 827, 834, 841, 847,
    854, 861, 868, 874, 881, 888, 895, 902, 908, 915, 922, 929, 936, 943, 950,
    956, 963, 970, 977, 984, 991, 998, 1005, 1012, 1019, 1026, 1033, 1039, 1047,
    1054, 1061, 1068, 1074, 1082, 1089, 1096, 1103, 1110, 1117, 1124, 1131, 1138,
    1146, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 7, 7, 8, 9, 10, 11, 12, 13, 14, 15, 17,
    18, 20, 21, 23, 25, 27, 30, 33, 36, 39, 43, 47, 51, 56, 61, 67, 72, 78, 84,
    90, 96, 102, 108, 114, 121, 127, 133, 139, 145, 151, 158, 164, 170, 176, 183,
    189, 195, 201, 208, 214, 220, 227, 233, 240, 246, 252, 258, 265, 271, 278,
    284, 290, 297, 303, 309, 316, 322, 329, 335, 341, 348, 354, 361, 367, 373,
    380, 386, 393, 399, 405, 412, 418, 425, 431, 438, 444, 450, 457, 463, 470,
    476, 482, 489, 495, 502, 508, 515, 521, 528, 534, 540, 547, 553, 560, 566,
    573, 579, 585, 592, 598, 605, 612, 618, 625, 631, 638, 644, 651, 657, 664,
    670, 677, 684, 690, 697, 704, 710, 717, 723, 730, 737, 743, 750, 757, 763,
    770, 776, 783, 790, 797, 803, 810, 817, 823, 830, 837, 844, 850, 857, 864,
    871, 877, 884, 891, 898, 905, 911, 918, 925, 932, 939, 946, 952, 959, 966,
    973, 980, 987, 994, 1001, 1007, 1014, 1021, 1028, 1035, 1042, 1049, 1056,
    1063, 1070, 1077, 1084, 1091, 1098, 1105, 1113, 1120, 1127, 1134, 1141, 0, 1,
    1, 2, 3, 3, 4, 4, 5, 6, 7, 7, 8, 9, 10, 11, 12, 13, 14, 15, 17, 18, 20, 21,
    23, 25, 27, 30, 32, 35, 39, 42, 46, 51, 56, 61, 66, 72, 77, 83, 89, 95, 101,
    107, 113, 120, 126, 132, 138, 144, 150, 157, 163, 169, 175, 182, 188, 194,
    200, 206, 213, 219, 226, 232, 238, 244, 251, 257, 263, 270, 276, 282, 289,
    295, 301, 308, 314, 321, 327, 333, 340, 346, 352, 359, 365, 372, 378, 384,
    391, 397, 403, 410, 416, 422, 429, 435, 442, 448, 454, 461, 467, 474, 480,
    487, 493, 499, 506, 512, 519, 525, 531, 538, 544, 551, 557, 563, 570, 577,
    583, 589, 596, 602, 609, 615, 622, 628, 635, 641, 648, 654, 661, 667, 674,
    681, 687, 694, 700, 707, 714, 720, 727, 733, 740, 747, 753, 760, 766, 773,
    780, 786, 793, 800, 807, 813, 820, 826, 833, 840, 847, 853, 860, 867, 874,
    880, 887, 894, 901, 908, 914, 921, 928, 935, 942, 948, 955, 962, 969, 976,
    983, 989, 996, 1003, 1010, 1017, 1024, 1031, 1038, 1045, 1052, 1059, 1065,
    1073, 1080, 1087, 1094, 1101, 1108, 1115, 1122, 1129, 1136, 0, 1, 1, 2, 3, 3,
    4, 4, 5, 6, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 18, 19, 21, 23, 25, 27,
    29, 32, 35, 38, 42, 46, 50, 55, 60, 65, 71, 76, 82, 88, 94, 100, 106, 112,
    119, 125, 131, 137, 143, 149, 156, 162, 168, 174, 180, 187, 193, 199, 205,
    212, 218, 224, 230, 237, 243, 249, 255, 262, 268, 274, 281, 287, 293, 300,
    306, 312, 319, 325, 331, 338, 344, 351, 357, 363, 370, 376, 382, 389, 395,
    401, 408, 414, 420, 427, 433, 440, 446, 452, 459, 465, 471, 478, 484, 491,
    497, 503, 510, 516, 523, 529, 535, 542, 548, 555, 561, 567, 574, 580, 587,
    593, 600, 606, 613, 619, 626, 632, 639, 645, 651, 658, 664, 671, 678, 684,
    691, 697, 704, 710, 717, 724, 730, 737, 743, 750, 756, 763, 770, 776, 783,
    790, 796, 803, 810, 816, 823, 830, 836, 843, 850, 857, 863, 870, 877, 883,
    890, 897, 904, 910, 917, 924, 931, 938, 944, 951, 958, 965, 972, 978, 985,
    992, 999, 1006, 1013, 1020, 1026, 1034, 1040, 1047, 1054, 1061, 1068, 1075,
    1082, 1089, 1096, 1103, 1110, 1117, 1124, 1132, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6,
    6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 18, 19, 21, 23, 25, 27, 29, 32, 35,
    38, 42, 46, 50, 54, 59, 65, 70, 76, 82, 87, 93, 99, 105, 112, 118, 124, 130,
    136, 142, 148, 155, 161, 167, 173, 179, 185, 192, 198, 204, 210, 216, 223,
    229, 235, 241, 248, 254, 260, 267, 273, 279, 286, 292, 298, 304, 311, 317,
    323, 330, 336, 342, 349, 355, 361, 368, 374, 380, 387, 393, 399, 406, 412,
    418, 425, 431, 437, 444, 450, 456, 463, 469, 476, 482, 488, 495, 501, 507,
    514, 520, 526, 533, 539, 546, 552, 558, 565, 571, 577, 584, 590, 597, 603,
    610, 616, 623, 629, 636, 642, 648, 655, 661, 668, 675, 681, 688, 694, 701,
    707, 714, 720, 727, 734, 740, 747, 753, 760, 766, 773, 780, 786, 793, 800,
    806, 813, 819, 826, 833, 839, 846, 853, 860, 866, 873, 880, 886, 893, 900,
    907, 913, 920, 927, 934, 940, 947, 954, 961, 968, 974, 981, 988, 995, 1002,
    1009, 1015, 1022, 1029, 1036, 1043, 1050, 1057, 1064, 1071, 1078, 1085, 1092,
    1099, 1106, 1113, 1120, 1127, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9, 10,
    11, 12, 13, 14, 15, 16, 18, 19, 21, 23, 25, 27, 29, 32, 34, 38, 41, 45, 49,
    54, 59, 64, 69, 75, 81, 87, 93, 98, 105, 111, 117, 123, 129, 135, 141, 147,
    153, 160, 166, 172, 178, 184, 190, 196, 203, 209, 215, 221, 228, 234, 240,
    246, 252, 259, 265, 271, 278, 284, 290, 297, 303, 309, 315, 322, 328, 334,
    340, 347, 353, 360, 366, 372, 378, 385, 391, 397, 404, 410, 416, 423, 429,
    435, 441, 448, 454, 461, 467, 473, 480, 486, 492, 499, 505, 511, 518, 524,
    530, 537, 543, 549, 556, 562, 569, 575, 581, 588, 594, 601, 607, 613, 620,
    626, 633, 639, 645, 652, 658, 665, 672, 678, 685, 691, 698, 704, 711, 717,
    724, 730, 737, 743, 750, 756, 763, 770, 776, 783, 789, 796, 803, 809, 816,
    823, 829, 836, 843, 849, 856, 863, 869, 876, 883, 889, 896, 903, 909, 916,
    923, 930, 936, 943, 950, 957, 963, 970, 977, 984, 991, 998, 1004, 1011, 1018,
    1025, 1032, 1039, 1046, 1052, 1059, 1066, 1073, 1080, 1087, 1094, 1101, 1108,
    1115, 1122, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15,
    16, 18, 19, 21, 22, 24, 26, 29, 31, 34, 37, 41, 45, 49, 53, 58, 63, 69, 74,
    80, 86, 92, 98, 104, 110, 116, 122, 128, 134, 140, 146, 152, 158, 165, 171,
    177, 183, 189, 195, 201, 208, 214, 220, 226, 232, 239, 245, 251, 257, 264,
    270, 276, 282, 288, 295, 301, 307, 314, 320, 326, 333, 339, 345, 351, 358,
    364, 370, 376, 383, 389, 395, 402, 408, 414, 421, 427, 433, 439, 446, 452,
    458, 465, 471, 477, 484, 490, 496, 503, 509, 515, 522, 528, 534, 541, 547,
    553, 560, 566, 572, 579, 585, 592, 598, 604, 611, 617, 624, 630, 636, 643,
    649, 655, 662, 669, 675, 682, 688, 695, 701, 708, 714, 721, 727, 734, 740,
    746, 753, 760, 766, 773, 779, 786, 793, 799, 806, 812, 819, 826, 832, 839,
    846, 852, 859, 866, 872, 879, 886, 892, 899, 905, 912, 919, 926, 932, 939,
    946, 953, 959, 966, 973, 980, 987, 993, 1000, 1007, 1014, 1021, 1028, 1034,
    1041, 1048, 1055, 1062, 1069, 1076, 1083, 1090, 1097, 1104, 1110, 1118, 0, 1,
    1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 19, 21,
    22, 24, 26, 29, 31, 34, 37, 40, 44, 48, 53, 58, 63, 68, 74, 79, 85, 91, 97,
    103, 109, 115, 121, 127, 133, 139, 145, 151, 157, 163, 169, 176, 182, 188,
    194, 200, 206, 212, 219, 225, 231, 237, 243, 249, 256, 262, 268, 275, 281,
    287, 293, 300, 306, 312, 318, 324, 331, 337, 343, 350, 356, 362, 368, 374,
    381, 387, 393, 399, 406, 412, 419, 425, 431, 437, 443, 450, 456, 462, 469,
    475, 481, 488, 494, 500, 507, 513, 519, 525, 532, 538, 544, 551, 557, 563,
    570, 576, 582, 589, 595, 602, 608, 614, 621, 627, 634, 640, 646, 653, 659,
    666, 672, 679, 685, 691, 698, 704, 711, 717, 724, 730, 737, 743, 750, 756,
    763, 770, 776, 783, 789, 796, 802, 809, 816, 822, 829, 835, 842, 849, 855,
    862, 868, 875, 882, 888, 895, 902, 908, 915, 922, 929, 935, 942, 949, 955,
    962, 969, 976, 982, 989, 996, 1003, 1009, 1016, 1023, 1030, 1037, 1044, 1051,
    1058, 1064, 1071, 1078, 1085, 1092, 1099, 1106, 1113, 0, 1, 1, 2, 3, 3, 4, 4,
    5, 6, 6, 7, 8, 9, 10, 11, 11, 13, 14, 15, 16, 17, 19, 20, 22, 24, 26, 28, 31,
    34, 37, 40, 44, 48, 52, 57, 62, 67, 73, 78, 84, 90, 96, 102, 108, 114, 120,
    126, 132, 138, 144, 150, 156, 162, 168, 175, 181, 187, 193, 199, 205, 211,
    217, 224, 230, 236, 242, 248, 254, 261, 267, 273, 279, 285, 292, 298, 304,
    310, 317, 323, 329, 335, 341, 348, 354, 360, 366, 373, 379, 385, 391, 397,
    404, 410, 416, 423, 429, 435, 441, 448, 454, 460, 467, 473, 479, 485, 492,
    498, 504, 510, 517, 523, 529, 536, 542, 548, 554, 561, 567, 573, 580, 586,
    592, 599, 605, 612, 618, 624, 631, 637, 643, 650, 656, 663, 669, 676, 682,
    688, 695, 701, 708, 714, 721, 727, 734, 740, 746, 753, 760, 766, 773, 779,
    786, 792, 799, 805, 812, 819, 825, 832, 838, 845, 852, 858, 865, 871, 878,
    885, 891, 898, 905, 911, 918, 925, 931, 938, 945, 951, 958, 965, 972, 978,
    985, 992, 999, 1005, 1012, 1019, 1026, 1033, 1039, 1046, 1053, 1060, 1067,
    1074, 1081, 1088, 1095, 1101, 1109, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9,
    10, 10, 11, 12, 14, 15, 16, 17, 19, 20, 22, 24, 26, 28, 31, 33, 36, 40, 43,
    47, 52, 56, 61, 67, 72, 78, 83, 89, 95, 101, 107, 113, 119, 125, 131, 137,
    143, 149, 155, 161, 167, 173, 179, 186, 191, 198, 204, 210, 216, 222, 228,
    234, 241, 247, 253, 259, 265, 271, 278, 284, 290, 296, 302, 309, 315, 321,
    327, 333, 340, 346, 352, 358, 365, 371, 377, 383, 389, 396, 402, 408, 414,
    421, 427, 433, 439, 445, 452, 458, 464, 471, 477, 483, 489, 495, 502, 508,
    514, 521, 527, 533, 539, 545, 552, 558, 564, 571, 577, 583, 590, 596, 602,
    609, 615, 621, 628, 634, 641, 647, 653, 660, 666, 673, 679, 685, 692, 698,
    705, 711, 718, 724, 730, 737, 743, 750, 756, 763, 769, 776, 782, 789, 795,
    802, 809, 815, 822, 828, 835, 841, 848, 854, 861, 868, 874, 881, 888, 894,
    901, 907, 914, 921, 927, 934, 941, 947, 954, 961, 968, 974, 981, 988, 994,
    1001, 1008, 1015, 1022, 1028, 1035, 1042, 1049, 1056, 1063, 1069, 1076, 1083,
    1090, 1097, 1104, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9, 10, 10, 11, 12,
    13, 15, 16, 17, 19, 20, 22, 24, 26, 28, 30, 33, 36, 39, 43, 47, 51, 56, 61,
    66, 71, 77, 83, 88, 94, 100, 106, 112, 118, 124, 130, 136, 142, 148, 154,
    160, 166, 172, 178, 184, 190, 196, 203, 208, 215, 221, 227, 233, 239, 245,
    251, 258, 264, 270, 276, 282, 288, 295, 301, 307, 313, 319, 326, 332, 338,
    344, 350, 357, 363, 369, 375, 381, 388, 394, 400, 406, 412, 419, 425, 431,
    437, 443, 450, 456, 462, 468, 474, 481, 487, 493, 499, 506, 512, 518, 524,
    531, 537, 543, 549, 556, 562, 568, 574, 581, 587, 593, 600, 606, 612, 619,
    625, 631, 638, 644, 650, 657, 663, 670, 676, 682, 689, 695, 702, 708, 714,
    721, 727, 734, 740, 746, 753, 760, 766, 773, 779, 786, 792, 798, 805, 812,
    818, 825, 831, 838, 844, 851, 857, 864, 871, 877, 884, 890, 897, 904, 910,
    917, 923, 930, 937, 943, 950, 957, 963, 970, 977, 984, 990, 997, 1004, 1011,
    1017, 1024, 1031, 1038, 1045, 1051, 1058, 1065, 1072, 1079, 1086, 1092, 1100,
    0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9, 10, 10, 11, 12, 13, 15, 16, 17, 19,
    20, 22, 24, 26, 28, 30, 33, 36, 39, 43, 46, 51, 55, 60, 65, 71, 76, 82, 87,
    93, 99, 105, 111, 117, 123, 129, 135, 141, 147, 153, 159, 165, 171, 177, 183,
    189, 195, 201, 207, 213, 220, 226, 232, 238, 244, 250, 256, 262, 268, 275,
    281, 287, 293, 299, 305, 311, 317, 324, 330, 336, 342, 349, 355, 361, 367,
    373, 379, 386, 392, 398, 404, 410, 416, 423, 429, 435, 441, 447, 454, 460,
    466, 472, 478, 485, 491, 497, 503, 509, 516, 522, 528, 534, 540, 547, 553,
    559, 566, 572, 578, 584, 591, 597, 603, 610, 616, 622, 628, 635, 641, 648,
    654, 660, 667, 673, 679, 686, 692, 698, 705, 711, 718, 724, 730, 737, 743,
    750, 756, 763, 769, 776, 782, 789, 795, 802, 808, 815, 821, 828, 834, 841,
    847, 854, 860, 867, 873, 880, 886, 893, 900, 906, 913, 920, 926, 933, 940,
    946, 953, 959, 966, 973, 980, 986, 993, 1000, 1006, 1013, 1020, 1026, 1033,
    1040, 1047, 1054, 1061, 1068, 1074, 1081, 1088, 1095, 0, 1, 1, 2, 3, 3, 4, 4,
    5, 6, 6, 7, 8, 9, 9, 10, 11, 12, 13, 15, 16, 17, 18, 20, 22, 23, 25, 27, 30,
    32, 35, 39, 42, 46, 50, 55, 60, 65, 70, 75, 81, 87, 92, 98, 104, 110, 116,
    122, 128, 134, 140, 146, 152, 158, 164, 170, 176, 182, 188, 194, 200, 206,
    212, 218, 224, 230, 236, 242, 249, 255, 261, 267, 273, 279, 285, 291, 298,
    304, 310, 316, 322, 328, 334, 341, 347, 353, 359, 365, 371, 377, 384, 390,
    396, 402, 408, 414, 421, 427, 433, 439, 445, 451, 458, 464, 470, 476, 482,
    489, 495, 501, 507, 513, 519, 526, 532, 538, 544, 551, 557, 563, 569, 576,
    582, 588, 594, 601, 607, 613, 619, 625, 632, 638, 645, 651, 657, 664, 670,
    676, 683, 689, 695, 702, 708, 715, 721, 727, 734, 740, 747, 753, 759, 766,
    772, 779, 785, 791, 798, 805, 811, 818, 824, 831, 837, 844, 850, 857, 863,
    870, 876, 883, 889, 896, 903, 909, 916, 922, 929, 936, 942, 949, 955, 962,
    969, 975, 982, 989, 996, 1002, 1009, 1016, 1022, 1029, 1036, 1043, 1050,
    1056, 1063, 1070, 1077, 1083, 1091, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9,
    9, 10, 11, 12, 13, 14, 16, 17, 18, 20, 21, 23, 25, 27, 30, 32, 35, 38, 42,
    46, 50, 54, 59, 64, 69, 75, 80, 86, 92, 97, 103, 109, 115, 121, 127, 133,
    139, 145, 151, 157, 163, 169, 175, 181, 187, 193, 199, 205, 211, 217, 223,
    229, 235, 241, 247, 253, 259, 265, 271, 277, 284, 290, 296, 302, 308, 314,
    320, 326, 333, 339, 345, 351, 357, 363, 369, 376, 382, 388, 394, 400, 406,
    412, 419, 424, 431, 437, 443, 449, 455, 462, 468, 474, 480, 486, 492, 499,
    505, 511, 517, 523, 529, 535, 542, 548, 554, 560, 566, 573, 579, 585, 592,
    598, 604, 610, 617, 623, 629, 635, 642, 648, 654, 661, 667, 673, 680, 686,
    692, 699, 705, 711, 718, 724, 730, 737, 743, 750, 756, 763, 769, 775, 782,
    788, 795, 801, 808, 814, 821, 827, 834, 840, 847, 853, 860, 866, 873, 879,
    886, 892, 899, 905, 912, 918, 925, 932, 938, 945, 951, 958, 965, 971, 978,
    985, 991, 998, 1005, 1012, 1018, 1025, 1032, 1038, 1045, 1052, 1059, 1066,
    1072, 1079, 1086, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9, 9, 10, 11, 12,
    13, 14, 16, 17, 18, 20, 21, 23, 25, 27, 29, 32, 35, 38, 41, 45, 49, 54, 58,
    63, 68, 74, 79, 85, 91, 97, 102, 108, 114, 120, 126, 132, 138, 144, 150, 156,
    161, 168, 174, 180, 185, 191, 197, 203, 210, 216, 222, 227, 234, 239, 246,
    252, 258, 264, 270, 276, 282, 288, 294, 300, 306, 312, 319, 325, 331, 337,
    343, 349, 355, 361, 368, 374, 380, 386, 392, 398, 404, 410, 416, 422, 429,
    435, 441, 447, 453, 459, 465, 472, 478, 484, 490, 496, 502, 508, 515, 521,
    527, 533, 539, 545, 551, 558, 564, 570, 576, 583, 589, 595, 601, 608, 614,
    620, 626, 632, 639, 645, 651, 658, 664, 670, 677, 683, 689, 696, 702, 708,
    715, 721, 727, 734, 740, 747, 753, 759, 766, 772, 779, 785, 791, 798, 804,
    811, 817, 824, 830, 836, 843, 849, 856, 862, 869, 875, 882, 888, 895, 902,
    908, 915, 921, 928, 934, 941, 948, 954, 961, 967, 974, 980, 987, 994, 1001,
    1007, 1014, 1021, 1027, 1034, 1041, 1048, 1054, 1061, 1068, 1075, 1082, 0, 1,
    1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 9, 9, 10, 11, 12, 13, 14, 15, 17, 18, 20,
    21, 23, 25, 27, 29, 32, 34, 38, 41, 45, 49, 53, 58, 63, 68, 73, 79, 84, 90,
    96, 102, 107, 113, 119, 125, 131, 137, 143, 149, 155, 160, 166, 172, 178,
    184, 190, 196, 202, 208, 214, 220, 226, 232, 238, 244, 250, 256, 262, 268,
    274, 281, 287, 293, 299, 305, 311, 317, 323, 329, 335, 341, 347, 353, 359,
    366, 372, 378, 384, 390, 396, 402, 408, 414, 420, 426, 433, 439, 445, 451,
    457, 463, 469, 475, 482, 488, 494, 500, 506, 512, 518, 525, 530, 537, 543,
    549, 555, 561, 568, 574, 580, 586, 592, 599, 605, 611, 617, 624, 630, 636,
    642, 649, 655, 661, 667, 674, 680, 686, 693, 699, 705, 712, 718, 724, 730,
    737, 743, 750, 756, 762, 769, 775, 781, 788, 794, 801, 807, 814, 820, 826,
    833, 839, 846, 852, 859, 865, 872, 878, 885, 891, 898, 904, 911, 917, 924,
    930, 937, 944, 950, 957, 963, 970, 976, 983, 990, 997, 1003, 1010, 1017,
    1023, 1030, 1037, 1043, 1050, 1057, 1064, 1070, 1077, 0, 1, 1, 2, 3, 3, 4, 4,
    5, 6, 6, 7, 8, 8, 9, 10, 11, 12, 13, 14, 15, 17, 18, 19, 21, 23, 25, 27, 29,
    31, 34, 37, 41, 44, 48, 52, 57, 62, 67, 72, 78, 83, 89, 95, 101, 106, 112,
    118, 124, 130, 136, 142, 148, 153, 159, 165, 171, 177, 183, 189, 195, 201,
    207, 213, 219, 225, 231, 237, 243, 249, 255, 261, 267, 273, 279, 285, 291,
    297, 303, 309, 315, 321, 327, 333, 340, 346, 352, 358, 364, 370, 376, 382,
    388, 394, 400, 406, 412, 418, 424, 430, 437, 443, 449, 455, 461, 467, 473,
    479, 485, 492, 498, 504, 510, 516, 522, 528, 534, 540, 546, 553, 559, 565,
    571, 577, 584, 590, 596, 602, 608, 614, 621, 627, 633, 639, 646, 652, 658,
    664, 671, 677, 683, 690, 696, 702, 708, 715, 721, 727, 734, 740, 746, 753,
    759, 765, 772, 778, 785, 791, 797, 804, 810, 817, 823, 829, 836, 842, 849,
    855, 862, 868, 875, 881, 888, 894, 900, 907, 914, 920, 927, 933, 940, 946,
    953, 959, 966, 972, 979, 986, 992, 999, 1005, 1012, 1019, 1026, 1032, 1039,
    1046, 1053, 1059, 1066, 1073, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 8, 9,
    10, 11, 12, 13, 14, 15, 17, 18, 19, 21, 23, 24, 26, 29, 31, 34, 37, 40, 44,
    48, 52, 56, 61, 66, 72, 77, 83, 88, 94, 100, 106, 111, 117, 123, 129, 135,
    141, 146, 152, 158, 164, 170, 176, 182, 188, 194, 199, 206, 212, 218, 223,
    229, 235, 241, 247, 253, 259, 265, 271, 277, 283, 289, 296, 302, 307, 314,
    319, 326, 332, 338, 344, 350, 356, 362, 368, 374, 380, 386, 392, 398, 404,
    410, 416, 422, 428, 435, 441, 447, 453, 459, 465, 471, 477, 483, 489, 495,
    501, 507, 514, 520, 526, 532, 538, 544, 550, 556, 562, 569, 575, 581, 587,
    593, 599, 606, 612, 618, 624, 630, 637, 643, 649, 655, 662, 668, 674, 680,
    687, 693, 699, 705, 711, 718, 724, 731, 737, 743, 749, 756, 762, 769, 775,
    781, 788, 794, 800, 807, 813, 819, 826, 832, 839, 845, 852, 858, 864, 871,
    877, 884, 890, 897, 903, 910, 916, 923, 929, 936, 942, 949, 955, 962, 968,
    975, 982, 988, 995, 1001, 1008, 1015, 1022, 1028, 1035, 1042, 1048, 1055,
    1061, 1068, 0, 1, 1, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 8, 9, 10, 11, 12, 13, 14,
    15, 16, 18, 19, 21, 22, 24, 26, 28, 31, 34, 37, 40, 43, 47, 51, 56, 61, 66,
    71, 76, 82, 87, 93, 99, 105, 110, 116, 122, 128, 134, 140, 145, 151, 157,
    163, 169, 175, 181, 186, 193, 198, 204, 210, 216, 222, 228, 234, 240, 246,
    252, 258, 264, 270, 276, 282, 288, 294, 300, 306, 312, 318, 324, 330, 336,
    342, 348, 354, 360, 366, 372, 378, 384, 390, 396, 402, 408, 414, 420, 426,
    432, 439, 445, 451, 457, 463, 469, 475, 481, 487, 493, 499, 505, 511, 517,
    523, 529, 536, 541, 548, 554, 560, 566, 572, 578, 584, 591, 597, 603, 609,
    615, 621, 628, 634, 640, 646, 652, 659, 665, 671, 677, 684, 690, 696, 702,
    708, 715, 721, 727, 734, 740, 746, 753, 759, 765, 771, 778, 784, 791, 797,
    803, 810, 816, 822, 829, 835, 842, 848, 854, 861, 867, 874, 880, 887, 893,
    899, 906, 912, 919, 925, 932, 938, 945, 951, 958, 964, 971, 978, 984, 991,
    997, 1004, 1011, 1017, 1024, 1031, 1037, 1044, 1051, 1057, 1064, 0, 1, 1, 2,
    2, 3, 4, 4, 5, 5, 6, 7, 8, 8, 9, 10, 11, 12, 13, 14, 15, 16, 18, 19, 21, 22,
    24, 26, 28, 31, 33, 36, 39, 43, 47, 51, 55, 60, 65, 70, 76, 81, 87, 92, 98,
    104, 109, 115, 121, 127, 133, 139, 144, 150, 156, 162, 168, 174, 179, 185,
    191, 197, 203, 209, 215, 221, 227, 232, 239, 245, 251, 256, 262, 268, 274,
    280, 286, 292, 298, 304, 310, 316, 322, 328, 334, 340, 346, 352, 358, 364,
    370, 376, 382, 388, 394, 400, 406, 412, 418, 424, 430, 436, 442, 448, 454,
    460, 467, 473, 479, 485, 491, 497, 503, 509, 515, 521, 527, 533, 539, 545,
    551, 557, 563, 570, 576, 582, 588, 594, 600, 606, 612, 618, 625, 631, 637,
    643, 649, 656, 662, 668, 674, 681, 687, 693, 699, 705, 711, 718, 724, 730,
    737, 743, 749, 756, 762, 768, 775, 781, 787, 794, 800, 806, 813, 819, 825,
    832, 838, 844, 851, 857, 864, 870, 876, 883, 889, 896, 902, 909, 915, 922,
    928, 934, 941, 947, 954, 960, 967, 974, 980, 987, 993, 1000, 1007, 1013,
    1020, 1026, 1033, 1040, 1046, 1053, 1060, 0, 1, 1, 2, 2, 3, 4, 4, 5, 5, 6, 7,
    8, 8, 9, 10, 11, 12, 13, 14, 15, 16, 18, 19, 20, 22, 24, 26, 28, 30, 33, 36,
    39, 42, 46, 50, 55, 59, 64, 70, 75, 80, 86, 91, 97, 103, 109, 114, 120, 126,
    131, 137, 143, 149, 155, 161, 167, 172, 178, 184, 190, 196, 202, 208, 214,
    219, 225, 231, 237, 243, 249, 255, 261, 267, 273, 279, 285, 291, 297, 302,
    309, 314, 321, 327, 333, 338, 344, 350, 356, 362, 368, 374, 380, 386, 392,
    398, 404, 410, 416, 422, 428, 434, 440, 446, 452, 458, 464, 470, 476, 482,
    488, 494, 500, 506, 512, 518, 525, 531, 536, 543, 549, 555, 561, 567, 573,
    579, 585, 591, 597, 603, 610, 616, 622, 628, 634, 640, 647, 653, 659, 665,
    671, 678, 684, 690, 696, 702, 708, 715, 721, 727, 734, 740, 746, 752, 759,
    765, 771, 778, 784, 790, 796, 803, 809, 815, 822, 828, 834, 841, 847, 853,
    860, 866, 873, 879, 886, 892, 898, 905, 911, 918, 924, 931, 937, 944, 950,
    956, 963, 970, 976, 983, 989, 996, 1002, 1009, 1016, 1022, 1029, 1035, 1042,
    1049, 1055, 0, 1, 1, 2, 2, 3, 4, 4, 5, 5, 6, 7, 8, 8, 9, 10, 11, 12, 13, 14,
    15, 16, 18, 19, 20, 22, 24, 26, 28, 30, 33, 36, 39, 42, 46, 50, 54, 59, 64,
    69, 74, 80, 85, 91, 96, 102, 108, 113, 119, 125, 130, 136, 142, 148, 154,
    160, 166, 171, 177, 183, 189, 195, 201, 206, 212, 218, 224, 230, 236, 242,
    248, 254, 259, 265, 271, 277, 283, 289, 295, 301, 307, 313, 319, 325, 331,
    337, 343, 348, 355, 361, 367, 372, 378, 384, 390, 396, 402, 408, 414, 420,
    426, 432, 438, 444, 450, 456, 462, 468, 474, 480, 486, 492, 498, 504, 510,
    516, 522, 528, 534, 540, 546, 552, 558, 564, 570, 577, 583, 589, 595, 601,
    607, 613, 619, 625, 631, 638, 644, 650, 656, 662, 668, 675, 681, 687, 693,
    699, 705, 712, 718, 724, 730, 737, 743, 749, 755, 761, 768, 774, 780, 787,
    793, 799, 806, 812, 818, 825, 831, 837, 844, 850, 856, 863, 869, 875, 882,
    888, 895, 901, 907, 914, 920, 927, 933, 940, 946, 952, 959, 966, 972, 979,
    985, 992, 998, 1005, 1011, 1018, 1025, 1031, 1038, 1044, 1051, 0, 1, 1, 2, 2,
    3, 4, 4, 5, 5, 6, 7, 8, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 19, 20, 22, 24,
    26, 28, 30, 32, 35, 38, 42, 45, 49, 54, 58, 63, 68, 73, 79, 84, 90, 95, 101,
    107, 112, 118, 124, 129, 135, 141, 147, 153, 159, 164, 170, 176, 182, 188,
    193, 199, 205, 211, 217, 223, 228, 234, 240, 246, 252, 258, 264, 270, 276,
    282, 288, 293, 299, 305, 311, 317, 323, 329, 335, 341, 347, 353, 359, 365,
    370, 377, 382, 388, 394, 400, 406, 412, 418, 424, 430, 436, 442, 448, 454,
    460, 466, 472, 478, 484, 490, 496, 502, 508, 514, 520, 526, 532, 538, 544,
    550, 556, 562, 568, 574, 580, 586, 592, 598, 604, 610, 616, 623, 629, 635,
    641, 647, 653, 659, 665, 672, 678, 684, 690, 696, 702, 708, 715, 721, 727,
    733, 740, 746, 752, 758, 765, 771, 777, 783, 790, 796, 802, 809, 815, 821,
    827, 834, 840, 846, 853, 859, 865, 872, 878, 884, 891, 897, 904, 910, 916,
    923, 929, 936, 942, 948, 955, 962, 968, 975, 981, 988, 994, 1001, 1007, 1014,
    1020, 1027, 1034, 1040, 1047, 0, 1, 1, 2, 2, 3, 4, 4, 5, 5, 6, 7, 8, 8, 9,
    10, 11, 12, 13, 14, 15, 16, 17, 19, 20, 22, 23, 25, 27, 30, 32, 35, 38, 41,
    45, 49, 53, 58, 62, 67, 73, 78, 83, 89, 95, 100, 106, 112, 117, 123, 129,
    134, 140, 146, 152, 157, 163, 169, 175, 180, 186, 192, 198, 204, 210, 215,
    221, 227, 233, 239, 245, 251, 257, 262, 268, 274, 280, 286, 292, 298, 304,
    309, 315, 321, 327, 333, 339, 345, 351, 357, 363, 369, 375, 380, 387, 392,
    398, 404, 410, 416, 422, 428, 434, 440, 446, 452, 458, 464, 470, 476, 481,
    487, 493, 499, 505, 511, 517, 523, 529, 535, 541, 547, 553, 559, 565, 571,
    577, 583, 589, 595, 602, 607, 614, 620, 626, 632, 638, 644, 650, 656, 663,
    669, 675, 681, 687, 693, 699, 705, 712, 718, 724, 730, 736, 743, 749, 755,
    761, 768, 774, 780, 786, 793, 799, 805, 811, 818, 824, 830, 837, 843, 849,
    855, 862, 868, 874, 881, 887, 894, 900, 906, 913, 919, 925, 932, 938, 945,
    951, 958, 964, 971, 977, 984, 990, 997, 1003, 1010, 1016, 1023, 1029, 1036,
    1043, 0, 1, 1, 2, 2, 3, 4, 4, 5, 5, 6, 7, 8, 8, 9, 10, 11, 12, 13, 14, 15,
    16, 17, 19, 20, 22, 23, 25, 27, 29, 32, 35, 38, 41, 44, 48, 53, 57, 62, 67,
    72, 77, 83, 88, 94, 99, 105, 111, 116, 122, 128, 133, 139, 145, 150, 156,
    162, 168, 173, 179, 185, 191, 197, 203, 208, 214, 220, 226, 232, 238, 243,
    249, 255, 261, 267, 273, 279, 284, 290, 296, 302, 308, 314, 320, 326, 331,
    337, 343, 349, 355, 361, 367, 373, 378, 385, 390, 396, 402, 408, 414, 420,
    426, 432, 438, 444, 450, 456, 461, 467, 473, 479, 485, 491, 497, 503, 509,
    515, 521, 527, 533, 539, 545, 551, 557, 563, 569, 575, 581, 587, 593, 599,
    605, 611, 617, 623, 629, 635, 641, 647, 653, 660, 666, 672, 678, 684, 690,
    696, 702, 709, 715, 721, 727, 733, 739, 746, 752, 758, 764, 771, 777, 783,
    789, 795, 802, 808, 814, 820, 827, 833, 839, 846, 852, 858, 864, 871, 877,
    883, 890, 896, 903, 909, 915, 922, 928, 934, 941, 947, 954, 960, 967, 973,
    980, 986, 993, 999, 1006, 1012, 1019, 1025, 1032, 1038, 0, 1, 1, 2, 2, 3, 4,
    4, 5, 5, 6, 7, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 20, 21, 23, 25,
    27, 29, 32, 34, 37, 41, 44, 48, 52, 56, 61, 66, 71, 76, 82, 87, 93, 98, 104,
    110, 115, 121, 127, 132, 138, 144, 149, 155, 161, 167, 172, 178, 184, 190,
    196, 201, 207, 213, 219, 224, 230, 236, 242, 248, 254, 259, 265, 271, 277,
    283, 289, 294, 300, 306, 312, 318, 324, 330, 336, 341, 347, 353, 359, 365,
    371, 377, 383, 389, 394, 400, 406, 412, 418, 424, 430, 436, 442, 447, 453,
    459, 465, 471, 477, 483, 489, 495, 501, 506, 513, 518, 524, 530, 536, 542,
    548, 554, 560, 566, 572, 578, 584, 590, 596, 602, 608, 614, 620, 626, 632,
    638, 645, 651, 657, 663, 669, 675, 681, 687, 693, 699, 706, 712, 718, 724,
    730, 736, 742, 748, 755, 761, 767, 773, 780, 786, 792, 798, 804, 811, 817,
    823, 829, 836, 842, 848, 855, 861, 867, 873, 880, 886, 892, 899, 905, 911,
    918, 924, 931, 937, 943, 950, 956, 963, 969, 976, 982, 988, 995, 1001, 1008,
    1015, 1021, 1027, 1034, 0, 1, 1, 2, 2, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 10, 11,
    12, 13, 14, 15, 16, 17, 18, 20, 21, 23, 25, 27, 29, 31, 34, 37, 40, 44, 47,
    52, 56, 61, 65, 71, 76, 81, 87, 92, 98, 103, 109, 114, 120, 126, 131, 137,
    143, 148, 154, 160, 166, 171, 177, 183, 188, 194, 200, 206, 211, 217, 223,
    229, 235, 241, 246, 252, 258, 264, 270, 275, 281, 287, 293, 299, 304, 310,
    316, 322, 328, 334, 340, 346, 351, 357, 363, 369, 375, 381, 387, 392, 398,
    404, 410, 416, 422, 428, 434, 439, 445, 451, 457, 463, 469, 475, 481, 487,
    492, 498, 504, 510, 516, 522, 528, 534, 540, 546, 552, 558, 564, 570, 576,
    582, 587, 594, 599, 606, 612, 618, 624, 630, 636, 642, 648, 654, 660, 666,
    672, 678, 684, 690, 696, 703, 709, 715, 721, 727, 733, 739, 745, 752, 758,
    764, 770, 776, 782, 789, 795, 801, 807, 814, 820, 826, 832, 838, 845, 851,
    857, 864, 870, 876, 882, 889, 895, 901, 908, 914, 920, 927, 933, 939, 946,
    952, 959, 965, 972, 978, 984, 991, 997, 1004, 1010, 1017, 1023, 1030, 0, 1,
    1, 2, 2, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
    20, 21, 23, 25, 27, 29, 31, 34, 37, 40, 43, 47, 51, 55, 60, 65, 70, 75, 80,
    86, 91, 97, 102, 108, 113, 119, 125, 130, 136, 142, 147, 153, 159, 165, 170,
    176, 182, 187, 193, 199, 205, 210, 216, 222, 228, 233, 239, 245, 251, 256,
    262, 268, 274, 280, 286, 291, 297, 303, 309, 315, 320, 326, 332, 338, 344,
    350, 355, 361, 367, 373, 379, 385, 391, 396, 402, 408, 414, 420, 426, 431,
    437, 443, 449, 455, 461, 467, 472, 478, 484, 490, 496, 502, 508, 514, 519,
    525, 531, 537, 543, 549, 555, 561, 567, 573, 579, 585, 591, 597, 603, 609,
    615, 621, 627, 633, 639, 645, 651, 657, 663, 669, 675, 681, 687, 693, 699,
    706, 712, 718, 724, 730, 736, 742, 748, 755, 761, 767, 773, 779, 785, 792,
    798, 804, 810, 816, 822, 829, 835, 841, 847, 854, 860, 866, 872, 879, 885,
    891, 898, 904, 910, 917, 923, 929, 936, 942, 948, 955, 961, 968, 974, 980,
    987, 993, 1000, 1006, 1013, 1019, 1026, 0, 1, 1, 2, 2, 3, 4, 4, 5, 5, 6, 7,
    7, 8, 9, 10, 11, 12, 12, 13, 15, 16, 17, 18, 20, 21, 23, 25, 26, 29, 31, 33,
    36, 39, 43, 46, 50, 55, 59, 64, 69, 74, 80, 85, 90, 96, 101, 107, 112, 118,
    124, 129, 135, 141, 146, 152, 158, 163, 169, 175, 180, 186, 192, 198, 203,
    209, 215, 220, 226, 232, 238, 244, 249, 255, 261, 267, 272, 278, 284, 290,
    296, 301, 307, 313, 319, 325, 330, 336, 342, 348, 354, 359, 365, 371, 377,
    383, 389, 394, 400, 406, 412, 418, 424, 429, 435, 441, 447, 453, 459, 464,
    470, 476, 482, 488, 494, 499, 505, 511, 517, 523, 529, 535, 541, 547, 553,
    559, 564, 570, 576, 582, 588, 594, 600, 606, 612, 618, 624, 630, 636, 642,
    648, 654, 660, 666, 672, 678, 684, 690, 696, 703, 709, 715, 721, 727, 733,
    739, 745, 751, 757, 764, 770, 776, 782, 788, 794, 800, 807, 813, 819, 825,
    831, 838, 844, 850, 856, 863, 869, 875, 881, 888, 894, 900, 906, 913, 919,
    925, 932, 938, 944, 951, 957, 964, 970, 976, 983, 989, 996, 1002, 1009, 1015,
    1022, 0, 1, 1, 2, 2, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 10, 11, 11, 12, 13, 14,
    16, 17, 18, 19, 21, 23, 24, 26, 28, 31, 33, 36, 39, 42, 46, 50, 54, 59, 63,
    68, 74, 79, 84, 90, 95, 101, 106, 111, 117, 123, 128, 134, 140, 145, 151,
    157, 162, 168, 173, 179, 185, 191, 196, 202, 208, 213, 219, 225, 231, 236,
    242, 248, 253, 259, 265, 271, 277, 282, 288, 294, 300, 306, 311, 317, 323,
    329, 334, 340, 346, 352, 357, 363, 369, 375, 381, 387, 392, 398, 404, 410,
    416, 421, 427, 433, 439, 445, 451, 456, 462, 468, 474, 480, 486, 491, 497,
    503, 509, 515, 521, 526, 532, 538, 544, 550, 556, 562, 568, 574, 579, 586,
    591, 597, 603, 609, 615, 621, 627, 633, 639, 645, 651, 657, 663, 669, 675,
    681, 687, 693, 699, 706, 712, 718, 724, 730, 736, 742, 748, 754, 760, 766,
    773, 779, 785, 791, 797, 803, 809, 816, 822, 828, 834, 840, 847, 853, 859,
    865, 871, 878, 884, 890, 896, 903, 909, 915, 921, 928, 934, 941, 947, 953,
    960, 966, 972, 979, 985, 992, 998, 1005, 1011, 1017, 0, 1, 1, 2, 2, 3, 4, 4,
    5, 5, 6, 7, 7, 8, 9, 10, 11, 11, 12, 13, 14, 16, 17, 18, 19, 21, 22, 24, 26,
    28, 30, 33, 36, 39, 42, 46, 49, 54, 58, 63, 68, 73, 78, 83, 89, 94, 100, 105,
    111, 116, 122, 127, 133, 139, 144, 150, 156, 161, 167, 172, 178, 184, 189,
    195, 201, 206, 212, 218, 224, 229, 235, 241, 246, 252, 258, 264, 269, 275,
    281, 286, 292, 298, 304, 310, 315, 321, 327, 333, 339, 344, 350, 356, 362,
    367, 373, 379, 385, 390, 396, 402, 408, 414, 419, 425, 431, 437, 443, 448,
    454, 460, 466, 472, 477, 483, 489, 495, 501, 507, 512, 518, 524, 530, 536,
    542, 548, 553, 559, 565, 571, 577, 583, 589, 595, 601, 607, 613, 619, 624,
    630, 636, 642, 648, 654, 660, 666, 672, 678, 684, 690, 696, 702, 709, 715,
    721, 727, 733, 739, 745, 751, 757, 763, 769, 775, 781, 788, 794, 800, 806,
    812, 818, 824, 831, 837, 843, 849, 855, 862, 868, 874, 880, 886, 893, 899,
    905, 912, 918, 924, 930, 937, 943, 949, 956, 962, 968, 975, 981, 988, 994,
    1000, 1007, 1013, 0, 1, 1, 2, 2, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 10, 10, 11,
    12, 13, 14, 15, 17, 18, 19, 21, 22, 24, 26, 28, 30, 33, 35, 38, 42, 45, 49,
    53, 58, 62, 67, 72, 77, 83, 88, 93, 99, 104, 110, 115, 121, 126, 132, 138,
    143, 149, 154, 160, 165, 171, 177, 182, 188, 194, 200, 205, 211, 216, 222,
    228, 234, 239, 245, 251, 256, 262, 268, 274, 279, 285, 291, 296, 302, 308,
    314, 320, 325, 331, 337, 343, 348, 354, 360, 365, 371, 377, 383, 388, 394,
    400, 406, 412, 417, 423, 429, 435, 441, 446, 452, 458, 464, 469, 475, 481,
    487, 492, 498, 504, 510, 516, 522, 528, 533, 539, 545, 551, 557, 563, 569,
    574, 580, 586, 592, 598, 604, 610, 616, 622, 628, 634, 640, 646, 651, 657,
    663, 669, 675, 681, 687, 693, 699, 705, 711, 718, 724, 729, 736, 742, 748,
    754, 760, 766, 772, 778, 784, 790, 796, 803, 809, 815, 821, 827, 833, 839,
    846, 852, 858, 864, 870, 877, 883, 889, 895, 901, 908, 914, 920, 927, 933,
    939, 945, 952, 958, 964, 971, 977, 984, 990, 996, 1003, 1009, 0, 1, 1, 2, 2,
    3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 10, 10, 11, 12, 13, 14, 15, 17, 18, 19, 21, 22,
    24, 26, 28, 30, 32, 35, 38, 41, 45, 48, 53, 57, 62, 66, 71, 76, 82, 87, 92,
    98, 103, 109, 114, 120, 125, 131, 137, 142, 148, 153, 159, 164, 170, 176,
    181, 187, 193, 198, 204, 210, 215, 221, 227, 232, 238, 244, 249, 255, 261,
    266, 272, 278, 283, 289, 295, 301, 306, 312, 318, 324, 329, 335, 341, 347,
    352, 358, 364, 369, 375, 381, 386, 392, 398, 404, 410, 415, 421, 427, 433,
    438, 444, 450, 456, 461, 467, 473, 479, 485, 490, 496, 502, 508, 514, 519,
    525, 531, 537, 543, 548, 554, 560, 566, 572, 578, 583, 589, 595, 601, 607,
    613, 619, 625, 631, 637, 643, 649, 655, 661, 666, 672, 678, 684, 690, 696,
    702, 708, 714, 720, 726, 732, 739, 745, 751, 757, 763, 769, 775, 781, 787,
    793, 799, 805, 811, 818, 824, 830, 836, 842, 848, 854, 861, 867, 873, 879,
    885, 892, 898, 904, 910, 916, 923, 929, 935, 941, 948, 954, 961, 967, 973,
    980, 986, 992, 998, 1005, 0, 1, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 7, 8, 9, 10,
    10, 11, 12, 13, 14, 15, 16, 18, 19, 20, 22, 24, 26, 28, 30, 32, 35, 38, 41,
    44, 48, 52, 56, 61, 66, 71, 76, 81, 86, 92, 97, 102, 108, 113, 119, 125, 130,
    136, 141, 147, 152, 158, 163, 169, 175, 180, 186, 191, 197, 202, 208, 214,
    220, 225, 231, 237, 242, 248, 254, 259, 265, 271, 276, 282, 288, 293, 299,
    305, 310, 316, 322, 327, 333, 339, 345, 350, 356, 362, 368, 373, 379, 385,
    390, 396, 402, 408, 413, 419, 425, 431, 436, 442, 448, 454, 459, 465, 471,
    477, 482, 488, 494, 500, 505, 511, 517, 523, 529, 534, 540, 546, 552, 558,
    563, 569, 575, 581, 587, 593, 599, 604, 610, 616, 622, 628, 634, 640, 646,
    652, 658, 663, 669, 675, 681, 687, 693, 699, 705, 711, 717, 723, 729, 735,
    741, 747, 753, 759, 766, 772, 778, 784, 790, 796, 802, 808, 814, 820, 826,
    832, 839, 845, 851, 857, 863, 869, 875, 882, 888, 894, 900, 906, 913, 919,
    925, 931, 937, 944, 950, 957, 963, 969, 976, 982, 988, 994, 1001, 0, 1, 1, 2,
    2, 3, 3, 4, 5, 5, 6, 7, 7, 8, 9, 10, 10, 11, 12, 13, 14, 15, 16, 18, 19, 20,
    22, 24, 25, 27, 29, 32, 34, 37, 40, 44, 48, 51, 56, 60, 65, 70, 75, 80, 85,
    91, 96, 102, 107, 113, 118, 124, 129, 135, 140, 146, 151, 157, 162, 168, 173,
    179, 185, 190, 196, 201, 207, 212, 218, 224, 229, 235, 241, 246, 252, 258,
    263, 269, 275, 280, 286, 292, 297, 303, 309, 314, 320, 326, 332, 337, 343,
    348, 354, 360, 366, 371, 377, 383, 388, 394, 400, 406, 411, 417, 423, 428,
    434, 440, 446, 451, 457, 463, 469, 474, 480, 486, 492, 497, 503, 509, 514,
    520, 526, 532, 538, 543, 549, 555, 561, 567, 573, 578, 584, 590, 596, 602,
    608, 613, 619, 625, 631, 637, 643, 649, 655, 660, 666, 672, 678, 684, 690,
    696, 702, 708, 714, 720, 726, 732, 738, 744, 750, 756, 762, 768, 774, 780,
    786, 792, 799, 804, 811, 817, 823, 829, 835, 841, 847, 853, 860, 866, 872,
    878, 884, 890, 897, 903, 909, 915, 921, 928, 934, 940, 946, 953, 959, 965,
    972, 978, 984, 990, 997, 0, 1, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 7, 8, 9, 10, 10,
    11, 12, 13, 14, 15, 16, 18, 19, 20, 22, 23, 25, 27, 29, 32, 34, 37, 40, 43,
    47, 51, 55, 60, 64, 69, 74, 79, 85, 90, 95, 101, 106, 112, 117, 123, 128,
    134, 139, 145, 150, 156, 161, 167, 172, 178, 183, 189, 195, 200, 206, 211,
    217, 223, 228, 234, 239, 245, 251, 256, 262, 268, 273, 279, 285, 290, 296,
    302, 307, 313, 319, 324, 330, 336, 341, 347, 353, 358, 364, 370, 375, 381,
    386, 392, 398, 404, 409, 415, 421, 426, 432, 438, 444, 449, 455, 461, 466,
    472, 478, 483, 489, 495, 500, 506, 512, 518, 524, 529, 535, 541, 547, 553,
    558, 564, 570, 576, 582, 587, 593, 599, 605, 611, 617, 622, 628, 634, 640,
    646, 652, 658, 663, 669, 676, 681, 687, 693, 699, 705, 711, 717, 723, 729,
    735, 741, 747, 753, 759, 765, 771, 777, 783, 789, 795, 801, 807, 813, 819,
    825, 832, 838, 844, 850, 856, 862, 868, 874, 880, 887, 893, 899, 905, 911,
    918, 924, 930, 936, 942, 949, 955, 961, 968, 974, 980, 986, 993, 0, 1, 1, 2,
    2, 3, 3, 4, 5, 5, 6, 7, 7, 8, 9, 9, 10, 11, 12, 13, 14, 15, 16, 17, 19, 20,
    22, 23, 25, 27, 29, 31, 34, 37, 40, 43, 47, 50, 55, 59, 64, 69, 74, 79, 84,
    89, 94, 100, 105, 111, 116, 122, 127, 133, 138, 144, 149, 155, 160, 165, 171,
    176, 182, 188, 193, 199, 204, 210, 216, 221, 227, 232, 238, 243, 249, 255,
    260, 266, 272, 277, 283, 288, 294, 300, 306, 311, 317, 322, 328, 334, 339,
    345, 351, 356, 362, 368, 373, 379, 385, 390, 396, 402, 407, 413, 419, 424,
    430, 436, 441, 447, 453, 458, 464, 470, 476, 481, 487, 493, 498, 504, 510,
    516, 521, 527, 533, 539, 544, 550, 556, 561, 567, 573, 579, 585, 591, 596,
    602, 608, 614, 620, 626, 631, 637, 643, 649, 655, 661, 666, 673, 678, 684,
    690, 696, 702, 708, 714, 720, 726, 732, 738, 744, 750, 756, 762, 768, 774,
    780, 786, 792, 798, 804, 810, 816, 822, 828, 834, 840, 846, 852, 858, 865,
    871, 877, 883, 889, 895, 901, 908, 914, 920, 926, 932, 939, 945, 951, 957,
    964, 970, 976, 982, 989, 0, 1, 1, 2, 2, 3, 3, 4, 5, 5, 6, 7, 7, 8, 9, 9, 10,
    11, 12, 13, 14, 15, 16, 17, 19, 20, 21, 23, 25, 27, 29, 31, 34, 36, 39, 43,
    46, 50, 54, 58, 63, 68, 73, 78, 83, 88, 94, 99, 104, 110, 115, 121, 126, 132,
    137, 143, 148, 153, 159, 164, 170, 175, 181, 187, 192, 197, 203, 209, 214,
    220, 225, 231, 237, 242, 248, 253, 259, 265, 270, 276, 281, 287, 293, 298,
    304, 310, 315, 321, 326, 332, 338, 343, 349, 354, 360, 366, 372, 377, 383,
    388, 394, 400, 405, 411, 417, 422, 428, 434, 439, 445, 451, 456, 462, 468,
    473, 479, 485, 490, 496, 502, 507, 513, 519, 525, 530, 536, 542, 548, 553,
    559, 565, 570, 576, 582, 588, 594, 600, 605, 611, 617, 623, 629, 634, 640,
    646, 652, 658, 664, 670, 676, 681, 687, 693, 699, 705, 711, 717, 723, 729,
    735, 741, 747, 753, 759, 765, 770, 776, 782, 788, 794, 800, 807, 813, 819,
    825, 831, 837, 843, 849, 855, 861, 867, 873, 879, 885, 891, 898, 904, 910,
    916, 922, 929, 935, 941, 947, 953, 960, 966, 972, 978, 985, 0, 1, 1, 2, 2, 3,
    3, 4, 5, 5, 6, 6, 7, 8, 9, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 20, 21, 23,
    25, 27, 29, 31, 33, 36, 39, 42, 46, 49, 54, 58, 62, 67, 72, 77, 82, 87, 93,
    98, 103, 109, 114, 120, 125, 131, 136, 141, 147, 152, 158, 163, 169, 174,
    180, 185, 191, 196, 202, 207, 213, 219, 224, 230, 235, 241, 246, 252, 258,
    263, 269, 274, 280, 285, 291, 297, 302, 308, 313, 319, 325, 330, 336, 341,
    347, 353, 358, 364, 370, 375, 381, 386, 392, 398, 403, 409, 415, 420, 426,
    432, 437, 443, 448, 454, 460, 465, 471, 477, 482, 488, 494, 499, 505, 511,
    517, 522, 528, 534, 539, 545, 551, 556, 562, 568, 574, 580, 585, 591, 597,
    603, 608, 614, 620, 626, 632, 638, 643, 649, 655, 661, 667, 673, 678, 684,
    690, 696, 702, 708, 714, 720, 726, 732, 737, 743, 749, 755, 761, 767, 773,
    779, 785, 791, 797, 803, 809, 815, 821, 827, 833, 839, 845, 851, 857, 864,
    870, 876, 882, 888, 894, 900, 906, 912, 918, 925, 931, 937, 943, 950, 956,
    962, 968, 974, 981, 0, 1, 1, 2, 2, 3, 3, 4, 5, 5, 6, 6, 7, 8, 9, 9, 10, 11,
    12, 13, 14, 15, 16, 17, 18, 20, 21, 23, 25, 26, 28, 31, 33, 36, 39, 42, 45,
    49, 53, 57, 62, 66, 71, 76, 81, 87, 92, 97, 102, 108, 113, 119, 124, 130,
    135, 140, 146, 151, 157, 162, 168, 173, 179, 184, 190, 195, 201, 206, 212,
    217, 223, 228, 234, 239, 245, 250, 256, 262, 267, 273, 278, 284, 289, 295,
    301, 306, 312, 317, 323, 329, 334, 340, 345, 351, 357, 362, 368, 373, 379,
    384, 390, 396, 401, 407, 413, 418, 424, 429, 435, 441, 446, 452, 458, 463,
    469, 474, 480, 486, 491, 497, 503, 508, 514, 520, 525, 531, 537, 543, 548,
    554, 560, 565, 571, 577, 583, 588, 594, 600, 606, 612, 617, 623, 629, 635,
    641, 646, 652, 658, 664, 670, 676, 681, 687, 693, 699, 705, 711, 717, 723,
    728, 734, 740, 746, 752, 758, 764, 770, 776, 782, 788, 794, 800, 806, 812,
    818, 824, 830, 836, 842, 848, 854, 860, 866, 872, 878, 884, 890, 896, 903,
    909, 915, 921, 927, 933, 939, 946, 952, 958, 964, 970, 977, 0, 1, 1, 2, 2, 3,
    3, 4, 5, 5, 6, 6, 7, 8, 9, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 20, 21, 23,
    24, 26, 28, 30, 33, 35, 38, 41, 45, 48, 52, 57, 61, 66, 71, 76, 81, 86, 91,
    96, 102, 107, 112, 118, 123, 129, 134, 139, 145, 150, 156, 161, 167, 172,
    177, 183, 188, 194, 199, 205, 210, 216, 221, 227, 232, 238, 244, 249, 255,
    260, 266, 271, 277, 282, 288, 293, 299, 305, 310, 316, 321, 327, 332, 338,
    344, 349, 355, 360, 366, 371, 377, 382, 388, 394, 399, 405, 411, 416, 422,
    427, 433, 439, 444, 450, 455, 461, 467, 472, 478, 484, 489, 495, 500, 506,
    512, 517, 523, 529, 534, 540, 546, 551, 557, 563, 569, 574, 580, 586, 592,
    597, 603, 609, 615, 620, 626, 632, 638, 643, 649, 655, 661, 667, 673, 678,
    684, 690, 696, 702, 708, 714, 719, 725, 731, 737, 743, 749, 755, 761, 767,
    773, 779, 784, 790, 796, 802, 808, 814, 820, 826, 832, 838, 844, 850, 856,
    862, 868, 874, 880, 887, 893, 899, 905, 911, 917, 923, 929, 936, 942, 948,
    954, 960, 966, 973, 0, 1, 1, 2, 2, 3, 3, 4, 5, 5, 6, 6, 7, 8, 9, 9, 10, 11,
    12, 13, 14, 15, 16, 17, 18, 20, 21, 23, 24, 26, 28, 30, 32, 35, 38, 41, 44,
    48, 52, 56, 60, 65, 70, 75, 80, 85, 90, 96, 101, 106, 111, 117, 122, 128,
    133, 138, 144, 149, 154, 160, 165, 171, 176, 182, 187, 193, 198, 203, 209,
    215, 220, 226, 231, 236, 242, 248, 253, 259, 264, 270, 275, 281, 286, 292,
    297, 303, 309, 314, 320, 325, 331, 336, 342, 347, 353, 359, 364, 369, 375,
    381, 386, 392, 397, 403, 409, 414, 420, 425, 431, 436, 442, 448, 453, 459,
    464, 470, 476, 481, 487, 493, 498, 504, 509, 515, 521, 526, 532, 538, 543,
    549, 555, 560, 566, 572, 577, 583, 589, 595, 600, 606, 612, 618, 623, 629,
    635, 640, 646, 652, 658, 664, 670, 675, 681, 687, 693, 699, 705, 710, 716,
    722, 728, 734, 740, 746, 752, 757, 763, 769, 775, 781, 787, 793, 799, 805,
    811, 817, 823, 829, 835, 841, 847, 853, 859, 865, 871, 877, 883, 889, 895,
    901, 907, 913, 920, 926, 932, 938, 944, 950, 956, 962, 969, 0, 1, 1, 2, 2, 3,
    3, 4, 5, 5, 6, 6, 7, 8, 9, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 21, 22,
    24, 26, 28, 30, 32, 35, 38, 41, 44, 47, 51, 55, 60, 64, 69, 74, 79, 84, 89,
    95, 100, 105, 110, 116, 121, 127, 132, 137, 143, 148, 153, 159, 164, 170,
    175, 181, 186, 191, 197, 202, 208, 213, 219, 224, 230, 235, 241, 246, 252,
    257, 263, 268, 274, 279, 285, 290, 296, 301, 307, 312, 318, 323, 329, 334,
    340, 345, 351, 357, 362, 368, 373, 379, 384, 390, 396, 401, 407, 412, 418,
    423, 429, 434, 440, 446, 451, 457, 462, 468, 473, 479, 484, 490, 496, 501,
    507, 513, 518, 524, 530, 535, 541, 546, 552, 558, 564, 569, 575, 581, 586,
    592, 598, 603, 609, 615, 621, 626, 632, 638, 643, 649, 655, 661, 667, 673,
    678, 684, 690, 696, 702, 707, 713, 719, 725, 731, 737, 743, 748, 754, 760,
    766, 772, 778, 784, 790, 796, 802, 807, 813, 819, 825, 831, 837, 843, 849,
    855, 861, 867, 873, 879, 885, 891, 898, 903, 910, 916, 922, 928, 934, 940,
    946, 953, 959, 965, 0, 1, 1, 2, 2, 3, 3, 4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 11,
    12, 13, 14, 15, 16, 17, 18, 19, 21, 22, 24, 26, 28, 30, 32, 34, 37, 40, 43,
    47, 51, 55, 59, 64, 69, 73, 78, 83, 89, 94, 99, 104, 109, 115, 120, 126, 131,
    136, 142, 147, 152, 158, 163, 168, 174, 179, 185, 190, 196, 201, 207, 212,
    217, 223, 228, 234, 239, 245, 250, 256, 261, 267, 272, 278, 283, 289, 294,
    300, 305, 311, 316, 322, 327, 333, 338, 344, 349, 355, 360, 366, 371, 377,
    383, 388, 394, 399, 405, 410, 416, 421, 427, 432, 438, 443, 449, 455, 460,
    465, 471, 477, 482, 488, 493, 499, 505, 510, 516, 522, 527, 533, 538, 544,
    550, 555, 561, 567, 572, 578, 584, 589, 595, 601, 606, 612, 618, 624, 629,
    635, 641, 646, 652, 658, 664, 670, 675, 681, 687, 693, 699, 704, 710, 716,
    722, 728, 734, 739, 745, 751, 757, 763, 769, 774, 780, 786, 792, 798, 804,
    810, 816, 822, 828, 834, 840, 846, 852, 858, 864, 869, 876, 882, 888, 894,
    900, 906, 912, 918, 924, 930, 936, 943, 949, 955, 961, 0, 1, 1, 2, 2, 3, 3,
    4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 11, 12, 13, 13, 15, 16, 17, 18, 19, 21, 22,
    24, 25, 27, 29, 32, 34, 37, 40, 43, 47, 50, 54, 59, 63, 68, 73, 78, 83, 88,
    93, 98, 103, 109, 114, 119, 125, 130, 135, 141, 146, 151, 157, 162, 167, 173,
    178, 184, 189, 194, 200, 205, 211, 216, 222, 227, 232, 238, 243, 249, 254,
    260, 265, 271, 276, 282, 287, 293, 298, 304, 309, 315, 320, 326, 331, 337,
    342, 348, 353, 359, 364, 369, 375, 381, 386, 392, 397, 403, 408, 414, 419,
    425, 430, 436, 441, 447, 452, 458, 463, 469, 475, 480, 486, 491, 497, 502,
    508, 514, 519, 525, 530, 536, 541, 547, 553, 558, 564, 570, 575, 581, 587,
    592, 598, 604, 609, 615, 621, 627, 632, 638, 644, 649, 655, 661, 667, 672,
    678, 684, 690, 696, 701, 707, 713, 719, 725, 730, 736, 742, 748, 754, 760,
    765, 771, 777, 783, 789, 795, 801, 807, 813, 818, 824, 830, 836, 842, 848,
    854, 860, 866, 872, 878, 884, 890, 896, 902, 908, 914, 920, 926, 933, 939,
    945, 951, 957, 0, 1, 1, 2, 2, 3, 3, 4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 11, 12,
    13, 13, 14, 16, 17, 18, 19, 20, 22, 24, 25, 27, 29, 31, 34, 37, 39, 43, 46,
    50, 54, 58, 62, 67, 72, 77, 82, 87, 92, 97, 103, 108, 113, 118, 124, 129,
    134, 140, 145, 150, 155, 161, 166, 172, 177, 182, 188, 193, 198, 204, 209,
    215, 220, 226, 231, 237, 242, 247, 253, 258, 264, 269, 274, 280, 286, 291,
    297, 302, 307, 313, 318, 324, 329, 335, 340, 346, 351, 357, 362, 368, 373,
    379, 384, 390, 395, 401, 406, 412, 417, 423, 428, 434, 439, 445, 450, 456,
    461, 467, 472, 478, 483, 489, 494, 500, 506, 511, 517, 522, 528, 534, 539,
    545, 550, 556, 562, 567, 573, 578, 584, 590, 595, 601, 607, 612, 618, 624,
    629, 635, 641, 647, 652, 658, 664, 670, 675, 681, 687, 693, 698, 704, 710,
    716, 721, 727, 733, 739, 745, 751, 756, 762, 768, 774, 780, 786, 791, 797,
    803, 809, 815, 821, 827, 833, 839, 845, 851, 857, 862, 868, 874, 880, 886,
    892, 898, 905, 911, 917, 923, 929, 935, 941, 947, 953, 0, 1, 1, 2, 2, 3, 3,
    4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 11, 12, 12, 13, 14, 15, 17, 18, 19, 20, 22,
    23, 25, 27, 29, 31, 34, 36, 39, 42, 46, 49, 53, 57, 62, 66, 71, 76, 81, 86,
    91, 96, 102, 107, 112, 117, 123, 128, 133, 139, 144, 149, 154, 160, 165, 171,
    176, 181, 186, 192, 197, 203, 208, 214, 219, 224, 230, 235, 241, 246, 251,
    257, 262, 268, 273, 279, 284, 289, 295, 300, 306, 311, 317, 322, 327, 333,
    338, 344, 350, 355, 360, 366, 371, 377, 382, 388, 393, 399, 404, 410, 415,
    421, 426, 432, 437, 443, 448, 454, 459, 465, 470, 475, 481, 486, 492, 498,
    503, 509, 514, 520, 526, 531, 536, 542, 548, 553, 559, 565, 570, 576, 581,
    587, 593, 598, 604, 610, 615, 621, 627, 632, 638, 644, 649, 655, 661, 667,
    672, 678, 684, 690, 695, 701, 707, 713, 718, 724, 730, 736, 742, 747, 753,
    759, 765, 771, 776, 782, 788, 794, 800, 806, 812, 817, 823, 829, 835, 841,
    847, 853, 859, 865, 871, 877, 883, 889, 895, 901, 907, 913, 919, 925, 931,
    937, 943, 949, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 8, 9, 10, 11, 12,
    12, 13, 14, 15, 16, 18, 19, 20, 22, 23, 25, 27, 29, 31, 33, 36, 39, 42, 45,
    49, 53, 57, 61, 66, 71, 75, 80, 85, 90, 95, 101, 106, 111, 116, 122, 127,
    132, 138, 143, 148, 153, 159, 164, 169, 175, 180, 185, 191, 196, 201, 207,
    212, 218, 223, 228, 234, 239, 245, 250, 255, 261, 266, 271, 277, 282, 288,
    293, 299, 304, 310, 315, 321, 326, 331, 337, 342, 348, 353, 358, 364, 369,
    375, 380, 386, 391, 397, 402, 408, 413, 419, 424, 430, 435, 440, 446, 451,
    457, 462, 468, 473, 479, 484, 490, 495, 501, 506, 512, 518, 523, 529, 534,
    540, 545, 551, 556, 562, 568, 573, 579, 585, 590, 596, 601, 607, 613, 618,
    624, 629, 635, 641, 647, 652, 658, 664, 669, 675, 681, 687, 692, 698, 704,
    710, 715, 721, 727, 733, 738, 744, 750, 756, 761, 767, 773, 779, 785, 791,
    796, 802, 808, 814, 820, 826, 832, 838, 844, 849, 855, 861, 867, 873, 879,
    885, 891, 897, 903, 909, 915, 921, 927, 933, 939, 945, 0, 1, 1, 2, 2, 3, 3,
    4, 4, 5, 6, 6, 7, 8, 8, 9, 10, 11, 11, 12, 13, 14, 15, 16, 18, 19, 20, 22,
    23, 25, 27, 29, 31, 33, 36, 38, 41, 45, 48, 52, 56, 61, 65, 70, 75, 80, 85,
    90, 95, 100, 105, 110, 116, 121, 126, 131, 137, 142, 147, 152, 158, 163, 168,
    174, 179, 184, 190, 195, 200, 206, 211, 216, 222, 227, 232, 238, 243, 249,
    254, 259, 265, 270, 276, 281, 286, 292, 297, 302, 308, 313, 319, 324, 330,
    335, 341, 346, 351, 357, 362, 367, 373, 379, 384, 389, 395, 400, 406, 411,
    417, 422, 427, 433, 438, 444, 449, 455, 460, 466, 471, 477, 482, 488, 493,
    499, 504, 510, 515, 521, 526, 532, 537, 543, 548, 554, 560, 565, 571, 576,
    582, 587, 593, 599, 604, 610, 616, 621, 627, 632, 638, 644, 649, 655, 661,
    667, 672, 678, 684, 689, 695, 701, 706, 712, 718, 724, 729, 735, 741, 747,
    753, 758, 764, 770, 776, 781, 787, 793, 799, 805, 811, 816, 822, 828, 834,
    840, 846, 852, 858, 864, 870, 876, 881, 887, 893, 899, 905, 911, 917, 923,
    929, 935, 942, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 8, 9, 10, 11, 11,
    12, 13, 14, 15, 16, 17, 19, 20, 21, 23, 25, 26, 28, 30, 33, 35, 38, 41, 44,
    48, 52, 56, 60, 64, 69, 74, 79, 84, 89, 94, 99, 104, 109, 115, 120, 125, 130,
    136, 141, 146, 151, 157, 162, 167, 172, 178, 183, 188, 193, 199, 204, 210,
    215, 220, 225, 231, 236, 242, 247, 252, 258, 263, 268, 274, 279, 285, 290,
    296, 301, 306, 312, 317, 322, 328, 333, 339, 344, 350, 355, 360, 366, 371,
    377, 382, 387, 393, 398, 404, 409, 415, 420, 425, 431, 436, 442, 447, 452,
    458, 463, 469, 474, 480, 485, 491, 496, 502, 507, 513, 518, 524, 529, 535,
    540, 546, 551, 557, 563, 568, 574, 579, 585, 590, 596, 602, 607, 613, 618,
    624, 630, 635, 641, 647, 652, 658, 664, 669, 675, 681, 686, 692, 698, 703,
    709, 715, 721, 726, 732, 738, 744, 749, 755, 761, 767, 772, 778, 784, 790,
    796, 801, 807, 813, 819, 825, 831, 836, 842, 848, 854, 860, 866, 872, 878,
    884, 890, 896, 902, 908, 914, 920, 926, 931, 938, 0, 1, 1, 2, 2, 3, 3, 4, 4,
    5, 6, 6, 7, 8, 8, 9, 10, 11, 11, 12, 13, 14, 15, 16, 17, 19, 20, 21, 23, 24,
    26, 28, 30, 32, 35, 38, 41, 44, 47, 51, 55, 59, 64, 68, 73, 78, 83, 88, 93,
    98, 103, 108, 114, 119, 124, 129, 135, 140, 145, 150, 155, 161, 166, 171,
    177, 182, 187, 192, 198, 203, 208, 214, 219, 224, 230, 235, 240, 246, 251,
    256, 262, 267, 272, 278, 283, 289, 294, 299, 305, 310, 315, 321, 326, 331,
    337, 342, 348, 353, 358, 364, 369, 375, 380, 386, 391, 396, 402, 407, 413,
    418, 423, 429, 434, 440, 445, 450, 456, 461, 467, 472, 477, 483, 489, 494,
    499, 505, 510, 516, 521, 527, 532, 538, 543, 549, 555, 560, 566, 571, 577,
    582, 588, 593, 599, 605, 610, 616, 621, 627, 633, 638, 644, 649, 655, 661,
    666, 672, 678, 683, 689, 695, 700, 706, 712, 718, 723, 729, 735, 740, 746,
    752, 758, 763, 769, 775, 781, 786, 792, 798, 804, 810, 815, 821, 827, 833,
    839, 845, 851, 856, 862, 868, 874, 880, 886, 892, 898, 904, 910, 916, 922,
    928, 934, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 8, 9, 10, 11, 11, 12, 13,
    14, 15, 16, 17, 19, 20, 21, 23, 24, 26, 28, 30, 32, 35, 37, 40, 43, 47, 51,
    55, 59, 63, 68, 72, 77, 82, 87, 92, 97, 102, 108, 113, 118, 123, 128, 133,
    139, 144, 149, 154, 159, 165, 170, 175, 180, 186, 191, 196, 202, 207, 212,
    218, 223, 228, 234, 239, 244, 250, 255, 260, 265, 271, 276, 282, 287, 292,
    298, 303, 308, 314, 319, 325, 330, 335, 341, 346, 351, 357, 362, 367, 373,
    378, 384, 389, 394, 400, 405, 411, 416, 421, 427, 432, 437, 443, 448, 454,
    459, 464, 470, 475, 481, 486, 492, 497, 503, 508, 514, 519, 524, 530, 535,
    541, 546, 552, 558, 563, 569, 574, 580, 585, 591, 596, 602, 607, 613, 618,
    624, 630, 635, 641, 647, 652, 658, 663, 669, 675, 680, 686, 692, 697, 703,
    709, 714, 720, 726, 732, 737, 743, 748, 754, 760, 766, 772, 777, 783, 789,
    795, 800, 806, 812, 818, 824, 830, 835, 841, 847, 853, 859, 865, 870, 876,
    882, 888, 894, 900, 906, 912, 918, 924, 930, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6,
    6, 7, 8, 8, 9, 10, 10, 11, 12, 13, 14, 15, 16, 17, 18, 20, 21, 23, 24, 26,
    28, 30, 32, 34, 37, 40, 43, 46, 50, 54, 58, 62, 67, 72, 76, 81, 86, 91, 96,
    101, 107, 112, 117, 122, 127, 132, 138, 143, 148, 153, 158, 164, 169, 174,
    179, 185, 190, 195, 200, 206, 211, 216, 221, 227, 232, 238, 243, 248, 253,
    259, 264, 269, 275, 280, 285, 291, 296, 301, 307, 312, 317, 323, 328, 334,
    339, 344, 349, 355, 360, 366, 371, 376, 382, 387, 392, 398, 403, 409, 414,
    419, 425, 430, 435, 441, 446, 452, 457, 462, 468, 473, 479, 484, 489, 495,
    500, 506, 511, 517, 522, 528, 533, 539, 544, 549, 555, 560, 566, 572, 577,
    583, 588, 594, 599, 605, 610, 616, 621, 627, 633, 638, 644, 649, 655, 661,
    666, 672, 677, 683, 689, 694, 700, 706, 711, 717, 723, 728, 734, 740, 745,
    751, 757, 763, 768, 774, 780, 786, 791, 797, 803, 809, 814, 820, 826, 832,
    838, 844, 849, 855, 861, 867, 873, 879, 885, 891, 896, 902, 908, 914, 920,
    926, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 8, 8, 9, 10, 10, 11, 12, 13, 14,
    15, 16, 17, 18, 20, 21, 22, 24, 26, 28, 30, 32, 34, 37, 40, 43, 46, 50, 53,
    58, 62, 66, 71, 76, 81, 86, 90, 96, 100, 106, 111, 116, 121, 126, 131, 137,
    142, 147, 152, 157, 163, 168, 173, 178, 183, 189, 194, 199, 204, 210, 215,
    220, 226, 231, 236, 241, 247, 252, 257, 262, 268, 273, 279, 284, 289, 294,
    300, 305, 310, 316, 321, 326, 332, 337, 342, 348, 353, 358, 364, 369, 374,
    380, 385, 390, 396, 401, 407, 412, 417, 423, 428, 433, 439, 444, 449, 455,
    460, 466, 471, 476, 482, 487, 493, 498, 503, 509, 514, 520, 525, 530, 536,
    542, 547, 552, 558, 563, 569, 574, 580, 585, 591, 597, 602, 607, 613, 618,
    624, 630, 635, 641, 647, 652, 658, 663, 669, 675, 680, 686, 691, 697, 703,
    708, 714, 720, 725, 731, 737, 742, 748, 754, 759, 765, 771, 777, 782, 788,
    794, 800, 805, 811, 817, 823, 828, 834, 840, 846, 852, 858, 863, 869, 875,
    881, 887, 893, 899, 905, 911, 916, 922, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6,
    7, 7, 8, 9, 10, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 21, 22, 24, 26, 27,
    29, 31, 34, 36, 39, 42, 46, 49, 53, 57, 61, 66, 70, 75, 80, 85, 90, 95, 100,
    105, 110, 115, 120, 125, 130, 136, 141, 146, 151, 156, 161, 167, 172, 177,
    182, 187, 193, 198, 203, 208, 214, 219, 224, 229, 235, 240, 245, 250, 256,
    261, 266, 272, 277, 282, 288, 293, 298, 304, 309, 314, 319, 325, 330, 335,
    341, 346, 351, 356, 362, 367, 373, 378, 383, 389, 394, 399, 405, 410, 415,
    421, 426, 431, 437, 442, 447, 453, 458, 463, 469, 474, 479, 485, 490, 496,
    501, 506, 512, 517, 523, 528, 534, 539, 545, 550, 555, 561, 566, 572, 577,
    583, 588, 594, 599, 605, 610, 616, 621, 627, 633, 638, 644, 649, 655, 660,
    666, 672, 677, 683, 688, 694, 700, 705, 711, 717, 722, 728, 733, 739, 745,
    750, 756, 762, 768, 773, 779, 785, 790, 796, 802, 808, 813, 819, 825, 831,
    837, 842, 848, 854, 860, 866, 871, 877, 883, 889, 895, 901, 907, 913, 919, 0,
    1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 7, 8, 9, 10, 10, 11, 12, 13, 14, 15, 16,
    17, 18, 19, 21, 22, 24, 25, 27, 29, 31, 34, 36, 39, 42, 45, 49, 52, 56, 61,
    65, 70, 74, 79, 84, 89, 94, 99, 104, 109, 114, 119, 124, 129, 135, 140, 145,
    150, 155, 160, 166, 171, 176, 181, 186, 192, 197, 202, 207, 212, 217, 223,
    228, 233, 239, 244, 249, 254, 259, 265, 270, 275, 281, 286, 291, 297, 302,
    307, 312, 318, 323, 328, 334, 339, 344, 349, 355, 360, 365, 371, 376, 381,
    387, 392, 397, 403, 408, 413, 418, 424, 429, 434, 440, 445, 450, 456, 461,
    466, 472, 477, 483, 488, 493, 499, 504, 510, 515, 520, 526, 531, 537, 542,
    547, 553, 558, 564, 569, 575, 580, 586, 591, 597, 602, 608, 613, 619, 624,
    630, 635, 641, 646, 652, 657, 663, 669, 674, 680, 685, 691, 697, 702, 708,
    713, 719, 725, 730, 736, 742, 747, 753, 759, 764, 770, 776, 781, 787, 793,
    799, 804, 810, 816, 822, 827, 833, 839, 845, 850, 856, 862, 868, 874, 880,
    885, 891, 897, 903, 909, 915, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 7, 8, 9,
    10, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 21, 22, 24, 25, 27, 29, 31, 33,
    36, 38, 41, 45, 48, 52, 56, 60, 64, 69, 73, 78, 83, 88, 93, 98, 103, 108,
    113, 118, 123, 128, 134, 139, 144, 149, 154, 159, 164, 170, 175, 180, 185,
    190, 195, 201, 206, 211, 216, 222, 227, 232, 237, 242, 248, 253, 258, 263,
    269, 274, 279, 284, 290, 295, 300, 306, 311, 316, 321, 327, 332, 337, 342,
    348, 353, 358, 364, 369, 374, 379, 385, 390, 395, 401, 406, 411, 416, 422,
    427, 432, 438, 443, 448, 453, 459, 464, 470, 475, 480, 486, 491, 496, 502,
    507, 512, 518, 523, 529, 534, 540, 545, 550, 556, 561, 567, 572, 578, 583,
    589, 594, 599, 605, 610, 616, 621, 627, 633, 638, 644, 649, 654, 660, 666,
    671, 677, 682, 688, 694, 699, 705, 710, 716, 722, 727, 733, 738, 744, 750,
    755, 761, 767, 772, 778, 784, 789, 795, 801, 807, 812, 818, 824, 830, 835,
    841, 847, 852, 858, 864, 870, 876, 882, 888, 894, 899, 905, 911, 0, 1, 1, 2,
    2, 3, 3, 4, 4, 5, 6, 6, 7, 7, 8, 9, 10, 10, 11, 12, 13, 14, 15, 16, 17, 18,
    19, 20, 22, 23, 25, 27, 29, 31, 33, 35, 38, 41, 44, 48, 51, 55, 59, 64, 68,
    73, 77, 82, 87, 92, 97, 102, 107, 112, 117, 122, 127, 133, 137, 143, 148,
    153, 158, 163, 168, 173, 179, 184, 189, 194, 199, 205, 210, 215, 220, 225,
    231, 236, 241, 246, 251, 257, 262, 267, 272, 278, 283, 288, 293, 299, 304,
    309, 314, 319, 325, 330, 335, 341, 346, 351, 356, 362, 367, 372, 378, 383,
    388, 393, 399, 404, 409, 414, 420, 425, 430, 435, 441, 446, 451, 457, 462,
    467, 473, 478, 483, 489, 494, 499, 505, 510, 516, 521, 526, 532, 537, 542,
    548, 553, 559, 564, 570, 575, 580, 586, 591, 597, 602, 608, 613, 619, 624,
    630, 635, 641, 646, 652, 657, 663, 668, 674, 679, 685, 691, 696, 702, 707,
    713, 719, 724, 730, 735, 741, 747, 752, 758, 763, 769, 775, 780, 786, 792,
    798, 803, 809, 815, 820, 826, 832, 838, 843, 849, 855, 861, 866, 872, 878,
    884, 890, 896, 901, 907, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 6, 6, 7, 7, 8, 9, 10,
    10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 23, 25, 27, 28, 30, 33, 35,
    38, 41, 44, 47, 51, 55, 59, 63, 67, 72, 77, 82, 86, 91, 96, 101, 106, 111,
    116, 121, 126, 132, 136, 142, 147, 152, 157, 162, 167, 172, 178, 183, 188,
    193, 198, 203, 208, 213, 219, 224, 229, 234, 240, 245, 250, 255, 260, 266,
    271, 276, 281, 286, 292, 297, 302, 307, 313, 318, 323, 328, 334, 339, 344,
    349, 355, 360, 365, 370, 376, 381, 386, 391, 397, 402, 407, 412, 418, 423,
    428, 433, 439, 444, 449, 455, 460, 465, 471, 476, 481, 486, 492, 497, 502,
    508, 513, 518, 524, 529, 535, 540, 545, 551, 556, 562, 567, 572, 578, 583,
    589, 594, 599, 605, 611, 616, 621, 627, 632, 638, 643, 649, 654, 660, 665,
    671, 677, 682, 688, 693, 699, 704, 710, 715, 721, 726, 732, 738, 743, 749,
    755, 760, 766, 772, 777, 783, 788, 794, 800, 806, 811, 817, 823, 828, 834,
    840, 845, 851, 857, 863, 869, 874, 880, 886, 892, 898, 904, 0, 1, 1, 2, 2, 3,
    3, 4, 4, 5, 6, 6, 7, 7, 8, 9, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    22, 23, 25, 26, 28, 30, 32, 35, 37, 40, 43, 47, 50, 54, 58, 62, 67, 71, 76,
    81, 85, 90, 95, 100, 105, 110, 115, 120, 125, 131, 135, 141, 146, 151, 156,
    161, 166, 171, 176, 181, 187, 192, 197, 202, 207, 212, 218, 223, 228, 233,
    238, 243, 249, 254, 259, 264, 269, 275, 280, 285, 290, 295, 301, 306, 311,
    316, 322, 327, 332, 337, 342, 347, 353, 358, 363, 368, 374, 379, 384, 389,
    395, 400, 405, 410, 416, 421, 426, 431, 437, 442, 447, 452, 458, 463, 468,
    474, 479, 484, 490, 495, 500, 505, 511, 516, 521, 527, 532, 538, 543, 548,
    554, 559, 564, 570, 575, 581, 586, 591, 597, 602, 608, 613, 619, 624, 630,
    635, 641, 646, 652, 657, 663, 668, 674, 679, 685, 690, 696, 701, 707, 712,
    718, 723, 729, 735, 740, 746, 751, 757, 763, 768, 774, 779, 785, 791, 796,
    802, 808, 813, 819, 825, 831, 836, 842, 848, 854, 859, 865, 871, 877, 882,
    888, 894, 900, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 9, 10, 11, 12,
    13, 14, 15, 16, 17, 18, 19, 20, 21, 23, 24, 26, 28, 30, 32, 35, 37, 40, 43,
    46, 50, 53, 57, 62, 66, 71, 75, 80, 85, 90, 94, 99, 104, 109, 114, 119, 124,
    130, 134, 139, 145, 150, 155, 160, 165, 170, 175, 180, 185, 191, 196, 201,
    206, 211, 216, 221, 227, 232, 237, 242, 247, 252, 258, 263, 268, 273, 278,
    283, 289, 294, 299, 304, 309, 314, 320, 325, 330, 335, 340, 346, 351, 356,
    361, 367, 372, 377, 382, 387, 393, 398, 403, 408, 414, 419, 424, 429, 435,
    440, 445, 450, 455, 461, 466, 471, 477, 482, 487, 493, 498, 503, 508, 514,
    519, 524, 530, 535, 540, 546, 551, 557, 562, 567, 573, 578, 583, 589, 594,
    600, 605, 611, 616, 621, 627, 632, 638, 643, 649, 654, 660, 665, 671, 676,
    682, 687, 693, 698, 704, 709, 715, 720, 726, 731, 737, 743, 748, 754, 759,
    765, 771, 776, 782, 787, 793, 799, 804, 810, 816, 821, 827, 833, 838, 844,
    850, 856, 861, 867, 873, 879, 885, 890, 896, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5,
    6, 7, 7, 8, 9, 9, 10, 11, 12, 13, 14, 14, 15, 16, 18, 19, 20, 21, 23, 24, 26,
    28, 30, 32, 34, 37, 40, 42, 46, 49, 53, 57, 61, 65, 70, 74, 79, 84, 89, 94,
    99, 104, 109, 113, 119, 124, 129, 133, 138, 144, 149, 154, 159, 164, 169,
    174, 179, 184, 189, 194, 200, 205, 210, 215, 220, 225, 230, 235, 240, 246,
    251, 256, 261, 266, 272, 277, 282, 287, 292, 297, 302, 308, 313, 318, 323,
    328, 333, 339, 344, 349, 354, 360, 365, 370, 375, 380, 386, 391, 396, 401,
    406, 412, 417, 422, 427, 432, 438, 443, 448, 453, 459, 464, 469, 474, 480,
    485, 490, 495, 501, 506, 511, 517, 522, 527, 533, 538, 543, 549, 554, 559,
    565, 570, 575, 581, 586, 591, 597, 602, 608, 613, 619, 624, 630, 635, 640,
    646, 651, 657, 662, 668, 673, 679, 684, 690, 695, 701, 706, 712, 717, 723,
    728, 734, 739, 745, 751, 756, 762, 767, 773, 779, 784, 790, 795, 801, 806,
    812, 818, 824, 829, 835, 841, 846, 852, 858, 864, 869, 875, 881, 887, 893, 0,
    1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 9, 10, 11, 12, 13, 13, 14, 15,
    16, 17, 19, 20, 21, 23, 24, 26, 28, 30, 32, 34, 36, 39, 42, 45, 49, 52, 56,
    60, 65, 69, 74, 78, 83, 88, 93, 98, 103, 108, 112, 118, 123, 128, 132, 137,
    143, 147, 153, 158, 163, 168, 173, 178, 183, 188, 193, 198, 203, 208, 214,
    219, 224, 229, 234, 239, 244, 249, 255, 260, 265, 270, 275, 280, 285, 291,
    296, 301, 306, 311, 316, 322, 327, 332, 337, 342, 347, 353, 358, 363, 368,
    373, 378, 384, 389, 394, 399, 404, 410, 415, 420, 425, 430, 436, 441, 446,
    451, 456, 462, 467, 472, 477, 483, 488, 493, 498, 504, 509, 514, 520, 525,
    530, 536, 541, 546, 551, 557, 562, 568, 573, 578, 583, 589, 594, 600, 605,
    611, 616, 621, 627, 632, 637, 643, 648, 654, 659, 665, 670, 676, 681, 687,
    692, 698, 703, 709, 714, 720, 725, 731, 736, 742, 747, 753, 758, 764, 770,
    775, 781, 786, 792, 798, 803, 809, 815, 820, 826, 831, 837, 843, 849, 854,
    860, 866, 872, 877, 883, 889, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9,
    9, 10, 11, 12, 12, 13, 14, 15, 16, 17, 19, 20, 21, 23, 24, 26, 27, 29, 31,
    34, 36, 39, 42, 45, 48, 52, 56, 60, 64, 68, 73, 78, 82, 87, 92, 97, 102, 107,
    111, 117, 122, 127, 131, 136, 142, 146, 152, 157, 162, 166, 172, 177, 182,
    187, 192, 197, 202, 207, 212, 217, 222, 228, 233, 238, 243, 248, 253, 258,
    263, 268, 274, 279, 284, 289, 294, 299, 304, 309, 315, 320, 325, 330, 335,
    340, 346, 351, 356, 361, 366, 371, 377, 382, 387, 392, 397, 402, 408, 413,
    418, 423, 428, 433, 438, 444, 449, 454, 459, 465, 470, 475, 480, 486, 491,
    496, 501, 506, 512, 517, 522, 528, 533, 538, 544, 549, 554, 560, 565, 570,
    576, 581, 586, 591, 597, 602, 608, 613, 619, 624, 629, 635, 640, 646, 651,
    656, 662, 667, 673, 678, 684, 689, 695, 700, 706, 711, 717, 722, 728, 733,
    739, 744, 750, 755, 761, 766, 772, 777, 783, 789, 794, 800, 805, 811, 817,
    822, 828, 834, 839, 845, 851, 856, 862, 868, 874, 879, 885, 0, 1, 1, 2, 2, 3,
    3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 9, 10, 11, 12, 12, 13, 14, 15, 16, 17, 18, 20,
    21, 22, 24, 25, 27, 29, 31, 33, 36, 38, 41, 44, 48, 51, 55, 59, 63, 68, 72,
    77, 81, 86, 91, 96, 101, 106, 111, 116, 121, 126, 130, 135, 140, 145, 150,
    155, 160, 165, 171, 175, 181, 186, 191, 196, 201, 206, 211, 216, 221, 226,
    231, 236, 241, 246, 252, 257, 262, 267, 272, 277, 282, 287, 293, 298, 303,
    308, 313, 318, 323, 328, 333, 339, 344, 349, 354, 359, 364, 370, 375, 380,
    385, 390, 395, 400, 406, 411, 416, 421, 426, 431, 436, 442, 447, 452, 457,
    462, 468, 473, 478, 483, 489, 494, 499, 504, 510, 515, 520, 525, 531, 536,
    541, 546, 552, 557, 562, 568, 573, 578, 583, 589, 594, 600, 605, 610, 616,
    621, 627, 632, 637, 643, 648, 654, 659, 664, 670, 675, 681, 686, 692, 697,
    703, 708, 713, 719, 724, 730, 735, 741, 746, 752, 758, 763, 769, 774, 780,
    785, 791, 796, 802, 808, 813, 819, 824, 830, 836, 842, 847, 853, 859, 864,
    870, 876, 881, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 9, 10, 11, 12,
    12, 13, 14, 15, 16, 17, 18, 20, 21, 22, 24, 25, 27, 29, 31, 33, 35, 38, 41,
    44, 47, 51, 55, 59, 63, 67, 71, 76, 81, 85, 90, 95, 100, 105, 110, 115, 120,
    125, 129, 134, 139, 144, 149, 154, 159, 164, 169, 174, 179, 184, 189, 194,
    200, 204, 210, 215, 220, 225, 230, 235, 240, 245, 250, 255, 260, 265, 271,
    275, 281, 286, 291, 296, 301, 306, 311, 316, 322, 327, 332, 337, 342, 347,
    352, 357, 363, 368, 373, 378, 383, 388, 393, 398, 404, 409, 414, 419, 424,
    429, 434, 440, 445, 450, 455, 460, 465, 471, 476, 481, 486, 491, 497, 502,
    507, 512, 518, 523, 528, 533, 539, 544, 549, 555, 560, 565, 570, 576, 581,
    586, 592, 597, 602, 608, 613, 618, 624, 629, 635, 640, 645, 651, 656, 661,
    667, 672, 678, 683, 689, 694, 699, 705, 710, 716, 721, 727, 732, 738, 743,
    749, 754, 760, 765, 771, 776, 782, 788, 793, 799, 804, 810, 815, 821, 827,
    832, 838, 844, 849, 855, 861, 866, 872, 878, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5,
    6, 7, 7, 8, 9, 9, 10, 11, 12, 12, 13, 14, 15, 16, 17, 18, 19, 21, 22, 24, 25,
    27, 29, 31, 33, 35, 38, 41, 44, 47, 50, 54, 58, 62, 66, 71, 75, 80, 85, 89,
    94, 99, 104, 109, 114, 119, 124, 128, 133, 138, 143, 148, 153, 158, 163, 168,
    173, 178, 183, 188, 193, 198, 203, 208, 213, 218, 223, 228, 233, 239, 243,
    249, 254, 259, 264, 269, 274, 279, 284, 289, 294, 300, 304, 310, 315, 320,
    325, 330, 335, 340, 345, 350, 356, 361, 366, 371, 376, 381, 386, 391, 396,
    402, 407, 412, 417, 422, 427, 432, 437, 442, 448, 453, 458, 463, 468, 474,
    479, 484, 489, 494, 499, 505, 510, 515, 520, 526, 531, 536, 541, 547, 552,
    557, 563, 568, 573, 578, 584, 589, 594, 600, 605, 610, 616, 621, 626, 632,
    637, 642, 648, 653, 659, 664, 669, 675, 680, 686, 691, 696, 702, 707, 713,
    718, 724, 729, 735, 740, 746, 751, 757, 762, 768, 773, 779, 784, 790, 795,
    801, 806, 812, 817, 823, 829, 834, 840, 846, 851, 857, 863, 868, 874, 0, 1,
    1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 9, 10, 11, 11, 12, 13, 14, 15, 16,
    17, 18, 19, 21, 22, 23, 25, 27, 28, 30, 33, 35, 37, 40, 43, 46, 50, 53, 57,
    61, 66, 70, 75, 79, 84, 88, 93, 98, 103, 108, 113, 118, 123, 127, 132, 137,
    142, 147, 152, 157, 162, 167, 172, 177, 182, 187, 192, 197, 202, 207, 212,
    217, 222, 227, 232, 237, 242, 247, 252, 257, 262, 268, 272, 278, 283, 288,
    293, 298, 303, 308, 313, 318, 323, 328, 333, 338, 344, 349, 354, 359, 364,
    369, 374, 379, 384, 389, 394, 400, 405, 410, 415, 420, 425, 430, 435, 440,
    446, 451, 456, 461, 466, 471, 477, 482, 487, 492, 497, 502, 508, 513, 518,
    523, 528, 534, 539, 544, 549, 555, 560, 565, 570, 576, 581, 586, 592, 597,
    602, 608, 613, 618, 623, 629, 634, 640, 645, 650, 656, 661, 666, 672, 677,
    683, 688, 693, 699, 704, 710, 715, 721, 726, 731, 737, 742, 748, 753, 759,
    764, 770, 775, 781, 786, 792, 797, 803, 809, 814, 820, 825, 831, 837, 842,
    848, 854, 859, 865, 871, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 9, 9,
    10, 11, 11, 12, 13, 14, 15, 16, 17, 18, 19, 21, 22, 23, 25, 26, 28, 30, 32,
    35, 37, 40, 43, 46, 49, 53, 57, 61, 65, 69, 74, 78, 83, 88, 93, 97, 102, 107,
    112, 117, 122, 126, 131, 136, 141, 146, 151, 156, 161, 166, 171, 176, 181,
    186, 191, 196, 201, 206, 211, 216, 221, 226, 231, 236, 241, 246, 251, 256,
    261, 266, 271, 276, 281, 286, 291, 296, 301, 306, 311, 316, 321, 326, 331,
    337, 342, 347, 352, 357, 362, 367, 372, 377, 382, 387, 392, 398, 403, 408,
    413, 418, 423, 428, 433, 438, 443, 449, 454, 459, 464, 469, 474, 479, 484,
    490, 495, 500, 505, 510, 516, 521, 526, 531, 536, 542, 547, 552, 557, 563,
    568, 573, 578, 584, 589, 594, 600, 605, 610, 615, 621, 626, 631, 637, 642,
    647, 653, 658, 663, 669, 674, 680, 685, 690, 696, 701, 707, 712, 717, 723,
    728, 734, 739, 745, 750, 756, 761, 767, 772, 778, 783, 789, 794, 800, 805,
    811, 816, 822, 827, 833, 839, 844, 850, 856, 861, 867, 0, 1, 1, 2, 2, 3, 3,
    4, 4, 5, 5, 6, 7, 7, 8, 8, 9, 10, 11, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    22, 23, 25, 26, 28, 30, 32, 34, 37, 39, 42, 45, 49, 52, 56, 60, 64, 69, 73,
    77, 82, 87, 92, 96, 101, 106, 111, 116, 121, 125, 130, 135, 140, 145, 150,
    155, 160, 165, 169, 175, 180, 185, 189, 194, 199, 204, 209, 214, 219, 224,
    229, 234, 239, 244, 249, 254, 259, 264, 269, 275, 280, 285, 289, 295, 300,
    305, 310, 315, 320, 325, 330, 335, 340, 345, 350, 355, 360, 365, 370, 375,
    380, 385, 391, 396, 401, 406, 411, 416, 421, 426, 431, 436, 441, 446, 452,
    457, 462, 467, 472, 477, 482, 487, 492, 498, 503, 508, 513, 518, 524, 529,
    534, 539, 544, 550, 555, 560, 565, 570, 576, 581, 586, 592, 597, 602, 607,
    613, 618, 623, 629, 634, 639, 645, 650, 655, 661, 666, 671, 677, 682, 687,
    693, 698, 704, 709, 714, 720, 725, 731, 736, 741, 747, 752, 758, 763, 769,
    774, 780, 785, 791, 796, 802, 807, 813, 818, 824, 830, 835, 841, 846, 852,
    857, 863, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 8, 9, 10, 11, 11, 12,
    13, 14, 15, 16, 17, 18, 19, 20, 22, 23, 24, 26, 28, 30, 32, 34, 36, 39, 42,
    45, 48, 52, 56, 59, 64, 68, 72, 77, 81, 86, 91, 96, 100, 105, 110, 115, 120,
    124, 129, 134, 139, 144, 149, 154, 159, 164, 168, 173, 178, 183, 188, 193,
    198, 203, 208, 213, 218, 223, 228, 233, 238, 243, 248, 253, 258, 263, 268,
    273, 278, 283, 288, 293, 298, 303, 308, 313, 318, 323, 328, 333, 338, 343,
    348, 353, 358, 363, 368, 373, 378, 383, 389, 394, 399, 404, 409, 414, 419,
    424, 429, 434, 439, 444, 449, 454, 460, 465, 470, 475, 480, 485, 490, 495,
    501, 506, 511, 516, 521, 526, 532, 537, 542, 547, 552, 558, 563, 568, 573,
    578, 584, 589, 594, 599, 605, 610, 615, 621, 626, 631, 636, 642, 647, 652,
    658, 663, 668, 674, 679, 684, 690, 695, 700, 706, 711, 717, 722, 727, 733,
    738, 744, 749, 755, 760, 765, 771, 776, 782, 787, 793, 798, 804, 809, 815,
    821, 826, 832, 837, 843, 848, 854, 860, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6,
    7, 7, 8, 8, 9, 10, 11, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 23, 24,
    26, 28, 29, 32, 34, 36, 39, 41, 44, 48, 51, 55, 59, 63, 67, 72, 76, 81, 85,
    90, 95, 99, 104, 109, 114, 119, 123, 128, 133, 138, 143, 148, 153, 157, 162,
    167, 172, 177, 182, 187, 192, 197, 202, 207, 212, 217, 222, 226, 232, 236,
    242, 246, 251, 256, 261, 266, 271, 276, 281, 286, 291, 296, 301, 306, 311,
    316, 321, 326, 331, 336, 341, 346, 351, 356, 361, 366, 372, 377, 382, 387,
    392, 397, 402, 407, 412, 417, 422, 427, 432, 437, 442, 447, 452, 457, 462,
    467, 473, 478, 483, 488, 493, 498, 503, 508, 514, 519, 524, 529, 534, 539,
    545, 550, 555, 560, 565, 570, 576, 581, 586, 591, 597, 602, 607, 612, 618,
    623, 628, 634, 639, 644, 649, 655, 660, 665, 671, 676, 681, 687, 692, 697,
    703, 708, 713, 719, 724, 730, 735, 740, 746, 751, 757, 762, 768, 773, 779,
    784, 789, 795, 800, 806, 811, 817, 823, 828, 834, 839, 845, 850, 856, 0, 1,
    1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 11, 11, 12, 13, 14, 15, 16,
    17, 18, 19, 20, 21, 23, 24, 26, 27, 29, 31, 33, 36, 38, 41, 44, 47, 51, 54,
    58, 62, 66, 71, 75, 80, 84, 89, 94, 99, 103, 108, 113, 118, 122, 127, 132,
    137, 142, 147, 152, 156, 161, 166, 171, 176, 181, 186, 191, 195, 201, 205,
    210, 215, 220, 225, 230, 235, 240, 245, 250, 255, 260, 265, 270, 275, 280,
    285, 290, 295, 300, 305, 310, 315, 320, 325, 330, 335, 340, 345, 350, 355,
    360, 365, 370, 375, 380, 385, 390, 395, 400, 405, 410, 415, 420, 425, 430,
    435, 440, 445, 450, 455, 460, 465, 470, 475, 481, 485, 491, 496, 501, 506,
    511, 516, 521, 527, 532, 537, 542, 547, 552, 557, 563, 568, 573, 578, 584,
    589, 594, 599, 604, 610, 615, 620, 625, 631, 636, 641, 647, 652, 657, 662,
    668, 673, 678, 683, 689, 694, 700, 705, 710, 716, 721, 726, 732, 737, 743,
    748, 753, 759, 764, 770, 775, 781, 786, 792, 797, 803, 808, 814, 819, 825,
    830, 836, 841, 847, 852, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9,
    10, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 23, 24, 26, 27, 29, 31,
    33, 35, 38, 41, 44, 47, 50, 54, 58, 62, 66, 70, 74, 79, 83, 88, 93, 98, 102,
    107, 112, 117, 121, 126, 131, 136, 141, 146, 150, 155, 160, 165, 170, 175,
    180, 185, 189, 194, 199, 204, 209, 214, 219, 224, 229, 234, 239, 244, 249,
    253, 258, 263, 268, 273, 278, 283, 288, 293, 298, 303, 308, 313, 318, 323,
    328, 333, 338, 343, 348, 353, 358, 363, 368, 373, 378, 383, 388, 393, 398,
    402, 408, 413, 418, 423, 428, 433, 438, 443, 448, 453, 458, 463, 468, 473,
    478, 483, 488, 493, 499, 504, 509, 514, 519, 524, 529, 534, 540, 545, 550,
    555, 560, 565, 571, 576, 581, 586, 591, 597, 602, 607, 612, 617, 623, 628,
    633, 638, 644, 649, 654, 659, 665, 670, 675, 680, 686, 691, 697, 702, 707,
    713, 718, 723, 729, 734, 739, 745, 750, 756, 761, 766, 772, 777, 783, 788,
    793, 799, 805, 810, 816, 821, 827, 832, 838, 843, 849, 0, 1, 1, 2, 2, 3, 3,
    4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 10, 11, 12, 13, 14, 15, 15, 17, 18, 19, 20,
    21, 22, 24, 25, 27, 29, 31, 33, 35, 38, 40, 43, 46, 50, 53, 57, 61, 65, 69,
    74, 78, 83, 87, 92, 97, 101, 106, 111, 116, 120, 125, 130, 135, 140, 145,
    149, 154, 159, 164, 169, 174, 178, 183, 188, 193, 198, 203, 208, 213, 218,
    222, 227, 232, 237, 242, 247, 252, 257, 262, 267, 272, 277, 281, 287, 291,
    296, 301, 306, 311, 316, 321, 326, 331, 336, 341, 346, 351, 356, 361, 366,
    371, 376, 381, 386, 391, 396, 400, 406, 411, 415, 421, 425, 431, 436, 441,
    446, 451, 456, 461, 466, 471, 476, 481, 486, 491, 496, 501, 506, 511, 517,
    522, 527, 532, 537, 542, 547, 552, 557, 563, 568, 573, 578, 583, 589, 594,
    599, 604, 609, 615, 620, 625, 630, 636, 641, 646, 651, 657, 662, 667, 672,
    677, 683, 688, 694, 699, 704, 709, 715, 720, 725, 731, 736, 742, 747, 752,
    758, 763, 769, 774, 779, 785, 790, 796, 801, 807, 812, 818, 823, 829, 834,
    839, 845, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 10, 11, 12,
    13, 14, 15, 15, 16, 17, 19, 20, 21, 22, 24, 25, 27, 29, 31, 33, 35, 37, 40,
    43, 46, 49, 53, 56, 60, 64, 69, 73, 77, 82, 87, 91, 96, 100, 105, 110, 115,
    119, 124, 129, 134, 139, 143, 148, 153, 158, 163, 168, 172, 177, 182, 187,
    192, 197, 202, 206, 211, 216, 221, 226, 231, 236, 241, 246, 251, 255, 260,
    265, 270, 275, 280, 285, 290, 295, 300, 305, 309, 314, 319, 324, 329, 334,
    339, 344, 349, 354, 359, 364, 369, 374, 379, 384, 389, 394, 398, 404, 409,
    413, 419, 423, 429, 434, 438, 444, 449, 454, 459, 464, 468, 474, 479, 484,
    489, 494, 499, 504, 509, 514, 519, 524, 529, 535, 540, 545, 550, 555, 560,
    565, 570, 576, 581, 586, 591, 596, 601, 607, 612, 617, 622, 627, 633, 638,
    643, 648, 654, 659, 664, 669, 674, 680, 685, 690, 696, 701, 706, 712, 717,
    722, 728, 733, 738, 744, 749, 754, 760, 765, 771, 776, 781, 787, 792, 798,
    803, 809, 814, 820, 825, 831, 836, 842, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6,
    6, 7, 8, 8, 9, 10, 10, 11, 12, 13, 14, 14, 15, 16, 17, 18, 20, 21, 22, 24,
    25, 27, 28, 30, 32, 35, 37, 40, 42, 45, 49, 52, 56, 60, 64, 68, 72, 77, 81,
    86, 90, 95, 99, 104, 109, 114, 118, 123, 128, 133, 138, 142, 147, 152, 157,
    161, 166, 171, 176, 181, 186, 190, 195, 200, 205, 210, 215, 220, 225, 229,
    234, 239, 244, 249, 254, 259, 264, 269, 274, 278, 283, 288, 293, 298, 303,
    308, 313, 318, 323, 328, 332, 337, 342, 347, 352, 357, 362, 367, 372, 377,
    382, 387, 392, 396, 402, 407, 411, 416, 421, 426, 431, 436, 441, 446, 451,
    456, 461, 466, 471, 476, 481, 486, 491, 497, 502, 507, 512, 517, 522, 527,
    532, 537, 542, 547, 552, 557, 563, 568, 573, 578, 583, 588, 594, 599, 604,
    609, 614, 619, 625, 630, 635, 640, 645, 651, 656, 661, 666, 671, 677, 682,
    687, 693, 698, 703, 709, 714, 719, 724, 730, 735, 740, 746, 751, 756, 762,
    767, 773, 778, 783, 789, 794, 800, 805, 811, 816, 822, 827, 832, 838, 0, 1,
    1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 10, 11, 12, 13, 14, 14, 15,
    16, 17, 18, 20, 21, 22, 23, 25, 26, 28, 30, 32, 34, 37, 39, 42, 45, 48, 52,
    55, 59, 63, 67, 71, 76, 80, 85, 89, 94, 99, 103, 108, 113, 117, 122, 127,
    132, 136, 141, 146, 151, 156, 160, 165, 170, 175, 180, 184, 189, 194, 199,
    204, 209, 213, 218, 223, 228, 233, 238, 243, 248, 252, 257, 262, 267, 272,
    277, 282, 286, 292, 296, 301, 306, 311, 316, 321, 326, 331, 336, 341, 345,
    350, 355, 360, 365, 370, 375, 380, 385, 390, 394, 400, 404, 409, 414, 419,
    424, 429, 434, 439, 444, 449, 454, 459, 464, 469, 474, 479, 484, 489, 494,
    499, 504, 509, 514, 519, 524, 530, 535, 540, 545, 550, 555, 560, 565, 570,
    575, 581, 586, 591, 596, 601, 606, 611, 617, 622, 627, 632, 637, 643, 648,
    653, 658, 663, 668, 674, 679, 684, 690, 695, 700, 705, 711, 716, 721, 727,
    732, 737, 743, 748, 753, 759, 764, 769, 775, 780, 785, 791, 796, 802, 807,
    813, 818, 824, 829, 834, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9,
    10, 10, 11, 12, 13, 13, 14, 15, 16, 17, 18, 19, 21, 22, 23, 25, 26, 28, 30,
    32, 34, 36, 39, 42, 44, 48, 51, 55, 58, 62, 66, 71, 75, 79, 84, 89, 93, 98,
    102, 107, 112, 116, 121, 126, 131, 135, 140, 145, 150, 154, 159, 164, 169,
    174, 178, 183, 188, 193, 198, 202, 207, 212, 217, 222, 226, 232, 236, 241,
    246, 251, 256, 261, 266, 270, 275, 280, 285, 290, 295, 300, 304, 309, 314,
    319, 324, 329, 334, 339, 344, 348, 353, 358, 363, 368, 373, 378, 383, 388,
    392, 398, 402, 407, 412, 417, 422, 427, 432, 437, 442, 447, 452, 457, 462,
    467, 472, 477, 482, 487, 492, 497, 502, 507, 512, 517, 522, 527, 532, 537,
    542, 547, 552, 557, 563, 568, 573, 578, 583, 588, 593, 598, 604, 609, 614,
    619, 624, 629, 634, 640, 645, 650, 655, 660, 665, 671, 676, 681, 687, 692,
    697, 702, 708, 713, 718, 723, 729, 734, 739, 745, 750, 755, 761, 766, 771,
    777, 782, 787, 793, 798, 804, 809, 815, 820, 825, 831, 0, 1, 1, 2, 2, 3, 3,
    4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 10, 11, 12, 13, 13, 14, 15, 16, 17, 18, 19,
    21, 22, 23, 25, 26, 28, 30, 32, 34, 36, 38, 41, 44, 47, 50, 54, 58, 62, 66,
    70, 74, 79, 83, 88, 92, 97, 102, 106, 111, 115, 120, 125, 129, 134, 139, 144,
    148, 153, 158, 163, 168, 172, 177, 182, 187, 192, 196, 201, 206, 211, 215,
    220, 225, 230, 235, 240, 245, 249, 254, 259, 264, 269, 274, 279, 283, 288,
    293, 298, 303, 308, 312, 317, 322, 327, 332, 337, 342, 347, 352, 356, 361,
    366, 371, 376, 381, 386, 390, 396, 400, 405, 410, 415, 420, 425, 430, 435,
    440, 445, 450, 455, 459, 465, 469, 474, 479, 484, 489, 494, 499, 504, 509,
    514, 520, 525, 530, 535, 540, 545, 550, 555, 560, 565, 570, 575, 580, 585,
    590, 596, 601, 606, 611, 616, 621, 626, 632, 637, 642, 647, 652, 657, 662,
    668, 673, 678, 683, 689, 694, 699, 704, 710, 715, 720, 725, 731, 736, 741,
    746, 752, 757, 763, 768, 773, 779, 784, 789, 795, 800, 806, 811, 816, 822,
    827, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 10, 10, 11, 12, 13,
    13, 14, 15, 16, 17, 18, 19, 20, 22, 23, 24, 26, 28, 29, 31, 33, 36, 38, 41,
    44, 47, 50, 53, 57, 61, 65, 69, 74, 78, 82, 87, 91, 96, 101, 105, 110, 114,
    119, 124, 128, 133, 138, 143, 147, 152, 157, 162, 166, 171, 176, 181, 185,
    190, 195, 200, 205, 209, 214, 219, 224, 229, 233, 238, 243, 248, 253, 258,
    262, 267, 272, 277, 282, 287, 291, 296, 301, 306, 311, 316, 321, 325, 330,
    335, 340, 345, 350, 355, 359, 364, 369, 374, 379, 384, 388, 393, 398, 403,
    408, 413, 418, 423, 428, 433, 438, 443, 447, 452, 457, 462, 467, 472, 477,
    482, 487, 492, 497, 502, 507, 512, 517, 522, 527, 532, 537, 542, 547, 552,
    557, 562, 567, 573, 578, 583, 588, 593, 598, 603, 608, 613, 618, 624, 629,
    634, 639, 644, 649, 655, 660, 665, 670, 675, 680, 686, 691, 696, 701, 707,
    712, 717, 722, 728, 733, 738, 743, 749, 754, 759, 765, 770, 775, 781, 786,
    791, 797, 802, 808, 813, 818, 824, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7,
    8, 8, 9, 10, 10, 11, 12, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 23, 24, 26,
    27, 29, 31, 33, 35, 38, 40, 43, 46, 49, 53, 57, 60, 64, 68, 73, 77, 82, 86,
    91, 95, 100, 104, 109, 113, 118, 123, 127, 132, 137, 142, 146, 151, 156, 161,
    165, 170, 175, 179, 184, 189, 194, 199, 203, 208, 213, 218, 222, 227, 232,
    237, 242, 246, 251, 256, 261, 266, 270, 275, 280, 285, 290, 295, 299, 304,
    309, 314, 319, 324, 328, 333, 338, 343, 348, 353, 357, 362, 367, 372, 377,
    382, 386, 391, 396, 401, 406, 411, 416, 421, 426, 431, 435, 440, 445, 450,
    455, 460, 465, 470, 475, 480, 485, 490, 495, 500, 505, 510, 515, 520, 525,
    530, 534, 539, 544, 550, 555, 560, 565, 570, 575, 580, 585, 590, 595, 600,
    605, 611, 616, 621, 626, 631, 636, 641, 646, 652, 657, 662, 667, 672, 677,
    683, 688, 693, 698, 703, 709, 714, 719, 724, 730, 735, 740, 745, 751, 756,
    761, 766, 772, 777, 783, 788, 793, 799, 804, 809, 815, 820, 0, 1, 1, 2, 2, 3,
    3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 9, 10, 11, 12, 12, 13, 14, 15, 16, 17, 18,
    19, 20, 21, 23, 24, 26, 27, 29, 31, 33, 35, 37, 40, 43, 46, 49, 52, 56, 60,
    64, 68, 72, 76, 81, 85, 90, 94, 99, 103, 108, 112, 117, 122, 126, 131, 136,
    141, 145, 150, 154, 159, 164, 169, 173, 178, 183, 188, 192, 197, 202, 207,
    211, 216, 221, 226, 231, 235, 240, 245, 250, 255, 259, 264, 269, 274, 278,
    283, 288, 293, 298, 302, 307, 312, 317, 322, 327, 331, 336, 341, 346, 351,
    356, 360, 365, 370, 375, 380, 384, 389, 394, 399, 404, 409, 414, 419, 423,
    428, 433, 438, 443, 448, 453, 458, 462, 468, 472, 477, 482, 487, 492, 497,
    502, 507, 512, 517, 522, 527, 532, 537, 542, 547, 552, 557, 562, 567, 572,
    577, 582, 587, 592, 598, 603, 608, 613, 618, 623, 628, 633, 638, 643, 649,
    654, 659, 664, 669, 674, 680, 685, 690, 695, 700, 705, 711, 716, 721, 726,
    732, 737, 742, 747, 753, 758, 763, 768, 774, 779, 784, 790, 795, 800, 806,
    811, 817, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 11, 12,
    12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 23, 24, 25, 27, 29, 31, 33, 35, 37,
    40, 42, 45, 48, 52, 55, 59, 63, 67, 71, 75, 80, 84, 89, 93, 98, 102, 107,
    111, 116, 121, 125, 130, 135, 139, 144, 149, 153, 158, 163, 168, 172, 177,
    182, 186, 191, 196, 201, 205, 210, 215, 219, 224, 229, 234, 239, 243, 248,
    253, 258, 263, 267, 272, 277, 282, 287, 291, 296, 301, 306, 311, 315, 320,
    325, 330, 335, 339, 344, 349, 354, 359, 363, 368, 373, 378, 382, 387, 392,
    397, 402, 407, 412, 416, 421, 426, 431, 436, 441, 446, 450, 455, 460, 465,
    470, 475, 480, 485, 490, 495, 500, 505, 510, 515, 520, 525, 529, 534, 539,
    545, 550, 555, 560, 565, 570, 575, 579, 585, 590, 595, 600, 605, 610, 615,
    620, 625, 630, 635, 641, 646, 651, 656, 661, 666, 671, 676, 682, 687, 692,
    697, 702, 707, 713, 718, 723, 728, 733, 739, 744, 749, 755, 760, 765, 770,
    776, 781, 786, 792, 797, 802, 808, 813, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6,
    6, 7, 7, 8, 9, 9, 10, 11, 12, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 24,
    25, 27, 28, 30, 32, 34, 37, 39, 42, 45, 48, 51, 55, 58, 62, 66, 70, 75, 79,
    83, 88, 92, 97, 101, 106, 110, 115, 120, 124, 129, 134, 138, 143, 148, 152,
    157, 162, 166, 171, 176, 180, 185, 190, 195, 199, 204, 209, 214, 218, 223,
    228, 233, 237, 242, 247, 252, 256, 261, 266, 271, 275, 280, 285, 290, 294,
    299, 304, 309, 314, 318, 323, 328, 333, 337, 342, 347, 352, 357, 361, 366,
    371, 376, 380, 385, 390, 395, 400, 405, 410, 414, 419, 424, 429, 434, 439,
    443, 448, 453, 458, 463, 468, 473, 478, 483, 487, 492, 497, 502, 507, 512,
    517, 522, 527, 532, 537, 542, 547, 552, 557, 562, 567, 572, 577, 582, 587,
    592, 597, 602, 607, 612, 617, 622, 627, 633, 638, 643, 648, 653, 658, 663,
    668, 673, 679, 684, 689, 694, 699, 704, 709, 715, 720, 725, 730, 735, 741,
    746, 751, 756, 762, 767, 772, 778, 783, 788, 793, 799, 804, 810, 0, 1, 1, 2,
    2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 11, 11, 12, 13, 14, 15, 16, 17,
    18, 19, 20, 21, 22, 24, 25, 27, 28, 30, 32, 34, 36, 39, 41, 44, 47, 51, 54,
    58, 62, 65, 70, 74, 78, 83, 87, 91, 96, 101, 105, 109, 114, 119, 123, 128,
    133, 137, 142, 147, 151, 156, 161, 165, 170, 175, 179, 184, 189, 193, 198,
    203, 207, 212, 217, 222, 226, 231, 236, 241, 245, 250, 255, 260, 264, 269,
    274, 279, 283, 288, 293, 297, 302, 307, 312, 317, 321, 326, 331, 336, 340,
    345, 350, 355, 360, 364, 369, 374, 378, 383, 388, 393, 398, 402, 407, 412,
    417, 422, 427, 432, 436, 441, 446, 451, 456, 461, 466, 470, 475, 480, 485,
    490, 495, 500, 505, 510, 515, 520, 524, 529, 534, 539, 544, 549, 554, 559,
    564, 569, 574, 579, 584, 589, 594, 599, 604, 609, 614, 619, 625, 630, 635,
    640, 645, 650, 655, 660, 665, 670, 675, 681, 686, 691, 696, 701, 706, 711,
    717, 722, 727, 732, 737, 743, 748, 753, 758, 764, 769, 774, 779, 785, 790,
    795, 800, 806, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 11,
    11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 25, 26, 28, 30, 32, 34,
    36, 38, 41, 44, 47, 50, 54, 57, 61, 65, 69, 73, 77, 82, 86, 90, 95, 100, 104,
    109, 113, 118, 122, 127, 132, 136, 141, 145, 150, 155, 159, 164, 169, 173,
    178, 183, 187, 192, 197, 201, 206, 211, 215, 220, 225, 230, 234, 239, 244,
    248, 253, 258, 263, 267, 272, 277, 282, 286, 291, 296, 300, 305, 310, 315,
    320, 324, 329, 334, 339, 343, 348, 353, 358, 362, 367, 372, 376, 381, 386,
    391, 396, 400, 405, 410, 415, 420, 425, 429, 434, 439, 444, 449, 453, 458,
    463, 468, 473, 478, 483, 488, 492, 497, 502, 507, 512, 517, 522, 527, 532,
    537, 542, 547, 552, 557, 562, 567, 571, 576, 581, 586, 591, 596, 602, 607,
    612, 617, 622, 627, 632, 637, 642, 647, 652, 657, 662, 667, 672, 677, 683,
    688, 693, 698, 703, 708, 713, 719, 724, 729, 734, 739, 745, 750, 755, 760,
    765, 771, 776, 781, 786, 792, 797, 802, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6,
    6, 7, 7, 8, 9, 9, 10, 11, 11, 12, 13, 14, 15, 16, 17, 17, 19, 20, 21, 22, 23,
    25, 26, 28, 30, 31, 34, 36, 38, 41, 43, 46, 50, 53, 57, 60, 64, 68, 72, 77,
    81, 85, 90, 94, 99, 103, 108, 112, 117, 121, 126, 131, 135, 140, 144, 149,
    154, 158, 163, 167, 172, 177, 181, 186, 191, 195, 200, 205, 209, 214, 219,
    223, 228, 233, 238, 242, 247, 252, 256, 261, 266, 270, 275, 280, 285, 289,
    294, 299, 304, 308, 313, 318, 323, 327, 332, 337, 341, 346, 351, 356, 360,
    365, 370, 375, 379, 384, 389, 394, 398, 403, 408, 413, 418, 422, 427, 432,
    437, 441, 446, 451, 456, 461, 466, 471, 475, 480, 485, 490, 495, 500, 505,
    510, 515, 519, 524, 529, 534, 539, 544, 549, 554, 559, 564, 569, 574, 579,
    584, 589, 594, 599, 604, 609, 614, 619, 624, 629, 634, 639, 644, 649, 654,
    659, 664, 669, 674, 679, 685, 690, 695, 700, 705, 710, 715, 720, 726, 731,
    736, 741, 746, 752, 757, 762, 767, 772, 778, 783, 788, 793, 799, 0, 1, 1, 2,
    2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 11, 11, 12, 13, 14, 15, 15, 16,
    17, 18, 20, 21, 22, 23, 25, 26, 28, 29, 31, 33, 35, 38, 40, 43, 46, 49, 52,
    56, 60, 63, 67, 71, 76, 80, 84, 89, 93, 98, 102, 107, 111, 116, 120, 125,
    129, 134, 138, 143, 148, 152, 157, 162, 166, 171, 175, 180, 185, 189, 194,
    199, 203, 208, 213, 217, 222, 227, 231, 236, 241, 245, 250, 255, 259, 264,
    269, 274, 278, 283, 288, 292, 297, 302, 307, 311, 316, 321, 325, 330, 335,
    340, 344, 349, 354, 358, 363, 368, 373, 377, 382, 387, 392, 396, 401, 406,
    411, 415, 420, 425, 430, 435, 439, 444, 449, 454, 459, 463, 468, 473, 478,
    483, 488, 492, 497, 502, 507, 512, 517, 522, 527, 532, 536, 541, 546, 551,
    556, 561, 566, 571, 576, 581, 586, 591, 596, 601, 606, 611, 616, 621, 626,
    631, 636, 641, 646, 651, 656, 661, 666, 671, 676, 681, 686, 692, 697, 702,
    707, 712, 717, 722, 727, 733, 738, 743, 748, 753, 759, 764, 769, 774, 779,
    785, 790, 795, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 11,
    11, 12, 13, 14, 15, 15, 16, 17, 18, 19, 21, 22, 23, 24, 26, 27, 29, 31, 33,
    35, 37, 40, 43, 45, 48, 52, 55, 59, 63, 67, 71, 75, 79, 84, 88, 92, 97, 101,
    106, 110, 115, 119, 124, 128, 133, 137, 142, 146, 151, 156, 160, 165, 170,
    174, 179, 183, 188, 193, 197, 202, 207, 211, 216, 221, 225, 230, 235, 239,
    244, 249, 253, 258, 263, 267, 272, 277, 281, 286, 291, 295, 300, 305, 310,
    314, 319, 324, 328, 333, 338, 342, 347, 352, 357, 361, 366, 371, 375, 380,
    385, 390, 394, 399, 404, 409, 413, 418, 423, 428, 432, 437, 442, 447, 452,
    456, 461, 466, 471, 476, 480, 485, 490, 495, 500, 505, 509, 514, 519, 524,
    529, 534, 539, 544, 549, 554, 558, 563, 568, 573, 578, 583, 588, 593, 598,
    603, 608, 613, 618, 623, 628, 633, 638, 643, 648, 653, 658, 663, 668, 673,
    678, 683, 688, 693, 699, 704, 709, 714, 719, 724, 729, 734, 739, 745, 750,
    755, 760, 766, 771, 776, 781, 786, 792, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6,
    6, 7, 7, 8, 9, 9, 10, 11, 11, 12, 13, 14, 14, 15, 16, 17, 18, 19, 20, 22, 23,
    24, 26, 27, 29, 31, 33, 35, 37, 39, 42, 45, 48, 51, 55, 58, 62, 66, 70, 74,
    78, 83, 87, 91, 96, 100, 105, 109, 114, 118, 123, 127, 132, 136, 141, 145,
    150, 155, 159, 164, 168, 173, 178, 182, 187, 191, 196, 200, 205, 210, 215,
    219, 224, 228, 233, 238, 242, 247, 252, 256, 261, 266, 270, 275, 280, 284,
    289, 294, 298, 303, 308, 312, 317, 322, 326, 331, 336, 341, 345, 350, 355,
    359, 364, 369, 373, 378, 383, 388, 392, 397, 402, 406, 411, 416, 421, 425,
    430, 435, 440, 444, 449, 454, 459, 464, 468, 473, 478, 483, 488, 492, 497,
    502, 507, 512, 517, 521, 526, 531, 536, 541, 546, 551, 556, 560, 566, 570,
    575, 580, 585, 590, 595, 600, 605, 610, 615, 620, 625, 630, 635, 640, 645,
    650, 655, 660, 665, 670, 675, 680, 685, 690, 695, 700, 706, 710, 716, 721,
    726, 731, 736, 741, 747, 752, 757, 762, 767, 772, 778, 783, 788, 0, 1, 1, 2,
    2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 11, 11, 12, 13, 14, 14, 15, 16,
    17, 18, 19, 20, 22, 23, 24, 26, 27, 29, 30, 32, 34, 37, 39, 42, 44, 47, 51,
    54, 58, 61, 65, 69, 73, 78, 82, 86, 91, 95, 99, 104, 108, 113, 117, 122, 126,
    131, 135, 140, 144, 149, 153, 158, 163, 167, 172, 176, 181, 185, 190, 195,
    199, 204, 208, 213, 218, 222, 227, 232, 236, 241, 246, 250, 255, 259, 264,
    269, 273, 278, 283, 287, 292, 297, 301, 306, 311, 315, 320, 325, 329, 334,
    339, 343, 348, 353, 357, 362, 367, 371, 376, 381, 385, 390, 395, 400, 404,
    409, 414, 418, 423, 428, 433, 437, 442, 447, 452, 456, 461, 466, 471, 476,
    480, 485, 490, 495, 500, 504, 509, 514, 519, 524, 529, 534, 538, 543, 548,
    553, 558, 563, 568, 573, 578, 582, 587, 592, 597, 602, 607, 612, 617, 622,
    627, 632, 637, 642, 647, 652, 657, 662, 667, 672, 677, 682, 687, 692, 697,
    702, 707, 712, 718, 723, 728, 733, 738, 743, 748, 753, 759, 764, 769, 774,
    779, 785, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 11, 11,
    12, 13, 14, 14, 15, 16, 17, 18, 19, 20, 21, 23, 24, 25, 27, 29, 30, 32, 34,
    36, 39, 41, 44, 47, 50, 53, 57, 61, 65, 68, 73, 77, 81, 85, 90, 94, 98, 103,
    107, 112, 116, 121, 125, 130, 134, 139, 143, 148, 152, 157, 161, 166, 170,
    175, 180, 184, 189, 193, 198, 203, 207, 212, 216, 221, 226, 230, 235, 239,
    244, 249, 253, 258, 262, 267, 272, 276, 281, 286, 290, 295, 300, 304, 309,
    314, 318, 323, 327, 332, 337, 341, 346, 351, 355, 360, 365, 369, 374, 379,
    383, 388, 393, 397, 402, 407, 412, 416, 421, 426, 430, 435, 440, 445, 449,
    454, 459, 464, 468, 473, 478, 483, 488, 492, 497, 502, 507, 511, 516, 521,
    526, 531, 536, 541, 545, 550, 555, 560, 565, 570, 575, 580, 585, 589, 594,
    599, 604, 609, 614, 619, 624, 629, 634, 639, 644, 649, 654, 659, 664, 669,
    674, 679, 684, 689, 694, 699, 704, 709, 714, 719, 724, 729, 735, 740, 745,
    750, 755, 760, 765, 771, 776, 781, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7,
    7, 8, 9, 9, 10, 10, 11, 12, 13, 13, 14, 15, 16, 17, 18, 19, 20, 21, 23, 24,
    25, 27, 28, 30, 32, 34, 36, 38, 41, 44, 46, 50, 53, 56, 60, 64, 68, 72, 76,
    80, 84, 89, 93, 97, 102, 106, 111, 115, 120, 124, 129, 133, 138, 142, 147,
    151, 156, 160, 165, 169, 174, 178, 183, 187, 192, 196, 201, 206, 210, 215,
    219, 224, 229, 233, 238, 242, 247, 252, 256, 261, 266, 270, 275, 279, 284,
    288, 293, 298, 302, 307, 312, 316, 321, 326, 330, 335, 340, 344, 349, 353,
    358, 363, 367, 372, 377, 381, 386, 391, 395, 400, 405, 409, 414, 419, 423,
    428, 433, 437, 442, 447, 452, 457, 461, 466, 471, 476, 480, 485, 490, 495,
    499, 504, 509, 514, 519, 523, 528, 533, 538, 543, 548, 552, 557, 562, 567,
    572, 577, 582, 587, 592, 596, 601, 606, 611, 616, 621, 626, 631, 636, 641,
    646, 651, 656, 661, 666, 671, 676, 681, 686, 691, 696, 701, 706, 711, 716,
    721, 726, 731, 736, 741, 746, 752, 757, 762, 767, 772, 777, 0, 1, 1, 2, 2, 3,
    3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 12, 13, 13, 14, 15, 16, 17,
    18, 19, 20, 21, 22, 24, 25, 27, 28, 30, 32, 34, 36, 38, 40, 43, 46, 49, 52,
    56, 59, 63, 67, 71, 75, 79, 83, 88, 92, 96, 101, 105, 110, 114, 119, 123,
    127, 132, 136, 141, 145, 150, 154, 159, 163, 168, 172, 177, 182, 186, 191,
    195, 200, 204, 209, 213, 218, 223, 227, 232, 236, 241, 246, 250, 255, 259,
    264, 269, 273, 278, 282, 287, 292, 296, 301, 305, 310, 315, 319, 324, 328,
    333, 338, 342, 347, 351, 356, 361, 365, 370, 374, 379, 384, 389, 393, 398,
    403, 407, 412, 417, 421, 426, 431, 435, 440, 445, 449, 454, 459, 464, 468,
    473, 478, 483, 487, 492, 497, 502, 506, 511, 516, 521, 526, 530, 535, 540,
    545, 550, 555, 559, 564, 569, 574, 579, 584, 589, 594, 598, 603, 608, 613,
    618, 623, 628, 633, 638, 643, 648, 653, 658, 662, 667, 672, 677, 682, 687,
    692, 697, 702, 708, 713, 718, 723, 728, 733, 738, 743, 748, 753, 758, 763,
    768, 774, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11,
    12, 13, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 24, 25, 26, 28, 30, 31, 33,
    35, 38, 40, 43, 45, 48, 52, 55, 59, 62, 66, 70, 74, 78, 82, 87, 91, 96, 100,
    104, 109, 113, 118, 122, 126, 131, 135, 140, 144, 149, 153, 158, 162, 167,
    171, 176, 180, 185, 189, 194, 198, 203, 207, 212, 217, 221, 226, 230, 235,
    239, 244, 248, 253, 258, 262, 267, 271, 276, 280, 285, 290, 294, 299, 304,
    308, 313, 317, 322, 327, 331, 336, 340, 345, 350, 354, 359, 363, 368, 372,
    377, 382, 386, 391, 396, 400, 405, 410, 414, 419, 424, 428, 433, 438, 442,
    447, 452, 457, 461, 466, 471, 475, 480, 485, 490, 494, 499, 504, 509, 513,
    518, 523, 528, 533, 537, 542, 547, 552, 557, 562, 566, 571, 576, 581, 586,
    591, 595, 600, 605, 610, 615, 620, 625, 630, 635, 640, 645, 649, 654, 659,
    664, 669, 674, 679, 684, 689, 694, 699, 704, 709, 714, 719, 724, 729, 734,
    740, 745, 750, 755, 760, 765, 770, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7,
    7, 8, 8, 9, 10, 10, 11, 12, 13, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23,
    25, 26, 28, 29, 31, 33, 35, 37, 40, 42, 45, 48, 51, 54, 58, 62, 65, 69, 73,
    78, 82, 86, 90, 95, 99, 103, 108, 112, 116, 121, 125, 130, 134, 138, 143,
    148, 152, 156, 161, 165, 170, 174, 179, 183, 188, 192, 197, 201, 206, 211,
    215, 220, 224, 229, 233, 238, 242, 247, 252, 256, 261, 265, 270, 274, 279,
    283, 288, 293, 297, 302, 306, 311, 315, 320, 325, 329, 334, 338, 343, 348,
    352, 357, 361, 366, 370, 375, 380, 384, 389, 394, 398, 403, 408, 412, 417,
    421, 426, 431, 435, 440, 445, 449, 454, 459, 464, 468, 473, 478, 482, 487,
    492, 496, 501, 506, 511, 516, 520, 525, 530, 535, 540, 544, 549, 554, 559,
    564, 568, 573, 578, 583, 588, 593, 597, 602, 607, 612, 617, 622, 627, 632,
    636, 641, 646, 651, 656, 661, 666, 671, 676, 681, 686, 691, 696, 701, 706,
    711, 716, 721, 726, 731, 736, 741, 746, 751, 756, 761, 767, 0, 1, 1, 2, 2, 3,
    3, 4, 4, 4, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 12, 12, 13, 14, 15, 16, 17,
    18, 19, 20, 21, 22, 23, 25, 26, 27, 29, 31, 33, 35, 37, 39, 42, 45, 47, 51,
    54, 57, 61, 65, 69, 73, 77, 81, 85, 89, 94, 98, 102, 107, 111, 115, 120, 124,
    128, 133, 137, 142, 146, 151, 155, 160, 164, 169, 173, 178, 182, 187, 191,
    196, 200, 205, 209, 214, 218, 223, 227, 232, 236, 241, 245, 250, 254, 259,
    264, 268, 273, 277, 282, 286, 291, 295, 300, 305, 309, 314, 318, 323, 327,
    332, 336, 341, 346, 350, 355, 359, 364, 368, 373, 378, 382, 387, 391, 396,
    401, 405, 410, 415, 419, 424, 428, 433, 438, 442, 447, 452, 456, 461, 466,
    470, 475, 480, 485, 489, 494, 499, 503, 508, 513, 518, 522, 527, 532, 537,
    541, 546, 551, 556, 561, 566, 570, 575, 580, 585, 590, 594, 599, 604, 609,
    614, 619, 624, 628, 633, 638, 643, 648, 653, 658, 663, 668, 673, 678, 683,
    687, 692, 697, 702, 707, 712, 717, 722, 727, 732, 738, 743, 748, 753, 758,
    763, 0, 1, 1, 2, 2, 3, 3, 4, 4, 4, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 12,
    12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 26, 27, 29, 31, 32, 34,
    37, 39, 41, 44, 47, 50, 53, 57, 60, 64, 68, 72, 76, 80, 84, 88, 93, 97, 101,
    106, 110, 114, 119, 123, 127, 132, 136, 141, 145, 150, 154, 158, 163, 167,
    172, 176, 181, 185, 190, 194, 199, 203, 208, 212, 217, 221, 226, 230, 235,
    239, 244, 248, 253, 257, 262, 266, 271, 275, 280, 285, 289, 294, 298, 303,
    307, 312, 316, 321, 325, 330, 335, 339, 344, 348, 353, 357, 362, 366, 371,
    375, 380, 385, 389, 394, 398, 403, 408, 412, 417, 422, 426, 431, 435, 440,
    445, 449, 454, 459, 463, 468, 473, 477, 482, 487, 491, 496, 501, 506, 510,
    515, 520, 525, 529, 534, 539, 544, 548, 553, 558, 563, 567, 572, 577, 582,
    587, 591, 596, 601, 606, 611, 616, 621, 625, 630, 635, 640, 645, 650, 655,
    660, 664, 669, 674, 679, 684, 689, 694, 699, 704, 709, 714, 719, 724, 729,
    734, 739, 744, 749, 754, 759, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 6, 6, 7, 7, 8,
    8, 9, 10, 10, 11, 12, 12, 13, 14, 15, 16, 17, 17, 18, 19, 21, 22, 23, 24, 26,
    27, 29, 30, 32, 34, 36, 38, 41, 44, 46, 49, 53, 56, 60, 63, 67, 71, 75, 79,
    83, 87, 92, 96, 100, 105, 109, 113, 118, 122, 126, 131, 135, 140, 144, 148,
    153, 157, 162, 166, 171, 175, 179, 184, 188, 193, 197, 202, 206, 211, 215,
    220, 224, 229, 233, 238, 242, 247, 251, 256, 260, 265, 269, 274, 278, 283,
    287, 292, 296, 301, 305, 310, 314, 319, 324, 328, 333, 337, 342, 346, 351,
    355, 360, 364, 369, 373, 378, 383, 387, 392, 396, 401, 405, 410, 415, 419,
    424, 428, 433, 438, 442, 447, 452, 456, 461, 466, 470, 475, 480, 484, 489,
    493, 498, 503, 508, 512, 517, 522, 527, 531, 536, 541, 546, 550, 555, 560,
    565, 569, 574, 579, 584, 589, 593, 598, 603, 608, 613, 617, 622, 627, 632,
    637, 642, 647, 651, 656, 661, 666, 671, 676, 681, 686, 691, 696, 701, 705,
    710, 715, 720, 725, 730, 735, 740, 745, 750, 756, 0, 1, 1, 2, 2, 3, 3, 3, 4,
    4, 5, 5, 6, 7, 7, 8, 8, 9, 10, 10, 11, 12, 12, 13, 14, 15, 16, 16, 17, 18,
    19, 20, 22, 23, 24, 25, 27, 28, 30, 32, 34, 36, 38, 41, 43, 46, 49, 52, 55,
    59, 62, 66, 70, 74, 78, 82, 87, 91, 95, 99, 104, 108, 112, 117, 121, 125,
    130, 134, 138, 143, 147, 152, 156, 160, 165, 169, 174, 178, 183, 187, 191,
    196, 200, 205, 209, 214, 218, 223, 227, 232, 236, 241, 245, 249, 254, 259,
    263, 267, 272, 276, 281, 286, 290, 295, 299, 304, 308, 313, 317, 322, 326,
    331, 335, 340, 344, 349, 353, 358, 362, 367, 371, 376, 380, 385, 390, 394,
    399, 403, 408, 412, 417, 421, 426, 431, 435, 440, 445, 449, 454, 458, 463,
    468, 472, 477, 482, 486, 491, 496, 500, 505, 510, 514, 519, 524, 529, 533,
    538, 543, 547, 552, 557, 562, 566, 571, 576, 581, 586, 590, 595, 600, 605,
    609, 614, 619, 624, 629, 634, 638, 643, 648, 653, 658, 663, 668, 673, 677,
    682, 687, 692, 697, 702, 707, 712, 717, 722, 727, 732, 737, 742, 747, 752, 0,
    1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 8, 9, 10, 10, 11, 12, 12, 13,
    14, 15, 15, 16, 17, 18, 19, 20, 21, 23, 24, 25, 27, 28, 30, 32, 34, 36, 38,
    40, 43, 45, 48, 51, 55, 58, 62, 66, 69, 73, 77, 81, 86, 90, 94, 98, 103, 107,
    111, 116, 120, 124, 129, 133, 137, 142, 146, 150, 155, 159, 164, 168, 172,
    177, 181, 186, 190, 194, 199, 203, 208, 212, 217, 221, 226, 230, 235, 239,
    244, 248, 252, 257, 261, 266, 270, 275, 279, 284, 288, 293, 297, 302, 306,
    311, 315, 320, 324, 329, 333, 338, 342, 347, 351, 356, 360, 365, 369, 374,
    378, 383, 387, 392, 396, 401, 406, 410, 415, 419, 424, 428, 433, 438, 442,
    447, 451, 456, 461, 465, 470, 474, 479, 484, 488, 493, 498, 502, 507, 512,
    516, 521, 526, 530, 535, 540, 545, 549, 554, 559, 564, 568, 573, 578, 583,
    587, 592, 597, 602, 606, 611, 616, 621, 626, 630, 635, 640, 645, 650, 655,
    659, 664, 669, 674, 679, 684, 689, 694, 698, 703, 708, 713, 718, 723, 728,
    733, 738, 743, 748, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 8, 9, 10,
    10, 11, 12, 12, 13, 14, 15, 15, 16, 17, 18, 19, 20, 21, 23, 24, 25, 27, 28,
    30, 31, 33, 35, 37, 40, 42, 45, 48, 51, 54, 57, 61, 65, 69, 72, 76, 81, 85,
    89, 93, 97, 102, 106, 110, 114, 119, 123, 127, 132, 136, 140, 145, 149, 154,
    158, 162, 167, 171, 175, 180, 184, 189, 193, 198, 202, 206, 211, 215, 220,
    224, 229, 233, 237, 242, 246, 251, 255, 260, 264, 269, 273, 278, 282, 287,
    291, 295, 300, 304, 309, 313, 318, 322, 327, 331, 336, 340, 345, 349, 354,
    358, 363, 367, 372, 376, 381, 385, 390, 394, 399, 403, 408, 412, 417, 421,
    426, 431, 435, 440, 444, 449, 453, 458, 463, 467, 472, 477, 481, 486, 490,
    495, 500, 504, 509, 514, 518, 523, 528, 532, 537, 542, 546, 551, 556, 561,
    565, 570, 575, 580, 584, 589, 594, 598, 603, 608, 613, 618, 622, 627, 632,
    637, 642, 646, 651, 656, 661, 666, 670, 675, 680, 685, 690, 695, 700, 705,
    710, 715, 720, 724, 729, 734, 739, 744, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5,
    6, 7, 7, 8, 8, 9, 9, 10, 11, 12, 12, 13, 14, 15, 15, 16, 17, 18, 19, 20, 21,
    22, 24, 25, 26, 28, 29, 31, 33, 35, 37, 39, 42, 44, 47, 50, 53, 57, 60, 64,
    68, 72, 75, 80, 84, 88, 92, 96, 101, 105, 109, 113, 118, 122, 126, 130, 135,
    139, 144, 148, 152, 156, 161, 165, 170, 174, 179, 183, 187, 192, 196, 201,
    205, 209, 214, 218, 223, 227, 231, 236, 240, 245, 249, 254, 258, 262, 267,
    271, 276, 280, 285, 289, 294, 298, 303, 307, 311, 316, 320, 325, 329, 334,
    338, 342, 347, 352, 356, 361, 365, 370, 374, 378, 383, 387, 392, 396, 401,
    405, 410, 414, 419, 424, 428, 433, 437, 442, 446, 451, 456, 460, 465, 469,
    474, 478, 483, 488, 492, 497, 502, 506, 511, 516, 520, 525, 530, 534, 539,
    544, 548, 553, 558, 562, 567, 572, 576, 581, 586, 590, 595, 600, 605, 610,
    614, 619, 624, 629, 633, 638, 643, 648, 653, 657, 662, 667, 672, 677, 682,
    687, 691, 696, 701, 706, 711, 716, 721, 726, 731, 735, 741, 0, 1, 1, 2, 2, 3,
    3, 3, 4, 4, 5, 5, 6, 7, 7, 8, 8, 9, 9, 10, 11, 11, 12, 13, 14, 14, 15, 16,
    17, 18, 19, 20, 21, 22, 23, 25, 26, 28, 29, 31, 33, 35, 37, 39, 41, 44, 47,
    50, 53, 56, 59, 63, 67, 71, 75, 79, 83, 87, 91, 95, 100, 104, 108, 112, 117,
    121, 125, 129, 134, 138, 142, 147, 151, 155, 160, 164, 168, 173, 177, 181,
    186, 190, 195, 199, 203, 208, 212, 217, 221, 225, 230, 234, 239, 243, 248,
    252, 256, 261, 265, 270, 274, 279, 283, 287, 292, 296, 301, 305, 310, 314,
    318, 323, 327, 332, 336, 340, 345, 350, 354, 358, 363, 367, 372, 376, 381,
    385, 390, 394, 399, 403, 408, 412, 417, 421, 426, 430, 435, 439, 444, 448,
    453, 458, 462, 467, 471, 476, 480, 485, 490, 494, 499, 504, 508, 513, 517,
    522, 527, 531, 536, 541, 545, 550, 555, 559, 564, 569, 573, 578, 583, 587,
    592, 597, 602, 606, 611, 616, 621, 625, 630, 635, 640, 644, 649, 654, 659,
    663, 668, 673, 678, 683, 688, 693, 697, 702, 707, 712, 717, 722, 727, 732,
    737, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 9, 10, 11, 11, 12,
    13, 14, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 25, 26, 27, 29, 31, 32, 34,
    36, 39, 41, 43, 46, 49, 52, 55, 59, 62, 66, 70, 74, 78, 82, 86, 90, 94, 98,
    103, 107, 111, 115, 120, 124, 128, 133, 137, 141, 145, 150, 154, 158, 163,
    167, 171, 176, 180, 184, 189, 193, 198, 202, 206, 211, 215, 220, 224, 228,
    233, 237, 241, 246, 250, 255, 259, 263, 268, 272, 277, 281, 286, 290, 294,
    299, 303, 308, 312, 316, 321, 325, 330, 334, 338, 343, 347, 352, 356, 361,
    365, 370, 374, 379, 383, 387, 392, 396, 401, 405, 410, 414, 419, 423, 428,
    432, 437, 441, 446, 451, 455, 460, 464, 469, 473, 478, 482, 487, 492, 496,
    501, 505, 510, 515, 519, 524, 528, 533, 538, 542, 547, 552, 556, 561, 566,
    570, 575, 580, 584, 589, 594, 598, 603, 608, 613, 617, 622, 627, 632, 636,
    641, 646, 651, 655, 660, 665, 670, 675, 679, 684, 689, 694, 699, 703, 708,
    713, 718, 723, 728, 733, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8,
    9, 9, 10, 11, 11, 12, 13, 14, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 26,
    27, 29, 30, 32, 34, 36, 38, 40, 43, 46, 49, 51, 55, 58, 62, 65, 69, 73, 77,
    81, 85, 89, 93, 97, 102, 106, 110, 114, 118, 123, 127, 131, 136, 140, 144,
    148, 153, 157, 161, 166, 170, 174, 179, 183, 187, 192, 196, 200, 205, 209,
    213, 218, 222, 227, 231, 235, 240, 244, 249, 253, 257, 262, 266, 271, 275,
    279, 284, 288, 292, 297, 301, 306, 310, 314, 319, 323, 328, 332, 336, 341,
    345, 350, 354, 358, 363, 367, 372, 376, 381, 385, 390, 394, 398, 403, 407,
    412, 416, 421, 425, 430, 434, 439, 443, 448, 452, 457, 462, 466, 470, 475,
    480, 484, 489, 493, 498, 503, 507, 512, 516, 521, 526, 530, 535, 539, 544,
    549, 553, 558, 563, 567, 572, 577, 581, 586, 591, 595, 600, 605, 609, 614,
    619, 623, 628, 633, 638, 642, 647, 652, 656, 661, 666, 671, 676, 680, 685,
    690, 695, 700, 705, 709, 714, 719, 724, 729, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5,
    5, 6, 6, 7, 8, 8, 9, 9, 10, 11, 11, 12, 13, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 22, 23, 24, 26, 27, 29, 30, 32, 34, 36, 38, 40, 43, 45, 48, 51, 54, 57,
    61, 65, 68, 72, 76, 80, 84, 88, 92, 96, 100, 105, 109, 113, 117, 122, 126,
    130, 134, 139, 143, 147, 151, 156, 160, 164, 169, 173, 177, 182, 186, 190,
    195, 199, 203, 208, 212, 216, 221, 225, 229, 234, 238, 243, 247, 251, 255,
    260, 264, 269, 273, 277, 282, 286, 291, 295, 299, 304, 308, 312, 317, 321,
    326, 330, 334, 339, 343, 347, 352, 356, 361, 365, 370, 374, 378, 383, 387,
    392, 396, 401, 405, 410, 414, 418, 423, 427, 432, 436, 441, 445, 450, 454,
    459, 463, 468, 472, 477, 482, 486, 491, 495, 500, 504, 509, 513, 518, 523,
    527, 532, 536, 541, 546, 550, 555, 559, 564, 569, 573, 578, 583, 587, 592,
    597, 601, 606, 611, 615, 620, 625, 629, 634, 639, 643, 648, 653, 658, 662,
    667, 672, 677, 681, 686, 691, 696, 701, 705, 710, 715, 720, 725, 0, 1, 1, 2,
    2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 9, 10, 11, 11, 12, 13, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 27, 28, 30, 32, 33, 35, 37, 40, 42,
    45, 47, 50, 53, 57, 60, 64, 67, 71, 75, 79, 83, 87, 91, 95, 99, 104, 108,
    112, 116, 120, 125, 129, 133, 137, 142, 146, 150, 154, 159, 163, 167, 172,
    176, 180, 184, 189, 193, 197, 202, 206, 210, 215, 219, 223, 228, 232, 236,
    241, 245, 250, 254, 258, 262, 267, 271, 276, 280, 284, 289, 293, 297, 302,
    306, 311, 315, 319, 324, 328, 332, 337, 341, 345, 350, 354, 359, 363, 367,
    372, 376, 381, 385, 389, 394, 398, 403, 407, 412, 416, 420, 425, 429, 434,
    438, 443, 447, 452, 456, 461, 465, 470, 474, 479, 483, 488, 492, 497, 501,
    506, 510, 515, 520, 524, 529, 533, 538, 542, 547, 552, 556, 561, 565, 570,
    575, 579, 584, 589, 593, 598, 603, 607, 612, 616, 621, 626, 630, 635, 640,
    645, 649, 654, 659, 663, 668, 673, 678, 682, 687, 692, 697, 701, 706, 711,
    716, 721, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 8, 9, 9, 10, 11,
    11, 12, 13, 13, 14, 15, 16, 17, 18, 19, 19, 21, 22, 23, 24, 25, 27, 28, 30,
    31, 33, 35, 37, 39, 42, 44, 47, 50, 53, 56, 59, 63, 67, 70, 74, 78, 82, 86,
    90, 94, 98, 103, 107, 111, 115, 119, 123, 128, 132, 136, 140, 145, 149, 153,
    157, 162, 166, 170, 174, 179, 183, 187, 192, 196, 200, 205, 209, 213, 218,
    222, 226, 231, 235, 239, 243, 248, 252, 256, 261, 265, 269, 274, 278, 282,
    287, 291, 295, 300, 304, 309, 313, 317, 322, 326, 330, 335, 339, 343, 348,
    352, 356, 361, 365, 370, 374, 378, 383, 387, 391, 396, 400, 405, 409, 414,
    418, 422, 427, 431, 436, 440, 445, 449, 454, 458, 462, 467, 471, 476, 480,
    485, 489, 494, 499, 503, 507, 512, 517, 521, 526, 530, 535, 539, 544, 549,
    553, 558, 562, 567, 571, 576, 581, 585, 590, 594, 599, 604, 608, 613, 618,
    622, 627, 632, 636, 641, 645, 650, 655, 660, 664, 669, 674, 678, 683, 688,
    693, 697, 702, 707, 712, 716, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8,
    8, 9, 9, 10, 11, 11, 12, 13, 13, 14, 15, 16, 17, 17, 18, 19, 20, 21, 23, 24,
    25, 26, 28, 29, 31, 33, 35, 37, 39, 41, 44, 46, 49, 52, 55, 59, 62, 66, 69,
    73, 77, 81, 85, 89, 93, 97, 102, 106, 110, 114, 118, 122, 127, 131, 135, 139,
    143, 147, 152, 156, 160, 165, 169, 173, 177, 181, 186, 190, 194, 199, 203,
    207, 212, 216, 220, 224, 229, 233, 237, 242, 246, 250, 255, 259, 263, 268,
    272, 276, 281, 285, 289, 294, 298, 302, 306, 311, 315, 319, 324, 328, 332,
    337, 341, 345, 350, 354, 359, 363, 367, 372, 376, 380, 385, 389, 393, 398,
    402, 407, 411, 415, 420, 424, 429, 433, 438, 442, 446, 451, 455, 460, 464,
    469, 473, 478, 482, 487, 491, 496, 500, 504, 509, 514, 518, 523, 527, 532,
    536, 541, 545, 550, 554, 559, 563, 568, 573, 577, 582, 586, 591, 596, 600,
    605, 609, 614, 619, 623, 628, 632, 637, 642, 646, 651, 656, 660, 665, 670,
    674, 679, 684, 688, 693, 698, 703, 707, 712, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5,
    5, 6, 6, 7, 8, 8, 9, 9, 10, 11, 11, 12, 13, 13, 14, 15, 16, 17, 17, 18, 19,
    20, 21, 22, 24, 25, 26, 28, 29, 31, 32, 34, 36, 38, 41, 43, 46, 48, 52, 55,
    58, 61, 65, 69, 72, 76, 80, 84, 88, 92, 96, 100, 105, 109, 113, 117, 121,
    125, 130, 134, 138, 142, 146, 151, 155, 159, 163, 167, 172, 176, 180, 184,
    189, 193, 197, 201, 206, 210, 214, 219, 223, 227, 231, 236, 240, 244, 248,
    253, 257, 261, 266, 270, 274, 279, 283, 287, 292, 296, 300, 304, 309, 313,
    317, 322, 326, 330, 335, 339, 343, 347, 352, 356, 361, 365, 369, 374, 378,
    382, 387, 391, 395, 400, 404, 409, 413, 417, 422, 426, 430, 435, 439, 444,
    448, 453, 457, 461, 466, 470, 475, 479, 484, 488, 493, 497, 501, 506, 511,
    515, 519, 524, 528, 533, 538, 542, 547, 551, 556, 560, 565, 569, 574, 578,
    583, 587, 592, 597, 601, 606, 610, 615, 619, 624, 629, 633, 638, 642, 647,
    652, 656, 661, 666, 670, 675, 680, 684, 689, 694, 698, 703, 708, 0, 1, 1, 2,
    2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 10, 11, 12, 13, 13, 14, 15,
    16, 16, 17, 18, 19, 20, 21, 22, 24, 25, 26, 27, 29, 31, 32, 34, 36, 38, 40,
    43, 45, 48, 51, 54, 57, 61, 64, 68, 72, 75, 79, 83, 87, 91, 95, 99, 103, 108,
    112, 116, 120, 124, 128, 132, 137, 141, 145, 149, 153, 158, 162, 166, 170,
    174, 179, 183, 187, 191, 196, 200, 204, 208, 213, 217, 221, 225, 230, 234,
    238, 243, 247, 251, 255, 260, 264, 268, 272, 277, 281, 285, 290, 294, 298,
    302, 307, 311, 315, 320, 324, 328, 332, 337, 341, 345, 350, 354, 358, 363,
    367, 371, 376, 380, 384, 389, 393, 397, 402, 406, 410, 415, 419, 423, 428,
    432, 437, 441, 445, 450, 454, 458, 463, 467, 472, 476, 481, 485, 490, 494,
    498, 503, 507, 512, 516, 521, 525, 530, 534, 539, 543, 548, 552, 557, 561,
    566, 570, 575, 579, 584, 588, 593, 597, 602, 606, 611, 616, 620, 625, 629,
    634, 638, 643, 648, 652, 657, 661, 666, 671, 675, 680, 684, 689, 694, 698,
    703, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 10, 11, 12,
    12, 13, 14, 15, 16, 16, 17, 18, 19, 20, 21, 22, 23, 25, 26, 27, 29, 30, 32,
    34, 36, 38, 40, 42, 45, 47, 50, 53, 57, 60, 63, 67, 71, 74, 78, 82, 86, 90,
    94, 98, 102, 106, 110, 115, 119, 123, 127, 131, 135, 139, 144, 148, 152, 156,
    160, 165, 169, 173, 177, 181, 186, 190, 194, 198, 202, 207, 211, 215, 219,
    224, 228, 232, 237, 241, 245, 249, 253, 258, 262, 266, 271, 275, 279, 283,
    288, 292, 296, 300, 305, 309, 313, 317, 322, 326, 330, 334, 339, 343, 347,
    352, 356, 360, 365, 369, 373, 377, 382, 386, 390, 395, 399, 403, 408, 412,
    416, 421, 425, 429, 434, 438, 443, 447, 451, 456, 460, 464, 469, 473, 478,
    482, 486, 491, 495, 500, 504, 509, 513, 517, 522, 526, 531, 535, 540, 544,
    549, 553, 557, 562, 567, 571, 576, 580, 585, 589, 594, 598, 603, 607, 612,
    616, 621, 625, 630, 634, 639, 643, 648, 652, 657, 662, 666, 671, 675, 680,
    684, 689, 694, 698, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9,
    10, 10, 11, 12, 12, 13, 14, 15, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 26,
    27, 28, 30, 32, 33, 35, 37, 39, 42, 44, 47, 50, 53, 56, 59, 63, 66, 70, 74,
    77, 81, 85, 89, 93, 97, 101, 105, 109, 113, 117, 122, 126, 130, 134, 138,
    142, 146, 151, 155, 159, 163, 167, 172, 176, 180, 184, 188, 193, 197, 201,
    205, 209, 214, 218, 222, 226, 231, 235, 239, 243, 247, 252, 256, 260, 264,
    269, 273, 277, 281, 286, 290, 294, 298, 303, 307, 311, 315, 319, 324, 328,
    332, 337, 341, 345, 349, 354, 358, 362, 366, 371, 375, 379, 384, 388, 392,
    396, 401, 405, 409, 414, 418, 422, 427, 431, 435, 440, 444, 448, 453, 457,
    461, 466, 470, 475, 479, 483, 488, 492, 497, 501, 505, 510, 514, 519, 523,
    527, 532, 536, 541, 545, 550, 554, 558, 563, 567, 572, 576, 581, 585, 590,
    594, 599, 603, 608, 612, 616, 621, 625, 630, 634, 639, 643, 648, 652, 657,
    662, 666, 671, 675, 680, 684, 689, 693, 0, 1, 1, 2, 2, 3, 3, 3, 4, 4, 5, 5,
    6, 6, 7, 7, 8, 9, 9, 10, 10, 11, 12, 12, 13, 14, 15, 15, 16, 17, 18, 19, 20,
    21, 22, 23, 24, 26, 27, 28, 30, 31, 33, 35, 37, 39, 41, 44, 46, 49, 52, 55,
    58, 62, 65, 69, 73, 76, 80, 84, 88, 92, 96, 100, 104, 108, 112, 116, 120,
    125, 129, 133, 137, 141, 145, 149, 153, 158, 162, 166, 170, 174, 178, 183,
    187, 191, 195, 199, 204, 208, 212, 216, 220, 224, 229, 233, 237, 241, 245,
    250, 254, 258, 262, 267, 271, 275, 279, 284, 288, 292, 296, 300, 305, 309,
    313, 317, 322, 326, 330, 334, 338, 343, 347, 351, 355, 360, 364, 368, 372,
    377, 381, 385, 389, 394, 398, 402, 407, 411, 415, 420, 424, 428, 432, 437,
    441, 445, 450, 454, 458, 463, 467, 471, 476, 480, 485, 489, 493, 498, 502,
    506, 511, 515, 519, 524, 528, 533, 537, 541, 546, 550, 555, 559, 563, 568,
    572, 577, 581, 586, 590, 594, 599, 603, 608, 612, 617, 621, 625, 630, 634,
    639, 643, 648, 652, 657, 661, 666, 670, 675, 679, 683, 688, 0, 1, 1, 2, 2, 3,
    3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 10, 11, 12, 12, 13, 14, 15, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 27, 28, 30, 31, 33, 35, 37, 39, 41,
    43, 46, 48, 51, 54, 58, 61, 64, 68, 72, 75, 79, 83, 87, 91, 95, 99, 103, 107,
    111, 115, 119, 123, 127, 131, 136, 139, 144, 148, 152, 156, 160, 164, 169,
    173, 177, 181, 185, 189, 194, 198, 202, 206, 210, 214, 219, 223, 227, 231,
    235, 239, 244, 248, 252, 256, 260, 265, 269, 273, 277, 281, 286, 290, 294,
    298, 302, 307, 311, 315, 319, 323, 328, 332, 336, 340, 345, 349, 353, 357,
    361, 366, 370, 374, 378, 382, 387, 391, 395, 400, 404, 408, 412, 417, 421,
    425, 430, 434, 438, 442, 447, 451, 455, 460, 464, 468, 473, 477, 481, 485,
    490, 494, 499, 503, 507, 512, 516, 520, 525, 529, 533, 538, 542, 546, 551,
    555, 559, 564, 568, 573, 577, 581, 586, 590, 594, 599, 603, 608, 612, 616,
    621, 625, 630, 634, 638, 643, 647, 652, 656, 660, 665, 669, 674, 678, 683, 0,
    1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 10, 11, 12, 12, 13,
    14, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 28, 29, 31, 32, 34,
    36, 38, 40, 43, 45, 48, 51, 54, 57, 60, 63, 67, 71, 74, 78, 82, 86, 90, 94,
    98, 102, 106, 110, 114, 118, 122, 126, 130, 134, 138, 142, 146, 151, 155,
    159, 163, 167, 171, 175, 179, 184, 188, 192, 196, 200, 204, 209, 213, 217,
    221, 225, 229, 234, 238, 242, 246, 250, 254, 259, 263, 267, 271, 275, 279,
    284, 288, 292, 296, 300, 304, 309, 313, 317, 321, 325, 329, 334, 338, 342,
    346, 350, 355, 359, 363, 367, 371, 376, 380, 384, 388, 393, 397, 401, 405,
    409, 414, 418, 422, 426, 431, 435, 439, 443, 448, 452, 456, 461, 465, 469,
    473, 478, 482, 486, 491, 495, 499, 504, 508, 512, 516, 521, 525, 529, 534,
    538, 542, 547, 551, 555, 560, 564, 568, 573, 577, 581, 586, 590, 594, 599,
    603, 607, 611, 616, 620, 625, 629, 633, 638, 642, 646, 651, 655, 659, 664,
    668, 672, 677, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10,
    10, 11, 12, 12, 13, 14, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26,
    28, 29, 31, 32, 34, 36, 38, 40, 42, 45, 47, 50, 53, 56, 59, 63, 66, 70, 73,
    77, 81, 85, 89, 93, 97, 101, 104, 109, 112, 117, 121, 125, 129, 133, 137,
    141, 145, 149, 153, 157, 161, 166, 169, 174, 178, 182, 186, 190, 194, 198,
    203, 207, 211, 215, 219, 223, 227, 232, 236, 240, 244, 248, 252, 257, 261,
    265, 269, 273, 277, 281, 286, 290, 294, 298, 302, 306, 310, 315, 319, 323,
    327, 331, 335, 340, 344, 348, 352, 356, 360, 365, 369, 373, 377, 381, 386,
    390, 394, 398, 402, 407, 411, 415, 419, 423, 428, 432, 436, 440, 444, 449,
    453, 457, 462, 466, 470, 474, 478, 483, 487, 491, 496, 500, 504, 508, 513,
    517, 521, 525, 530, 534, 538, 542, 547, 551, 555, 560, 564, 568, 572, 577,
    581, 585, 589, 594, 598, 602, 606, 611, 615, 619, 624, 628, 632, 636, 641,
    645, 649, 654, 658, 662, 666, 671, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
    7, 7, 8, 8, 9, 10, 10, 11, 12, 12, 13, 14, 14, 15, 16, 17, 18, 19, 19, 20,
    21, 23, 24, 25, 26, 27, 29, 30, 32, 34, 35, 37, 40, 42, 44, 47, 49, 52, 55,
    59, 62, 65, 69, 72, 76, 80, 84, 88, 92, 95, 99, 103, 107, 111, 115, 119, 123,
    127, 131, 135, 140, 144, 148, 152, 156, 160, 164, 168, 172, 176, 180, 184,
    189, 193, 197, 201, 205, 209, 213, 217, 221, 226, 230, 234, 238, 242, 246,
    250, 254, 259, 263, 267, 271, 275, 279, 283, 287, 292, 296, 300, 304, 308,
    312, 316, 320, 325, 329, 333, 337, 341, 345, 349, 354, 358, 362, 366, 370,
    374, 379, 383, 387, 391, 395, 399, 404, 408, 412, 416, 420, 424, 429, 433,
    437, 441, 445, 450, 454, 458, 462, 466, 471, 475, 479, 483, 487, 492, 496,
    500, 504, 509, 513, 517, 521, 525, 530, 534, 538, 542, 546, 551, 555, 559,
    563, 567, 572, 576, 580, 584, 588, 593, 597, 601, 605, 610, 614, 618, 622,
    626, 631, 635, 639, 643, 648, 652, 656, 660, 665, 0, 1, 1, 2, 2, 2, 3, 3, 4,
    4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 12, 12, 13, 14, 14, 15, 16, 17, 18,
    18, 19, 20, 21, 22, 23, 25, 26, 27, 29, 30, 32, 33, 35, 37, 39, 41, 44, 46,
    49, 52, 55, 58, 61, 64, 68, 71, 75, 79, 83, 86, 90, 94, 98, 102, 106, 110,
    114, 118, 122, 126, 130, 134, 138, 142, 146, 150, 154, 158, 162, 166, 171,
    175, 179, 183, 187, 191, 195, 199, 203, 207, 211, 215, 220, 224, 228, 232,
    236, 240, 244, 248, 252, 257, 261, 265, 269, 273, 277, 281, 285, 289, 293,
    297, 302, 306, 310, 314, 318, 322, 326, 330, 334, 339, 343, 347, 351, 355,
    359, 363, 367, 371, 376, 380, 384, 388, 392, 396, 400, 405, 409, 413, 417,
    421, 425, 429, 433, 438, 442, 446, 450, 454, 459, 463, 467, 471, 475, 479,
    483, 488, 492, 496, 500, 504, 508, 513, 517, 521, 525, 529, 533, 538, 542,
    546, 550, 554, 558, 562, 567, 571, 575, 579, 583, 587, 591, 595, 600, 604,
    608, 612, 616, 620, 625, 629, 633, 637, 641, 646, 650, 654, 658, 0, 1, 1, 2,
    2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 11, 12, 13, 14, 14,
    15, 16, 17, 17, 18, 19, 20, 21, 22, 23, 24, 26, 27, 28, 30, 31, 33, 35, 37,
    39, 41, 43, 46, 48, 51, 54, 57, 60, 63, 67, 71, 74, 78, 82, 85, 89, 93, 97,
    101, 105, 109, 113, 117, 121, 125, 129, 133, 137, 141, 145, 149, 153, 157,
    161, 165, 169, 173, 177, 181, 185, 189, 193, 197, 201, 205, 210, 213, 218,
    222, 226, 230, 234, 238, 242, 246, 250, 254, 258, 263, 267, 271, 275, 279,
    283, 287, 291, 295, 299, 303, 307, 311, 315, 320, 323, 328, 332, 336, 340,
    344, 348, 352, 356, 360, 364, 368, 373, 377, 381, 385, 389, 393, 397, 401,
    405, 409, 414, 418, 422, 426, 430, 434, 438, 442, 446, 451, 455, 459, 463,
    467, 471, 475, 479, 483, 488, 492, 496, 500, 504, 508, 512, 516, 520, 524,
    528, 533, 537, 541, 545, 549, 553, 557, 561, 565, 569, 573, 577, 581, 586,
    590, 594, 598, 602, 606, 610, 614, 619, 623, 627, 631, 635, 639, 644, 648,
    652, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 11,
    12, 13, 13, 14, 15, 16, 17, 17, 18, 19, 20, 21, 22, 23, 24, 26, 27, 28, 30,
    31, 33, 34, 36, 38, 40, 43, 45, 47, 50, 53, 56, 59, 63, 66, 70, 73, 77, 80,
    84, 88, 92, 96, 100, 104, 107, 111, 115, 119, 123, 127, 131, 135, 139, 143,
    147, 151, 155, 159, 163, 167, 171, 175, 179, 183, 187, 191, 196, 200, 203,
    208, 212, 216, 220, 224, 228, 232, 236, 240, 244, 248, 252, 256, 260, 264,
    268, 272, 276, 281, 285, 289, 293, 297, 301, 305, 309, 313, 317, 321, 325,
    329, 333, 337, 341, 345, 349, 353, 357, 361, 365, 369, 374, 378, 382, 386,
    390, 394, 398, 402, 406, 410, 414, 418, 422, 426, 430, 434, 438, 443, 447,
    451, 455, 459, 463, 467, 471, 475, 479, 483, 487, 491, 495, 499, 503, 507,
    511, 515, 519, 523, 527, 531, 535, 539, 543, 547, 551, 555, 559, 563, 568,
    572, 576, 580, 584, 588, 592, 596, 600, 604, 608, 612, 617, 621, 625, 629,
    633, 638, 642, 646, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9,
    10, 10, 11, 11, 12, 13, 13, 14, 15, 16, 16, 17, 18, 19, 20, 21, 22, 23, 24,
    25, 27, 28, 29, 31, 32, 34, 36, 38, 40, 42, 44, 47, 50, 52, 55, 58, 62, 65,
    69, 72, 76, 79, 83, 87, 91, 95, 98, 102, 106, 110, 114, 118, 122, 126, 130,
    134, 138, 142, 146, 150, 153, 158, 161, 166, 170, 174, 178, 182, 185, 190,
    194, 198, 202, 206, 210, 214, 218, 222, 226, 230, 234, 238, 242, 246, 250,
    254, 258, 262, 266, 270, 274, 278, 282, 286, 290, 294, 298, 302, 306, 310,
    314, 318, 322, 326, 330, 334, 338, 342, 346, 350, 354, 358, 362, 366, 370,
    374, 378, 382, 386, 390, 394, 398, 402, 406, 410, 414, 418, 422, 426, 430,
    434, 438, 442, 446, 450, 454, 458, 462, 466, 470, 474, 478, 482, 486, 490,
    494, 498, 502, 506, 510, 514, 518, 522, 526, 530, 534, 538, 542, 546, 550,
    554, 558, 562, 566, 570, 574, 578, 582, 586, 590, 594, 598, 602, 606, 611,
    615, 619, 623, 627, 632, 636, 640, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
    7, 7, 8, 8, 9, 10, 10, 11, 11, 12, 13, 13, 14, 15, 16, 16, 17, 18, 19, 20,
    21, 22, 23, 24, 25, 26, 28, 29, 31, 32, 34, 36, 37, 39, 41, 44, 46, 49, 52,
    55, 58, 61, 64, 68, 71, 74, 78, 82, 86, 89, 93, 97, 101, 105, 109, 113, 117,
    120, 124, 128, 132, 136, 140, 144, 148, 152, 156, 160, 164, 168, 172, 176,
    180, 184, 188, 192, 196, 200, 204, 208, 212, 216, 220, 224, 228, 232, 236,
    240, 244, 248, 252, 256, 260, 264, 268, 272, 276, 280, 284, 288, 292, 295,
    299, 303, 307, 311, 315, 319, 323, 327, 331, 335, 339, 343, 347, 351, 355,
    359, 363, 367, 371, 375, 379, 383, 387, 391, 395, 399, 402, 406, 410, 414,
    418, 422, 426, 430, 434, 438, 442, 446, 450, 453, 458, 461, 465, 469, 473,
    477, 481, 485, 489, 493, 497, 501, 504, 508, 512, 516, 520, 524, 528, 532,
    536, 540, 544, 548, 552, 556, 560, 564, 568, 572, 576, 580, 584, 588, 592,
    597, 601, 605, 609, 613, 617, 622, 626, 630, 634, 0, 1, 1, 2, 2, 2, 3, 3, 4,
    4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 11, 11, 12, 13, 13, 14, 15, 16, 16, 17,
    18, 19, 20, 21, 22, 23, 24, 25, 26, 28, 29, 30, 32, 33, 35, 37, 39, 41, 43,
    46, 48, 51, 54, 57, 60, 63, 67, 70, 73, 77, 81, 84, 88, 92, 96, 100, 103,
    107, 111, 115, 119, 123, 127, 131, 135, 139, 142, 146, 150, 154, 158, 162,
    166, 170, 174, 178, 182, 186, 190, 194, 198, 202, 206, 210, 214, 218, 221,
    225, 229, 233, 237, 241, 245, 249, 253, 257, 261, 265, 269, 273, 277, 281,
    285, 289, 293, 297, 301, 304, 308, 312, 316, 320, 324, 328, 332, 336, 340,
    344, 348, 352, 355, 359, 363, 367, 371, 375, 379, 383, 387, 391, 395, 398,
    402, 406, 410, 414, 418, 422, 426, 429, 433, 437, 441, 445, 449, 453, 456,
    460, 464, 468, 472, 476, 480, 483, 487, 491, 495, 499, 503, 507, 511, 514,
    518, 522, 526, 530, 534, 538, 542, 546, 550, 554, 558, 562, 566, 570, 574,
    578, 583, 587, 591, 595, 599, 603, 608, 612, 616, 620, 624, 629, 0, 1, 1, 2,
    2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 11, 11, 12, 13, 13, 14,
    15, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 29, 30, 32, 33, 35,
    37, 39, 41, 43, 45, 48, 50, 53, 56, 59, 62, 66, 69, 72, 76, 79, 83, 87, 91,
    94, 98, 102, 106, 110, 114, 117, 121, 125, 129, 133, 137, 141, 145, 148, 153,
    156, 160, 164, 168, 172, 176, 180, 184, 188, 192, 196, 200, 204, 208, 212,
    215, 219, 223, 227, 231, 235, 239, 243, 247, 251, 255, 259, 263, 266, 270,
    274, 278, 282, 286, 290, 294, 298, 301, 305, 309, 313, 317, 321, 325, 329,
    333, 336, 340, 344, 348, 352, 356, 360, 363, 367, 371, 375, 379, 383, 386,
    390, 394, 398, 402, 405, 409, 413, 417, 421, 425, 428, 432, 436, 440, 443,
    447, 451, 455, 459, 463, 466, 470, 474, 478, 482, 486, 489, 493, 497, 501,
    505, 509, 513, 517, 521, 525, 529, 532, 536, 540, 544, 549, 553, 557, 561,
    565, 569, 573, 577, 581, 585, 590, 594, 598, 602, 606, 611, 615, 619, 623, 0,
    1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 11, 11, 12, 13,
    13, 14, 15, 15, 16, 17, 18, 19, 20, 20, 21, 22, 24, 25, 26, 27, 28, 30, 31,
    33, 34, 36, 38, 40, 42, 44, 47, 50, 52, 55, 58, 61, 65, 68, 71, 75, 78, 82,
    86, 89, 93, 97, 101, 105, 108, 112, 116, 120, 124, 128, 131, 135, 139, 143,
    147, 151, 155, 159, 162, 166, 170, 174, 178, 182, 186, 190, 194, 198, 201,
    205, 209, 213, 217, 221, 225, 229, 233, 237, 240, 244, 248, 252, 256, 260,
    264, 268, 271, 275, 279, 283, 287, 291, 295, 298, 302, 306, 310, 314, 318,
    321, 325, 329, 333, 337, 340, 344, 348, 352, 356, 359, 363, 367, 371, 374,
    378, 382, 386, 389, 393, 397, 401, 404, 408, 412, 416, 419, 423, 427, 431,
    434, 438, 442, 446, 450, 453, 457, 461, 465, 469, 472, 476, 480, 484, 488,
    491, 495, 499, 503, 507, 511, 515, 519, 523, 527, 531, 535, 539, 543, 547,
    551, 555, 559, 563, 568, 572, 576, 580, 584, 588, 593, 597, 601, 605, 610,
    614, 618, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 11,
    11, 12, 12, 13, 14, 15, 15, 16, 17, 18, 19, 19, 20, 21, 22, 23, 25, 26, 27,
    28, 30, 31, 32, 34, 36, 38, 40, 42, 44, 46, 49, 51, 54, 57, 60, 64, 67, 70,
    74, 77, 81, 84, 88, 92, 96, 99, 103, 107, 111, 114, 118, 122, 126, 130, 134,
    137, 141, 145, 149, 153, 157, 161, 164, 168, 172, 176, 180, 184, 188, 191,
    195, 199, 203, 207, 211, 215, 219, 222, 226, 230, 234, 238, 242, 246, 249,
    253, 257, 261, 265, 269, 272, 276, 280, 284, 288, 291, 295, 299, 303, 306,
    310, 314, 318, 322, 325, 329, 333, 336, 340, 344, 348, 351, 355, 359, 363,
    366, 370, 374, 377, 381, 385, 388, 392, 396, 399, 403, 407, 411, 414, 418,
    422, 425, 429, 433, 437, 440, 444, 448, 452, 455, 459, 463, 467, 471, 475,
    478, 482, 486, 490, 494, 498, 502, 506, 510, 514, 518, 522, 526, 530, 534,
    538, 542, 546, 550, 554, 558, 562, 567, 571, 575, 579, 583, 587, 592, 596,
    600, 604, 609, 613, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9,
    9, 10, 11, 11, 12, 12, 13, 14, 15, 15, 16, 17, 18, 18, 19, 20, 21, 22, 23,
    24, 25, 27, 28, 29, 31, 32, 34, 35, 37, 39, 41, 43, 46, 48, 51, 53, 56, 59,
    63, 66, 69, 73, 76, 80, 83, 87, 90, 94, 98, 102, 105, 109, 113, 117, 120,
    124, 128, 132, 136, 139, 143, 147, 151, 155, 159, 162, 166, 170, 174, 178,
    182, 185, 189, 193, 197, 201, 205, 209, 212, 216, 220, 224, 228, 231, 235,
    239, 243, 247, 250, 254, 258, 262, 265, 269, 273, 277, 280, 284, 288, 292,
    295, 299, 303, 307, 310, 314, 318, 321, 325, 329, 332, 336, 340, 343, 347,
    351, 354, 358, 362, 365, 369, 372, 376, 380, 383, 387, 391, 394, 398, 402,
    405, 409, 413, 416, 420, 424, 427, 431, 435, 439, 443, 446, 450, 454, 458,
    462, 465, 469, 473, 477, 481, 485, 489, 493, 497, 501, 505, 509, 513, 517,
    521, 525, 529, 533, 537, 541, 545, 549, 553, 557, 562, 566, 570, 574, 578,
    582, 587, 591, 595, 599, 604, 608, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
    7, 7, 8, 8, 9, 9, 10, 11, 11, 12, 12, 13, 14, 14, 15, 16, 17, 18, 18, 19, 20,
    21, 22, 23, 24, 25, 26, 28, 29, 30, 32, 33, 35, 37, 39, 41, 43, 45, 47, 50,
    53, 55, 58, 62, 65, 68, 71, 75, 78, 82, 85, 89, 93, 96, 100, 104, 108, 111,
    115, 119, 123, 126, 130, 134, 138, 141, 145, 149, 153, 157, 160, 164, 168,
    172, 176, 179, 183, 187, 191, 195, 198, 202, 206, 210, 213, 217, 221, 225,
    229, 232, 236, 240, 244, 247, 251, 255, 259, 262, 266, 270, 273, 277, 281,
    284, 288, 292, 295, 299, 303, 306, 310, 313, 317, 321, 324, 328, 332, 335,
    339, 342, 346, 349, 353, 357, 360, 364, 367, 371, 375, 378, 382, 385, 389,
    393, 396, 400, 404, 407, 411, 415, 419, 422, 426, 430, 434, 437, 441, 445,
    449, 453, 457, 460, 464, 468, 472, 476, 480, 484, 488, 492, 496, 500, 504,
    508, 512, 516, 520, 524, 528, 532, 536, 540, 544, 548, 552, 557, 561, 565,
    569, 573, 578, 582, 586, 590, 594, 599, 603, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5,
    5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 12, 12, 13, 14, 14, 15, 16, 17, 17,
    18, 19, 20, 21, 22, 23, 24, 25, 26, 28, 29, 30, 32, 33, 35, 36, 38, 40, 42,
    44, 47, 49, 52, 55, 57, 60, 64, 67, 70, 73, 77, 80, 84, 88, 91, 95, 99, 102,
    106, 110, 113, 117, 121, 125, 128, 132, 136, 139, 143, 147, 151, 155, 158,
    162, 166, 169, 173, 177, 181, 185, 188, 192, 196, 200, 203, 207, 211, 214,
    218, 222, 226, 229, 233, 237, 241, 244, 248, 252, 255, 259, 262, 266, 270,
    273, 277, 281, 284, 288, 291, 295, 298, 302, 306, 309, 313, 316, 320, 323,
    327, 330, 334, 337, 341, 345, 348, 352, 355, 359, 362, 366, 370, 373, 377,
    380, 384, 388, 391, 395, 399, 402, 406, 410, 414, 417, 421, 425, 429, 432,
    436, 440, 444, 448, 452, 455, 459, 463, 467, 471, 475, 479, 483, 487, 491,
    495, 499, 503, 507, 511, 515, 519, 523, 527, 531, 535, 539, 544, 548, 552,
    556, 560, 564, 569, 573, 577, 581, 585, 590, 594, 598, 0, 1, 1, 2, 2, 2, 3,
    3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 12, 12, 13, 14, 14, 15,
    16, 17, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 29, 30, 31, 33, 34, 36,
    38, 40, 42, 44, 46, 48, 51, 54, 57, 59, 62, 66, 69, 72, 76, 79, 83, 86, 90,
    93, 97, 101, 104, 108, 112, 115, 119, 123, 126, 130, 134, 137, 141, 145, 149,
    152, 156, 160, 164, 167, 171, 175, 178, 182, 186, 189, 193, 197, 201, 204,
    208, 212, 215, 219, 223, 226, 230, 234, 237, 241, 244, 248, 252, 255, 259,
    262, 266, 269, 273, 277, 280, 284, 287, 291, 294, 298, 301, 305, 308, 312,
    315, 318, 322, 325, 329, 333, 336, 340, 343, 347, 350, 354, 357, 361, 365,
    368, 372, 375, 379, 383, 386, 390, 394, 397, 401, 405, 409, 412, 416, 420,
    424, 428, 431, 435, 439, 443, 447, 451, 455, 458, 462, 466, 470, 474, 478,
    482, 486, 490, 494, 498, 502, 506, 510, 514, 518, 522, 527, 530, 535, 539,
    543, 547, 551, 555, 560, 564, 568, 572, 576, 581, 585, 589, 593, 0, 1, 1, 2,
    2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 12, 12, 13, 14,
    14, 15, 16, 17, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 30, 31, 32,
    34, 36, 37, 39, 41, 43, 45, 48, 50, 53, 56, 58, 61, 64, 68, 71, 74, 78, 81,
    85, 88, 92, 95, 99, 103, 106, 110, 113, 117, 121, 124, 128, 132, 135, 139,
    143, 147, 150, 154, 158, 161, 165, 169, 172, 176, 179, 183, 187, 191, 194,
    198, 201, 205, 209, 212, 216, 219, 223, 227, 230, 234, 237, 241, 244, 248,
    251, 255, 258, 262, 265, 269, 272, 276, 279, 282, 286, 289, 293, 296, 300,
    303, 307, 310, 314, 317, 321, 324, 328, 331, 335, 338, 342, 345, 349, 352,
    356, 360, 363, 367, 370, 374, 378, 382, 385, 389, 393, 396, 400, 404, 408,
    411, 415, 419, 423, 427, 431, 434, 438, 442, 446, 450, 454, 458, 462, 466,
    470, 474, 478, 482, 486, 490, 494, 498, 502, 506, 510, 514, 518, 522, 526,
    530, 534, 538, 543, 547, 551, 555, 559, 563, 568, 572, 576, 580, 584, 589, 0,
    1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11, 12, 12,
    13, 14, 14, 15, 16, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
    31, 32, 34, 35, 37, 39, 41, 43, 45, 47, 49, 52, 55, 57, 60, 63, 66, 70, 73,
    76, 80, 83, 87, 90, 94, 97, 101, 104, 108, 111, 115, 119, 122, 126, 130, 133,
    137, 140, 144, 148, 151, 155, 159, 162, 166, 170, 173, 177, 180, 184, 188,
    191, 195, 198, 202, 205, 209, 212, 216, 220, 223, 227, 230, 233, 237, 240,
    244, 247, 251, 254, 258, 261, 264, 268, 271, 275, 278, 281, 285, 288, 292,
    295, 299, 302, 305, 309, 312, 316, 319, 323, 326, 330, 333, 337, 340, 344,
    348, 351, 355, 359, 362, 366, 369, 373, 377, 381, 384, 388, 392, 396, 399,
    403, 407, 411, 414, 418, 422, 426, 430, 434, 438, 442, 445, 449, 453, 457,
    461, 465, 469, 473, 477, 481, 485, 489, 493, 497, 501, 505, 509, 513, 517,
    521, 526, 530, 534, 538, 542, 546, 550, 555, 559, 563, 567, 571, 576, 580,
    584, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9, 10, 10, 11,
    12, 12, 13, 14, 14, 15, 16, 16, 17, 18, 19, 20, 21, 21, 22, 23, 24, 26, 27,
    28, 29, 30, 32, 33, 35, 36, 38, 40, 42, 44, 46, 49, 51, 54, 56, 59, 62, 65,
    68, 72, 75, 78, 82, 85, 88, 92, 96, 99, 103, 106, 110, 113, 117, 120, 124,
    128, 131, 135, 138, 142, 145, 149, 153, 156, 160, 163, 167, 170, 174, 177,
    181, 184, 188, 192, 195, 198, 202, 205, 209, 212, 216, 219, 223, 226, 229,
    233, 236, 240, 243, 246, 250, 253, 256, 260, 263, 266, 270, 273, 277, 280,
    284, 287, 290, 294, 297, 301, 304, 308, 311, 315, 318, 322, 325, 329, 332,
    336, 339, 343, 347, 350, 354, 358, 361, 365, 369, 372, 376, 380, 384, 387,
    391, 395, 399, 402, 406, 410, 414, 418, 422, 425, 429, 433, 437, 441, 445,
    449, 453, 457, 461, 465, 469, 473, 477, 481, 485, 489, 493, 497, 501, 505,
    509, 513, 517, 521, 525, 529, 533, 537, 542, 546, 550, 554, 558, 563, 567,
    571, 575, 580, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8, 8, 9, 9,
    10, 10, 11, 12, 12, 13, 13, 14, 15, 16, 16, 17, 18, 19, 20, 20, 21, 22, 23,
    24, 25, 26, 28, 29, 30, 32, 33, 34, 36, 38, 39, 41, 43, 46, 48, 50, 53, 55,
    58, 61, 64, 67, 70, 73, 77, 80, 83, 87, 90, 94, 97, 101, 104, 108, 111, 115,
    118, 122, 125, 129, 132, 136, 139, 143, 146, 150, 153, 157, 160, 164, 167,
    171, 174, 178, 181, 185, 188, 191, 195, 198, 202, 205, 208, 212, 215, 219,
    222, 225, 229, 232, 235, 239, 242, 245, 249, 252, 255, 259, 262, 265, 269,
    272, 275, 279, 282, 286, 289, 293, 296, 299, 303, 306, 310, 314, 317, 321,
    324, 328, 331, 335, 339, 342, 346, 350, 353, 357, 360, 364, 368, 372, 375,
    379, 383, 387, 391, 394, 398, 402, 406, 410, 413, 417, 421, 425, 429, 433,
    437, 441, 445, 448, 452, 456, 460, 464, 468, 472, 476, 480, 484, 488, 492,
    496, 500, 504, 509, 513, 517, 521, 525, 529, 533, 537, 541, 546, 550, 554,
    558, 562, 567, 571, 575, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7, 8,
    8, 9, 9, 10, 10, 11, 11, 12, 13, 13, 14, 15, 15, 16, 17, 18, 19, 19, 20, 21,
    22, 23, 24, 25, 26, 27, 29, 30, 31, 33, 34, 36, 37, 39, 41, 43, 45, 47, 49,
    52, 54, 57, 60, 63, 66, 69, 72, 75, 78, 82, 85, 88, 92, 95, 99, 102, 105,
    109, 112, 116, 119, 123, 126, 130, 133, 137, 140, 144, 147, 150, 154, 157,
    161, 164, 167, 171, 174, 178, 181, 184, 188, 191, 194, 198, 201, 204, 208,
    211, 214, 218, 221, 224, 227, 231, 234, 237, 241, 244, 247, 251, 254, 257,
    261, 264, 267, 271, 274, 278, 281, 285, 288, 292, 295, 299, 302, 306, 309,
    313, 316, 320, 323, 327, 331, 334, 338, 342, 345, 349, 352, 356, 360, 364,
    367, 371, 375, 379, 382, 386, 390, 394, 398, 402, 405, 409, 413, 417, 421,
    425, 429, 432, 436, 440, 444, 448, 452, 456, 460, 464, 468, 472, 476, 480,
    484, 488, 492, 496, 500, 504, 508, 512, 516, 521, 525, 529, 533, 537, 541,
    545, 550, 554, 558, 562, 566, 571, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6,
    6, 7, 8, 8, 9, 9, 10, 10, 11, 11, 12, 13, 13, 14, 15, 15, 16, 17, 18, 18, 19,
    20, 21, 22, 23, 24, 25, 26, 27, 28, 30, 31, 32, 34, 35, 37, 38, 40, 42, 44,
    46, 49, 51, 53, 56, 59, 61, 64, 67, 70, 74, 77, 80, 83, 87, 90, 93, 97, 100,
    103, 107, 110, 113, 117, 120, 124, 127, 130, 134, 137, 141, 144, 147, 151,
    154, 157, 161, 164, 167, 171, 174, 177, 181, 184, 187, 190, 194, 197, 200,
    203, 207, 210, 213, 216, 220, 223, 226, 230, 233, 236, 240, 243, 246, 250,
    253, 256, 260, 263, 266, 270, 273, 277, 280, 284, 287, 291, 294, 298, 301,
    305, 308, 312, 316, 319, 323, 326, 330, 334, 337, 341, 345, 348, 352, 356,
    360, 363, 367, 371, 375, 378, 382, 386, 390, 394, 397, 401, 405, 409, 413,
    417, 421, 424, 428, 432, 436, 440, 444, 448, 452, 456, 460, 464, 468, 472,
    476, 480, 484, 488, 492, 496, 500, 504, 508, 512, 516, 520, 524, 529, 533,
    537, 541, 545, 549, 554, 558, 562, 566, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5, 5,
    5, 6, 6, 7, 7, 8, 9, 9, 10, 10, 11, 11, 12, 13, 13, 14, 15, 15, 16, 17, 18,
    18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 31, 32, 33, 35, 36, 38, 40,
    42, 44, 45, 48, 50, 52, 55, 57, 60, 63, 66, 69, 72, 75, 78, 81, 85, 88, 91,
    94, 98, 101, 104, 108, 111, 114, 118, 121, 124, 127, 131, 134, 137, 141, 144,
    147, 151, 154, 157, 160, 164, 167, 170, 173, 176, 180, 183, 186, 189, 193,
    196, 199, 202, 206, 209, 212, 215, 219, 222, 225, 228, 232, 235, 238, 242,
    245, 248, 252, 255, 259, 262, 266, 269, 273, 276, 280, 283, 286, 290, 294,
    297, 301, 304, 308, 311, 315, 319, 322, 326, 330, 333, 337, 341, 344, 348,
    352, 356, 359, 363, 367, 371, 374, 378, 382, 386, 390, 393, 397, 401, 405,
    409, 413, 416, 420, 424, 428, 432, 436, 440, 444, 448, 452, 456, 460, 464,
    468, 472, 476, 480, 484, 488, 492, 496, 500, 504, 508, 512, 516, 520, 524,
    528, 533, 537, 541, 545, 549, 553, 557, 562, 0, 1, 1, 2, 2, 2, 3, 3, 4, 4, 5,
    5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 10, 11, 11, 12, 13, 13, 14, 15, 15, 16, 17,
    18, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 32, 33, 34, 36, 37,
    39, 41, 43, 45, 47, 49, 51, 54, 56, 59, 61, 64, 67, 70, 73, 76, 79, 82, 86,
    89, 92, 95, 98, 102, 105, 108, 111, 115, 118, 121, 124, 128, 131, 134, 137,
    141, 144, 147, 150, 153, 156, 160, 163, 166, 169, 172, 175, 179, 182, 185,
    188, 192, 195, 198, 201, 204, 208, 211, 214, 218, 221, 224, 228, 231, 234,
    238, 241, 244, 248, 251, 255, 258, 262, 265, 269, 272, 275, 279, 282, 286,
    289, 293, 297, 300, 304, 308, 311, 315, 318, 322, 326, 329, 333, 337, 340,
    344, 348, 352, 355, 359, 363, 367, 370, 374, 378, 382, 386, 389, 393, 397,
    401, 405, 409, 412, 416, 420, 424, 428, 432, 436, 440, 444, 448, 452, 456,
    459, 463, 467, 471, 475, 479, 483, 487, 492, 495, 500, 504, 508, 512, 516,
    520, 524, 528, 532, 536, 541, 545, 549, 553, 557, 0, 1, 1, 2, 2, 2, 3, 3, 4,
    4, 5, 5, 5, 6, 6, 7, 7, 8, 9, 9, 10, 10, 11, 11, 12, 13, 13, 14, 15, 15, 16,
    17, 17, 18, 19, 20, 21, 22, 23, 23, 24, 25, 27, 28, 29, 30, 31, 33, 34, 36,
    37, 39, 40, 42, 44, 46, 48, 50, 53, 55, 58, 60, 63, 66, 68, 71, 74, 77, 80,
    83, 87, 90, 93, 96, 99, 102, 105, 109, 112, 115, 118, 121, 124, 127, 131,
    134, 137, 140, 143, 146, 149, 152, 156, 159, 162, 165, 168, 171, 174, 178,
    181, 184, 187, 191, 194, 197, 200, 204, 207, 210, 213, 217, 220, 223, 227,
    230, 234, 237, 240, 244, 247, 251, 254, 258, 261, 265, 268, 272, 275, 278,
    282, 286, 289, 293, 296, 300, 304, 307, 311, 315, 318, 322, 325, 329, 333,
    336, 340, 344, 348, 351, 355, 359, 363, 366, 370, 374, 378, 382, 385, 389,
    393, 397, 401, 405, 408, 412, 416, 420, 424, 428, 432, 436, 440, 444, 448,
    451, 455, 459, 463, 467, 471, 475, 479, 483, 487, 491, 495, 499, 504, 508,
    512, 516, 520, 524, 528, 532, 536, 540, 545, 549, 553, 0, 1, 1, 2, 2, 2, 3,
    3, 4, 4, 5, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 11, 12, 13, 13, 14, 14,
    15, 16, 17, 17, 18, 19, 20, 21, 21, 22, 23, 24, 25, 26, 27, 29, 30, 31, 32,
    34, 35, 37, 38, 40, 41, 43, 45, 47, 49, 51, 54, 56, 59, 61, 64, 67, 69, 72,
    75, 78, 81, 84, 87, 90, 93, 96, 99, 102, 105, 108, 111, 115, 118, 121, 124,
    127, 130, 133, 136, 139, 142, 145, 148, 151, 154, 158, 161, 164, 167, 170,
    173, 177, 180, 183, 186, 190, 193, 196, 200, 203, 206, 209, 213, 216, 220,
    223, 226, 230, 233, 236, 240, 243, 247, 250, 254, 257, 261, 264, 268, 271,
    275, 278, 282, 285, 289, 293, 296, 300, 303, 307, 311, 314, 318, 322, 325,
    329, 333, 336, 340, 344, 348, 351, 355, 359, 363, 366, 370, 374, 378, 382,
    385, 389, 393, 397, 401, 405, 408, 412, 416, 420, 424, 428, 432, 436, 440,
    443, 447, 451, 455, 459, 463, 467, 471, 475, 479, 483, 487, 491, 495, 499,
    503, 507, 511, 516, 520, 524, 528, 532, 536, 540, 544, 548, 0, 1, 1, 2, 2, 2,
    3, 3, 4, 4, 5, 5, 5, 6, 6, 7, 7, 8, 8, 9, 10, 10, 11, 11, 12, 12, 13, 14, 14,
    15, 16, 17, 17, 18, 19, 20, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 31, 32,
    33, 35, 36, 38, 39, 41, 43, 44, 46, 48, 50, 53, 55, 57, 60, 62, 65, 67, 70,
    73, 76, 79, 82, 84, 87, 90, 93, 96, 99, 102, 105, 108, 111, 114, 117, 120,
    123, 126, 129, 132, 135, 138, 141, 144, 147, 150, 154, 157, 160, 163, 166,
    169, 173, 176, 179, 182, 186, 189, 192, 196, 199, 202, 206, 209, 212, 216,
    219, 222, 226, 229, 233, 236, 239, 243, 247, 250, 253, 257, 261, 264, 268,
    271, 275, 278, 282, 285, 289, 293, 296, 300, 303, 307, 311, 314, 318, 322,
    325, 329, 333, 336, 340, 344, 348, 351, 355, 359, 363, 366, 370, 374, 378,
    381, 385, 389, 393, 397, 401, 404, 408, 412, 416, 420, 424, 428, 432, 436,
    439, 443, 447, 451, 455, 459, 463, 467, 471, 475, 479, 483, 487, 491, 495,
    499, 503, 507, 511, 515, 519, 524, 528, 532, 536, 540, 544 };

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  emxInit_real_T(&b_X, 2);

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  /*  set up inputs */
  /*  reset X,w so have a group for each input sample */
  i11 = b_X->size[0] * b_X->size[1];
  b_X->size[0] = X->size[0];
  b_X->size[1] = X->size[1];
  emxEnsureCapacity((emxArray__common *)b_X, i11, (int)sizeof(double));
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  loop_ub = X->size[0] * X->size[1];
  for (i11 = 0; i11 < loop_ub; i11++) {
    b_X->data[i11] = X->data[i11];
  }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

  b_emxInit_real_T(&U, 1);
  repmat(b_X, u->size[1], X);
  b_repmat(w, u->size[1], W);
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

  /*  set up U */
  uBlockSize = (double)X->size[1] / (double)u->size[1];
  i11 = U->size[0];
  U->size[0] = X->size[1];
  emxEnsureCapacity((emxArray__common *)U, i11, (int)sizeof(double));
  loop_ub = X->size[1];
  emxFree_real_T(&b_X);
  for (i11 = 0; i11 < loop_ub; i11++) {
    U->data[i11] = 0.0;
  }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

  i = 0;
  emxInit_int32_T(&r787, 2);
  while (i <= u->size[1] - 1) {
    x = ((1.0 + (double)i) * uBlockSize - uBlockSize) + 1.0;
    d2 = (1.0 + (double)i) * uBlockSize;
    if (x > d2) {
      i11 = 0;
      i12 = 0;
    } else {
      i11 = (int)x - 1;
      i12 = (int)d2;
    }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

    loop_ub = r787->size[0] * r787->size[1];
    r787->size[0] = 1;
    r787->size[1] = i12 - i11;
    emxEnsureCapacity((emxArray__common *)r787, loop_ub, (int)sizeof(int));
    loop_ub = i12 - i11;
    for (i12 = 0; i12 < loop_ub; i12++) {
      r787->data[r787->size[0] * i12] = i11 + i12;
    }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

    loop_ub = r787->size[0] * r787->size[1];
    for (i11 = 0; i11 < loop_ub; i11++) {
      U->data[r787->data[i11]] = u->data[i];
    }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

    i++;
  }

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  emxFree_int32_T(&r787);

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  /*  initialize EOL & RUL */
  i11 = EOL->size[0] * EOL->size[1];
  EOL->size[0] = 1;
  emxEnsureCapacity((emxArray__common *)EOL, i11, (int)sizeof(double));
  loop_ub = X->size[1];
  i11 = EOL->size[0] * EOL->size[1];
  EOL->size[1] = loop_ub;
  emxEnsureCapacity((emxArray__common *)EOL, i11, (int)sizeof(double));
  loop_ub = X->size[1];
  for (i11 = 0; i11 < loop_ub; i11++) {
    EOL->data[i11] = 1.0E+99;
  }

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  i11 = RUL->size[0] * RUL->size[1];
  RUL->size[0] = 1;
  RUL->size[1] = EOL->size[1];
  emxEnsureCapacity((emxArray__common *)RUL, i11, (int)sizeof(double));
  loop_ub = EOL->size[0] * EOL->size[1];
  for (i11 = 0; i11 < loop_ub; i11++) {
    RUL->data[i11] = EOL->data[i11];
  }

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  emxInit_real_T(&c_X, 2);

  /*  load the data */
  /*  convert X to SOC values */
  /*  extract states */
  /*  compute apparent SOC */
  /* SOC = qnS/P.qnSMax;	% note that qnSMax already takes the 0.6 factor into consideration */
  /*  compute nominal SOC */
  loop_ub = X->size[1];
  i11 = c_X->size[0] * c_X->size[1];
  c_X->size[0] = 1;
  c_X->size[1] = loop_ub;
  emxEnsureCapacity((emxArray__common *)c_X, i11, (int)sizeof(double));
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  for (i11 = 0; i11 < loop_ub; i11++) {
    c_X->data[c_X->size[0] * i11] = X->data[3 + X->size[0] * i11] + X->data[2 +
      X->size[0] * i11];
  }

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  emxInit_real_T(&SOCs, 2);
  rdivide(c_X, 7600.0, SOCs);
  i = 0;
  emxFree_real_T(&c_X);
  while (i <= SOCs->size[1] - 1) {
    /*  For given SOC, future constant P, given lookup table, determine */
    /*  corresponding RTD */
    /*  load map if not given */
    /*  round SOC */
    uBlockSize = rt_roundd_snf(SOCs->data[i] / 0.005) * 0.005;
    b_bool = false;
    if (input != 'P') {
    } else {
      b_bool = true;
    }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

    if (b_bool) {
      loop_ub = 0;
    } else {
      b_bool = false;
      if (input != 'R') {
      } else {
        b_bool = true;
      }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

      if (b_bool) {
        loop_ub = 1;
      } else {
        b_bool = false;
        if (input != 'I') {
        } else {
          b_bool = true;
        }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

        if (b_bool) {
          loop_ub = 2;
        } else {
          loop_ub = -1;
        }
      }
    }

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
    switch (loop_ub) {
     case 0:
      /*  find the indices */
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      uBlockSize /= 0.005;
      if (uBlockSize < 0.0) {
        x = ceil(uBlockSize);
      } else {
        x = floor(uBlockSize);
      }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);

      uBlockSize = (rt_roundd_snf(U->data[i] / 0.05) * 0.05 - 5.0) / 0.05;
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      if (uBlockSize < 0.0) {
        uBlockSize = ceil(uBlockSize);
      } else {
        uBlockSize = floor(uBlockSize);
      }

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      if ((x + 1.0 < 1.0) || (x + 1.0 > 201.0) || (uBlockSize + 1.0 < 1.0) ||
          (uBlockSize + 1.0 > 401.0)) {
        loop_ub = 54000;
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      } else {
        /*  lookup EOD */
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
// OS_printf("R2U2: prognostics: %s %d: index=%d\n",__FILE__,__LINE__,
//        ((int)(x + 1.0) + 201 * ((int)(uBlockSize + 1.0) - 1)) - 1);
        loop_ub = iv0[((int)(x + 1.0) + 201 * ((int)(uBlockSize + 1.0) - 1)) - 1];
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      }
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      break;

     case 1:
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      /*  find the indices */
      /*  lookup EOD */
      loop_ub = -1;
      break;

     case 2:
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      /*  find the indices */
      /*  lookup EOD */
      loop_ub = -1;
      break;

     default:
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
      loop_ub = 0;
      break;
    }

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
    RUL->data[i] = loop_ub;
    EOL->data[i] = t0 + RUL->data[i];
    i++;
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  }

// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
  emxFree_real_T(&SOCs);
  emxFree_real_T(&U);
// OS_printf("R2U2: prognostics: %s %d\n",__FILE__,__LINE__);
}

/*
 * File trailer for CellConstantInputPredictLookup.c
 *
 * [EOF]
 */
